/*  Wilbur - file indexing and search utility - CD launch program
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// CDWilburDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CDWilbur.h"
#include "CDWilburDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCDWilburDlg dialog

CCDWilburDlg::CCDWilburDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCDWilburDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCDWilburDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CCDWilburDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCDWilburDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCDWilburDlg, CDialog)
	//{{AFX_MSG_MAP(CCDWilburDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_README, OnReadme)
	ON_BN_CLICKED(IDC_INSTALL, OnInstall)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCDWilburDlg message handlers

BOOL CCDWilburDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	CString sTitle = thisApp()->getValue("title");
	if( sTitle.IsEmpty() )
		sTitle = "CD Launcher for Wilbur";
	SetWindowText( sTitle );

	CString sHeading = thisApp()->getValue("heading");

	CString sNumber = thisApp()->getValue("headingheight");
	int nHeight = 10;
	if( !sNumber.IsEmpty() )
		nHeight = atoi( sNumber );

	sNumber = thisApp()->getValue("headingweight");
	int nWeight = 200;
	if( !sNumber.IsEmpty() )
		nWeight = atoi( sNumber );

	bool bItalic = false;
	sNumber = thisApp()->getValue("headingitalic");
	if( !sNumber.IsEmpty() && sNumber != "0" )
		bItalic = true;

	CString sFont = thisApp()->getValue("headingfont");
	if( sFont.IsEmpty() )
		sFont = "Times New Roman";

	LOGFONT lf;                        // Used to create the CFont.
	memset(&lf, 0, sizeof(LOGFONT));   // Clear out structure.

	lf.lfHeight = nHeight;                  // Request a 20-pixel-high font
	lf.lfWeight = nWeight;                 // Bold
	lf.lfItalic = bItalic;

	strcpy(lf.lfFaceName, sFont);
	m_logoFont.CreateFontIndirect(&lf);    // Create the font.
	// Use the font to paint a control. This code assumes
	// a control named IDC_TEXT1 in the dialog box.
	GetDlgItem(IDC_LOGO)->SetFont(&m_logoFont);
	SetDlgItemText(IDC_LOGO, sHeading );


	memset(&lf, 0, sizeof(LOGFONT));   // Clear out structure.

	lf.lfHeight = 40;                  // Request a 20-pixel-high font
	lf.lfWeight = 700;                 // Bold
	lf.lfItalic = TRUE;

	strcpy(lf.lfFaceName, "Times New Roman");
	m_searchFont.CreateFontIndirect(&lf);    // Create the font.
	// Use the font to paint a control. This code assumes
	// a control named IDC_TEXT1 in the dialog box.
	GetDlgItem(IDOK)->SetFont(&m_searchFont);
   
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CCDWilburDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CCDWilburDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CCDWilburDlg::OnReadme() 
{
	CString sPath = thisApp()->startDir() + "\\readme.txt";
	ShellExecute( NULL, NULL, sPath, NULL, NULL, SW_SHOW );	
}

void CCDWilburDlg::OnOK() 
{
	thisApp()->checkInit();	
	
	CDialog::OnOK();
}

void CCDWilburDlg::OnInstall() 
{
	CString sMsg = "This will install the Wilbur program on your hard disk.\n";
	sMsg += "This is not required to view this CD but will let you create your own indexes.\n\n";
	sMsg += "Do you want to continue with this installation?";
	if( AfxMessageBox( sMsg, MB_YESNO ) == IDYES )
	{
		CString sPath = thisApp()->startDir() + "\\wilbur\\WilburSetup.exe";
		ShellExecute( NULL, NULL, sPath, NULL, NULL, SW_SHOW );
	}
}
