/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// contents.cpp : implementation file
//

#include "stdafx.h"

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __CONTENTS_H__
#include "contents.h"
#endif

#ifndef __DIALOGS_H__
#include "dialogs.h"
#endif

#if !defined(AFX_CONTENTSSEARCHDIALOG_H__064E1351_F19B_11D1_B2D6_006008A8E2D0__INCLUDED_)
#include "ContentsSearchDialog.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAutoContents - used for Automation
CAutoContents::CAutoContents( CWilburDoc* pDoc )
{
	m_pDocument = pDoc;
	setInternationalFlag( pDoc->oleGetInternational() );
	setOEMTextFlag( pDoc->oleGetDOSText() );
	setNumberInclusion( (enum CFileIndex::NumberInclusion) pDoc->oleGetNumberHandling() );
}

CAutoContents::~CAutoContents()
{
	m_pDocument = NULL;
}


/////////////////////////////////////////////////////////////////////////////
// CContentsView


IMPLEMENT_DYNCREATE(CContentsView, CView)

CContentsView::CContentsView()
	:
		m_bFont( FALSE ),
		m_nLineHeight( 8 ), m_nAvgCharWidth( 8 ),
		m_nHorizOffset( 0 ), m_dwInternational( CHAR_ENGLISH ),
		m_pszBuffer( NULL ),
		m_pszCursor( NULL ),
		m_pszStartSel( NULL ),
		m_pszTopOfView( NULL ),
		m_pszMouseDown( NULL ),
		m_pszLastMouse( NULL ),
		m_nWheelMoves( 0 ),
		m_bDoubleClick( FALSE ),
		m_bOutline( FALSE ),
		m_bHideHtml( TRUE ),
		m_nWordWrap( 80 ),
		m_nTabWidth( 4)

{
	int nOutline = regGetInt( HKEY_CURRENT_USER,"Contents View", "Outline", 1 );
	m_bOutline = (nOutline != 0);

	int nHideHtml = regGetInt( HKEY_CURRENT_USER,"Contents View", "HideHtml", 1 );
	m_bHideHtml = (nHideHtml != 0);

	int nNotFoundAtTop = regGetInt( HKEY_CURRENT_USER,"Contents View", "NotFoundAtTop", 1 );
	m_bNotFoundAtTop = (nNotFoundAtTop != 0 );

	setCharMask();

	// set the printable non alphabetic characters
	memset( m_bNonAlphaChar, 0, 256 * sizeof( bool ) );

	m_bNonAlphaChar[ 9 ] = true;  // horizontal tab

	for( int i = 32; i <= 47; i++ )
		m_bNonAlphaChar[ i ] = true;

	for( i = 58; i <= 64; i++ )
		m_bNonAlphaChar[ i ] = true;

	for( i = 91; i <= 96; i++ )
		m_bNonAlphaChar[ i ] = true;

	for( i = 123; i <= 126; i++ )
		m_bNonAlphaChar[ i ] = true;

	for( i = 130; i <= 137; i++ )
		m_bNonAlphaChar[ i ] = true;

	m_bNonAlphaChar[ 139 ];

	for( i = 145; i <= 153; i++ )
		m_bNonAlphaChar[ i ] = true;

	m_bNonAlphaChar[ 155 ] = true;

	for( i = 160; i <= 191; i++ )
		m_bNonAlphaChar[ i ] = true;

	m_bNonAlphaChar[ 215 ] = true;
	m_bNonAlphaChar[ 247 ] = true;

	m_htmlCharMap.SetAt("quot", '"' );
    m_htmlCharMap.SetAt("amp", '&' );
    m_htmlCharMap.SetAt("lt", '<' );
    m_htmlCharMap.SetAt("gt", '>' );
	m_htmlCharMap.SetAt("apos", '\'');
    m_htmlCharMap.SetAt("Agrave", '' );
    m_htmlCharMap.SetAt("Aacute", '' );
    m_htmlCharMap.SetAt("Acirc", '' );
    m_htmlCharMap.SetAt("Atilde", '' );
    m_htmlCharMap.SetAt("Auml", '' );
    m_htmlCharMap.SetAt("Aring", '' );
    m_htmlCharMap.SetAt("AElig", '' );
    m_htmlCharMap.SetAt("Ccedil", '' );
    m_htmlCharMap.SetAt("Egrave", '' );
    m_htmlCharMap.SetAt("Eacute", '' );
    m_htmlCharMap.SetAt("Ecirc", '' );
    m_htmlCharMap.SetAt("Euml", '' );
    m_htmlCharMap.SetAt("Igrave", '' );
    m_htmlCharMap.SetAt("Iacute", '' );
    m_htmlCharMap.SetAt("Icirc", '' );
    m_htmlCharMap.SetAt("Iuml", '' );
    m_htmlCharMap.SetAt("ETH", '' );
    m_htmlCharMap.SetAt("Ntilde", '' );
    m_htmlCharMap.SetAt("Ograve", '' );
    m_htmlCharMap.SetAt("Oacute", '' );
    m_htmlCharMap.SetAt("Ocirc", '' );
    m_htmlCharMap.SetAt("Otilde", '' );
    m_htmlCharMap.SetAt("Ouml", '' );
    m_htmlCharMap.SetAt("slash", '' );
    m_htmlCharMap.SetAt("Ugrave", '' );
    m_htmlCharMap.SetAt("Uacute", '' );
    m_htmlCharMap.SetAt("Ucirc", '' );
    m_htmlCharMap.SetAt("Uuml", '' );
    m_htmlCharMap.SetAt("Yacute", '' );
    m_htmlCharMap.SetAt("THORN", '' );
    m_htmlCharMap.SetAt("szlig", '' );
    m_htmlCharMap.SetAt("agrave", '' );
    m_htmlCharMap.SetAt("aacute", '' );
    m_htmlCharMap.SetAt("acirc", '' );
    m_htmlCharMap.SetAt("atilde", '' );
    m_htmlCharMap.SetAt("auml", '' );
    m_htmlCharMap.SetAt("aring", '' );
    m_htmlCharMap.SetAt("aelig", '' );
    m_htmlCharMap.SetAt("ccedil", '' );
    m_htmlCharMap.SetAt("egrave", '' );
    m_htmlCharMap.SetAt("eacute", '' );
    m_htmlCharMap.SetAt("ecirc", '' );
    m_htmlCharMap.SetAt("euml", '' );
    m_htmlCharMap.SetAt("igrave", '' );
    m_htmlCharMap.SetAt("iacute", '' );
    m_htmlCharMap.SetAt("icirc", '' );
    m_htmlCharMap.SetAt("iuml", '' );
    m_htmlCharMap.SetAt("eth", '' );
    m_htmlCharMap.SetAt("tilde", '' );
    m_htmlCharMap.SetAt("ngrave", '' );
    m_htmlCharMap.SetAt("oacute", '' );
    m_htmlCharMap.SetAt("ocirc", '' );
    m_htmlCharMap.SetAt("otilde", '' );
    m_htmlCharMap.SetAt("ouml", '' );
    m_htmlCharMap.SetAt("slash", '' );
    m_htmlCharMap.SetAt("ugrave", '' );
    m_htmlCharMap.SetAt("uacute", '' );
    m_htmlCharMap.SetAt("ucirc", '' );
    m_htmlCharMap.SetAt("uuml", '' );
    m_htmlCharMap.SetAt("yacute", '' );
    m_htmlCharMap.SetAt("thorn", '' );
    m_htmlCharMap.SetAt("yuml", '' );
}

CContentsView::~CContentsView()
{
	if( !m_sTempZipName.IsEmpty() )
	{
		try
		{
			CFileStatus fileStatus;
			if( CFile::GetStatus( m_sTempZipName, fileStatus ) )
			{
				fileStatus.m_attribute &= 0x38;  // clear read, sys and hidden
				CFile::SetStatus( m_sTempZipName, fileStatus );
			}

			CFile::Remove( m_sTempZipName );
			m_sTempZipName.Empty();
		}
		catch( CException* e )
		{
			e->Delete();
		}
	}

	thisApp()->cleanUpTempDir();

	if( m_pszBuffer )
		delete[] m_pszBuffer;
}


BEGIN_MESSAGE_MAP(CContentsView, CView)
	//{{AFX_MSG_MAP(CContentsView)
	ON_WM_CHAR()
	ON_WM_VSCROLL()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_WM_KEYDOWN()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_HSCROLL()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_EDIT_EXPORT, OnEditExport)
	ON_UPDATE_COMMAND_UI(ID_EDIT_EXPORT, OnUpdateEditExport)
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CContentsView drawing

void CContentsView::OnDraw(CDC* pDC)
{
	CFont* pOldFont = pDC->SelectObject( &m_font );
	
	TEXTMETRIC tm;
	if( !m_pszBuffer || !pDC->GetTextMetrics( &tm ) )
	{
		pDC->SelectObject( pOldFont );
		return;
	}

	CRect rtClient;
	GetClientRect( &rtClient );

	CRect rtUpdate;
	pDC->GetClipBox( &rtUpdate );

	COLORREF clrFore;

	if( GetFocus() == this )
		clrFore = RGB( 0, 0, 0 );
	else
		clrFore = ::GetSysColor( COLOR_GRAYTEXT );

	pDC->SetTextColor( clrFore );

	COLORREF clrBack;
	if( m_bOutline )
		clrBack = ::GetSysColor( COLOR_INFOBK );
	else
		clrBack = ::GetSysColor( COLOR_WINDOW );

	pDC->SetBkColor( clrBack );
	CBrush bkBrush( clrBack );
	pDC->FillRect( &rtUpdate, &bkBrush );

	m_nLineHeight = tm.tmHeight;
	m_nAvgCharWidth = tm.tmAveCharWidth;

	char* pszText = m_pszTopOfView;
	char* pszLineStart = pszText;
	int nYPos = 0;
	int nXPos = -m_nHorizOffset * m_nAvgCharWidth + 5;

	COLORREF clrOldBack = pDC->GetBkColor();
	COLORREF clrOldFore = pDC->GetTextColor();

	char* pszLastSel = m_pszCursor;
	if( pszLastSel >= m_pszBuffer ) pszLastSel--;

	if( ( pszLastSel != m_pszStartSel || pszLastSel < pszText ) &&
	      m_bCaretShown && GetFocus() == this )
	{
		m_bCaretShown = FALSE;
		HideCaret();
	}

	if( pszText >= m_pszStartSel && pszText <= pszLastSel && m_pszStartSel <= pszLastSel )
		setSelColor( pDC );

	while( *pszText )
	{
		if( pszText == m_pszStartSel && m_pszStartSel <= pszLastSel )
		{
			// dump line to this point
			if( nYPos >= rtUpdate.top && nYPos <= rtUpdate.bottom )
				pDC->TextOut( nXPos, nYPos, pszLineStart, pszText - pszLineStart );

			CSize size = pDC->GetTextExtent( pszLineStart, pszText - pszLineStart );
			nXPos += size.cx;  // set x start position
			pszLineStart = pszText;
			setSelColor( pDC );
		}

		if( pszText == pszLastSel && m_pszStartSel <= pszLastSel )
			{
				// dump line to this point
				int nChars = pszText - pszLineStart;
				if( *pszText != '\n' && *pszText != '\t' )
					nChars++;

				if( nYPos >= rtUpdate.top && nYPos <= rtUpdate.bottom )
					pDC->TextOut( nXPos, nYPos, pszLineStart, nChars );

				CSize size = pDC->GetTextExtent( pszLineStart, nChars );
				nXPos += size.cx;  // set x start position
				pszLineStart = pszText + 1;
				setColor( pDC, clrOldFore, clrOldBack );
			}

		if( pszText == m_pszStartSel && GetFocus() == this && m_pszStartSel == pszLastSel + 1 )
		{
			// get position
			int nChars = pszText - pszLineStart;

			if( nYPos >= rtUpdate.top && nYPos <= rtUpdate.bottom )
			{
				CSize size = pDC->GetTextExtent( pszLineStart, nChars );
				CPoint ptCaret( nXPos + size.cx, nYPos );
				SetCaretPos( ptCaret );
				if( !m_bCaretShown )
				{
					m_bCaretShown = TRUE;
					ShowCaret();
				}
			}
			else if( m_bCaretShown )
			{
				m_bCaretShown = FALSE;
				HideCaret();
			}
		}

		if( *pszText == '\t' )
		{
			// dump line to this point
			int nChars = pszText - pszLineStart;

			if( nChars < 0 )
				nChars = 0;

			if( nChars > 0 && nYPos >= rtUpdate.top && nYPos <= rtUpdate.bottom )
				pDC->TextOut( nXPos, nYPos, pszLineStart, nChars );

			CSize size = pDC->GetTextExtent( pszLineStart, nChars );
			nXPos += size.cx;

			size =pDC->GetTextExtent( "A", 1 );
			int nTabWidth = size.cx * m_nTabWidth;

			if( nXPos > -nTabWidth )
				nXPos += nTabWidth;  // set x start position
			nXPos = ( nXPos / nTabWidth ) * nTabWidth;
	
			if( m_nHorizOffset )
				nXPos += nTabWidth - ( m_nHorizOffset*m_nAvgCharWidth ) % nTabWidth;
			pszLineStart = pszText + 1;
		}		
		else if( *pszText == '\n' || pszText - pszLineStart > 2 * MaxDisplayOffset )
		{
			if( nYPos >= rtUpdate.top && nYPos <= rtUpdate.bottom && pszText > pszLineStart )
				pDC->TextOut( nXPos, nYPos, pszLineStart, pszText - pszLineStart );

			while( *pszText != '\n' && *pszText ) pszText++;

			if( !*pszText )
				break;

			pszLineStart = pszText + 1;
			nYPos += m_nLineHeight;
			nXPos = -m_nHorizOffset * m_nAvgCharWidth + 5;

			if( nYPos + m_nLineHeight > rtClient.bottom )
			{
				if( pszLastSel + 1 > pszText && m_bCaretShown && GetFocus() == this )
				{
					m_bCaretShown = FALSE;
					HideCaret();
				}

				break;
			}
		}

		pszText++;
	}

	// clean up any dangling lines
	if( nYPos >= rtUpdate.top && nYPos <= rtUpdate.bottom && pszText > pszLineStart )
		pDC->TextOut( nXPos, nYPos, pszLineStart, pszText - pszLineStart );

	pDC->SelectObject( pOldFont );
}

void CContentsView::setSelColor( CDC* pDC )
{
	pDC->SetBkColor( RGB( 0, 0, 128 ) );
	pDC->SetTextColor( RGB( 255, 255, 255 ) );
}

void CContentsView::setColor( CDC* pDC, COLORREF clrFore, COLORREF clrBack )
{
	pDC->SetBkColor( clrBack );
	pDC->SetTextColor( clrFore );
}

/////////////////////////////////////////////////////////////////////////////
// CContentsView diagnostics

#ifdef _DEBUG
void CContentsView::AssertValid() const
{
	CView::AssertValid();
}

void CContentsView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CWilburDoc* CContentsView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWilburDoc)));
	return (CWilburDoc*)m_pDocument;
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CContentsView message handlers

void CContentsView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	m_nTabWidth = regGetInt( HKEY_CURRENT_USER, "Contents View", "TabWidth", 4 );
	m_nWordWrap = regGetInt( HKEY_CURRENT_USER, "Contents View", "WordWrap", 80 );

	SetScrollRange( SB_VERT, 0, 200, FALSE );
	SetScrollRange( SB_HORZ, 0, MaxDisplayOffset,  FALSE );

	if( m_bFont )
		setFont( m_logFont );  // if font was set before window active, give another chance
	else if( !m_font.m_hObject )
	{
		LOGFONT lf;
		::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

		m_font.CreateFontIndirect(&lf);
	}
}

void CContentsView::setTabSize( int nSize )
{
	m_nTabWidth = nSize;
	Invalidate();
}

void CContentsView::setWordWrap( int nChar )
{
	m_nWordWrap = nChar;
	GetDocument()->refreshContents();
}

void CContentsView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{

	CView::OnUpdate( pSender, lHint, pHint );
}

bool CContentsView::readFile( CString sFileName, bool bOutline, int nMaxBuffer )
	// when nMaxBuffer is 0, it is read from registry
{
	CString sLowerFileName = makeLower( sFileName);
	bool bSuccess = false;

	if( !m_sTempZipName.IsEmpty() )
	{
		try
		{
			CFileIndex::removeTemporaryFile( m_sTempZipName, 7);
			m_sTempZipName.Empty();
			thisApp()->removeZipDirectories();
		}
		catch( CException* e )
		{
			m_sTempZipName.Empty();
			e->Delete();
		}
	}

	if( !sFileName.IsEmpty() )
	{
		char currentDir[ MAX_PATH ];
		GetCurrentDirectory( MAX_PATH, currentDir );

		int nZip = CFileIndex::getZipExtIndex(sLowerFileName);
		if( nZip != -1 )
		{
			m_sTempZipName = CFileIndex::getTempZip( sFileName, nZip );
			if(m_sTempZipName.IsEmpty() )
			{
				emptyBuffer();
				return false;
			}

			sFileName = m_sTempZipName;
		}
		
		try
		{
			int nLength = sFileName.GetLength();
			if( nLength && sFileName[ nLength - 1] == '\\' )
			{
				// entry is a directory
				CString sSearchPath = sFileName + "*";

				WIN32_FIND_DATA fileData;
				HANDLE hFind = FindFirstFile( sSearchPath, &fileData );
				if( hFind != INVALID_HANDLE_VALUE )
				{
					int nBufferLength = 10000;

					delete[] m_pszBuffer;
					m_pszBuffer = NULL;
					m_pszBuffer = new char[ nBufferLength ];
					m_pszCursor = m_pszStartSel = m_pszMouseDown = m_pszLastMouse = m_pszBuffer;
					CString sMsg; sMsg.LoadString(IDS_FOLDERHEADER);
					CString sFormat; sFormat.LoadString(IDS_DIRLISTFORMAT);
					const char* pszHeader = (const char*) sMsg;

					setTopOfView( m_pszBuffer, FALSE );

					char* pszNext = m_pszBuffer;
					int nBufferLeft = nBufferLength;
					while( *pszHeader )
					{
						*pszNext++ = *pszHeader++;
						nBufferLeft--;
					}

					do
					{
						if( thisApp()->getHaltStatus() )
						{
							AfxThrowUserException();
						}

						if( nBufferLeft < 100 )
						{
							int nOldLength = nBufferLength;
							nBufferLength *= 2;
							char* pNewBuffer = new char[ nBufferLength ];
							memcpy( pNewBuffer, m_pszBuffer, nOldLength );
							pszNext = pNewBuffer + nOldLength - nBufferLeft;
							nBufferLeft += nOldLength;

							delete[] m_pszBuffer;
							m_pszBuffer = pNewBuffer;
							m_pszCursor = m_pszStartSel = m_pszMouseDown = m_pszLastMouse = m_pszBuffer;

							setTopOfView( m_pszBuffer, FALSE );
						}

						if( fileData.cFileName[0] == '.' &&
							( fileData.cFileName[1] == '\0' ||
							    (fileData.cFileName[1] == '.' && fileData.cFileName[2] == '\0' ) ) )
							continue;

						CTime timeLastWritten( fileData.ftLastWriteTime );
						CString sTimeFormat; sTimeFormat.LoadString(IDS_DATETIMEFORMAT);
						CString sTime = timeLastWritten.Format( sTimeFormat );

						CString sAttrib;
						DWORD dwAttrib = fileData.dwFileAttributes;

						if( dwAttrib & FILE_ATTRIBUTE_SYSTEM )
							sAttrib += 's';

						if( dwAttrib & FILE_ATTRIBUTE_HIDDEN )
							sAttrib += 'h';

						if( dwAttrib & FILE_ATTRIBUTE_DIRECTORY )
							sAttrib += 'd';

						if( dwAttrib & FILE_ATTRIBUTE_READONLY )
							sAttrib += 'r';

						if( dwAttrib & FILE_ATTRIBUTE_COMPRESSED )
							sAttrib += 'c';

						if( dwAttrib & FILE_ATTRIBUTE_ARCHIVE )
							sAttrib += 'a';

						CString sLine;
						sLine.Format(sFormat, sTime, fileData.nFileSizeLow, sAttrib, fileData.cFileName );

						const char* pszLine = sLine;
						while( *pszLine )
						{
							*pszNext++ = *pszLine++;
							nBufferLeft--;
						}

						*pszNext++ = '\n';
						nBufferLeft--;

					} while ( FindNextFile( hFind, &fileData ) );
					
					*pszNext++ = '\0';
					m_nBufferSize = pszNext - m_pszBuffer;

					FindClose( hFind );
				}
			}
			else if( bOutline && GetDocument()->getSearchText().IsEmpty() )
			{
				// no need to read - just clear buffer
				emptyBuffer();
			}
			else if( nLength )
			{
				CString sTempName = sFileName;
				CString sExt = sTempName.Right( 5 );
				sExt.MakeLower();
				bool bPDF = false;
				if (sExt.Find(".pdf") != -1 && thisApp()->hasPDFtoText())
				{
					sTempName = thisApp()->translatePDF(sTempName, false);
					bPDF = true;
				}

				CFile file( sTempName, CFile::modeRead | CFile::shareDenyNone );

				if( m_bHideHtml )
				{
					CString sExt = sFileName.Right( 5 );
					sExt.MakeLower();

					if( sExt.Find(".htm") != -1 || sExt.Find(".xml") != -1 || sExt.Find(".php") != -1 || sExt.Find(".mht") != -1)
						m_bHtml = true;
					else
						m_bHtml = false;
				}
				else
					m_bHtml = false;

				int nFileLength = file.GetLength();
				int nBufferLength = nFileLength + nFileLength / 10;

				if (nMaxBuffer == 0 )
					nMaxBuffer = regGetInt( HKEY_CURRENT_USER, "Contents View", "MaxBuffer", 1000000 );

				if( nBufferLength > nMaxBuffer )
					nBufferLength = nMaxBuffer;
				
				if( nBufferLength < 500 )
					nBufferLength = 500;   // enough for error messages

				delete[] m_pszBuffer;
				m_pszBuffer = NULL;
				m_pszBuffer = new char[ nBufferLength ];
				m_pszCursor = m_pszStartSel = m_pszMouseDown = m_pszLastMouse = m_pszBuffer;
				*m_pszBuffer = '\0';

				setTopOfView( m_pszBuffer, FALSE );

				char* pszNext = m_pszBuffer;
				int nBufferLeft = nBufferLength - 50;  // the 50 is for exhausted or interrupted message

				const int nSegmentSize = 0xffff;
				char buffer[ nSegmentSize ];

				TextBlockSize nBytesRead = 0;
		
				int nSkippingHtml = 0;
				int nTotalRead = 0;
				while( (nBytesRead = file.Read( &buffer, nSegmentSize ) ) )
				{
					int nBytesOut = stripBuffer( buffer, nBytesRead, pszNext, nBufferLeft, nSkippingHtml );
					nBufferLeft -= nBytesOut;
					pszNext += nBytesOut;
					
					nTotalRead += nBytesRead;
					CString sMsg;
					sMsg.Format(IDS_READ_BYTES_FMT, nTotalRead, sFileName );
					thisApp()->setStatus( sMsg );

					if( thisApp()->getHaltStatus() )
					{
						CString sMsg;
						sMsg.LoadString(IDS_READHALTED);
						int nLength = sMsg.GetLength();
						strcpy( pszNext, sMsg );
						pszNext += nLength;

						break;
						// AfxThrowUserException();
					}
					else if( nBufferLeft <= 1 )
					{
						simpleMessageBox(IDS_CONTENTSFULL );

						CString sMsg; sMsg.LoadString(IDS_BUFFERFULL);
						int nLength = sMsg.GetLength();
						strcpy( pszNext, sMsg );
						pszNext += nLength;
						
						break;
					}
				}
				m_nBufferSize = pszNext - m_pszBuffer;

				if( bOutline )
					convertToOutline();

				file.Close();
				if( bPDF )
				{
					if( !thisApp()->usePDFCache() )
						CFileIndex::removeTemporaryFile(sTempName, 7);
				}
			}

			bSuccess = true;
		}
		catch( CException* e )
		{
			emptyBuffer();
			if( m_pszBuffer )
			{
				if( e->IsKindOf( RUNTIME_CLASS( CUserException) ) ) {
					CString sMsg; sMsg.LoadString(IDS_READHALTED);
					strcpy( m_pszBuffer, (const char *) sMsg );
				}
				else
				{
					CString sMsg; sMsg.LoadString(IDS_UNABLETOREAD);
					strcpy( m_pszBuffer, (const char*) sMsg );
					int nLength = strlen( m_pszBuffer );
					e->GetErrorMessage( m_pszBuffer + nLength, 500 - nLength );
				}

				m_nBufferSize = strlen( m_pszBuffer );

				setTopOfView( m_pszBuffer, FALSE );
			}

			e->Delete();
		}

		SetCurrentDirectory( currentDir );
	}
	else
	{
		emptyBuffer();
	}

	return bSuccess;
}

void CContentsView::newFileSelected()
{
	Invalidate();

	CWilburDoc* pDoc = GetDocument();

	readFile( pDoc->getCurrentFile(), m_bOutline != 0 );
	
	if( GetDocument()->isAutomaticQuickView() )
		GetDocument()->quickView();
}

void CContentsView::convertToOutline()
{
	char* pszText = m_pszBuffer;
	BOOL bUseSoundex = GetDocument()->isSoundex();

	CString sSearch = makeLower( GetDocument()->getSearchText() );
	parseSearchText( sSearch );
	const char* pszSearch = sSearch;

	char* pszFrom = searchBuffer( pszText, pszSearch, bUseSoundex, 0, FALSE );;
	while( *pszFrom )
	{
		while( pszFrom > m_pszBuffer && *(pszFrom - 1) != '\n' )
			pszFrom--;

		do
		{
			if( *pszFrom )
				*pszText++ = *pszFrom;
		}
		while( *pszFrom && *pszFrom++ != '\n' );

		pszFrom = searchBuffer( pszFrom, pszSearch, bUseSoundex, 0, FALSE );
	}

	*pszText = '\0';
	m_nBufferSize = pszText - m_pszBuffer;
}

void CContentsView::emptyBuffer()
{
	delete[] m_pszBuffer;
	m_pszBuffer = NULL;
	m_pszBuffer = new char[ 200 ];  // if it can't get this much then I'm dead
	m_nBufferSize = 0;
	m_pszCursor = m_pszStartSel = m_pszTopOfView = m_pszMouseDown = m_pszLastMouse = m_pszBuffer;

	setTopOfView( m_pszBuffer, FALSE );

	strcpy( m_pszBuffer, "" );
}

unsigned char CContentsView::htmlChar( char* pszIn )
{
	unsigned char chIn;

	if( *pszIn == '#' )
	{
		pszIn++;
		int nChar = 0;
		while( (chIn = *pszIn ) && chIn >= '0' && chIn <= '9' )
		{
			nChar = nChar * 10 + chIn - '0';
			pszIn++;
		}

		return (unsigned char) nChar;
	}

	if( m_htmlCharMap.Lookup( pszIn, chIn ) != 0 )
		return chIn;
	else
		return ' ';
}

int CContentsView::stripBuffer( char* pszIn, int nBytesIn, char* pszOut, int nMaxOut, int& nSkippingHtml )
{
	if( nMaxOut ) nMaxOut--;   // leave room for eos
	int nSinceNewLine = 0;
	int nRepeatIntl = 0;
	const int nMaxIntl = 4;

	int  nPhraseLength = 0;
	bool bValid = false;
	bool bNonAlpha = false;
	bool bIntl = false;

	char* pszInitial = pszOut;
	char* pszLastForcedSpace = pszOut;

	bool bText = true;
	bool bRtf = false;
	if( pszIn[ 0 ] == '{' && pszIn[ 1 ] == '\\' && pszIn[ 2 ] == 'r' && pszIn[ 3 ] == 't' && pszIn[ 4 ] == 'f' )
		bRtf = true;
	else
	{
		for( int i = 0; i < 24; i++ )
		{
			unsigned char chIn = pszIn[ i ];
			if( ( chIn < ' ' || chIn > '~' ) && chIn != '\t' && chIn != '\r' && chIn != '\n' )
			{
				bText = FALSE;
				break;
			}
		}
	}

	if( bText && m_bOEMTextChar )
		OemToChar( pszIn, pszIn );

	char* pszLastOut = pszOut;
	unsigned char chIn = '\0';
	bool bSkippedZero = true;
	for( ; nBytesIn-- && nMaxOut; pszIn++ )
	{
		if( *pszIn == '\0' && !bSkippedZero && isValid( chIn ) )
		{
			bSkippedZero = true;
			continue;  // kludge for normal ascii in Unicode double bytes
		}
		else
			bSkippedZero = false;

		unsigned char chLast = chIn;
		chIn = *pszIn;
		if( chIn == 0x80 )
			chIn = ' ';  // Word Perfect uses 0x80 as word separators
		                 // this could be removed if more generic stuff is used in includes

		if( m_bHtml )
		{
			if( chIn == '<' )
			{
				if( nBytesIn > 2 && makeLower((int)*(pszIn + 1)) == 'p' && (*(pszIn + 2) == '>' || *(pszIn + 2) == ' '))
				{
					chIn = '\n';
					pszIn += 2; nBytesIn -= 2;
					while( *pszIn && *pszIn != '>' ) {
						pszIn++; nBytesIn--;
						if( nBytesIn <= 0 ) {
							nSkippingHtml++;
							break;
						}
					}
				}
				else if( nBytesIn > 3 && makeLower((int)*(pszIn + 1)) == 'b' &&
					     makeLower((int)*(pszIn + 2)) == 'r' && *(pszIn + 3) == '>')
				{
					chIn = '\n';
					pszIn += 3;
					nBytesIn -= 3;
				}
				else
					nSkippingHtml++;
			}
			else if( chIn == '>' && nSkippingHtml > 0 )
			{
				nSkippingHtml--;
				if( pszOut <= pszLastOut && nBytesIn )
				{
					unsigned char cNext = *(pszIn + 1);
					unsigned char cLast = cNext;
					while( nBytesIn && ( cNext == '\r' || cNext == '\n' ||
						   cNext == '\t' ||
						   ( cNext == ' ' && chLast == ' ' ) ) )
					{
						pszIn++; nBytesIn--;
						cNext = *(pszIn + 1);
					}

					pszLastOut = pszOut;
				}
				else
					pszLastOut = pszOut + 2;

				continue;
			}

			if( nSkippingHtml )
				continue;

			if( nBytesIn )
			{
				if( chIn == '&' )
				{
					pszIn++; nBytesIn--;
					const nStringMax = 32;
					char pszHtmlString[ nStringMax ];
									
					for( int i = 0; i < nStringMax - 1 && nBytesIn && *pszIn != ';'; i++, pszIn++, nBytesIn-- )
						pszHtmlString[ i ] = *pszIn;

					pszHtmlString[ i ] = '\0';

					chIn = htmlChar( pszHtmlString );
				}
				else if( (chIn & 0xe0) == 0xc0 )  // check for UTF-8 encoding of latin
				{
					pszIn++; nBytesIn--;
					chIn = ((chIn & 0x3) << 6) | (*pszIn & 0x3f);
				}
			}
		}
		else if( bRtf )
		{
			if( chIn == '\\' )
			{
				nBytesIn--; pszIn++;
				if( nBytesIn > 3 && pszIn[0] == 'p' && pszIn[1] == 'a' && pszIn[2] == 'r' )
				{   // paragraph
					chIn = '\n';
					pszIn += 3;
					nBytesIn -= 3;
					while( nBytesIn && *pszIn >= 'a' && *pszIn <= 'z' )
					{
						nBytesIn--; pszIn++;
					}
				}
				else if( nBytesIn  && *pszIn == '\r')
				{  // paragraph another way
					chIn = '\n';
				}
				else if( nBytesIn > 4 && pszIn[0] == 'q' && pszIn[2] == ' ' )
				{
					chIn = chLast;
					nBytesIn -= 2; pszIn += 2;
					continue;
				}
				else if( nBytesIn && *pszIn == '\'' )
				{	// character code
					pszIn++; nBytesIn--;
					if( nBytesIn > 1 )
					{
						char pHex[3];
						pHex[0] = *pszIn++; nBytesIn--;
						pHex[1] = *pszIn;
						pHex[2] = '\0';
						chIn = (char) strtol( pHex, NULL, 16);
					}
				}

				else if( nBytesIn && *pszIn >= 'a' && *pszIn <= 'z' )
				{	// delimited control
					while( nBytesIn && *pszIn >= 'a' && *pszIn <= 'z' )
					{
						nBytesIn--; pszIn++;
					}

					// now figure out delimiter
					if( nBytesIn && *pszIn == ' ')
					{
						nBytesIn--; pszIn++;
					}

					while( nBytesIn && ( *pszIn >= '0' && *pszIn <= '9' ) || *pszIn == '-' )
					{
						nBytesIn--; pszIn++;
					}

					chIn = chLast;
					nBytesIn++; pszIn--;
					continue;
				}
				else if( nBytesIn && *pszIn != '\\')
				{ // not just escaped backslash
					nBytesIn++; pszIn--;
					chIn = ' ';  // non delimited control - just replace with space
				}
			}
			else if( chIn == '{' || chIn == '}' )
			{
				chIn = chLast;
				continue;
			}
			else if( chIn == '\r' )
			{
				if( nBytesIn > 1 && pszIn[1] == '\n' )
					{ nBytesIn--; pszIn++; }
				continue;
			}
		}


		bool bWasValid = bValid;
		bValid = isValid( chIn );

		bool bWasNonAlpha = bNonAlpha;
		bNonAlpha = isNonAlpha( chIn );

		bool bWasIntl = bIntl;
		bIntl = (chIn >= 128 );

		bool bAddChar = false;

		if( bNonAlpha && ( bWasValid || bValid ) )
		{
			bValid = true;
			bNonAlpha = false;
		}

		if( bValid )
		{
			if( bWasValid )
				bAddChar = true;
			else
			{
				if( bWasNonAlpha )
				{
					bWasValid = true;
					bAddChar  = true;
				}
				else
					bAddChar = true;
			}
		}
		else
		{
			if( bNonAlpha )
				bAddChar = true;
			else
			{
				if( bWasNonAlpha )
				{
					while( nPhraseLength && pszOut > pszInitial && isNonAlpha( (unsigned char) *(pszOut - 1) ) )
					{
						pszOut--;
						nMaxOut++;
						nSinceNewLine--;
						nPhraseLength--;
					}
					// nPhraseLength = 0;
				}
				else
				{
					if( !bText && nPhraseLength == 1 )
					{
						if( pszOut > pszInitial )
						{
							pszOut--;
							nMaxOut++;
							nSinceNewLine--;
						}
					}
					else if( nPhraseLength )
					{
						if( chIn != '\r' && chIn != '\n' )
							*pszOut++ = ' ';
						pszLastForcedSpace = pszOut;

						nMaxOut--;
						nSinceNewLine++;
					}

					nPhraseLength = 0;
				}

				if( chIn == '\n' || chIn == '\r' || chIn == 227 )  // 227 for BBS files
				{
					if( chIn == '\r' )
					{
						if( nBytesIn && nMaxOut && *(pszIn + 1) != '\n' )
						{
							*pszOut++ = '\n';
							nMaxOut--;
						}
					}
					else
					{
						*pszOut++ = '\n';
						nMaxOut--;
					}

					nSinceNewLine = 0;
					bValid = true;
				}

			}
		}

		// try to trim repetitive strings of international characters
		// probably not real
		if( chIn < 128 || bValid )
		{
			if( nRepeatIntl >= nMaxIntl )
			{
				while( pszOut > pszInitial && (unsigned char) *(pszOut - 1) >= 128 )
				{
					pszOut--;
					nMaxOut++;
					nSinceNewLine--;
				}
				nPhraseLength = 0;
			}

			nRepeatIntl = 0;
		}
		else
			nRepeatIntl++;

//		if( ( chIn >= ' ' && chIn <= '~' ) || chIn == '\t' ||
//			isValid( chIn ) )
//			( m_bInternational && chIn >= 192 ) ||
//			( chIn >= 145 && chIn <= 148) )
		if( bAddChar )
		{
			*pszOut++ = chIn;
			nMaxOut--;
			nSinceNewLine++;
			nPhraseLength++;
		}

		if( nSinceNewLine > m_nWordWrap && (chIn == ' ' || chIn == '\t' || nPhraseLength == 0) )
		{
			*pszOut++ = '\n';
			nMaxOut--;

			nSinceNewLine = 0;
			bValid = true;
		}
	}

	*pszOut = '\0';
	return pszOut - pszInitial;
}

CString CContentsView::getCursorWord()
{
	CString sWord;

	if( m_pszBuffer )
	{
		char* pszStart = m_pszStartSel;
		char* pszEnd   = m_pszCursor;
		if( pszStart == pszEnd )	
		{
			while( pszStart > m_pszBuffer && isValid( *(pszStart - 1 ) ) )
				pszStart--;

			while( isValid( *pszEnd ) ) pszEnd++;
		}

		sWord = CString( pszStart, pszEnd - pszStart );
	}

	return sWord;
}

void CContentsView::parseSearchText( CString& sSearch )
{
	int nIndex = 0;

	// replace all of the < operators with the { dummy
	while( ( nIndex = sSearch.Find('<') ) >= 1 )
		sSearch.SetAt( nIndex, '{' );

	// look for a '>' character
	while( ( nIndex = sSearch.FindOneOf(">:{") ) >= 1 )
	{
		int nLength = sSearch.GetLength();

		// find preceding term
		int nFirstEnd = nIndex - 1;
		while( nFirstEnd && !isValidSearchChar( sSearch[ nFirstEnd ] ) && sSearch[ nFirstEnd ] != '\"' )
			nFirstEnd--;

		if( nFirstEnd < 1 )
			return;

		// find start of first term
		int nFirstStart = nFirstEnd;

		if( sSearch[ nFirstEnd ] == '\"' )
		{
			while( nFirstStart > 0 && sSearch[ nFirstEnd ] != '\"' ) nFirstStart--;
		}
		else
		{
			while( nFirstStart > 0 && isValidSearchChar( sSearch[ nFirstStart - 1] ) ) nFirstStart--;
		}

		// find start of second term
		int nSecondStart = nIndex + 1;

		// first skip any count numbers
		while( nSecondStart < nLength && sSearch[ nSecondStart ] >= '0' && sSearch[ nSecondStart ] <= '9' )
			nSecondStart++;

		while( nSecondStart < nLength && !isValidSearchChar( sSearch[ nSecondStart ] ) &&
			sSearch[ nSecondStart ] != '\"' ) nSecondStart++;

		if( nSecondStart >= nLength )
			return;

		// find end of second term
		int nSecondEnd = nSecondStart;

		if( sSearch[ nSecondStart ] == '\"' )
		{
			while( nSecondEnd + 1 < nLength && sSearch[ nSecondEnd ] != '\"' ) nSecondEnd++;
		}
		else
		{
			while( nSecondEnd + 1 < nLength && isValidSearchChar( sSearch[ nSecondEnd + 1 ] ) ) nSecondEnd++;
		}
			
		CString sFirst = sSearch.Mid( nFirstStart, nFirstEnd - nFirstStart + 1 );
		CString sSecond = sSearch.Mid( nSecondStart, nSecondEnd - nSecondStart + 1 );
		CString sCount = sSearch.Mid( nIndex + 1, nSecondStart - nIndex - 1 );
		
		CString sNew;
		if( sSearch[ nIndex ] == ':' )
		{
			// bidirectional near - replace with doubled term
			sNew = sSearch.Left( nFirstStart ) + "((" + sFirst + " <" +sCount + sSecond + ")|(" +
						   sSecond + " <" + sCount + sFirst + "))";

		}
		else if( sSearch[ nIndex ] == '>' )
		{
			// just reverse
			sNew = sSearch.Left( nFirstStart ) + '(' + sSecond + " <" + sCount + sFirst + ')';
		}
		else
		{
			sNew = sSearch.Left( nFirstStart ) + '(' + sFirst + " <" + sCount + sSecond + ')';
			//sNew = sSearch.Left( nFirstStart ) + '(' + sFirst + " <" + sCount +
			//	sSearch.Mid( nSecondStart ) + ')';
		}

		if( nSecondEnd < nLength )
			sNew += sSearch.Mid( nSecondEnd + 1 );

		sSearch = sNew;
	}

	// look for quoted strings
	nIndex = sSearch.Find( '\"' );
	if( nIndex != -1 )
	{
		int nLength = sSearch.GetLength();
		int nStart = 0;
		CString sNew;
		while( nIndex < nLength )
		{
			sNew += sSearch.Mid(nStart, nIndex - nStart ) + '(';
			nIndex++;
			while( nIndex < nLength && sSearch[ nIndex ] == ' ' ) nIndex++;

			nStart = nIndex;
			int nEnd = nStart;
			while( nEnd < nLength && sSearch[ nEnd ] != '\"' )
			{
				while( nEnd < nLength && isValidSearchChar( sSearch[ nEnd ] ) ) nEnd++;

				sNew += sSearch.Mid( nStart, nEnd - nStart );

				while( nEnd < nLength && sSearch[ nEnd ] == ' ' ) nEnd++;

				if( nEnd < nLength  && sSearch[ nEnd ] != '\"' )
				{
					sNew += " <1 ";
					while( nEnd < nLength && !isValidSearchChar( sSearch[ nEnd ] ) && sSearch[ nEnd ] != '\"' ) nEnd++;
					nStart = nEnd;
				}
			}

			sNew += ')';
			nIndex = nEnd + 1;
			nStart = nIndex;
			while( nIndex < nLength && sSearch[ nIndex ] != '\"' ) nIndex++;
		}
		
		if( nStart < nLength )
			sNew += sSearch.Mid( nStart );

		sSearch = sNew;
	}
}




BOOL CContentsView::find( CString sSearchText, BOOL bUseSoundex, BOOL bReverse )
{
	if( !m_pszBuffer || sSearchText.IsEmpty() ||
		( bReverse && m_pszStartSel == m_pszBuffer ) )
		return FALSE;

	sSearchText = makeLower( sSearchText );
	parseSearchText( sSearchText );
	const char* pszMaskStart = sSearchText;

	char* pszWord = searchBuffer( m_pszCursor, pszMaskStart, bUseSoundex, 0, bReverse );

	if( *pszWord  && pszWord >= m_pszBuffer )
	{
		invalidateText( m_pszStartSel, m_pszCursor );
		m_pszStartSel = pszWord++;
		while( isValid( *pszWord ) ) pszWord++;
		m_pszCursor = pszWord;

		// is it visible?
		CRect rtClient;
		GetClientRect( &rtClient );
		rtClient.bottom -= m_nLineHeight * 2;
		if( rtClient.bottom < 0 )
			rtClient.bottom = m_nLineHeight;
		
		if( pszWord < getCharAt( rtClient.TopLeft() ) ||
		    pszWord > getCharAt( rtClient.BottomRight() ) )
		{
			// no so put it near middle of view
			int nLines = rtClient.bottom / m_nLineHeight / 2; // move back a half page
			char* pszChar = pszWord;
			while( --pszChar > m_pszBuffer && nLines )
				if( *pszChar == '\n' )
					nLines--;
		
			if( pszChar != m_pszBuffer ) pszChar++;
			setTopOfView( pszChar );
		}

		// do we need horizontal scrolling?
		// what line are we at
		char* pszText = m_pszTopOfView;
		char* pszBegin = pszText;
		int nLines = 0;
		while( pszText < pszWord )
			if( *pszText++	== '\n' )
			{
				pszBegin = pszText;
				nLines++;
			}

		CPoint point = rtClient.BottomRight();
		point.y = nLines * m_nLineHeight + m_nLineHeight / 2;

		int nOldOffset = m_nHorizOffset;
		m_nHorizOffset = 0;

		if( *pszWord == '\n' )
			pszWord--;

		while( getCharAt( point ) < pszWord && m_nHorizOffset < MaxDisplayOffset )
			OnHScroll( SB_PAGEDOWN, 0, NULL );

		if( nOldOffset == m_nHorizOffset )
			invalidateText( m_pszStartSel, m_pszCursor );
		else
		{
			Invalidate();
			SetScrollPos( SB_HORZ, m_nHorizOffset );
		}

		return TRUE;
	}
	else if( bReverse )
	{
		m_pszCursor = m_pszBuffer;
		m_pszStartSel = m_pszCursor;
	}

	else if( pszWord > m_pszBuffer )
	{
		m_pszCursor = pszWord - 1;
		m_pszStartSel = m_pszCursor;
	}


	if( m_bNotFoundAtTop )
		setTopOfView( m_pszBuffer );
	else if( *pszWord )
		setTopOfView( pszWord );
	else
		setToBottom();

	return FALSE;
}

bool CContentsView::containsHit( const CString& sFileName, const char* pszMaskStart, bool bUseSoundex )
{
	bool bFound = false;

	if( readFile( sFileName, false ) )
	{
		char* pszText = m_pszBuffer;
		//bFound = checkBufferForHit( true, m_pszBuffer, pszMaskStart, bUseSoundex );
		if( *(searchBuffer( pszText, pszMaskStart, bUseSoundex, 0, false )) )
			bFound = true;
	}

	return bFound;
}

char* CContentsView::getContentsOf( const CString& sFileName, bool bOutline, int nMaxChars )
{
	readFile( sFileName, bOutline, nMaxChars );
	return m_pszBuffer;
}

bool CContentsView::checkBufferForHit( bool bLHS, char* pszBuffer, const char* pszMaskStart, bool bUseSoundex )
{
	const char* pszMask = pszMaskStart;
	
	while( *pszMask )
	{
		char op = '&';
		bool bRHS = true;

		while( *pszMask && (*pszMask == ' ' || *pszMask == '+' ) ) *pszMask++;

		if( *pszMask == '|' )
		{
			if( bLHS )
				return true;
			op = '|';
			pszMask++;
			while( *pszMask && *pszMask == ' ' ) *pszMask++;
		}

		if( *pszMask == '&' )
		{
			pszMask++;
			op = '&';
			while( *pszMask && *pszMask == ' ' ) *pszMask++;
		}

		if ( !bLHS && op == '&')
			return false;

		if( *pszMask == '(' )
		{
			const char* pszEnd = ++pszMask;
			int nParen = 1;
			int nMaxDepth = 1;

			while( *pszEnd )
			{
				if( *pszEnd == '(' )
					nParen++;
				else if( *pszEnd == ')' )
					nParen--;

				nMaxDepth = max( nMaxDepth, nParen );

				if( nParen == 0 )
				{
					// get contents of paren and call this function to get rhs
					int nLength = pszEnd - pszMask;
					char* pszNewMask = new char[ nLength + 1 ];
					strncpy( pszNewMask, pszMask, nLength );
					pszNewMask[ nLength ] = '\0';

					if( nMaxDepth > 1 ) // more nested parens
						bRHS = checkBufferForHit( true, pszBuffer, pszNewMask, bUseSoundex );
					else
					{
						if( *(searchBuffer( pszBuffer, pszNewMask, bUseSoundex, 0, FALSE )) )
							bRHS = true;
						else
							bRHS = false;
					}

					delete[] pszNewMask;
					pszMask = pszEnd + 1;
					break;
				}
				pszEnd++;
			}
		}
		else
		{
			bRHS = true;
			while( *pszMask && *pszMask != ' ' ) pszMask++;
		}

		if( op == '|' )
			bLHS = (bLHS || bRHS);
		else
			bLHS = (bLHS && bRHS);
	}

	return bLHS;
}

char* CContentsView::searchBuffer( char* pszWord, const char* pszMaskStart,
								   BOOL bUseSoundex, int nNear, BOOL bReverse )
{
	// if nNear is not equal to 0, then only the next nNear words are searched
	// if no match is found in nNear words, then the original pszWord is returned
	// if nNear is 0 then the search continues until a match is found or the end
	char* pszStartWord = pszWord;

	// find the begining of the first word
	if( bReverse )
	{
		if( pszWord > m_pszBuffer ) pszWord--;

		// back up to begining of word if in one
		while( pszWord > m_pszBuffer && isValid( *pszWord ) ) pszWord--;

		// now find end of previous word
		while( pszWord > m_pszBuffer && !isValid( *pszWord ) ) pszWord--;

		// now find begining of that word
		while( pszWord > m_pszBuffer && isValid( *(pszWord - 1) ) ) pszWord--;
	}
	else
		while( *pszWord && !isValid( *pszWord ) ) pszWord++;

	int nWord = 0;
	while( *pszWord )
	{
		const char* pszMask = pszMaskStart;

		// find first term in search string
		while( *pszMask && !isValid( *pszMask ) && *pszMask != '?' && *pszMask != '*' && *pszMask != '+' )
			pszMask++;

		// see if any of the term is the search string match the current word
		int nParen = 0;
		while( isValid(*pszMask) || ( *pszMask && *pszMask != '&' && *pszMask != '<' ) )
		{
			if( !isValid( *pszMask ) )
			{
				if( *pszMask == '(' )
				{
					nParen++;
					pszMask++;
					continue;
				}
				else if( *pszMask == ')' )
				{
					if( nParen == 0 )
						break;
					else
					{
						nParen--;
						pszMask++;
					}
					continue;
				}
			}

			// mask works prepended with + are skipped
			if( ( isValid(*pszMask) || *pszMask != '+' ) && isMatch( pszWord, pszMask, bUseSoundex ) )
			{
				const char* pszSaveMask = pszMask;
				// find end of term
				while( *pszMask && ( isValid( *pszMask ) || *pszMask == '?' || *pszMask == '*')	)
					pszMask++;

				while( *pszMask && !isValid( *pszMask ) && *pszMask != '?' &&
				       *pszMask != '*' && *pszMask != '<')
					pszMask++;

				if( !isValid(*pszMask) && *pszMask == '<' )
				{
					int nNear = GetDocument()->getNear();
					if( *++pszMask >= '0' && *pszMask <= '9' )
					{
						nNear = *pszMask++ - '0';
						while( *pszMask >= '0' && *pszMask <= '9' )
							nNear = nNear*10 + *pszMask++ - '0';
					}

					// find end of term	again
					//while( *pszMask && ( isValid( *pszMask ) || *pszMask == '?' || *pszMask == '*' || *pszMask == '&' )	)
					//	pszMask++;

					// start search at next character so things like end < end work
					char* pszNextWord = searchBuffer( pszWord + 1, pszMask, bUseSoundex, nNear, FALSE );
					if( *pszNextWord && pszNextWord > m_pszBuffer && pszNextWord != pszWord + 1 )
						return pszWord;
					
				}
				else
					return pszWord;

				pszMask = pszSaveMask;
			}

			// if + and followed by ( skip entire term
			if( *pszMask == '+' )
			{
				*pszMask++;
				if( *pszMask == '(' )
				{
					int nParen = 1;
					*pszMask++;

					while( *pszMask && nParen )
					{
						if( *pszMask == '(' )
							nParen++;
						else if( *pszMask == ')' )
							nParen--;

						pszMask++;
					}
				}
			}

			// find end of term
			while( *pszMask && ( isValid( *pszMask ) || *pszMask == '?' || *pszMask == '*' || *pszMask == '+' )	)
				pszMask++;

			// find beginning of next term
			while( *pszMask && !isValid( *pszMask ) && *pszMask != '?' && *pszMask != '*' &&
				   *pszMask != '<'  && *pszMask != '+' )
				pszMask++;

			while( !isValid(*pszMask) && *pszMask == '<' )
			{
				// failed to find first term of near - skip over next term
				pszMask++;

				// skip any count
				while( *pszMask >= '0' && *pszMask <= '9' ) pszMask++;

				// find beginning of next term
				while( *pszMask && !isValid( *pszMask ) && *pszMask != '?' && *pszMask != '*' &&
					   *pszMask != '<'  && *pszMask != '+' )
					pszMask++;

				// now skip to end of next term
				while( *pszMask && ( isValid( *pszMask ) || *pszMask == '?' || *pszMask == '*' || *pszMask == '+' )	)
					pszMask++;

				// find beginning of next term
				while( *pszMask && !isValid( *pszMask ) && *pszMask != '?' && *pszMask != '*' &&
					   *pszMask != '<'  && *pszMask != '+' )
					pszMask++;
			}
		}

		if( bReverse )
		{
			// if already at the beginning then give up
			if( pszWord == m_pszBuffer )
				return --pszWord;

			// find begining of current word
			while( pszWord > m_pszBuffer && isValid( *pszWord ) ) pszWord--;

			// find end of previous word
			while( pszWord > m_pszBuffer && !isValid( *pszWord ) ) pszWord--;

			// find beginning of previous word
			while( pszWord > m_pszBuffer && isValid( *( pszWord - 1 ) ) ) pszWord--;
		}
		else
		{
			// find end of current word
			while( *pszWord && isValid( *pszWord ) ) pszWord++;

			// find beginning of next word
			while( *pszWord && !isValid( *pszWord ) ) pszWord++;
		}

		if( nNear )
		{
			if( nWord++ >= nNear )
				return "";
		}
	}

	return pszWord;
}

void CContentsView::setTopOfView( char* pszTop, BOOL bCanScroll )
{
	if( !m_pszBuffer || !m_hWnd )
		return;

	// back up to beginning of line
	while( pszTop > m_pszBuffer && *(pszTop - 1) != '\n' )
		pszTop--;

	CRect rtClient;
	GetClientRect( &rtClient );
	int nScrollLines = 0;
	int nLinesInWindow = rtClient.bottom / m_nLineHeight;

	char* pszOldTop = m_pszTopOfView;
	m_pszTopOfView = pszTop;

	int nPos = 0;
	if( m_nBufferSize )
	{
		double rPos = ((double) ( pszTop - m_pszBuffer ) ) / m_nBufferSize;
		nPos = (int) ( rPos * 200. );
	}

	SetScrollPos( SB_VERT, nPos );

	if( !bCanScroll || !pszOldTop )
		Invalidate();

	else if( pszTop > pszOldTop )
	{
		char* pszText = pszOldTop;
		while( *pszText && pszText < pszTop && nScrollLines < nLinesInWindow )
		{
			if( *pszText++ == '\n' ) nScrollLines++;
		}

		if( pszText < pszTop ) // true if too far to scroll
			Invalidate();
		else
		{
			ScrollWindow( 0, -nScrollLines*m_nLineHeight );
			while( *pszText && nScrollLines < nLinesInWindow )
			{
				if( *pszText++ == '\n' ) nLinesInWindow--;
			}
			char* pszStartInvalid = pszText;
			

			while( *pszText && nLinesInWindow )
			{
				if( *pszText++ == '\n' ) nLinesInWindow--;
			}

			invalidateText( pszStartInvalid, pszText );
		}
	}
	else  if( pszTop < pszOldTop )
	{
		char* pszText = pszTop;
		while( *pszText && pszText < pszOldTop && nScrollLines < nLinesInWindow )
		{
			if( *pszText++ == '\n' ) nScrollLines++;
		}

		if( pszText < pszOldTop ) // true if too far to scroll
			Invalidate();
		else
		{
			ScrollWindow( 0, nScrollLines*m_nLineHeight );

			invalidateText( pszTop, pszOldTop );
		}
	}
}

BOOL CContentsView::isMatch( const char* pszWord, const char* pszMask, BOOL bUseSoundex )
{
	if( bUseSoundex )
	{
		DWORD sxWord, sxMask;
		CFileIndex::getSoundex( pszWord, (char*) &sxWord );
		CFileIndex::getSoundex( pszMask, (char*) &sxMask );

		return ( sxWord == sxMask );
	}
	else
	{
		for( ;; )
		{
			if( !isValid( *pszMask ) && *pszMask != '?' && *pszMask != '*' )
			{
				if( isValid( *pszWord ) )
					return FALSE;
				else
					return TRUE;
			}

			if( !isValid( *pszWord ) )
			{
				// trim any wildcards - must check isValid in case wild card is used as valid
				while( !isValid(*pszMask) && (*pszMask == '*' || *pszMask == '?') ) pszMask++;

				if( !isValid( *pszMask ) )
					return TRUE;
				else
					return FALSE;
			}

			if( !isValid(*pszMask) && *pszMask == '*' )
			{
				pszMask++;
				if( !isValid( *pszMask ) )
					return TRUE;

				while( isValid( *pszWord ) )
				{
					if( makeLower( *pszWord ) == *pszMask && isMatch( pszWord, pszMask, false ) )
						return TRUE;
					
					pszWord++;
				}

				return FALSE;
			}

			else if( (!isValid(*pszMask) && *pszMask == '?') || *pszMask == makeLower( *pszWord ) )
			{
				pszMask++; pszWord++;
			}

			else
				return FALSE;
		}
	}
}

void CContentsView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if( !thisApp()->isThreadRunning() && ( isValid( nChar ) || nChar == '*' || nChar == '?' ) )
	{
		CContentsSearchDialog dlg;
		dlg.m_pDocument = GetDocument();
		dlg.m_sSearch = nChar;
		dlg.m_bReverse = FALSE;
		dlg.m_sStatus = GetDocument()->getIndexAttributes();

		if( dlg.DoModal() == IDOK )
		{
			GetDocument()->setSearchText( dlg.m_sSearch );

			if( m_bOutline )
				newFileSelected();

			if( find( dlg.m_sSearch, dlg.m_bUseSoundex, dlg.m_bReverse ) )
			{
				CString sStatus;
				sStatus.LoadString( IDS_NEXT_OCCURRENCE );
			 	thisApp()->setStatus( dlg.m_sSearch + sStatus );
			}
			else
			{
				MessageBeep( MB_ICONEXCLAMATION );
				CString sStatus;
				sStatus.LoadString( IDS_NEXT_FILE );
				thisApp()->setStatus( sStatus );
			}
		}
	}
	else
		CView::OnChar(nChar, nRepCnt, nFlags);
}

void CContentsView::setNumberInclusion( CFileIndex::NumberInclusion newValue )
{
	m_numberHandling = newValue;
	setCharMask();
}

void CContentsView::setInternationalFlag( DWORD dwNewValue )
{
	m_dwInternational = dwNewValue;
	setCharMask();
}

void CContentsView::setOEMTextFlag( BOOL bNewValue )
{
	m_bOEMTextChar = bNewValue;
	setCharMask();
}

void CContentsView::setAddedChars( const CString sAdded )
{
	m_sAddedChars = sAdded;
	setCharMask();
}

void CContentsView::setCharMask()
{
	// set the valid character masks

	memset( m_bValidChar, 0, 256 * sizeof( bool ) );

	for( int i = 'a'; i <= 'z'; i++ )
	{
		m_bValidChar[ i ] = true;
		int j = i + 'A' - 'a';
		m_bValidChar[ j ] = true;
	}

	m_bValidChar[ '_' ] = true;

	// numbers are always valid in output, regardless of index setting
	for( i = '0'; i <= '9'; i++ )
		m_bValidChar[ i ] = true;

	if( m_dwInternational & CHAR_ALL )
	{
		for( int i = 191; i < 255; i++ )
			m_bValidChar[ i ] = true;
		for(i = 140; i <= 143; i++)
		{
			m_bValidChar[ i ] = true;
			m_bValidChar[ i + 16 ] = true;
		}
		m_bValidChar[ 138 ] = true;
		m_bValidChar[ 154 ] = true;
		m_bValidChar[ 163 ] = true;
		m_bValidChar[ 165 ] = true;
		m_bValidChar[ 170 ] = true;
		m_bValidChar[ 175 ] = true;
		m_bValidChar[ 179 ] = true;
		m_bValidChar[ 185 ] = true;
		m_bValidChar[ 186 ] = true;
		m_bValidChar[ 188 ] = true;
	}
	else if( m_dwInternational != CHAR_ENGLISH )
	{
		CString sLangChar = addedLangChars( m_dwInternational );
		int nLength = sLangChar.GetLength();
		for( i = 0; i < nLength; i++ )
		{
			BYTE nChar = sLangChar[ i ];
			m_bValidChar[ nChar ] = true;
		}
	}

	int nLength = m_sAddedChars.GetLength();
	for( i = 0; i < nLength; i++ )
	{
		BYTE nChar = m_sAddedChars[ i ];
		m_bValidChar[ nChar ] = true;
	}
}

void CContentsView::setOutline( BOOL bUseOutline )
{
	if( bUseOutline == m_bOutline )
		return;

	m_bOutline = bUseOutline;
	if( !m_pszBuffer )
		return;
	CString sSearch = makeLower( GetDocument()->getSearchText() );
	parseSearchText( sSearch );
	const char* pszSearch = sSearch;
	BOOL bUseSoundex = GetDocument()->isSoundex();

	int nCount = 0;	  // figure out where we are
	char* pszEOL = m_pszCursor;  // use end of current line
	while( *pszEOL && *pszEOL != '\n' )
		pszEOL++;

	char* pszNext = searchBuffer( m_pszBuffer, pszSearch, bUseSoundex, 0, FALSE );
	while( isValid(*pszNext ) ) pszNext++;

	while( *pszNext && pszNext < pszEOL )
	{
		pszNext = searchBuffer( pszNext + 1, pszSearch, bUseSoundex, 0, FALSE );
		while( isValid(*pszNext ) ) pszNext++;

		nCount++;
	}

	newFileSelected();

	while( nCount-- && find( pszSearch, bUseSoundex, FALSE ) );
}

void CContentsView::setHideHtml( bool bHideHtml )
{
	if( bHideHtml == m_bHideHtml )
		return;

	m_bHideHtml = bHideHtml;
	newFileSelected();
}

void CContentsView::changeFont() 
{
	try
	{
		CFont* pFont = GetFont();
		LOGFONT lf;
		if( m_bFont )
			lf = m_logFont;
		else
			::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

		CFontDialog dlg(&lf, CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT);
		if( dlg.DoModal() == IDOK )
		{
			// switch to new font.
			setFont( lf );
		}
	}
	catch( CException* e )
	{
		simpleMessageBox( IDS_SETFONTERR );
		e->Delete();
	}
}

BOOL CContentsView::getFont( LOGFONT& lf )
{
	if( m_bFont )
		lf = m_logFont;

	return m_bFont;
}

void CContentsView::setFont( LOGFONT& lf )
{
	m_bFont = TRUE;
	m_logFont = lf;

	if( IsWindow( m_hWnd ) )
	{
		// switch to new font.
		m_font.DeleteObject();
		if (m_font.CreateFontIndirect(&lf))
		{
			CWaitCursor wait;
			SetFont(&m_font);
		}

		Invalidate();
	}
}

void CContentsView::invalidateText( const char* pszFirst, const char* pszSecond )
{
	CRect rtInvalid;
	GetClientRect( rtInvalid );

	const char* pszStart;
	const char* pszEnd;
	if( pszFirst < pszSecond )
	{
		pszStart = pszFirst;
		pszEnd   = pszSecond;
	}
	else
	{
		pszStart = pszSecond;
		pszEnd   = pszFirst;
	}

	if( pszEnd < m_pszTopOfView )
		return;

	char* pszText = m_pszTopOfView;

	if( pszStart > m_pszTopOfView )
	{
		int nLine = 0;
		while( *pszText && pszText < pszStart && rtInvalid.top < rtInvalid.bottom )
			if( *pszText++ == '\n' )
				rtInvalid.top += m_nLineHeight;

		if( rtInvalid.top >= rtInvalid.bottom )
			return;
	}

	int nBottom = rtInvalid.top;
	
	while( *pszText && pszText < pszEnd && nBottom < rtInvalid.bottom )
		if( *pszText++ == '\n' )
			nBottom += m_nLineHeight;

	rtInvalid.bottom = nBottom + m_nLineHeight;
	InvalidateRect( rtInvalid );
}		 

BOOL CContentsView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	dwStyle |= WS_HSCROLL |WS_VSCROLL;	
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CContentsView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch( nSBCode )
	{
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
		{
			int nOffset = m_nBufferSize / 200 * nPos;
			setTopOfView( m_pszBuffer + nOffset );
			break;
		}

		case SB_PAGEDOWN:
		{
			CRect rtClient;
			GetClientRect( &rtClient );
			int nLines = rtClient.bottom / m_nLineHeight - 1;
			char* pszTop = m_pszTopOfView;
			while( *pszTop && nLines )
				if( *pszTop++ == '\n' )
					nLines--;

			setTopOfView( pszTop );
			break;
		}

		case SB_PAGEUP:
		{
			CRect rtClient;
			GetClientRect( &rtClient );
			int nLines = rtClient.bottom / m_nLineHeight - 1;
			char* pszTop = m_pszTopOfView;
			while( pszTop  > m_pszBuffer && nLines )
				if( *pszTop-- == '\n' )
					nLines--;

			setTopOfView( pszTop );
			break;
		}

		case SB_LINEDOWN:
		{
			char* pszTop = m_pszTopOfView;
			while( *pszTop &&  *pszTop++ != '\n' );

			setTopOfView( pszTop );
			break;
		}

		case SB_LINEUP:
		{
			char* pszTop = m_pszTopOfView;
			while( pszTop  > m_pszBuffer &&  *pszTop-- == '\n' );

			setTopOfView( pszTop );
			break;
		}
	}		
}

BOOL CContentsView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	m_nWheelMoves += zDelta;

	char* pszTop = m_pszTopOfView;

	while( m_nWheelMoves >= WHEEL_DELTA ) // going up
	{
		int nLines = 3;
		while( pszTop  > m_pszBuffer && nLines )
			if( *pszTop-- == '\n' )
				nLines--;

		m_nWheelMoves -= WHEEL_DELTA;
	}

	while( (-m_nWheelMoves) >= WHEEL_DELTA )
	{
		int nLines = 3;
		while( *pszTop && nLines )
			if( *pszTop++ == '\n' )
				nLines--;

		m_nWheelMoves += WHEEL_DELTA;
	}

	if( pszTop != m_pszTopOfView )
		setTopOfView( pszTop );

//		return CView::OnMouseWheel(nFlags, zDelta, pt);

	return TRUE;
}

void CContentsView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	switch( nSBCode )
	{
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
		{
			m_nHorizOffset = nPos;
			break;
		}

		case SB_PAGEUP:
		{
			CRect rtClient;
			GetClientRect( &rtClient );
			int nStep = rtClient.right / m_nAvgCharWidth;
			nStep = nStep / 4;
			m_nHorizOffset -= nStep;
			if( m_nHorizOffset < 0 )
				m_nHorizOffset = 0;
			break;
		}

		case SB_PAGEDOWN:
		{
			CRect rtClient;
			GetClientRect( &rtClient );
			int nStep = rtClient.right / m_nAvgCharWidth;
			nStep = nStep / 4;
			m_nHorizOffset += nStep;
			if( m_nHorizOffset > MaxDisplayOffset )
				m_nHorizOffset = MaxDisplayOffset;
			break;
		}

		case SB_LINEUP:
		{
			if( m_nHorizOffset > 0 )
				m_nHorizOffset--;
			break;
		}

		case SB_LINEDOWN:
		{
			if( m_nHorizOffset < MaxDisplayOffset )
				m_nHorizOffset++;
			break;
		}
	}
	
	SetScrollPos( SB_HORZ, m_nHorizOffset );
	Invalidate();		
}

void CContentsView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if( m_pszStartSel && m_pszCursor )
	{
		SetCapture();
		invalidateText( m_pszStartSel, m_pszCursor );
		m_pszStartSel = m_pszCursor = m_pszLastMouse = m_pszMouseDown = getCharAt( point );
		invalidateText( m_pszStartSel, m_pszCursor );
	}

	CView::OnLButtonDown(nFlags, point);
}

void CContentsView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if( m_idTimer )
	{
		KillTimer( m_idTimer );
		m_idTimer = 0;
	}

	if( GetCapture() == this )
		ReleaseCapture();

	if( m_pszMouseDown && !m_bDoubleClick )
	{
		char* pszMouseUp = getCharAt( point );
		if( pszMouseUp > m_pszMouseDown )
		{
			m_pszCursor   = pszMouseUp;
			m_pszStartSel = m_pszMouseDown;
		}
		else
		{
			m_pszCursor   = m_pszMouseDown;
			m_pszStartSel = pszMouseUp;
		}
	}

	m_bDoubleClick = FALSE;
	
	CView::OnLButtonUp(nFlags, point);
}

void CContentsView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if( m_pszBuffer )
	{
		m_pszCursor = getCharAt( point );
		m_pszStartSel = m_pszCursor;

		while( m_pszStartSel > m_pszBuffer && isValid( *(m_pszStartSel - 1 ) ) )
			m_pszStartSel--;

		while( isValid( *m_pszCursor ) ) m_pszCursor++;

		invalidateText( m_pszStartSel, m_pszCursor );
	}

	m_bDoubleClick = TRUE;    // so the up message doesn't screw things up
	CView::OnLButtonDblClk(nFlags, point);
}

void CContentsView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if( GetCapture() == this && nFlags & MK_LBUTTON )
	{
		if( m_idTimer )
		{
			KillTimer( m_idTimer );
			m_idTimer = 0;
		}

		CRect rtClient;
		GetClientRect( &rtClient );

		if( point.y >= rtClient.bottom )
		{
			m_idTimer = TimerScrollDown;
			SetTimer( m_idTimer, 50, NULL );

			scrollSelDown();
			m_pszLastMouse = getCharAt( point );
		}
		else if( point.y <= 0 )
		{
			m_idTimer = TimerScrollUp;
			SetTimer( m_idTimer, 50, NULL );

			scrollSelUp();
			m_pszLastMouse = m_pszBuffer;
		}
		else
		{
			char* pszNow  = getCharAt( point );

			if( pszNow > m_pszMouseDown )
			{
				m_pszCursor   = pszNow;
				m_pszStartSel = m_pszMouseDown;
			}
			else
			{
				m_pszCursor   = m_pszMouseDown;
				m_pszStartSel = pszNow;
			}

			invalidateText( pszNow, m_pszLastMouse );
			m_pszLastMouse = pszNow;
		}
	}
			
	CView::OnMouseMove(nFlags, point);
}

char* CContentsView::getCharAt( CPoint point )
{
	int nLine = point.y / m_nLineHeight;
	char *pszText = m_pszTopOfView;
	char * pszLineStart = pszText;

	while( *pszText && nLine)
		if( *pszText++ == '\n' )
		{
			nLine--;
			pszLineStart = pszText;
		}
			
	CClientDC dc( this );
	OnPrepareDC( &dc );
	CFont* pOldFont = dc.SelectObject( &m_font );

	pszText = pszLineStart;
	
	if( *pszText == '\n' )
		return pszText;

	int nXPos = -m_nHorizOffset * m_nAvgCharWidth + 5;
	while( *pszText && *pszText != '\n' )
	{
		if( *pszText == '\t' )
		{
			CSize size;
			if( pszText > pszLineStart )
			{
				size = dc.GetTextExtent( pszLineStart, pszText - pszLineStart );
				nXPos += size.cx;
			}

			size = dc.GetTextExtent( "A", 1 );
			int nTabWidth = size.cx * m_nTabWidth;

			if( nXPos > -nTabWidth )
				nXPos += nTabWidth;  // set x start position
			nXPos = ( nXPos / nTabWidth ) * nTabWidth;
	
			if( m_nHorizOffset )
				nXPos += nTabWidth - ( m_nHorizOffset*m_nAvgCharWidth ) % nTabWidth;
			pszLineStart = pszText + 1;
		}
		else
		{
			CSize size = dc.GetTextExtent( pszLineStart, pszText - pszLineStart + 1 );

			if( size.cx + nXPos > point.x )
				break;
		}


		pszText++;
	}

	//if( *pszText == '\n' && pszText > m_pszBuffer )
	//	pszText--;

	dc.SelectObject( pOldFont );

	return pszText;
}



void CContentsView::OnTimer(UINT nIDEvent) 
{
	switch( nIDEvent )
	{
		case TimerScrollDown:
		{
			scrollSelDown();
			break;
		}

		case TimerScrollUp:
		{
 			scrollSelUp();
			break;
		}

		default:
			CView::OnTimer(nIDEvent);
	}
}

void CContentsView::scrollSelDown()
{
	char* pszText = m_pszTopOfView;
	while( *pszText && *pszText++ != '\n' );

	setTopOfView( pszText );

	pszText = m_pszCursor;

	if( *pszText == '\n' ) pszText++;

	while( *pszText && *pszText != '\n' ) pszText++;

	if( pszText > m_pszMouseDown )
		m_pszCursor = pszText;
	else
		m_pszStartSel = pszText;
}

void CContentsView::scrollSelUp()
{
	char* pszText = m_pszTopOfView;
	if( pszText > m_pszBuffer ) pszText--;

	while( pszText > m_pszBuffer && *( pszText - 1) != '\n' ) pszText--;

	setTopOfView( pszText );

	if( pszText > m_pszMouseDown )
		m_pszCursor = pszText;
	else
		m_pszStartSel = pszText;
}

void CContentsView::OnEditCopy() 
{
	OpenClipboard();
	EmptyClipboard();

	char* pszText = m_pszStartSel;
	int nLines = 0;
	while( pszText < m_pszCursor )
		if( *pszText++ == '\n' )
			nLines++;

	HGLOBAL hText = GlobalAlloc( GMEM_MOVEABLE | GMEM_DDESHARE, m_pszCursor - m_pszStartSel + nLines + 1 );
	char* pszClip = (char*) GlobalLock( hText );
	pszText = m_pszStartSel;
	while( pszText < m_pszCursor )
	{
		if( *pszText == '\n' )
			*pszClip++ = '\r';
		*pszClip++ = *pszText++;
	}
	*pszClip = '\0';
			
	SetClipboardData( CF_TEXT, hText );
	GlobalUnlock( hText );
	CloseClipboard();
}

void CContentsView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_pszStartSel < m_pszCursor );	
}


void CContentsView::OnEditExport() 
{
	CString sName = thisApp()->getIndexPath() + "\\Export.txt";

	try
	{
		CStdioFile fileExport( sName, CFile::modeCreate | CFile::modeNoTruncate |
										CFile::modeWrite | CFile::typeText );
		fileExport.SeekToEnd();

		char* pszExport = new char[m_pszCursor - m_pszStartSel + 1 ];
		char* pszClip = pszExport;
		char* pszText = m_pszStartSel;
		while( pszText < m_pszCursor )
			*pszClip++ = *pszText++;

		*pszClip = '\0';

		fileExport.WriteString( pszExport );
		delete pszExport;
	}
	catch( ... )
	{
		CString sMsg; sMsg.LoadString(IDS_EXPORTFILEERR);
		sMsg +=	sName;
		AfxMessageBox( sMsg );
	}
}

void CContentsView::OnUpdateEditExport(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_pszStartSel < m_pszCursor );	
}

void CContentsView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch( nChar )
	{
		case VK_DOWN:
		{
			char* pszText = m_pszTopOfView;
			while( *pszText && *pszText++ != '\n' );

			setTopOfView( pszText );
			break;
		}

		case VK_UP:
		{
			char* pszText = m_pszTopOfView;
			if( pszText > m_pszBuffer ) pszText--;

			while( pszText > m_pszBuffer && *( pszText - 1) != '\n' ) pszText--;

			setTopOfView( pszText );
			break;
		}

		case VK_RIGHT:
			if( m_nHorizOffset < MaxDisplayOffset )
			{
				m_nHorizOffset++;
				SetScrollPos( SB_HORZ, m_nHorizOffset ); 
				Invalidate();
			}

			break;

		case VK_LEFT:
			if( m_nHorizOffset > 0 )
			{
				m_nHorizOffset--; 
				SetScrollPos( SB_HORZ, m_nHorizOffset ); 
				Invalidate();
			}

			break;

		case VK_HOME:
			setTopOfView( m_pszBuffer );
			break;

		case VK_END:
		{
			setToBottom();
			break;
		}

		case VK_NEXT:
		{
			CRect rtClient;
			GetClientRect( &rtClient );
			int nLines = rtClient.bottom / m_nLineHeight - 1;
			
			char* pszText = m_pszTopOfView;
			while( *pszText && nLines > 0 )
				if( *pszText++ == '\n' ) nLines--;

			setTopOfView( pszText );
			break;
		}

		case VK_PRIOR:
		{
			CRect rtClient;
			GetClientRect( &rtClient );
			int nLines = rtClient.bottom / m_nLineHeight - 1;
			
			char* pszText = m_pszTopOfView;
			while( pszText > m_pszBuffer && nLines > 0 )
				if( *pszText-- == '\n' ) nLines--;

			setTopOfView( pszText );
			break;
		}

		case VK_PAUSE:
			GetDocument()->minimizeWindow();
			break;

		case VK_TAB:
			GetDocument()->setFocusOnList();
			break;
	}

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CContentsView::setToBottom()
{
	CRect rtClient;
	GetClientRect( &rtClient );
	int nLines = rtClient.bottom / m_nLineHeight;

	char* pszText = m_pszBuffer + m_nBufferSize - 1;
	while( nLines > 0 && pszText > m_pszBuffer )
		if( *pszText-- == '\n' ) nLines--;

	setTopOfView( pszText );
}

void CContentsView::setCursorToEnd()
{
	m_pszCursor = m_pszBuffer + m_nBufferSize - 1;
	if( m_pszCursor < m_pszBuffer )
		m_pszCursor = m_pszBuffer;

	m_pszStartSel = m_pszCursor;
}

void CContentsView::selectAll()
{
	CWaitCursor wait;

	m_pszCursor = m_pszBuffer + m_nBufferSize - 1;
	if( m_pszCursor < m_pszBuffer )
		m_pszCursor = m_pszBuffer;

	m_pszStartSel = m_pszBuffer;
	Invalidate();
}

void CContentsView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);
	int nWidth = GetSystemMetrics( SM_CXBORDER ) * 2;
	CreateSolidCaret( nWidth, m_nLineHeight );
	m_bCaretShown = FALSE;
	Invalidate();
}

void CContentsView::OnKillFocus(CWnd* pNewWnd) 
{
	if( m_bCaretShown )
	{
		HideCaret();
		m_bCaretShown = FALSE;
	}

	DestroyCaret();
	CView::OnKillFocus(pNewWnd);
	Invalidate();
}


void CContentsView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	CWilburDoc* pDoc = GetDocument();
	pDoc->quickView();
	
	CView::OnRButtonUp(nFlags, point);
}

