/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// contents.h : header file
//

#ifndef __CONTENTS_H__
#define __CONTENTS_H__ 1

#ifndef __WILBUDOC_H__
#include "wilbudoc.h"
#endif

/////////////////////////////////////////////////////////////////////////////
// CContentsView view


#define MaxDisplayOffset 120

class CWilburDoc;

class CContentsView : public CView
{
	protected:
		CContentsView();           // protected constructor used by dynamic creation
		DECLARE_DYNCREATE(CContentsView)

		virtual ~CContentsView();

	// Attributes
	public:
		CWilburDoc* GetDocument();

	// Operations
	public:
		BOOL find( CString sSearchText, BOOL bUseSoundex, BOOL bReverse );
		void newFileSelected();
		void changeFont();
		void selectAll();
		BOOL getFont( LOGFONT& lf );
		void setFont( LOGFONT& lf );
		void setTabSize( int nSize );
		void setWordWrap( int nChar );
		CString getCursorWord();
		bool isValid( unsigned char c );
		bool isValidSearchChar( unsigned char c );
		bool isNonAlpha( unsigned char c );
		DWORD getInternationalFlag() { return m_dwInternational; }
		void setInternationalFlag( DWORD dwNewValue );
		void setNumberInclusion( CFileIndex::NumberInclusion newValue );

		BOOL getOEMTextFlag() { return m_bOEMTextChar; }
		void setOEMTextFlag( BOOL bNewValue );
		void setOutline( BOOL bUseOutline );
		BOOL getOutline() { return m_bOutline; }
		void setHideHtml( bool bHideHtml );
		bool getHideHtml() { return m_bHideHtml; }

		void setAddedChars( const CString sAdded );
		
		CString getTempZipName() { return m_sTempZipName; }
		void emptyBuffer();
		void setToBottom();
		void setCursorToEnd();

		bool containsHit( const CString& sFileName, const char* pszMaskStart, bool bUseSoundex );
		bool checkBufferForHit( bool bLHS, char* pszBuffer, const char* pszMaskStart, bool bUseSoundex );
		void parseSearchText( CString& sSearch );
		char* getContentsOf( const CString& sFileName, bool bOutline, int nMaxChars );

	// Overrides
		// ClassWizard generated virtual function overrides
		//{{AFX_VIRTUAL(CContentsView)
	public:
	virtual void OnInitialUpdate();
	virtual BOOL Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
	protected:
		virtual void OnDraw(CDC* pDC);      // overridden to draw this view
		virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

	// Implementation
	protected:
		#ifdef _DEBUG
			virtual void AssertValid() const;
			virtual void Dump(CDumpContext& dc) const;
		#endif

		// Generated message map functions
	protected:
		//{{AFX_MSG(CContentsView)
		afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnEditCopy();
	afx_msg void OnUpdateEditCopy(CCmdUI* pCmdUI);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnEditExport();
	afx_msg void OnUpdateEditExport(CCmdUI* pCmdUI);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	//}}AFX_MSG
		DECLARE_MESSAGE_MAP()

		CFont   m_font;
		LOGFONT	m_logFont;
		BOOL    m_bFont;
		BOOL    m_bCaretShown;
		BOOL    m_bOutline;
		bool    m_bHideHtml;
		bool    m_bHtml;
		bool    m_bNotFoundAtTop;

		int     m_nLineHeight;
		int     m_nAvgCharWidth;
		int     m_nHorizOffset;

		char*   m_pszBuffer;
		char*   m_pszCursor;
		char*   m_pszStartSel;
		char*   m_pszTopOfView;

		char*   m_pszMouseDown;
		char*   m_pszLastMouse;
		int     m_nWheelMoves;

		int     m_nBufferSize;
		int     m_nTabWidth;
		int     m_nWordWrap;

		DWORD   m_dwInternational;
		BOOL    m_bOEMTextChar;
		bool    m_bValidChar[ 256 ];
		bool    m_bNonAlphaChar[ 256 ];
		CString m_sAddedChars;
		CFileIndex::NumberInclusion m_numberHandling;
		CMap< CString, LPCSTR, unsigned char, unsigned char > m_htmlCharMap;

		UINT    m_idTimer;
		enum { TimerScrollDown = 1, TimerScrollUp } m_timerAction;

		BOOL    m_bDoubleClick;

		CString m_sTempZipName;

		int  stripBuffer( char* pszIn, int nBytesIn, char* pszOut, int nMaxOut, int& nSkippingHtml );
		void convertToOutline();

		void setTopOfView( char* pszTop, BOOL bCanScroll=TRUE );
		void setColor( CDC* pDC, COLORREF clrFore, COLORREF clrBack );
		void setSelColor( CDC* pDC );
		void setCharMask();

		void invalidateText( const char* pszFirst, const char* pszSecond );

		void scrollSelUp();
		void scrollSelDown();

		char* getCharAt( CPoint point );

		BOOL isMatch( const char* pszWord, const char* pszMask, BOOL bUseSoundex );
		bool readFile( CString sFileName, bool bOutline, int nMaxBuffer = 0 );
		char* searchBuffer( char* pszWord, const char* pszMask,
			                BOOL bUseSoundex, int nNear, BOOL bReverse );
		unsigned char htmlChar( char* pszIn );
};

class CAutoContents : public CContentsView
{
	public:
		CAutoContents( CWilburDoc* pDoc );
		virtual ~CAutoContents();
};

inline bool CContentsView::isValid( unsigned char c )
{
	return m_bValidChar[ c ];
}

bool inline CContentsView::isValidSearchChar( unsigned char c )
{
	 return CContentsView::isValid( c ) || c == '*' || c == '?';
}

inline bool CContentsView::isNonAlpha( unsigned char c )
{
	return m_bNonAlphaChar[ c ];
}

#ifndef _DEBUG  // debug version in wilbuvw.cpp
inline CWilburDoc* CContentsView::GetDocument()
   { return (CWilburDoc*)m_pDocument; }
#endif


/////////////////////////////////////////////////////////////////////////////
#endif
