/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// FileListView.cpp : implementation file
//

#include "stdafx.h"
#include <time.h>
#include <direct.h>
#include <io.h>
#include <sys/stat.h>

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __MAINFRM_H__
#include "mainfrm.h"
#endif

#ifndef __FILELISTVIEW_H__
#include "FileListView.h"
#endif

#ifndef __OVERWRITEDIALOG_H__
#include "overwritedialog.h"
#endif

#if !defined(__COMMANDDIALOG_H__)
#include "CommandDialog.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const int SORT_COLUMN_DIVISOR = 0x10000;

/////////////////////////////////////////////////////////////////////////////
// CFileListView

IMPLEMENT_DYNCREATE(CFileListView, CListView)

CFileListView::CFileListView() : m_nItems( 0 ), m_nNextItem( 0 ), m_pResults( NULL ), m_bFont( FALSE )
                                 
{
	m_headerctrl.SetCallback( this, (void (CWnd::*)(int, int))DragColumn );

	for( int i = clmFirst; i <= clmLast; i++)
	{
		m_nSortOrder[ i ] = m_nDisplayOrder[ i ] = i;
		m_bReverseSort[ i ] = false;
		m_bEnabled[ i ] = true;
	}
	
	m_bEnabled[ clmVolume ] = false;
	m_nDisplayOrder[ clmVolume ] = -1;
}

CFileListView::~CFileListView()
{
	if( m_pResults )
		delete[] m_pResults;
}


BEGIN_MESSAGE_MAP(CFileListView, CListView)
	//{{AFX_MSG_MAP(CFileListView)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnItemchanged)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_WM_DESTROY()
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	ON_COMMAND(ID_FILE_EXPORTLIST, OnFileExportlist)
	ON_COMMAND(ID_FILE_COPY, OnFileCopy)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetdispinfo)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_COLUMN_NAME, OnColumnName)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_NAME, OnUpdateColumnName)
	ON_COMMAND(ID_COLUMN_ATTRIB, OnColumnAttrib)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_ATTRIB, OnUpdateColumnAttrib)
	ON_COMMAND(ID_COLUMN_FOLDER, OnColumnFolder)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_FOLDER, OnUpdateColumnFolder)
	ON_COMMAND(ID_COLUMN_RANK, OnColumnRank)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_RANK, OnUpdateColumnRank)
	ON_COMMAND(ID_COLUMN_SIZE, OnColumnSize)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_SIZE, OnUpdateColumnSize)
	ON_COMMAND(ID_COLUMN_TIME, OnColumnTime)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_TIME, OnUpdateColumnTime)
	ON_COMMAND(ID_COLUMN_TYPE, OnColumnType)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_TYPE, OnUpdateColumnType)
	ON_COMMAND(ID_COLUMN_VOLUME, OnColumnVolume)
	ON_UPDATE_COMMAND_UI(ID_COLUMN_VOLUME, OnUpdateColumnVolume)
	ON_COMMAND(ID_FILE_COMMAND, OnFileCommand)
	ON_UPDATE_COMMAND_UI(ID_FILE_COPY, OnUpdateFileCopy)
	ON_UPDATE_COMMAND_UI(ID_FILE_COMMAND, OnUpdateFileCommand)
	ON_COMMAND(ID_FILE_PRINTLIST, OnFilePrintList)
	ON_NOTIFY_REFLECT(LVN_ODFINDITEM, OnOdfinditem)
	ON_COMMAND(ID_FILE_LIST_CLIP, OnFileListClip)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFileListView drawing

void CFileListView::OnDraw(CDC* pDC)
{
	// used for printing only
	int nTabs[ clmLast + 1 ];

	CRect rtPrint;
	pDC->GetClipBox( rtPrint );
	CSize szPrint( rtPrint.Width(), rtPrint.Height() );

	// find out total of column pixels
	int nScreenPixels = 0;
	int nCol = 0;

	for( int nClm = 0; nClm <= clmLast; nClm++ )
	{
		LV_COLUMN lvColumn;
		lvColumn.mask = LVCF_WIDTH;

		int nDisplay = m_nDisplayOrder[ nClm ];
		if( nDisplay >= 0 )
		{
			if( GetListCtrl().GetColumn( nCol, &lvColumn ) )
				m_columns[ nDisplay ].m_nWidth = lvColumn.cx;

			nScreenPixels += m_columns[ nDisplay ].m_nWidth;
			nCol++;
		}
	}
		

	CString sHeader;
	nCol = 0;
	int nTabStop = 0;
	for( nClm = 0; nClm <= clmLast; nClm++ )
	{
		int nDisplay = m_nDisplayOrder[ nClm ];
		if( nDisplay >= 0 )
		{
			nTabStop += m_columns[ nDisplay ].m_nWidth * szPrint.cx / nScreenPixels;
			nTabs[ nCol ] = nTabStop;
			sHeader += m_columns[ nDisplay ].m_sLabel + '\t';
			nCol++;
		}
	}
	
	int nTopOfLine = 0;
	CSize size = pDC->TabbedTextOut( 100, nTopOfLine, sHeader, nCol, nTabs, 0 );
	nTopOfLine += size.cy;
	pDC->MoveTo( 100, size.cy );
	pDC->LineTo( szPrint.cx, size.cy );
	nTopOfLine += size.cy;

	int nSelected = GetListCtrl().GetSelectedCount();

	for( int nItem = 0; nItem < m_nItems; nItem++ )
	{
		if( nSelected < 2 || GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
		{

			CString sLine = getItemTextLine( nItem );
			size = pDC->TabbedTextOut( 100, nTopOfLine, sLine, nCol, nTabs, 0 );
			nTopOfLine += size.cy;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CFileListView diagnostics

#ifdef _DEBUG
void CFileListView::AssertValid() const
{
	CListView::AssertValid();
}

void CFileListView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}
#endif //_DEBUG

CWilburDoc* CFileListView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWilburDoc)));
	return (CWilburDoc*)m_pDocument;
}

/////////////////////////////////////////////////////////////////////////////
// CFileListView operations

void CFileListView::clearList()
{
	CListCtrl& ctlList = GetListCtrl();
	ctlList.DeleteAllItems();

	setCount( 0 );

	UpdateWindow();
}

void CFileListView::setCount( int nCount )
{
	GetListCtrl().SetItemCount( nCount );
	delete[] m_pResults;
	m_pResults = NULL;

	m_nItems = 0;
	m_nNextItem = 0;
	try
	{
		m_pResults = new CFileResult[ nCount ];
		m_nItems = nCount;
	}
	catch( CMemoryException* e )
	{
		e->Delete();
		m_pResults = NULL;
		// there should be enough memory now
		simpleMessageBox( IDS_SEARCH_MEM_ERR );
		AfxThrowUserException();
	}
}

void CFileListView::setActualListCount()
{
	// must be the actual number of items added
	m_nItems = m_nNextItem;
	GetListCtrl().SetItemCount( m_nItems );
}

void CFileListView::addFile( CFileResult& fileResult )
{
	ASSERT( m_nNextItem < m_nItems );

	if( *fileResult.m_pFileInfo->m_pszName != (char) 0xff )  // check to see if marked obsolete
		m_pResults[ m_nNextItem++ ] = fileResult;
}

char* CFileListView::getLeafName( char* pPathName )
{
	int nLength = strlen( pPathName );

	while( nLength )
	{
		char* pName = pPathName + nLength;
		if( *pName == '/' || *pName == '\\' || *pName == ':' )
		{
			nLength += 1;
			break;
		}

		nLength--;
	}

	return pPathName + nLength;
}

int CFileListView::compareFileResult( CFileResult* pResult1, CFileResult* pResult2,
	        								   int nSortColumn, bool bReverseSort )
{
	CFileInfo* pFile1 = pResult1->m_pFileInfo;
	CFileInfo* pFile2 = pResult2->m_pFileInfo;

	CFileInfo* pDir1 = pResult1->m_pDirInfo;
	CFileInfo* pDir2 = pResult2->m_pDirInfo;

	int nReverseSort = bReverseSort ? -1 : 1;

	switch( nSortColumn )
	{
		case clmName:
		{
			if( pDir1 == NULL )
				return -nReverseSort;
			else if( pDir2 == NULL )
				return nReverseSort;

			char* pszText1 = pFile1->m_pszName;
			char* pszText2 = pFile2->m_pszName;
			return nReverseSort * _stricoll( pszText1, pszText2 );
		}

		case clmRank:
		{
			return nReverseSort * (	pResult2->m_nRank - pResult1->m_nRank );
		}

		case clmExtension:
		{
			if( pDir1 == NULL )
				return -nReverseSort;
			else if( pDir2 == NULL )
				return nReverseSort;

			char* pszText1 = pFile1->m_pszName;
			char* pszText2 = pFile2->m_pszName;

			char* p = NULL;
			while( *pszText1 ) {
				if( *pszText1 == '.' ) p = pszText1;
				pszText1++;
			}
			if( p ) pszText1 = p + 1;

			p = NULL;
			while( *pszText2 ) {
				if( *pszText2 == '.' ) p = pszText2;
				pszText2++;
			}
			if( p ) pszText2 = p + 1;

			int nCompare = _stricoll( pszText1, pszText2 );

			/* if( !nCompare )
			{
				pszText1 = getLeafName( pFile1->m_pszName );
				pszText2 = getLeafName( pFile2->m_pszName );
				nCompare = _stricoll( pszText1, pszText2 );
			}  */

			return nReverseSort * nCompare;
		}

		case clmPath:
		{
			char* pszText1 = pDir1 ? pDir1->m_pszName : pFile1->m_pszName;
			char* pszText2 = pDir2 ? pDir2->m_pszName : pFile2->m_pszName;
			return nReverseSort * _stricoll( pszText1, pszText2 );
		}

		case clmDate:
		{
			CTime time1 = pFile1->m_timeLastWritten;
			CTime time2 = pFile2->m_timeLastWritten;
			if( time1 < time2 )
				return nReverseSort;
			else if( time1 == time2 )
				return 0;
			else return -nReverseSort;
		}

		case clmSize:
		{
			return nReverseSort * (	pFile2->m_nSize - pFile1->m_nSize );
		}

		case clmAttributes:
		{
			DWORD dwMask = FILE_ATTRIBUTE_READONLY |
						   FILE_ATTRIBUTE_HIDDEN |
						   FILE_ATTRIBUTE_SYSTEM |
						   FILE_ATTRIBUTE_DIRECTORY |
				           FILE_ATTRIBUTE_ARCHIVE |
				           FILE_ATTRIBUTE_COMPRESSED;

			DWORD dwAttrib1 = pFile1->m_dwAttributes & dwMask;
			DWORD dwAttrib2 = pFile2->m_dwAttributes & dwMask;
			if( dwAttrib1 < dwAttrib2 )
				return nReverseSort;
			else if( dwAttrib1 == dwAttrib2 )
				return 0;

			else return -nReverseSort;
		}

		case clmVolume:
		{
			char* pszText1;
			char* pszText2;

			if( pDir1 && pDir1->m_nParent )
			{
				CFileInfo* pVolume = GetDocument()->getFileInfo( pDir1->m_nParent );
				ASSERT( pVolume );

				pszText1 = pVolume->m_pszName;
			}
			else
				pszText1 = "";

			if( pDir2 && pDir2->m_nParent )
			{
				CFileInfo* pVolume = GetDocument()->getFileInfo( pDir2->m_nParent );
				ASSERT( pVolume );

				pszText2 = pVolume->m_pszName;
			}
			else
				pszText2 = "";

			return nReverseSort * _stricoll( pszText1, pszText2 );
		}

		default:
			ASSERT( 0 );
			return 0;
	}
}

int CFileListView::compareFunc(CFileResult* pResult1, CFileResult* pResult2)
{
	if( pResult1 && pResult2 && !thisApp()->isThreadRunning() )
	{
		for( int i = clmFirst; i <= clmLast; i++ )
		{
			int nResult = compareFileResult( pResult1, pResult2, m_nSortOrder[ i ], m_bReverseSort[ i ] );
			if( nResult )
				return nResult;
		}
	}

	return 0;
}


//void CFileListView::sort()
//{
//	GetListCtrl().SortItems( compareFunc, 0x1234 /* m_nSortColumn */ );
//}

/* this parameter defines the cutoff between using quick sort and
   insertion sort for arrays; arrays with lengths shorter or equal to the
   below value use insertion sort */

#define CUTOFF 8            /* testing shows that this is good value */


/***
*qsort(base, num, wid, comp) - quicksort function for sorting arrays
*
*Purpose:
*       quicksort the array of elements
*       side effects:  sorts in place
*
*Entry:
*       char *base = pointer to base of array
*       unsigned num  = number of elements in the array
*       unsigned width = width in bytes of each array element
*       int (*comp)() = pointer to function returning analog of strcmp for
*               strings, but supplied by user for comparing the array elements.
*               it accepts 2 pointers to elements and returns neg if 1<2, 0 if
*               1=2, pos if 1>2.
*
*Exit:
*       returns void
*
*Exceptions:
*
void sort (
    void *base,
    unsigned num,
    unsigned width,
    int (__cdecl *comp)(const void *, const void *)
    )

*******************************************************************************/

static void swap ( CFileResult *a, CFileResult *b )
{
    CFileResult tmp;

    if ( a != b )
	{
        /* Do the swap one character at a time to avoid potential alignment
           problems. */
        tmp = *a;
        *a = *b;
        *b = tmp;
    }
}

/* sort the array between lo and hi (inclusive) */

void CFileListView::sort()
{
	CWaitCursor wait;

    CFileResult *lo, *hi;              /* ends of sub-array currently sorting */
    CFileResult *mid;                  /* points to middle of subarray */
    CFileResult *loguy, *higuy;        /* traveling pointers for partition step */
    unsigned size;              /* size of the sub-array */
    CFileResult *lostk[30], *histk[30];
    int stkptr;                 /* stack for saving sub-array to be processed */

    /* Note: the number of stack entries required is no more than
       1 + log2(size), so 30 is sufficient for any array */

	CFileResult* base = m_pResults;
	unsigned num  = m_nItems;

    if (num < 2 )
        return;                 /* nothing to do */

    stkptr = 0;                 /* initialize stack */

    lo = base;
    hi = base + num-1;          /* initialize limits */

    /* this entry point is for pseudo-recursion calling: setting
       lo and hi and jumping to here is like recursion, but stkptr is
       prserved, locals aren't, so we preserve stuff on the stack */
recurse:

    size = (hi - lo) + 1;        /* number of el's to sort */

    /* below a certain size, it is faster to use a O(n^2) sorting method */
    if (size <= CUTOFF) {
         shortsort( lo, hi );
    }
    else {
        /* First we pick a partititioning element.  The efficiency of the
           algorithm demands that we find one that is approximately the
           median of the values, but also that we select one fast.  Using
           the first one produces bad performace if the array is already
           sorted, so we use the middle one, which would require a very
           wierdly arranged array for worst case performance.  Testing shows
           that a median-of-three algorithm does not, in general, increase
           performance. */

        mid = lo + (size / 2);              /* find middle element */
        swap(mid, lo);                      /* swap it to beginning of array */

        /* We now wish to partition the array into three pieces, one
           consisiting of elements <= partition element, one of elements
           equal to the parition element, and one of element >= to it.  This
           is done below; comments indicate conditions established at every
           step. */

        loguy = lo;
        higuy = hi + 1;

        /* Note that higuy decreases and loguy increases on every iteration,
           so loop must terminate. */
        for (;;) {
            /* lo <= loguy < hi, lo < higuy <= hi + 1,
               A[i] <= A[lo] for lo <= i <= loguy,
               A[i] >= A[lo] for higuy <= i <= hi */

            do  {
                loguy += 1;
            } while (loguy <= hi && compareFunc(loguy, lo) <= 0);

            /* lo < loguy <= hi+1, A[i] <= A[lo] for lo <= i < loguy,
               either loguy > hi or A[loguy] > A[lo] */

            do  {
                higuy -= 1;
            } while (higuy > lo && compareFunc(higuy, lo) >= 0);

            /* lo-1 <= higuy <= hi, A[i] >= A[lo] for higuy < i <= hi,
               either higuy <= lo or A[higuy] < A[lo] */

            if (higuy < loguy)
                break;

            /* if loguy > hi or higuy <= lo, then we would have exited, so
               A[loguy] > A[lo], A[higuy] < A[lo],
               loguy < hi, highy > lo */

            swap( loguy, higuy );

            /* A[loguy] < A[lo], A[higuy] > A[lo]; so condition at top
               of loop is re-established */
        }

        /*     A[i] >= A[lo] for higuy < i <= hi,
               A[i] <= A[lo] for lo <= i < loguy,
               higuy < loguy, lo <= higuy <= hi
           implying:
               A[i] >= A[lo] for loguy <= i <= hi,
               A[i] <= A[lo] for lo <= i <= higuy,
               A[i] = A[lo] for higuy < i < loguy */

        swap(lo, higuy);     /* put partition element in place */

        /* OK, now we have the following:
              A[i] >= A[higuy] for loguy <= i <= hi,
              A[i] <= A[higuy] for lo <= i < higuy
              A[i] = A[lo] for higuy <= i < loguy    */

        /* We've finished the partition, now we want to sort the subarrays
           [lo, higuy-1] and [loguy, hi].
           We do the smaller one first to minimize stack usage.
           We only sort arrays of length 2 or more.*/

        if ( higuy - 1 - lo >= hi - loguy ) {
            if (lo + 1 < higuy) {
                lostk[stkptr] = lo;
                histk[stkptr] = higuy - 1;
                ++stkptr;
            }                           /* save big recursion for later */

            if (loguy < hi) {
                lo = loguy;
                goto recurse;           /* do small recursion */
            }
        }
        else {
            if (loguy < hi) {
                lostk[stkptr] = loguy;
                histk[stkptr] = hi;
                ++stkptr;               /* save big recursion for later */
            }

            if (lo + 1 < higuy) {
                hi = higuy - 1;
                goto recurse;           /* do small recursion */
            }
        }
    }

    /* We have sorted the array, except for any pending sorts on the stack.
       Check if there are any, and do them. */

    --stkptr;
    if (stkptr >= 0) {
        lo = lostk[stkptr];
        hi = histk[stkptr];
        goto recurse;           /* pop subarray from stack */
    }

    /* all subarrays done */
	Invalidate();
}


/***
*shortsort(hi, lo, width, comp) - insertion sort for sorting short arrays
*
*Purpose:
*       sorts the sub-array of elements between lo and hi (inclusive)
*       side effects:  sorts in place
*       assumes that lo < hi
*
*Entry:
*       char *lo = pointer to low element to sort
*       char *hi = pointer to high element to sort
*       unsigned width = width in bytes of each array element
*       int (*comp)() = pointer to function returning analog of strcmp for
*               strings, but supplied by user for comparing the array elements.
*               it accepts 2 pointers to elements and returns neg if 1<2, 0 if
*               1=2, pos if 1>2.
*
*Exit:
*       returns void
*
*Exceptions:
*
*******************************************************************************/

void CFileListView::shortsort ( CFileResult *lo, CFileResult *hi )
{
    CFileResult *p, *max;

    /* Note: in assertions below, i and j are alway inside original bound of
       array to sort. */

    while (hi > lo) {
        /* A[i] <= A[j] for i <= j, j > hi */
        max = lo;
        for (p = lo+1; p <= hi; p++) {
            /* A[i] <= A[max] for lo <= i < p */
            if (compareFunc(p, max) > 0) {
                max = p;
            }
            /* A[i] <= A[max] for lo <= i <= p */
        }

        /* A[i] <= A[max] for lo <= i <= hi */

        swap(max, hi);

        /* A[i] <= A[hi] for i <= hi, so A[i] <= A[j] for i <= j, j >= hi */

        hi--;

        /* A[i] <= A[j] for i <= j, j > hi, loop top condition established */
    }
    /* A[i] <= A[j] for i <= j, j > lo, which implies A[i] <= A[j] for i < j,
       so array is sorted */
}


/***
*swap(a, b, width) - swap two elements
*
*Purpose:
*       swaps the two array elements of size width
*
*Entry:
*       char *a, *b = pointer to two elements to swap
*       unsigned width = width in bytes of each array element
*
*Exit:
*       returns void
*
*Exceptions:
*
*******************************************************************************/

CString CFileListView::focusOn( int nIndex )
{
	CString sReturn;
	CListCtrl& listCtrl = GetListCtrl();

	if( nIndex < m_nItems )
	{
		LV_ITEM lvItem;
		int nSelected;
		while( (nSelected = listCtrl.GetNextItem( -1, LVIS_SELECTED )) != -1 )
		{
			lvItem.mask = LVIF_STATE;
			lvItem.stateMask =LVIS_SELECTED;
			lvItem.state = 0;
			VERIFY( listCtrl.SetItemState( nSelected, &lvItem ) );
		}

		lvItem.mask = LVIF_STATE;
		lvItem.stateMask = LVNI_FOCUSED | LVIS_SELECTED;
		lvItem.state = LVNI_FOCUSED | LVIS_SELECTED;
		if( listCtrl.SetItemState( nIndex, &lvItem ) )
			listCtrl.EnsureVisible( nIndex, FALSE );

		sReturn = getPath( nIndex );
	}
	return sReturn;	
}

void CFileListView::selectAll()
{
	CWaitCursor wait;
	CListCtrl& listCtrl = GetListCtrl();

	LV_ITEM lvItem;
	int nItem = -1;
	while( (nItem = listCtrl.GetNextItem( nItem, LVNI_ALL )) != -1 )
	{
		lvItem.mask = LVIF_STATE;
		lvItem.stateMask = LVIS_SELECTED;
		lvItem.state = LVIS_SELECTED;
		VERIFY( listCtrl.SetItemState( nItem, &lvItem ) );
	}
}


CString CFileListView::getName( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pFile = m_pResults[ nIndex ].m_pFileInfo;
	ASSERT( pFile );

	return pFile->m_pszName;
}

CString CFileListView::getType( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pFile = m_pResults[ nIndex ].m_pFileInfo;
	CFileInfo* pDir  = m_pResults[ nIndex ].m_pDirInfo;
	ASSERT( pFile );

	char* pType = pDir ? pFile->m_pszName : "<Folder>";
	char* pExt = "";
	while(*pType) {
		if(*pType == '.' ) pExt = pType; // get last dot
		pType++;
	}
	if( *pExt ) pExt++;
	return pExt;
}


CString CFileListView::getPath( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pFile = m_pResults[ nIndex ].m_pFileInfo;
	CFileInfo* pDir  = m_pResults[ nIndex ].m_pDirInfo;
	ASSERT( pFile );

	CString sPath;
	if( pDir && pDir->m_nParent )
	{
		sPath = pDir->m_pszName;
		sPath += pFile->m_pszName;
	}
	else
		sPath = pFile->m_pszName;

	return sPath;
}

CString CFileListView::getDirectoryPath( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pFile = m_pResults[ nIndex ].m_pFileInfo;
	CFileInfo* pDir  = m_pResults[ nIndex ].m_pDirInfo;
	ASSERT( pFile );

	CString sPath;
	if( pDir && pDir->m_nParent )
	{
		sPath = pDir->m_pszName;
	}
	else
		sPath = pFile->m_pszName;

	// trim trailing backslash
	int nCount = sPath.GetLength();

	if( nCount == 0 ) nCount = 1;

	return sPath.Left( nCount - 1) ;
}

CString CFileListView::getVolume( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pDir  = m_pResults[ nIndex ].m_pDirInfo;

	CString sVolume;

	if( pDir )
	{
		if( pDir->m_nParent )
		{
			CFileInfo* pVolume = GetDocument()->getFileInfo( pDir->m_nParent );
			ASSERT( pVolume );
			sVolume =  pVolume->m_pszName;
		}
		else
			sVolume =  pDir->m_pszName;
	}

	return sVolume;
}


CString CFileListView::getDate( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pFile = m_pResults[ nIndex ].m_pFileInfo;
	ASSERT( pFile );

	return pFile->m_timeLastWritten.Format( "%a %b %d, %Y" );	
}

CString CFileListView::getSize( int nIndex )
{
    ASSERT (-1 != nIndex );
	ASSERT ( nIndex < m_nItems );

	CFileInfo* pFile = m_pResults[ nIndex ].m_pFileInfo;
	ASSERT( pFile );

	char pszSize[20];
	_itoa( pFile->m_nSize, pszSize, 10 );
	return pszSize;
}

int CFileListView::getCurrentIndex()
{
	CListCtrl& listCtrl = GetListCtrl();
	
	return listCtrl.GetNextItem( -1, LVNI_FOCUSED );
}

CString CFileListView::nextFile()
{
	CString sReturn;
	int nCur = getCurrentIndex();

	if( nCur != -1 )
	{
		int nItems = GetListCtrl().GetItemCount();
		if( nCur + 1 < nItems )
		{
			nCur++;
		 	sReturn = focusOn( nCur );
		}
	}

	return sReturn;	
}

CString CFileListView::prevFile()
{
	CString sReturn;
	int nCur = getCurrentIndex();

	if( nCur > 0 )
	{
		nCur--;
		sReturn = focusOn( nCur );
	}

	return sReturn;	
}

CString CFileListView::firstFile()
{
	CString sReturn;
	if( m_nItems )
		sReturn = focusOn( 0 );

	return sReturn;
}

CString CFileListView::lastFile()
{
	CString sReturn;
	if( m_nItems )
		sReturn = focusOn( m_nItems - 1 );

	return sReturn;
}

void CFileListView::launchFile()
{
	int nSel = getCurrentIndex();

	if( nSel != -1 )
	{
		CString sText = getPath( nSel );
		CString sLower = sText;
		sLower.MakeLower();

		int nZip = CFileIndex::getZipExtIndex(sLower);

		if( nZip != -1 )
		{
			// is the leaf a zip?
			int nSlash = sText.ReverseFind('\\');
			if(  nSlash > nZip )
			{
				try
				{
					CString sSource = CFileIndex::getTempZip( sText, nZip, true );
					/*
					CString sLeaf = sText.Mid( nSlash + 1 );
					int nExt = sText.ReverseFind('.');
					CString sExt;
					if( nExt != -1 )
						sExt = sText.Mid( nExt + 1 );

					CFileDialog dlg( FALSE, sExt, sLeaf, OFN_HIDEREADONLY  | OFN_OVERWRITEPROMPT );
					if( dlg.DoModal() != IDOK )
						return;

					sText = dlg.GetPathName();
					
					if( !MoveFile( sSource, sText ) )
					{
						CString sMsg = "Sorry I could not put the file at\n";
						sMsg += dlg.GetPathName();
						simpleMessageBox( sMsg );
						return;
					}
					*/
					sText = sSource;
					_chmod( sText, _S_IREAD );
				}
				catch( ... )
				{
					return;  // shouldn't happen
				}
			}
		}
						
		if( !sText.IsEmpty() )
		{
			CListCtrl& listCtrl = GetListCtrl();
			if( listCtrl.GetItemState( nSel, LVIS_SELECTED ) )
				ShellExecute( m_hWnd, NULL, sText, NULL, NULL, SW_SHOW );
		}
	}
}

void CFileListView::printFile()
{
	int nSel = getCurrentIndex();

	if( nSel != -1 )
	{
		CString sText = getPath( nSel );
		CString sLower = sText;
		sLower.MakeLower();

		int nZip = CFileIndex::getZipExtIndex(sLower);

		if( nZip != -1 )
		{
			// is the leaf a zip?
			int nSlash = sText.ReverseFind('\\');
			if(  nSlash > nZip )
			{
				try
				{
					CString sSource = CFileIndex::getTempZip( sText, nZip, true );
					sText = sSource;

				}
				catch( ... )
				{
					return;  // shouldn't happen
				}
			}
		}
						
		if( !sText.IsEmpty() )
			ShellExecute( m_hWnd, "print", sText, NULL, NULL, SW_SHOW );

	}
}

void CFileListView::editTextFile()
{
	int nSel = getCurrentIndex();

	if( nSel != -1 )
	{
		CString sFile = getPath( nSel );

		CString sCommand = regGetString( HKEY_CURRENT_USER, "options", "editor", "notepad.exe" );
		if( !sFile.IsEmpty() && !sCommand.IsEmpty() )
		{
			const char* pszFile = sFile;
			ShellExecute( m_hWnd, NULL, sCommand, (char*) pszFile, NULL, SW_SHOW );
		}
	}
}

void CFileListView::changeFont() 
{
	try
	{
		CListCtrl& listCtrl = GetListCtrl();

		CFont* pFont = listCtrl.GetFont();
		LOGFONT lf;
		if (pFont != NULL)
			pFont->GetObject(sizeof(LOGFONT), &lf);
		else
			::GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &lf);

		CFontDialog dlg(&lf, CF_SCREENFONTS|CF_INITTOLOGFONTSTRUCT);
		if( dlg.DoModal() == IDOK )
		{
			// switch to new font.
			setFont(lf);
		}
	}
	catch( ... )
	{
		CString sMsg; sMsg.LoadString(IDS_FONTSETERROR);
		AfxMessageBox( sMsg );
	}
}

BOOL CFileListView::getFont( LOGFONT& lf )
{
	if( m_bFont )
		lf = m_logFont;

	return m_bFont;
}

void CFileListView::setFont( LOGFONT& lf )
{
	m_bFont = TRUE;
	m_logFont = lf;
	CListCtrl& listCtrl = GetListCtrl();

	// switch to new font.
	if( IsWindow( listCtrl.m_hWnd ) )
	{
		m_font.DeleteObject();
		if(m_font.CreateFontIndirect(&lf))
		{
			CWaitCursor wait;
			listCtrl.SetFont(&m_font);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CFileListView message handlers


int CFileListView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	lpCreateStruct->style |= LVS_REPORT | LVS_OWNERDATA | LVS_SHOWSELALWAYS;
	if (CListView::OnCreate(lpCreateStruct) == -1)
		return -1;

	ListView_SetExtendedListViewStyle(GetSafeHwnd(),LVS_EX_FULLROWSELECT);
	DWORD dwCheck = ListView_GetExtendedListViewStyle( GetSafeHwnd() );

	if( FAILED(dwCheck) || !( dwCheck & LVS_EX_FULLROWSELECT))
		simpleMessageBox( IDS_40COMUPD );

	if( m_bFont )
		setFont( m_logFont );  // if font was set before window active, give another chance

	int nWidth = lpCreateStruct->cx;

	m_columns[ clmName ].m_sLabel.LoadString( IDS_FILES_FOUND );
	m_columns[ clmName ].m_sKeyPrefix = "Name";
	m_columns[ clmName ].m_nWidth = nWidth *2  / 12;

	m_columns[ clmRank ].m_sLabel.LoadString( IDS_RANK );
	m_columns[ clmRank ].m_sKeyPrefix = "Rank";
	m_columns[ clmRank ].m_nWidth = nWidth / 12;

	m_columns[ clmExtension ].m_sLabel.LoadString( IDS_TYPE );
	m_columns[ clmExtension ].m_sKeyPrefix = "Type";
	m_columns[ clmExtension ].m_nWidth   = nWidth / 12;

	m_columns[ clmPath ].m_sLabel.LoadString( IDS_FOLDER );
	m_columns[ clmPath ].m_sKeyPrefix = "Path";
	m_columns[ clmPath ].m_nWidth = nWidth * 4 / 12;

	m_columns[ clmDate ].m_sLabel.LoadString( IDS_LAST_MODIFIED );
	m_columns[ clmDate ].m_sKeyPrefix = "Date";
	m_columns[ clmDate ].m_nWidth = nWidth * 2 / 12;

	m_columns[ clmSize ].m_sLabel.LoadString( IDS_SIZE );
	m_columns[ clmSize ].m_sKeyPrefix = "Size";
	m_columns[ clmSize ].m_nWidth = nWidth / 12;

	m_columns[ clmAttributes ].m_sLabel.LoadString( IDS_ATTRIBUTES );
	m_columns[ clmAttributes ].m_sKeyPrefix = "Attributes";
	m_columns[ clmAttributes ].m_nWidth = nWidth  / 12;

	m_columns[ clmVolume ].m_sLabel.LoadString( IDS_VOLUME );
	m_columns[ clmVolume ].m_sKeyPrefix = "Volume";
	m_columns[ clmVolume ].m_nWidth = nWidth / 12;
	
	for( int i = 0; i <= clmLast; i++ )
	{
		m_columns[ i ].m_nWidth = regGetInt( HKEY_CURRENT_USER, "List View", 
			                                 m_columns[ i ].m_sKeyPrefix + "Width",
								             m_columns[ i ].m_nWidth );

		m_columns[ i ].m_dwFormat = regGetInt( HKEY_CURRENT_USER, "List View", 
			                                   m_columns[ i ].m_sKeyPrefix + "Format",
								               LVCFMT_LEFT );

		char pszDisplay[ 20 ] = "Display Order ";
		itoa( i, pszDisplay + 14, 10 );

		m_nDisplayOrder[ i ] = regGetInt( HKEY_CURRENT_USER, "List View", pszDisplay, i );

		char pszSort[ 20 ] = "Sort Order ";
		itoa( i, pszSort + 11, 10 );
		m_nSortOrder[ i ] = regGetInt( HKEY_CURRENT_USER, "List View", pszSort, i );

		m_bEnabled[ i ] = false; // will reset below
	}

	int nCol = 0;
	for( i = 0; i <= clmLast; i++ )
	{
		int nDisplay = m_nDisplayOrder[ i ];
		if( nDisplay >= 0 )
		{
			m_bEnabled[ nDisplay ] = true;
			int nSubItem = ( nCol ) ? nCol : -1;
			GetListCtrl().InsertColumn(nCol, m_columns[ nDisplay ].m_sLabel,
										     m_columns[ nDisplay ].m_dwFormat, 
										     m_columns[ nDisplay ].m_nWidth, nSubItem );

			nCol++;
		}
	}

	return 0;
}

void CFileListView::OnDblclk(NMHDR* pNMHDR, LRESULT* pResult) 
{
	launchFile();
	
	*pResult = 0;
}

void CFileListView::OnItemchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	
	if( ( pNMListView->uChanged & LVIF_STATE ) &&
		( pNMListView->uNewState & LVIS_FOCUSED ) )
	{
		GetDocument()->setCurrentFile( getPath( pNMListView->iItem ), FALSE );
	}
	
	*pResult = 0;
}


void CFileListView::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	ASSERT( pNMListView->iSubItem >= clmFirst && pNMListView->iSubItem <= clmLast );

	//CString sMsg;
	//sMsg.Format("Column click\nSubitem %d\nFirst %d\nLast %d", pNMListView->iSubItem, clmFirst, clmLast );
	//simpleMessageBox( sMsg );

	sortColumn( pNMListView->iSubItem );

	*pResult = 0;
}

void CFileListView::sortColumn( int nNewSortColumn )
{
	ASSERT( nNewSortColumn >=0 && nNewSortColumn < clmLast );

	nNewSortColumn = m_nDisplayOrder[ nNewSortColumn ];

	for( int nPos = 0; nPos < clmLast; nPos++ )  // only goes to clmLast so a valid result is mandated
	{
		if( nNewSortColumn == abs( m_nSortOrder[ nPos ] ) )
			break;
	}

	if( nPos == 0 )
		m_bReverseSort[ 0 ] = !m_bReverseSort[ 0 ];
	else
	{
		// move everything down
		for( int i = nPos; i > 0; i-- )
		{
			m_nSortOrder[ i ] = m_nSortOrder[ i - 1 ];
			m_bReverseSort[ i ] = m_bReverseSort[ i - 1 ];
		}

		m_nSortOrder[ 0 ] = nNewSortColumn;
		m_bReverseSort[ 0 ] = false;
	}

	sort();
}

void CFileListView::OnDestroy() 
{
	LV_COLUMN lvColumn;
	lvColumn.mask = LVCF_WIDTH;

	int nCol = 0;
	for( int i = 0; i <= clmLast; i++ )
	{
		int nDisplay = m_nDisplayOrder[ i ];
		if( nDisplay >= 0 )
		{
			if( GetListCtrl().GetColumn( nCol, &lvColumn ) )
				regPutInt( HKEY_CURRENT_USER, "List View",
				           m_columns[ nDisplay ].m_sKeyPrefix + "Width",
						   lvColumn.cx );

			nCol++;
		}

		char pszDisplay[ 20 ] = "Display Order ";
		itoa( i, pszDisplay + 14, 10 );

		regPutInt( HKEY_CURRENT_USER, "List View", pszDisplay, m_nDisplayOrder[ i ] );

		char pszSort[ 20 ] = "Sort Order ";
		itoa( i, pszSort + 11, 10 );
		regPutInt( HKEY_CURRENT_USER, "List View", pszSort, m_nSortOrder[ i ] );
	}


	CListView::OnDestroy();
}

LRESULT CALLBACK menuCallBack(HWND hWnd, unsigned iMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT result;
	switch( iMsg )
	{
		case WM_CREATE:
		{
			// we are just retrieving the LPCONTEXTMENU2 passed to CreateWindow
			LPCONTEXTMENU2 lpcm2 = LPCONTEXTMENU2 (((LPCREATESTRUCT) lParam)->lpCreateParams);

			// save it
			SetWindowLong( hWnd, GWL_USERDATA, (long) lpcm2 );
			result = DefWindowProc( hWnd, iMsg, wParam, lParam );
			break;
		}

		case WM_DRAWITEM:
		case WM_MEASUREITEM:
		case WM_INITMENUPOPUP:
		{
			LPCONTEXTMENU2 lpcm2 = (LPCONTEXTMENU2) GetWindowLong( hWnd, GWL_USERDATA);
			ASSERT( lpcm2 );
			lpcm2->HandleMenuMsg( iMsg, wParam, lParam );

	        if( iMsg == WM_INITMENUPOPUP )
				result = 0;
			else
				result = 1;

			break;
		}

		default:
		{
			result = DefWindowProc( hWnd, iMsg, wParam, lParam );
			break;
		}
	}

	return result;
}

void CFileListView::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint point;
	::GetCursorPos((LPPOINT) &point);

	LPSHELLFOLDER lpsf=NULL;
	LPSHELLFOLDER pFolder = NULL;
	LPITEMIDLIST pidlReturn = NULL;

	try
	{
		// Allocate a shell memory object.
		m_pMalloc = NULL;
		HRESULT hrMalloc = ::SHGetMalloc( &m_pMalloc );
		if (FAILED(hrMalloc))
			AfxThrowUserException();

		HRESULT hrDesktop = SHGetDesktopFolder(&lpsf);

		if (SUCCEEDED(hrDesktop))
		{
			// create a list to hold the PIDL for all the selected entries
			int nSelected = GetListCtrl().GetSelectedCount();
			struct _ITEMIDLIST **pPidlList = new struct _ITEMIDLIST*[ nSelected ];
			int nPidl = 0;

			CFileInfo* pCommonDir = NULL;
			for( int nItem = 0; nItem < m_nItems; nItem++ )
			{
				if( GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
				{
					CFileInfo* pDir  = m_pResults[ nItem ].m_pDirInfo;
					if( pCommonDir )
					{
						if( pCommonDir != pDir )
						{
							simpleMessageBox( IDS_CONTEXT_FOLDER_ERR );
							if( pFolder )
							{
								pFolder->Release();
								pFolder = NULL;
							}
							break;
						}
					}
					else
						pCommonDir = pDir;

					CString sPath = getPath( nItem );

					LPITEMIDLIST pidl = getUIObject( sPath, lpsf, pFolder );
					if( pidl )
						pPidlList[ nPidl++ ] = pidl;
				}
			}

			HRESULT hr;
			LPCONTEXTMENU lpcm = NULL;
			if( pFolder )
			{
				hr = pFolder->GetUIObjectOf( GetParent()->m_hWnd,
										nPidl,     // number of objects to get attributes of
										(const struct _ITEMIDLIST **) pPidlList,
										IID_IContextMenu,
										0,
										(LPVOID *)&lpcm);
			}

			for( nItem = 0; nItem < nPidl; nItem++ )
				m_pMalloc->Free( pPidlList[ nItem ] );

			delete[] pPidlList;
			pPidlList = NULL;

			if( pFolder && (SUCCEEDED(hr)))  
			{
				bool bHasContext2;
				HWND hContext2;
				LPCONTEXTMENU2 lpcm2 = NULL;
				HRESULT hr2 =  lpcm->QueryInterface( IID_IContextMenu2, (LPVOID *) &lpcm2 );
				if( SUCCEEDED( hr2 ) )
				{
					bHasContext2 = true;
					WNDCLASSEX classData;
					memset( &classData, 0, sizeof( classData ) );
					classData.cbSize = sizeof( classData );
					classData.style = CS_PARENTDC;
					classData.lpfnWndProc = &menuCallBack;
					classData.hInstance = thisApp()->m_hInstance;
					classData.lpszClassName = "WilburMenuCallbackHelper";

					// perhaps I should check if already registered rather than letting this fail
					ATOM nClass = RegisterClassEx( &classData );

					hContext2 = CreateWindow( "WilburMenuCallbackHelper",
							                  "WilburMenuCallbackProcessor",
											   WS_POPUPWINDOW,
											   CW_USEDEFAULT, CW_USEDEFAULT,
											   CW_USEDEFAULT, CW_USEDEFAULT,
											   0, 0,
											   thisApp()->m_hInstance,
											   lpcm2);

				}
				else
					bHasContext2 = false;

				HMENU         hMenu;
				hMenu = CreatePopupMenu();

				if (hMenu)
				{
					// Get the context menu for the item.
					if( bHasContext2 )
						hr = lpcm2->QueryContextMenu(hMenu, 0, 1, 0x7fff, CMF_NORMAL);
					else
						hr = lpcm->QueryContextMenu(hMenu, 0, 1, 0x7fff, CMF_NORMAL);



					if (SUCCEEDED(hr))
					{
						HWND hWnd = hContext2 ? hContext2 : m_hWnd;

						int idCmd = TrackPopupMenu(hMenu, 
										TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, 
										point.x, point.y, 0, hWnd, NULL);

						if (idCmd)
						{
							// Invoke the command that was selected.
							CMINVOKECOMMANDINFO cmi;

							cmi.cbSize = sizeof(CMINVOKECOMMANDINFO);
							cmi.fMask  = 0;
							cmi.hwnd   = m_hWnd;
							cmi.lpVerb = MAKEINTRESOURCE(idCmd-1);
							cmi.lpParameters = NULL;
      						cmi.lpDirectory  = NULL;
							cmi.nShow        = SW_SHOWNORMAL;
							cmi.dwHotKey     = 0;
							cmi.hIcon        = NULL;
							if( bHasContext2 )
								hr = lpcm2->InvokeCommand(&cmi);
							else
								hr = lpcm->InvokeCommand(&cmi);
						}
					}

					DestroyMenu(hMenu);
				}
				if( bHasContext2 )
					lpcm2->Release();

				lpcm->Release();
			} 
 
			// Release the folder pointer.
			if( lpsf )
				lpsf->Release();

			if( pFolder )
				pFolder->Release();

			m_pMalloc->Release();
			m_pMalloc = NULL;

			// check to see if the files are still there (make sure not in zip)
			// must make list of files since list will become bogus after marking file obsolete
			FileID* pidDeletedFiles;
			pidDeletedFiles = new FileID[ m_nItems ];
			int nNextFile = 0;
			for( nItem = 0; nItem < m_nItems; nItem++ )
			{
				if( GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
				{
					CString sPath = getPath( nItem );

					CFileInfo* pDir  = m_pResults[ nItem ].m_pDirInfo;
					bool bFolder = ( !pDir || !pDir->m_nParent );

					CFileStatus fileStatus;
					sPath = makeLower( sPath );
					if( !bFolder && CFileIndex::getZipExtIndex(sPath) == -1 
						&& !CFile::GetStatus( sPath, fileStatus ) )
					{
						pidDeletedFiles[ nNextFile++ ] = GetDocument()->getFileIDFromPath( sPath );
					}
				}
			}

			if( nNextFile )
			{
				clearList();  // so it doesn't read something evil
				while( nNextFile-- )
					GetDocument()->markFileObsolete( pidDeletedFiles[ nNextFile ] );

				delete[] pidDeletedFiles;
				GetDocument()->doSearch( FALSE );
			}
			else
				delete[] pidDeletedFiles;

		}
	}
	catch( ... )
	{
		simpleMessageBox( IDS_CONTEXT_MENU_PROC_ERR );
		if( m_pMalloc )
		{
			m_pMalloc->Release();
			m_pMalloc = NULL;
		}

		if( lpsf )
			lpsf->Release();

		if( pFolder )
			pFolder->Release();
	}

	*pResult = 0;
}

LPITEMIDLIST CFileListView::getUIObject( const CString sPath, LPSHELLFOLDER lpsf, LPSHELLFOLDER& pFolder )
{
	LPITEMIDLIST pidlReturn = NULL;

	HRESULT hr;

	//
	// IShellFolder::ParseDisplayName requires the file name be in Unicode.
	//
	OLECHAR       olePath[MAX_PATH];
	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, sPath, -1,
						olePath, MAX_PATH);

	//
	// Convert the path to an ITEMIDLIST.
	//
	ULONG chEaten;
	ULONG dwAttributes = 0;
	LPITEMIDLIST  pidl, pidlStart;

	hr = lpsf->ParseDisplayName( NULL,
								 NULL,
								 olePath,
								 &chEaten,
								 &pidl,
								 &dwAttributes);

	pidlStart = pidl;

	if (SUCCEEDED(hr))
	{
		// pidl now contains a pointer to an ITEMIDLIST for .\readme.txt. This
		// ITEMIDLIST needs to be freed using the IMalloc allocator returned
		// from SHGetMalloc().

		LPSHELLFOLDER pParentFolder = lpsf;
		LPITEMIDLIST pNextPidl = pidl;

		LPSHELLFOLDER pPrevFolder = lpsf;

		// must walk down list binding to each subfolder in turn
		for (; pNextPidl != NULL; pNextPidl = GetNextItemID(pidl))
		{
			pidl = pNextPidl;

			LPITEMIDLIST pidlCopy; 
			LPSHELLFOLDER pSubFolder;

			// Copy the item identifier to a list by itself. 
			if ((pidlCopy = CopyItemID(pidl)) == NULL) 
				break; 

			// Bind to the subfolder. 
			if (!SUCCEEDED(pParentFolder->BindToObject( pidlCopy, NULL, 
					IID_IShellFolder, (void**) &pSubFolder)))
			{ 
				m_pMalloc->Free( pidlCopy ); 
				if( pFolder )
					pParentFolder->Release();
				else
					pFolder = pParentFolder;

				if( pPrevFolder != lpsf )
					pPrevFolder->Release();

				break; 
			} 

			// Free the copy of the item identifier. 
			m_pMalloc->Free( pidlCopy ); 

			// Release the parent folder and point to the 
			// subfolder.
			
			if( pPrevFolder != pFolder && pPrevFolder != lpsf )
				pPrevFolder->Release();

			pPrevFolder = pParentFolder;
			pParentFolder = pSubFolder;
		}

		if(!pFolder)
		{
			pFolder = pPrevFolder;
			
			if( pPrevFolder != pFolder && pPrevFolder != lpsf )
				pPrevFolder->Release();
		}

		if( pidl )
			pidlReturn = CopyItemID( pidl );

		m_pMalloc->Free(pidlStart);  // free the pidl that the shell gave us
	}

	return pidlReturn;
}



// CopyItemID - creates an item identifier list containing the first 
//     item identifier in the specified list. 
// Returns a PIDL if successful or NULL if out of memory. 

LPITEMIDLIST CFileListView::CopyItemID(LPITEMIDLIST pidl) 
{ 
    // Get the size of the specified item identifier. 
    int cb = pidl->mkid.cb; 
 
    // Allocate a new item identifier list. 
    LPITEMIDLIST pidlNew = (LPITEMIDLIST) 
        m_pMalloc->Alloc( cb + sizeof(USHORT) );

    if (pidlNew == NULL) 
        return NULL; 
 
    // Copy the specified item identifier. 
    CopyMemory(pidlNew, pidl, cb); 
 
    // Append a terminating zero. 
    *((USHORT *) (((LPBYTE) pidlNew) + cb)) = 0; 
 
    return pidlNew; 
} 

// GetNextItemID - points to the next element in an item identifier list. 
// Returns a PIDL if successful or NULL if at the end of the list. 
// pdil - previous element

LPITEMIDLIST CFileListView::GetNextItemID(LPITEMIDLIST pidl) 
{ 
    // Get the size of the specified item identifier. 
    int cb = pidl->mkid.cb; 
 
    // If the size is zero, it is the end of the list. 
    if (cb == 0) 
        return NULL; 
 
    // Add cb to pidl (casting to increment by bytes). 
    pidl = (LPITEMIDLIST) (((LPBYTE) pidl) + cb); 
 
    // Return NULL if it is null-terminating or a pidl otherwise. 
    return (pidl->mkid.cb == 0) ? NULL : pidl; 
} 
 
// StrRettoCString - converts the contents of a STRRET structure to a CString
// pidl - PIDL containing the display name of STRRET_OFFSET 
// lpStr - address of the STRRET structure 
CString CFileListView::StrRettoCString(LPITEMIDLIST pidl, LPSTRRET lpStr) 
{ 
    LPSTR lpsz; 
    int cch;
	CString sReturn;
 
    switch (lpStr->uType) { 
 
        case STRRET_WSTR: 
            cch = WideCharToMultiByte(CP_OEMCP, 0, 
                lpStr->pOleStr, -1, NULL, 0, NULL, NULL);
			
#ifdef _DEBUG
			if( !cch )
			{
				DWORD nErr = GetLastError();

				char* pszMessage;
				FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
							   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );

				CString sMsg; sMsg.LoadString(IDS_WIDECHARERROR);
				sMsg += pszMessage;

				AfxMessageBox(  sMsg );
				LocalFree( pszMessage );
			}
#endif

            lpsz = (LPSTR) m_pMalloc->Alloc( cch ); 
            if (lpsz != NULL) { 
                WideCharToMultiByte(CP_OEMCP, 0, 
                    lpStr->pOleStr, -1, lpsz, cch, NULL, NULL);

                sReturn = lpsz;
                m_pMalloc->Free( lpsz ); 
            } 
            break; 
 
        case STRRET_OFFSET: 
            sReturn =  ((char *) pidl) + lpStr->uOffset; 
            break; 
 
        case STRRET_CSTR: 
            sReturn = lpStr->cStr; 
            break; 
    } 

	return sReturn;
} 
 

#if 0  // Drag stuff
void CFileListView::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	int nSel = getCurrentIndex();

	if( nSel != -1 )
	{
		CString sPath = getPath( nSel );
		if( !sPath.IsEmpty() )
		{
			// Get a pointer to the desktop folder.
			LPSHELLFOLDER lpsf=NULL;

			HRESULT hr=SHGetDesktopFolder(&lpsf);

			if (SUCCEEDED(hr))
			{
				LPDATAOBJECT lpdo = NULL;

				hr = getUIObject( sPath, lpsf, (LPVOID &) lpdo );

				if (SUCCEEDED(hr))  
				{
					LPDROPSOURCE lpds= new CDropSource();
					DWORD          dwEffect;

					if (lpds)
						hr = ::DoDragDrop(lpdo,
								lpds,
								DROPEFFECT_COPY | DROPEFFECT_MOVE | DROPEFFECT_LINK,
								&dwEffect);     

					lpdo->Release();
					lpds->Release();
				}
			}
		}
	
	*pResult = 0;
	}
}

//---------------------------------------------------------------------
//                    IUnknown Methods
//---------------------------------------------------------------------


STDMETHODIMP CDropSource::QueryInterface(REFIID iid, void FAR* FAR* ppv) 
{
    if(iid == IID_IUnknown || iid == IID_IDropSource)
    {
      *ppv = this;
      ++m_refs;
      return NOERROR;
    }
    *ppv = NULL;

    return E_NOINTERFACE;
}


STDMETHODIMP_(ULONG) CDropSource::AddRef(void)
{
    return ++m_refs;
}


STDMETHODIMP_(ULONG) CDropSource::Release(void)
{
    if(--m_refs == 0)
    {
      delete this;
      return 0;
    }
    return m_refs;
}  

//---------------------------------------------------------------------
//                    CDropSource Constructor
//---------------------------------------------------------------------        
 
CDropSource::CDropSource()
{
    m_refs = 1;  
}

//---------------------------------------------------------------------
//                    IDropSource Methods
//---------------------------------------------------------------------  

STDMETHODIMP CDropSource::QueryContinueDrag(BOOL fEscapePressed, 
                                            DWORD grfKeyState)
{  

    if (fEscapePressed)
        return DRAGDROP_S_CANCEL;
    else if (!(grfKeyState & MK_LBUTTON) && !(grfKeyState & MK_RBUTTON))
        return DRAGDROP_S_DROP;
    else
        return NOERROR;                  
}

STDMETHODIMP CDropSource::GiveFeedback(DWORD dwEffect)
{
    return DRAGDROP_S_USEDEFAULTCURSORS;
}

#endif // drag stuff

CString CFileListView::getItemTextLine( int nItem )
{
	CFileInfo* pFile = m_pResults[ nItem ].m_pFileInfo;
	CFileInfo* pDir  = m_pResults[ nItem ].m_pDirInfo;

	CString sLine;
	CString sFolderWord; sFolderWord.LoadString(IDS_FOLDER);
	CString sFolder;     sFolder.Format("<%s>", sFolderWord);
	for( int nClm = 0; nClm <= clmLast; nClm++ )
	{
		int nDisplay = m_nDisplayOrder[ nClm ];
		if( m_bEnabled[ nDisplay ] )
		{
			if( !sLine.IsEmpty() )
				sLine += '\t';

			switch (nDisplay )
			{
				case clmName:
				{   //  Copy the main item text - filename.
					if( pDir && pDir->m_nParent )  sLine += pFile->m_pszName;
					else sLine += sFolder;
					break;
				}

				case clmRank:
				{   //  Copy subitem 1 text - rank.
					char pszRank[ 20 ];
					_itoa( m_pResults[ nItem ].m_nRank + 1, pszRank, 10 );
					sLine += pszRank;
					break;
				}

				case clmExtension:
				{   //  Copy subitem 2 text - type extension.
					char* pType = ( pDir && pDir->m_nParent ) ? pFile->m_pszName : "";
					char * pExt = NULL;
					while(*pType) {
						if(*pType == '.' ) pExt = pType; // get last dot
						pType++;
					}
					if(  pExt ) {
						if( *pExt ) pExt++;
					}
					else
						pExt = pType;
					sLine += pExt;
					break;
				}

				case clmPath:
				{   //  Copy subitem 3 text - dir name.
					const char* pDirName = ( pDir && pDir->m_nParent ) ? pDir->m_pszName : pFile->m_pszName;
					sLine += pDirName;
					break;
				}

				case clmDate:
				{   //  Copy subitem 4 text - time last written.
					CString sFormat; sFormat.LoadString(IDS_DATEFORMAT);
					sLine += pFile->m_timeLastWritten.Format( sFormat );
					break;
				}

				case clmSize:
				{   //  Copy subitem 5 text - size.
					char pszSize[20];
					_itoa( pFile->m_nSize, pszSize, 10 );

					sLine += pszSize;
					break;
				}

				case clmAttributes:
				{
					char attrib[10];
					char* pAttrib = attrib;
					DWORD dwAttrib = pFile->m_dwAttributes;

					if( dwAttrib & FILE_ATTRIBUTE_COMPRESSED )
						*pAttrib++ = 'c';

					if( dwAttrib & FILE_ATTRIBUTE_ARCHIVE )
						*pAttrib++ = 'a';

					if( dwAttrib & FILE_ATTRIBUTE_DIRECTORY )
						*pAttrib++ = 'd';

					if( dwAttrib & FILE_ATTRIBUTE_SYSTEM )
						*pAttrib++ = 's';

					if( dwAttrib & FILE_ATTRIBUTE_HIDDEN )
						*pAttrib++ = 'h';

					if( dwAttrib & FILE_ATTRIBUTE_READONLY )
						*pAttrib++ = 'r';

					*pAttrib = '\0';

					sLine += attrib;
					break;
				}

				case clmVolume:
				{
					if( pDir )
					{
						if( pDir->m_nParent )
						{
							CFileInfo* pVolume = GetDocument()->getFileInfo( pDir->m_nParent );
							ASSERT( pVolume );
							sLine += pVolume->m_pszName;
						}
						else
							sLine += pDir->m_pszName;
					}
					
					break;
				}

				default:
					break;
			}
		}
	}

	return sLine;
}

void CFileListView::OnFileExportlist() 
{
	char currentDir[ MAX_PATH ];
	GetCurrentDirectory( MAX_PATH, currentDir );

	// thisApp()->changeToIndexDirectory();

	CString sExportPath = regGetString( HKEY_CURRENT_USER, "Settings", "ExportPath", thisApp()->getIndexPath() );
	SetCurrentDirectory( sExportPath );

	CFileDialog dlg( FALSE, "txt", "FileList.txt",
		OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_OVERWRITEPROMPT );
	if( dlg.DoModal() == IDOK )
	{
		try
		{
			CFile file( dlg.GetPathName(), CFile::modeCreate | CFile::modeWrite );

			CString sDir = dlg.GetPathName();
			int nSlash = sDir.ReverseFind('\\');
			sDir = sDir.Left( nSlash );

			regPutString( HKEY_CURRENT_USER, "Settings", "ExportPath", sDir );
			int nSelected = GetListCtrl().GetSelectedCount();

			for( int nItem = 0; nItem < m_nItems; nItem++ )
			{
				if( nSelected < 2 || GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
				{

					CString sLine = getItemTextLine( nItem );
					sLine += "\r\n";
					file.Write( (const char*) sLine, sLine.GetLength() );
				}
			}
		}
		catch( ... )
		{
			CString sMsg; sMsg.LoadString(IDS_LISTEXPORTFAILED);
			AfxMessageBox(sMsg);
		}
	}

	SetCurrentDirectory( currentDir );
}

void CFileListView::OnFileListClip() 
{
	CWaitCursor wait;
	OpenClipboard();
	EmptyClipboard();

	int nSelected = GetListCtrl().GetSelectedCount();
	int nSize = 0;

	// Count the size
	for( int nItem = 0; nItem < m_nItems; nItem++ )
	{
		if( nSelected < 2 || GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
		{
			nSize += getItemTextLine( nItem ).GetLength() + 2;
		}
	}

	HGLOBAL hText = GlobalAlloc( GMEM_MOVEABLE | GMEM_DDESHARE, nSize + 1 );
	char* pszClip = (char*) GlobalLock( hText );
	char* pszText = pszClip;

	for( nItem = 0; nItem < m_nItems; nItem++ )
	{
		if( nSelected < 2 || GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
		{

			CString sLine = getItemTextLine( nItem );
			sLine += "\r\n";
			strcpy( pszText, sLine );
			pszText += sLine.GetLength();
		}
	}

			
	SetClipboardData( CF_TEXT, hText );
	GlobalUnlock( hText );
	CloseClipboard();
}


void CFileListView::OnFileCopy() 
{
	doFileOperation( fileOpCopy );
}

void CFileListView::OnUpdateFileCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetListCtrl().GetSelectedCount() != 0 );
}

void CFileListView::OnUpdateFileCommand(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetListCtrl().GetSelectedCount() != 0 );
}

CString CFileListView::getDesktopFolder( CString sDisplayName )
{
	// Allocate a shell memory object.
	m_pMalloc = NULL;
	;
	if (FAILED(::SHGetMalloc( &m_pMalloc )))
		return "";

    BROWSEINFO bi; 
    LPSTR lpBuffer; 
    LPITEMIDLIST pidlBrowse;    // PIDL selected by user 
 
    // Allocate a buffer to receive browse information. 
    if ((lpBuffer = (LPSTR) m_pMalloc->Alloc(  MAX_PATH)) == NULL) 
        return ""; 
 
    // Fill in the BROWSEINFO structure. 
    bi.hwndOwner = m_hWnd; 
    bi.pidlRoot = NULL; //pidlPrograms; 
    bi.pszDisplayName = lpBuffer; 
    bi.lpszTitle = sDisplayName; 
    bi.ulFlags = BIF_RETURNONLYFSDIRS; 
    bi.lpfn = NULL; 
    bi.lParam = 0; 
 
    // Browse for a folder and return its PIDL. 
    pidlBrowse = SHBrowseForFolder(&bi); 

    CString sTargetPath;
	if( pidlBrowse != NULL )
	{ 
		LPSHELLFOLDER pFolder; 
 
		// Start with the desktop folder. 
		if( SUCCEEDED(SHGetDesktopFolder(&pFolder)) )
		{
			LPITEMIDLIST pidl; 

			// Process each item identifier in the list. 
			for( pidl = pidlBrowse; pidl != NULL; pidl = GetNextItemID(pidl) )
			{ 
				STRRET sName; 
				LPSHELLFOLDER pSubFolder; 
				LPITEMIDLIST pidlCopy; 

				// Copy the item identifier to a list by itself. 
				if( (pidlCopy = CopyItemID(pidl)) == NULL )
					break; 

				// Display the name of the subfolder. 
				if( SUCCEEDED(pFolder->GetDisplayNameOf( pidlCopy, SHGDN_INFOLDER, &sName)) ) 
					sTargetPath += StrRettoCString(pidlCopy, &sName) + '\\'; 

				// Bind to the subfolder. 
				if( !SUCCEEDED(pFolder->BindToObject( pidlCopy, NULL, 
						IID_IShellFolder, (void**) &pSubFolder)) )
				{ 
					m_pMalloc->Free( pidlCopy );
					sTargetPath.Empty();
					break; 
				} 

				// Free the copy of the item identifier. 
				m_pMalloc->Free( pidlCopy ); 

				// Release the parent folder and point to the 
				// subfolder. 
				pFolder->Release(); 
				pFolder = pSubFolder; 
			} 

			// Release the last folder that was bound to. 
			if (pFolder != NULL) 
				pFolder->Release(); 
		} 

        // Free the PIDL returned by SHBrowseForFolder. 
        m_pMalloc->Free( pidlBrowse );
    }

	// Clean up. 
    m_pMalloc->Free( lpBuffer ); 
	m_pMalloc->Release();

	return sTargetPath;
}

void CFileListView::doFileOperation( FileOperation opType )
{
	CFileDialog dlg( FALSE, NULL, "*", OFN_NOVALIDATE | OFN_NOCHANGEDIR | OFN_HIDEREADONLY );

	CString sTargetPath;
	if( dlg.DoModal() == IDOK )
	{
		sTargetPath = dlg.GetPathName();
		int nLen = sTargetPath.GetLength();
		if( nLen && sTargetPath[nLen-1] == '*' )
			sTargetPath.SetAt(nLen - 1, '.');
	}
	else
		return;

	if( !sTargetPath.IsEmpty() )
	{
		// are there multiple selections?
		int nSelected = GetListCtrl().GetSelectedCount();
		bool bMultiSource;
		
		if( nSelected == 1 )
		{
			// only one selected - is it a folder?
			for( int i = 0; i < m_nItems; i++ )
				if( GetListCtrl().GetItemState( i, LVIS_SELECTED ) == LVIS_SELECTED )
					break;

			ASSERT( i < m_nItems );
			CFileInfo* pDir  = m_pResults[ i ].m_pDirInfo;
			if( !pDir || !pDir->m_nParent )
				bMultiSource = true;
			else
				bMultiSource = false;
		}
		else
			bMultiSource = true;

		// check path to see that everything exists
		// start with leaf
		CFileStatus targetStatus;
		bool isDirTarget = false;

		int nZip = CFileIndex::getZipExtIndex(sTargetPath);
		m_bCopyTargetIsZip = false;
		if( nZip != -1 )
		{
			sTargetPath = sTargetPath.Left( nZip + 4 );
			m_bCopyTargetIsZip = true;
			m_sZipSourceFiles.Empty();
		}
		else if( CFile::GetStatus( sTargetPath, targetStatus) )
		{
			// target leaf exists - is it a folder?
			if( targetStatus.m_attribute & CFile::directory )
			{
				isDirTarget = true;

				// if path ends in \. truncate them otherwise add the backslash
				int nLength = sTargetPath.GetLength();
				if( nLength >= 2 && sTargetPath[ nLength - 1 ] == '.' && sTargetPath[ nLength - 2 ] == '\\' )
					sTargetPath = sTargetPath.Left( nLength - 2 );
			}
		}
		else
		{
			// target leaf doesn't exist
			int nSlash = 0;
			const char *pszTarget = sTargetPath;

			while( *pszTarget )
			{
				// find next slash
				while( *pszTarget && *pszTarget != '\\' && *pszTarget != '/' )
					{ pszTarget++; nSlash++; }

				if( *pszTarget )
				{
					// see if it exists
					CString sPath = sTargetPath.Left( nSlash );
					if( !CFile::GetStatus( sPath, targetStatus ) )
					{
						// it doesn't, so create directory
						if( _mkdir( sPath ) == -1 )
						{
							CString sMsg;
							sMsg.LoadString( IDS_CREATE_FOLDER_ERR );
							sMsg += sPath;
							simpleMessageBox( sMsg );
							return;
						}
					}
					pszTarget++; nSlash++;
					if( *pszTarget == '\0' )
						isDirTarget = true;
				}
			}

			if( bMultiSource && !isDirTarget )
			{
				// non existant target and multiple source files
				// create target as directory
				if( _mkdir( sTargetPath ) == -1 )
				{
					CString sMsg;
					sMsg.LoadString( IDS_CREATE_FOLDER_ERR );
					sMsg += sTargetPath;
					simpleMessageBox( sMsg );
					return;
				}
				
				isDirTarget = true;
			}
		}
		
		try
		{
			int nOverWriteResponse = IDNO;

			for( int i = 0; i < m_nItems; i++ )
			{
				if( GetListCtrl().GetItemState( i, LVIS_SELECTED ) == LVIS_SELECTED )
				{
					CString sName = getName( i );
					CString sSource = getPath( i );

					CFileInfo* pDir  = m_pResults[ i ].m_pDirInfo;
					bool isDirSource = ( !pDir || !pDir->m_nParent );

					if( isDirSource )
					{
						// just want leaf of source dir name

						int nSlash = sName.GetLength();
						if( nSlash ) // get rid of trailing slash
							sName = sName.Left( nSlash - 1 );

						// now find the last slash
						nSlash = sName.ReverseFind('\\');
						if( nSlash != -1 )
							sName = sName.Mid( nSlash );
					}
					else
						sName = '\\' + sName;

					BOOL bSuccess;

					CString sTarget = ( isDirTarget ) ? sTargetPath + sName : sTargetPath;

					CFileStatus fileStatus;
					if( !m_bCopyTargetIsZip && CFile::GetStatus( sTarget, fileStatus ) )
					{
						if( nOverWriteResponse != IDOK )
						{
							COverWriteDialog dlg;
							CString sMsg;
							sMsg.LoadString(IDS_ALREADYEXISTS);
							dlg.m_sMessage = sTarget + sMsg;
							nOverWriteResponse = dlg.DoModal();
							
							if( nOverWriteResponse == IDCANCEL )
								break;
							else if( nOverWriteResponse == IDNO )
								continue;
						}
					}
					else if( isDirSource )
					{
						// target directory does not exist
						if( _mkdir( sTarget ) == -1 )
						{
							CString sMsg;
							sMsg.LoadString( IDS_CREATE_FOLDER_ERR );
							sMsg += sTargetPath;

							bSuccess = FALSE;
							if( simpleMessageBox( sMsg ) == IDCANCEL )
								break;
							else
								continue;
						}
					}	

					if( isDirSource )
						bSuccess = doDirCopy( sSource, sTarget, nOverWriteResponse );
					else
					{
						bSuccess = doFileCopy( sSource, sTarget );
					}

					if( !bSuccess )
					{
						CString sMsg;
						sMsg.LoadString( IDS_FILE_COPY_ERR );
						sMsg += sSource;
						CString sTo; sTo.LoadString(IDS_TO);
						sMsg += sTo;
						sMsg += sTarget;
						if( simpleMessageBox( sMsg, MB_OKCANCEL ) == IDCANCEL )
							break;
					}

				}
			}
		}
		catch( ... )
		{
			CString sMsg; sMsg.LoadString(IDS_LISTCOPYFAILED);
			AfxMessageBox(IDS_LISTCOPYFAILED);
		}

		if( m_bCopyTargetIsZip )
		{
			CString sStatus;
			sStatus.Format(IDS_ADDING_TO_ZIP_FMT, sTargetPath );
			thisApp()->setStatus( sStatus );

			if( !addToZip( m_sZipSourceFiles, sTargetPath ) )
			{
				CString sMsg;
				sMsg.LoadString( IDS_ADD_FILE_ERR );
				sMsg += sTargetPath;
				simpleMessageBox( sMsg );
			}
		}

		CString sStatus;
		sStatus.LoadString( IDS_DONE );
		thisApp()->setStatus( sStatus );
	}
}

BOOL CFileListView::doFileCopy( CString sSource, const CString& sTarget )
{
	int nZip = CFileIndex::getZipExtIndex(sSource);

	BOOL bSuccess = FALSE;
	if( nZip != -1 )
	{
		// is the leaf a zip?
		int nSlash = sSource.ReverseFind('\\');
		if(  nSlash > nZip )
		{
			sSource = CFileIndex::getTempZip( sSource, nZip, true );

			if( !m_bCopyTargetIsZip )
				bSuccess = MoveFile( sSource, sTarget );

			if( !bSuccess )
			{
				if( m_bCopyTargetIsZip )
				{
					sSource = '"' + sSource + '"';
					bSuccess = addToZip( sSource, sTarget );
				}
				else
					bSuccess = CopyFile( sSource, sTarget, FALSE );

				try
				{
					CFileStatus fileStatus;
					if( CFile::GetStatus( sSource, fileStatus ) )
					{
						fileStatus.m_attribute &= 0x38;  // clear read, sys and hidden
						CFile::SetStatus( sSource, fileStatus );
					}

					CFile::Remove( sSource );
				}
				catch( CException* e )
				{
					e->Delete();
				}
			}

			return bSuccess;
		}
	}

	if( m_bCopyTargetIsZip )
	{
		m_sZipSourceFiles += " \"" + sSource + '"';
		bSuccess = TRUE;
	}
	else
	{
		CString sStatus;
		sStatus.Format( IDS_COPYING_FMT, sSource, sTarget );
		thisApp()->setStatus( sStatus );
		bSuccess = CopyFile( sSource, sTarget, FALSE );
	}

	return bSuccess;
}

BOOL CFileListView::doDirCopy( CString sSourceDir, const CString& sTargetDir, int nOverWriteResponse )
{
	BOOL bSuccess = TRUE;

	WIN32_FIND_DATA sourceData;
	CString sSearchPath = sSourceDir + "*";
	HANDLE hFind = FindFirstFile( sSearchPath, &sourceData );
	if (hFind != INVALID_HANDLE_VALUE)
	{
		do
		{
			CString sNewSource = sSourceDir + sourceData.cFileName;
			CString sNewTarget = sTargetDir + '\\' + sourceData.cFileName;

			CFileStatus targetStatus;
			BOOL bTargetExists = CFile::GetStatus( sNewTarget, targetStatus );
			BOOL bSourceIsDir = sourceData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;

			if( bSourceIsDir && 
				!( sourceData.cFileName[0] != '.' || (sourceData.cFileName[1] != '.' && sourceData.cFileName[1] != '\0' ) ) )
				continue;

			if( bTargetExists )
			{
				if( nOverWriteResponse != IDOK )
				{
					COverWriteDialog dlg;
					CString sMsg;
					sMsg.LoadString(IDS_ALREADYEXISTS);
					dlg.m_sMessage = sNewTarget + sMsg;
					nOverWriteResponse = dlg.DoModal();
					
					if( nOverWriteResponse == IDCANCEL )
						break;
					else if( nOverWriteResponse == IDNO )
						continue;
				}
			}
			
			if( bSourceIsDir )
			{
				if( !bTargetExists )
				{
					// target directory does not exist, so make it
					if( _mkdir( sNewTarget ) == -1 )
					{
						CString sMsg;
						sMsg.LoadString( IDS_CREATE_FOLDER_ERR );
						sMsg += sNewTarget;

						bSuccess = FALSE;
						if( simpleMessageBox( sMsg ) == IDCANCEL )
							break;
						else
							continue;
					}

					bSuccess = doDirCopy( sNewSource, sNewTarget, nOverWriteResponse );
				}
			}
			else
			{
				bSuccess = doFileCopy( sNewSource, sNewTarget );
			}

		} while ( bSuccess && FindNextFile( hFind, &sourceData ) );
		FindClose( hFind );
	}

	return bSuccess;
}


void CFileListView::OnFileCommand() 
{
	CCommandDialog dlg;
	if( dlg.DoModal() == IDOK )
	{
		CWaitCursor cursor;

		try
		{
			char pszCmdFileName[MAX_PATH];

			if( !GetTempFileName( thisApp()->getZipPath(), "cmd", 0, pszCmdFileName ) )
			{
				DWORD nErr = GetLastError();

				char* pszMessage;
				FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
							   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );
				simpleMessageBox( pszMessage );

				return;
			}

			CString sCmdFile = pszCmdFileName;

			if( !dlg.m_bSingleExec )
			{
				sCmdFile = sCmdFile.Left( sCmdFile.GetLength() - 3 ) + "bat";
				CFile::Rename( pszCmdFileName, sCmdFile );
			}

			CString sExec;

			{
				CStdioFile cmdFile( sCmdFile, CFile::modeWrite );

				for( int nItem = 0; nItem < m_nItems; nItem++ )
				{
					CString sCommand;
					CString sTemplate = dlg.m_sCommand;
					if( GetListCtrl().GetItemState( nItem, LVIS_SELECTED ) == LVIS_SELECTED )
					{
						int nPercent;

						if( dlg.m_bSingleExec )
						{
							// skip to first space
							int nSpace = sTemplate.Find( '%' );
							if( nSpace == -1 )
								sTemplate.Empty();
							else
								sTemplate = sTemplate.Mid( nSpace );
						}

						while( ( nPercent = sTemplate.Find( '%' )) != -1 )
						{
							sCommand += sTemplate.Left( nPercent );
							switch( sTemplate[ nPercent + 1 ] )
							{
								case 'D':
								{
									sCommand += getDirectoryPath( nItem );
									break;
								}

								case 'E':
								{
									sCommand += getType( nItem );
									break;
								}

								case 'F':
								{
									sCommand += getPath( nItem );
									break;
								}

								case 'N':
								{
									sCommand += getName( nItem );
									break;
								}

								case 'P':
								{
									CString sPrefix = getName( nItem );
									int nDot = sPrefix.ReverseFind( '.' );
									if( nDot != -1 )
										sPrefix = sPrefix.Left( nDot );

									sCommand += sPrefix;
									break;
								}

								case 'V':
								{
									sCommand += getVolume( nItem );
									break;
								}

								case '%':
								{
									sCommand += '%';
									break;
								}
							}

							sTemplate = sTemplate.Mid( nPercent + 2 );
						}

						sCommand += sTemplate + '\n';
						cmdFile.WriteString( sCommand );
					}
				}

				CString sCleanup;
				
				if( dlg.m_bPause ) sCleanup += "pause\n";
				sCleanup += "del " + sCmdFile + '\n';

				if( dlg.m_bSingleExec )
				{
					CString sBatchFile = sCmdFile.Left( sCmdFile.GetLength() - 3 ) + "bat";
					CStdioFile batchFile( sBatchFile, CFile::modeCreate | CFile::modeWrite );

					CString sCommand = dlg.m_sCommand;

					int nSpace = sCommand.Find( '%' );
					if( nSpace != -1 )
						sCommand = sCommand.Left( nSpace );

					batchFile.WriteString( sCommand + sCmdFile + '\n' );

					batchFile.WriteString( sCleanup );
					batchFile.WriteString( "del " + sBatchFile );
					sExec = sBatchFile;
				}
				else
				{
					cmdFile.WriteString( sCleanup );
					sExec = sCmdFile;
				}
			}
				
			bool bDoRun;
			if( dlg.m_bPreview )
			{
				CString sCommand = regGetString( HKEY_CURRENT_USER, "options", "editor", "notepad.exe" );
				if( !sExec.IsEmpty() && !sCommand.IsEmpty() )
				{
					const char* pszFile = sExec;
					ShellExecute( m_hWnd, NULL, sCommand, (char*) pszFile, NULL, SW_SHOW );
				}

				if( dlg.m_bSingleExec && !sCmdFile.IsEmpty() )
				{
					const char* pszFile = sCmdFile;
					ShellExecute( m_hWnd, NULL, sCommand, (char*) pszFile, NULL, SW_SHOW );
				}

				if( simpleMessageBox( IDS_RUN_COMMAND_QUEST, MB_YESNO ) == IDYES )
					bDoRun = true;
				else
					bDoRun = false;
			}
			else
				bDoRun = true;

			if( bDoRun )
				ShellExecute( m_hWnd, NULL, sExec, NULL, NULL, SW_SHOW );
			else
			{
				CFile::Remove( sExec );
				if( dlg.m_bSingleExec )
					CFile::Remove( sCmdFile );
			}
		}
		catch( ... )
		{
			DWORD nErr = GetLastError();

			char* pszMessage;
			FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
						   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );
			CString sMsg;
			sMsg.LoadString( IDS_CMD_SYS_ERR );
			sMsg += pszMessage;
			
			simpleMessageBox( pszMessage );

			return;
		}
	}
}

void CFileListView::OnInitialUpdate() 
{
	CListView::OnInitialUpdate();
	
	// Add initialization code
	if( m_headerctrl.m_hWnd == NULL )
		m_headerctrl.SubclassWindow( ::GetDlgItem(m_hWnd,0) );
}

void CFileListView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch( nChar )
	{
		case VK_PAUSE:
			GetDocument()->minimizeWindow();
			break;

		case VK_TAB:
			GetDocument()->setFocusOnContents();
			break;
	}
	
	CListView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CFileListView::OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
	// TODO: Add your control notification handler code here

	int nItem = pDispInfo->item.iItem;
    ASSERT (-1 != nItem );
	ASSERT ( nItem < m_nItems );

	CFileInfo* pFile = m_pResults[ nItem ].m_pFileInfo;
	CFileInfo* pDir  = m_pResults[ nItem ].m_pDirInfo;
	ASSERT( pFile );

    if(pDispInfo->item.mask & LVIF_TEXT && pDispInfo->item.iSubItem >= clmFirst && pDispInfo->item.iSubItem <= clmLast )
	{
        //  Fill in the text information.
		CString sFolderWord; sFolderWord.LoadString(IDS_FOLDER);
		CString sFolder;     sFolder.Format("<%s>", sFolderWord);
        switch (m_nDisplayOrder[ pDispInfo->item.iSubItem ] )
		{
			case clmName:
			{   //  Copy the main item text - filename.
				if( pDir && pDir->m_nParent )  lstrcpy( pDispInfo->item.pszText, pFile->m_pszName);
				else lstrcpy( pDispInfo->item.pszText, sFolder.GetBuffer(2) );
                break;
			}

			case clmRank:
			{   //  Copy subitem 1 text - rank.
				char pszRank[ 20 ];
				_itoa( m_pResults[ nItem ].m_nRank + 1, pszRank, 10 );
                lstrcpy( pDispInfo->item.pszText, pszRank );
                break;
			}

			case clmExtension:
			{   //  Copy subitem 2 text - type extension.
				char* pType = ( pDir && pDir->m_nParent ) ? pFile->m_pszName : "";
				char* pLast = "";
				while( *pType ) {
					if( *pType == '.' ) pLast = pType; // find last dot
					pType++;
				}
				if( *pLast ) pLast++;
                lstrcpy( pDispInfo->item.pszText, pLast );
                break;
			}

			case clmPath:
			{   //  Copy subitem 3 text - dir name.
				const char* pDirName = ( pDir && pDir->m_nParent ) ? pDir->m_pszName : pFile->m_pszName;
                lstrcpy( pDispInfo->item.pszText, pDirName );
                break;
			}

			case clmDate:
			{   //  Copy subitem 4 text - time last written.
				CString sFormat; sFormat.LoadString(IDS_DATEFORMAT);
				lstrcpy( pDispInfo->item.pszText, pFile->m_timeLastWritten.Format( sFormat ));
                break;
			}

			case clmSize:
			{   //  Copy subitem 5 text - size.
				char pszSize[20];
				_itoa( pFile->m_nSize, pszSize, 10 );

				lstrcpy( pDispInfo->item.pszText, pszSize );
                break;
			}

			case clmAttributes:
			{
				char attrib[10];
				char* pAttrib = attrib;
				DWORD dwAttrib = pFile->m_dwAttributes;

				if( dwAttrib & FILE_ATTRIBUTE_COMPRESSED )
					*pAttrib++ = 'c';

				if( dwAttrib & FILE_ATTRIBUTE_ARCHIVE )
					*pAttrib++ = 'a';

				if( dwAttrib & FILE_ATTRIBUTE_DIRECTORY )
					*pAttrib++ = 'd';

				if( dwAttrib & FILE_ATTRIBUTE_SYSTEM )
					*pAttrib++ = 's';

				if( dwAttrib & FILE_ATTRIBUTE_HIDDEN )
					*pAttrib++ = 'h';

				if( dwAttrib & FILE_ATTRIBUTE_READONLY )
					*pAttrib++ = 'r';

				*pAttrib = '\0';

				lstrcpy( pDispInfo->item.pszText, attrib );
                break;
			}

			case clmVolume:
			{
				if( pDir )
				{
					if( pDir->m_nParent )
					{
						CFileInfo* pVolume = GetDocument()->getFileInfo( pDir->m_nParent );
						ASSERT( pVolume );
						lstrcpy( pDispInfo->item.pszText, pVolume->m_pszName );
					}
					else
						lstrcpy( pDispInfo->item.pszText, pDir->m_pszName );
				}
				
				break;
			}

			default:
                break;
		}
	}

	*pResult = 0;
}

void CFileListView::toggleColumn( ColumnType type )
{
	if( m_bEnabled[ type ] )
	{
		for(int i = clmFirst; i < clmLast; i++ )
			if( m_nDisplayOrder[ i ] == type )
				break;

		GetListCtrl().DeleteColumn( i );

		// adjust display order numbers
		for( i++; i <= clmLast; i++ )
			m_nDisplayOrder[ i - 1 ] = m_nDisplayOrder[ i ];

		m_nDisplayOrder[ clmLast ] = -1;
		m_bEnabled[ type ] = false;
	}
	else
	{
		for( int i = clmFirst; i < clmLast; i++ )
			if( m_nDisplayOrder[ i ] == -1 )
				break;

		int nSubItem = ( i ) ? i : -1;
		GetListCtrl().InsertColumn(i, m_columns[ type ].m_sLabel,
								      m_columns[ type ].m_dwFormat, 
									  m_columns[ type ].m_nWidth, nSubItem );

		m_nDisplayOrder[ i ] = type;
		m_bEnabled[ type ] = true;
	}
}

void CFileListView::OnColumnName() 
{
	toggleColumn( clmName );
}

void CFileListView::OnUpdateColumnName(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmName ] );
}


void CFileListView::OnColumnAttrib() 
{
	toggleColumn( clmAttributes );
}

void CFileListView::OnUpdateColumnAttrib(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmAttributes ] );
}

void CFileListView::OnColumnFolder() 
{
	toggleColumn( clmPath );
}

void CFileListView::OnUpdateColumnFolder(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmPath ] );
}

void CFileListView::OnColumnRank() 
{
	toggleColumn( clmRank );
}

void CFileListView::OnUpdateColumnRank(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmRank ] );
}

void CFileListView::OnColumnSize() 
{
	toggleColumn( clmSize );
}

void CFileListView::OnUpdateColumnSize(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmSize ] );
}

void CFileListView::OnColumnTime() 
{
	toggleColumn( clmDate );
}

void CFileListView::OnUpdateColumnTime(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmDate ] );
}

void CFileListView::OnColumnType() 
{
	toggleColumn( clmExtension );
}

void CFileListView::OnUpdateColumnType(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmExtension ] );
}

void CFileListView::OnColumnVolume() 
{
	toggleColumn( clmVolume );
}

void CFileListView::OnUpdateColumnVolume(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bEnabled[ clmVolume ] );
}

void CFileListView::DragColumn(int source, int dest)
{
	TCHAR sColText[160];

	ASSERT( source >= clmFirst && source <= clmLast );
	ASSERT( dest >= clmFirst );

	int nLast = m_nDisplayOrder[ source ];
	if( dest > source )
	{
		for( int i = source + 1; i < dest; i++ )
			m_nDisplayOrder[ i - 1 ] = m_nDisplayOrder[ i ];

		m_nDisplayOrder[ dest - 1 ] = nLast;
	}
	else
	{
		for( int i = source; i > dest; i-- )
			m_nDisplayOrder[ i ] = m_nDisplayOrder[ i - 1 ];

		m_nDisplayOrder[ dest ] = nLast;
	}

	// Insert a column at dest
	LV_COLUMN       lv_col;
	lv_col.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH | LVCF_SUBITEM;
	lv_col.pszText = sColText;
	lv_col.cchTextMax = 159;
	GetListCtrl().GetColumn( source, &lv_col );
	lv_col.iSubItem = dest;
	GetListCtrl().InsertColumn( dest, &lv_col );

	// Adjust source col number since it might have changed 
	// because a new column was inserted
	if( source > dest ) 
		source++;

	GetListCtrl().DeleteColumn( source );

	Invalidate();
}


/////////////////////////////////////////////////////////////////////////////
// CMyHeaderCtrl

CMyHeaderCtrl::CMyHeaderCtrl() 
	: marker_rect(0,0,0,0)
{
	m_pWidth = NULL;
	m_bDragging = FALSE;
	m_bCheckForDrag = FALSE;
	m_fpDragCol = NULL;
	m_pOwnerWnd = NULL;
}

CMyHeaderCtrl::CMyHeaderCtrl(CWnd *pWnd, void (CWnd::*fpDragCol)(int, int)) 
	: marker_rect(0,0,0,0)
{
	m_pWidth = NULL;
	m_bDragging = FALSE;
	m_bCheckForDrag = FALSE;
	m_fpDragCol = fpDragCol;
	m_pOwnerWnd = pWnd;
}

CMyHeaderCtrl::~CMyHeaderCtrl()
{
}


BEGIN_MESSAGE_MAP(CMyHeaderCtrl, CHeaderCtrl)
	//{{AFX_MSG_MAP(CMyHeaderCtrl)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMyHeaderCtrl message handlers

void CMyHeaderCtrl::OnMouseMove(UINT nFlags, CPoint point) 
{
	if( (MK_LBUTTON & nFlags) == 0)
	{
		// The left mouse button is not pressed - so reset flags
		m_bCheckForDrag = FALSE;
		m_bDragging = FALSE;
	}
	else if( m_bDragging )
	{
		// Get column number that falls under the mouse
		int i=0, cx = 0;
		if( point.x > 0 )
			for( i = 0; i < GetItemCount(); i++ )
			{
				if( point.x >= cx && point.x < cx + m_pWidth[i] )
					break;
				cx += m_pWidth[i];
			}

		if( i != m_nDropPos )
		{
			m_nDropPos = i;

			CRect rect;
			GetWindowRect( &rect );

			// Invalidate area occupied by previous marker
			InvalidateRect( &marker_rect );

			// Draw a new marker
			CClientDC dc(this);
			POINT pts[3];
			pts[0].x = cx; pts[1].x = cx -3; pts[2].x = cx +3;
			pts[0].y = rect.Height(); pts[1].y = pts[2].y = rect.Height() -7; 
			dc.Polygon( pts, 3 );

			// save marker information
			marker_rect.left = cx - 4;
			marker_rect.top = rect.Height() -8;
			marker_rect.right = cx + 4;
			marker_rect.bottom = rect.Height();
		}
		return;
	}
	else if( m_bCheckForDrag )
	{
		// The mouse button was pressed over a column header
		// and now the mouse has moved - so start drag
		m_bCheckForDrag = FALSE;

		m_bDragging = TRUE;
		m_nDropPos = m_nDragCol;

		SetCapture();

		// Store information for later use
		int iCount = GetItemCount();
		HD_ITEM hd_item;
		m_pWidth = new int[iCount];
		for( int i = 0; i < iCount; i++ )
		{
			hd_item.mask = HDI_WIDTH;
			GetItem( i, &hd_item );
			m_pWidth[i] = hd_item.cxy;

		}
		return;
	}
	
	CHeaderCtrl::OnMouseMove(nFlags, point);
}

void CMyHeaderCtrl::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ASSERT( m_pOwnerWnd != NULL && m_fpDragCol != NULL );

	if( m_bDragging )
	{
		m_bDragging = FALSE;
		delete[] m_pWidth;
		ReleaseCapture();
		Invalidate();

		// Call the callback function.
		if( m_nDragCol != m_nDropPos && m_nDragCol != m_nDropPos -1 )

			(m_pOwnerWnd->*m_fpDragCol)( m_nDragCol, m_nDropPos );
		else if( m_nDragCol == m_nDropPos)
		{
			CFileListView* pFileList = (CFileListView*) m_pOwnerWnd;
			pFileList->sortColumn( m_nDragCol );
			pFileList->Invalidate();
		}
	}
	CHeaderCtrl::OnLButtonUp(nFlags, point);
}


void CMyHeaderCtrl::SetCallback( CWnd* pWnd, void (CWnd::*fpDragCol)(int, int) )
{
	m_fpDragCol = fpDragCol;
	m_pOwnerWnd = pWnd;
}

void CMyHeaderCtrl::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Determine if mouse was pressed over a column header
	HD_HITTESTINFO hd_hittestinfo;
	hd_hittestinfo.pt = point;
	SendMessage(HDM_HITTEST, 0, (LPARAM)(&hd_hittestinfo));
	if( hd_hittestinfo.flags == HHT_ONHEADER )
	{
		m_nDragCol = hd_hittestinfo.iItem;
		m_bCheckForDrag = TRUE;
	}
	
	CHeaderCtrl::OnLButtonDown(nFlags, point);
}


void CFileListView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	CListView::OnPrint(pDC, pInfo);
}

BOOL CFileListView::OnPreparePrinting(CPrintInfo* pInfo) 
{
 	return DoPreparePrinting(pInfo);
	
//	return CListView::OnPreparePrinting(pInfo);
}


void CFileListView::OnFilePrintList() 
{
	CListView::OnFilePrint();
}

void CFileListView::OnOdfinditem(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLVFINDITEM* pFindInfo = (NMLVFINDITEM*)pNMHDR;

	int nFound = 0;
	if( pFindInfo->lvfi.flags & LVFI_STRING )
	{
		char cFind = tolower( *(pFindInfo->lvfi.psz) );

		// start by looking down from start
		for( nFound = pFindInfo->iStart; nFound < m_nItems; nFound++ )
		{
			CString sNext = getName( nFound );
			char c = tolower( sNext[0] );

			if( c == cFind )
				break;
		}

		// if the next one fits, us it
		if( nFound >= m_nItems )
		{
			// not found, look from top
			for( nFound = 0; nFound < m_nItems; nFound++ )
			{
				CString sNext = getName( nFound );
				char c = tolower( sNext[0] );

				if( c == cFind )
					break;
			}
		}


		// otherwise find first match
		if( nFound >= m_nItems )
			nFound = pFindInfo->iStart - 1;
	}

	if( nFound < 0 )
		nFound = 0;

	*pResult = nFound;
}

