/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// FileListView.h : header file
//

#ifndef __FILELISTVIEW_H__
#define __FILELISTVIEW_H__ 1

#ifndef _SHLOBJ_H_
#include <shlobj.h>
#endif

#ifndef __FILEINDX_H__
#include "fileindx.h"
#endif

#ifndef __WILBUDOC_H__
#include "wilbudoc.h"
#endif


struct CColumnInfo {
	CString m_sLabel;
	CString m_sKeyPrefix;
	int     m_nWidth;
	DWORD   m_dwFormat;
};


/////////////////////////////////////////////////////////////////////////////
// CMyHeaderCtrl window

class CMyHeaderCtrl : public CHeaderCtrl
{
// Construction
public:
	CMyHeaderCtrl();
	CMyHeaderCtrl(CWnd* pWnd, void (CWnd::*fpDragCol)(int, int));

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMyHeaderCtrl)
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMyHeaderCtrl();
	void SetCallback(CWnd* pWnd, void (CWnd::*fpDragCol)(int, int));

protected:
	BOOL	m_bCheckForDrag;
	BOOL	m_bDragging;
	int	*m_pWidth;
	int	m_nDragCol;
	int	m_nDropPos;
	CRect	marker_rect;
	void	(CWnd::*m_fpDragCol)(int, int);
	CWnd	*m_pOwnerWnd;



	// Generated message map functions
protected:
	//{{AFX_MSG(CMyHeaderCtrl)
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.


/////////////////////////////////////////////////////////////////////////////
// CFileListView view

class CFileListView : public CListView
{
	protected:
		CFileListView();           // protected constructor used by dynamic creation
		DECLARE_DYNCREATE(CFileListView)

	// Attributes
	public:
		CWilburDoc* GetDocument();

		enum ColumnType { clmFirst = 0, clmName = clmFirst, clmRank, clmExtension, clmPath, clmDate,
			clmSize, clmAttributes, clmVolume, clmLast = clmVolume };

		enum FileOperation { fileOpCopy, fileOpMove, fileOpDelete };

	// Operations
			void clearList();
			void setCount( int nItems );
			void setActualListCount();
			void addFile( CFileResult& fileResult );
			void sort();
			void sortColumn( int nNewSortColumn );
			CString nextFile();
			CString prevFile();
			CString firstFile();
			CString lastFile();
			void editTextFile();
			void launchFile();
			void printFile();
			void changeFont();
			void selectAll();
			BOOL getFont( LOGFONT& lf );
			void setFont( LOGFONT& lf );
			int getCurrentIndex();

	public:

	// Overrides
		// ClassWizard generated virtual function overrides
		//{{AFX_VIRTUAL(CFileListView)
	public:
	virtual void OnInitialUpdate();
	protected:
		virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

	// Implementation
	protected:
		virtual ~CFileListView();
	#ifdef _DEBUG
		virtual void AssertValid() const;
		virtual void Dump(CDumpContext& dc) const;
	#endif

		// Generated message map functions
	protected:
		//{{AFX_MSG(CFileListView)
		afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDblclk(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnItemchanged(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDestroy();
	afx_msg void OnRclick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnFileExportlist();
	afx_msg void OnFileCopy();
	afx_msg void OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnColumnName();
	afx_msg void OnUpdateColumnName(CCmdUI* pCmdUI);
	afx_msg void OnColumnAttrib();
	afx_msg void OnUpdateColumnAttrib(CCmdUI* pCmdUI);
	afx_msg void OnColumnFolder();
	afx_msg void OnUpdateColumnFolder(CCmdUI* pCmdUI);
	afx_msg void OnColumnRank();
	afx_msg void OnUpdateColumnRank(CCmdUI* pCmdUI);
	afx_msg void OnColumnSize();
	afx_msg void OnUpdateColumnSize(CCmdUI* pCmdUI);
	afx_msg void OnColumnTime();
	afx_msg void OnUpdateColumnTime(CCmdUI* pCmdUI);
	afx_msg void OnColumnType();
	afx_msg void OnUpdateColumnType(CCmdUI* pCmdUI);
	afx_msg void OnColumnVolume();
	afx_msg void OnUpdateColumnVolume(CCmdUI* pCmdUI);
	afx_msg void OnFileCommand();
	afx_msg void OnUpdateFileCopy(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileCommand(CCmdUI* pCmdUI);
	afx_msg void OnFilePrintList();
	afx_msg void OnOdfinditem(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnFileListClip();
	//}}AFX_MSG
		DECLARE_MESSAGE_MAP()

	private:
		int m_nItems;   // total number of items in list
		int m_nNextItem;  // used to count items when adding
		CFileResult* m_pResults;

		CMyHeaderCtrl m_headerctrl;
		void DragColumn(int source, int dest);

		CFont   m_font;
		LOGFONT m_logFont;
		BOOL    m_bFont;

		int     m_nSortOrder[ clmLast + 1 ];
		int     m_nDisplayOrder[ clmLast + 1 ];
		bool    m_bReverseSort[ clmLast + 1 ];
		bool    m_bEnabled[ clmLast + 1 ];

		bool    m_bCopyTargetIsZip;
		CString m_sZipSourceFiles;

		CColumnInfo m_columns[ clmLast + 1 ];

		CString focusOn( int nIndex );
		CString getName( int nIndex );
		CString getType( int nIndex );
		CString getPath( int nIndex );
		CString getDirectoryPath( int nIndex );
		CString getVolume( int nIndex );
		CString getDate( int nIndex );
		CString getSize( int nIndex );

		// CString getColumnText( int nIndex, int nColumn );

		static char* getLeafName( char* pPathName );

		LPMALLOC m_pMalloc;
		LPITEMIDLIST getUIObject( const CString sPath, LPSHELLFOLDER lpsf, LPSHELLFOLDER& pFolder );
		LPITEMIDLIST GetNextItemID(LPITEMIDLIST pidl);
		LPITEMIDLIST CopyItemID(LPITEMIDLIST pidl);
		CString StrRettoCString(LPITEMIDLIST pidl, LPSTRRET lpStr);
		int compareFunc(CFileResult* pResult1, CFileResult* pResult2 );
		int compareFileResult( CFileResult* pResult1, CFileResult* pResult2,
			                    int nSortColumn, bool bReverseSort );
		void toggleColumn( ColumnType type );

		void doFileOperation( FileOperation opType );

		void CFileListView::shortsort ( CFileResult *lo, CFileResult *hi );
		CString CFileListView::getDesktopFolder( CString sDisplayName );
		BOOL CFileListView::doFileCopy( CString sSource, const CString& sTarget );
		BOOL CFileListView::doDirCopy( CString sSource, const CString& sTarget, int nOverWriteResponse );
		bool CFileListView::addToZip( CString sFileList, CString sTargetZip );

		CString getItemTextLine( int nItem );
};

/////////////////////////////////////////////////////////////////////////////


// CDropSource

class FAR CDropSource : public IDropSource
{
public:    
    CDropSource();

    /* IUnknown methods */
    STDMETHOD(QueryInterface)(REFIID riid, void FAR* FAR* ppvObj);
    STDMETHOD_(ULONG, AddRef)(void);
    STDMETHOD_(ULONG, Release)(void);

    /* IDropSource methods */
    STDMETHOD(QueryContinueDrag)(BOOL fEscapePressed, DWORD grfKeyState);
    STDMETHOD(GiveFeedback)(DWORD dwEffect);
 
private:
    ULONG m_refs;     
};  


#endif
