/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
#ifndef __FILEMEMORY_H__
#define __FILEMEMORY_H__ 1

#include "mmfile.h"

typedef DWORD MemSize;
typedef DWORD MemOffset;

class CWilburDoc;

class CFileMemory : public CObject
{
	DECLARE_SERIAL( CFileMemory );

	public:
		CFileMemory();
		virtual ~CFileMemory();
		CFileMemory( MemSize nInitialIncrement, MemSize nMaxIncrement,
			         CString sFileExtension );

		MemOffset getNewSpace( MemSize nBytes );
		BYTE*     getPointer( MemOffset nOffset );
		void Serialize( CArchive& archive );
		BOOL enableWrite();
		BOOL disableWrite();
		CFile* createTemporaryFile();
		void mapFile( MemSize nSize = 0 );
		void closeFile();
		void reportError( const char* pszWhere );
		BOOL getFileStatus( CFileStatus& fileStatus ) { return m_file.GetStatus( fileStatus ); }

	private:
		CFile             m_file;
		BOOL              m_bFileOpen;
		CMemoryMappedFile m_fileMap;
		BOOL              m_bMapOpen;
		CString           m_sExtension;
		MemSize           m_nInitialIncrement;
		MemSize           m_nMaxIncrement;
		MemSize           m_nCurrentSize;
		MemSize           m_nNextNew;
		BYTE*             m_pBytes;
		CString           m_sStoredFileName;
		BOOL              m_bTemporaryFile;

		void clearAll();
		CString makeFileName( CWilburDoc* pDocument );
		void openMapFile( CWilburDoc* pDocument );
};

#endif
