/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// fileindx.cpp : implementation of the CFileIndex class
//
#include "stdafx.h"

#include <io.h>
#include <direct.h>

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __MAINFRM_H__
#include "mainfrm.h"
#endif

#ifndef __WILBUDOC_H__
#include "wilbudoc.h"
#endif

#ifndef __FILEINDX_H__
#include "fileindx.h"
#endif

#ifndef __DIALOGS_H__
#include "dialogs.h"
#endif

// H.W. begin --------------------------------------------
#ifndef __EXTENSIONS_HW_H__
#include "XtensionsHW.h"
#define __EXTENSIONS_HW_H__
#endif
// H.W. end ----------------------------------------------

IMPLEMENT_SERIAL( CTextStorage, CObject, 0 );
IMPLEMENT_SERIAL( CFileStorage, CTextStorage, 0 );
IMPLEMENT_SERIAL( CBuildLinksList, CObject, 0 );
IMPLEMENT_SERIAL( CFileLinks, CObject, 0 );
IMPLEMENT_DYNAMIC( CWordLinks, CObject );
IMPLEMENT_SERIAL( CFileIndex, CObject, 0 );


#if _DEBUG
void debugMsg( CString sMsg)
{
	thisApp()->getDocument()->oleAddMessage( sMsg );
}

void debugErr()
{
	DWORD nErr = GetLastError();

	char* pszMessage;
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
				   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );

	thisApp()->getDocument()->oleAddMessage( pszMessage );
	LocalFree( pszMessage );
}
#endif

/*
void dumpString( const CString& sText )
{
	CStdioFile dumpFile( "wildump.txt", CFile::modeCreate | CFile::modeNoTruncate | CFile::modeReadWrite );
	dumpFile.SeekToEnd();
	dumpFile.WriteString( sText );
}
*/

UINT indexThread( LPVOID pParam )
{
	// undocumented call to enable CMemoryExceptions in helper thread
	AfxSetNewHandler( AfxNewHandler );

	CFileIndex* pIndex = (CFileIndex*) pParam;

	ASSERT_VALID( pIndex );

	thisApp()->setHaltStatus( FALSE );
	thisApp()->setThreadRunning( TRUE );
	thisApp()->getDocument()->SetModifiedFlag( FALSE ); // so it doesn't prompt to save an interrupted index

	try
	{
		CTime timeStart = CTime::GetCurrentTime();
		pIndex->build();
		thisApp()->getDocument()->SetModifiedFlag( TRUE );

		if( !thisApp()->isBatch() )
		{
			CString sMessage;

			sMessage.Format(IDS_INDEX_RESULTS_FMT,
							 pIndex->numberOfFiles(), pIndex->numberOfContents(), pIndex->numberOfWords(),
							 pIndex->numberOfBytes() );

			CTimeSpan tsDuration = CTime::GetCurrentTime() - timeStart;

			sMessage += tsDuration.Format( IDS_SAVE_IMMEDIATE_FMT );

			if( simpleMessageBox( sMessage, MB_YESNO | MB_DEFBUTTON2 |MB_ICONQUESTION ) == IDYES )
			{
				thisApp()->getDocument()->doSaveAfterIndex();
			}
		}
	}
	catch( CMemoryException* e )
	{
		e->Delete();
		// there should be enough memory now
		simpleMessageBox( IDS_MEMORY_EXHAUSTED );
	}
	catch( CException* e )
	{
		e->Delete();			

		simpleMessageBox( IDS_INDEXING_ABORTED );
	}

	CWnd* pFrame = thisApp()->getMainFrame();
	if( pFrame )
		pFrame->PostMessage( WM_COMMAND, ID_INDEX_FINISHED, NULL );

	thisApp()->setHaltStatus( FALSE );
	thisApp()->setThreadRunning( FALSE );

	return 0;
}

UINT updateThread( LPVOID pParam )
{
	// undocumented call to enable CMemoryExceptions in helper thread
	AfxSetNewHandler( AfxNewHandler );

	CFileIndex* pIndex = (CFileIndex*) pParam;

	ASSERT_VALID( pIndex );

	thisApp()->setHaltStatus( FALSE );
	thisApp()->setThreadRunning( TRUE );
	thisApp()->getDocument()->SetModifiedFlag( TRUE );

	try
	{
		CTime timeStart = CTime::GetCurrentTime();
		pIndex->update();

		if( !thisApp()->isBatch() )
		{
			CString sMessage;
			CString sFormat;
			sFormat.LoadString( IDS_UPDATE_RESULTS );
			sMessage.Format( sFormat,
							 pIndex->numberOfFiles(), pIndex->numberOfContents(), pIndex->numberOfWords(),
							 pIndex->numberOfBytes() );

			CTimeSpan tsDuration = CTime::GetCurrentTime() - timeStart;
			sFormat.LoadString( IDS_INDEXING_TIME_FMT );
			sMessage += tsDuration.Format( sFormat );
			simpleMessageBox( sMessage );
		}
	}
	catch( CException* e )
	{
		thisApp()->getDocument()->SetModifiedFlag( FALSE );  // so it doesn't prompt to save the empty index

		if( e->IsKindOf( RUNTIME_CLASS( CMemoryException ) ) )
			simpleMessageBox( IDS_MEMORY_EXHAUSTED );
		else
			simpleMessageBox( IDS_INDEXING_ABORTED );

		e->Delete();			
		
		thisApp()->setHaltStatus( FALSE );
		thisApp()->setThreadRunning( FALSE );

		return 0;
	}

	if( thisApp()->getMainFrame() )
		thisApp()->getMainFrame()->PostMessage( WM_COMMAND, ID_UPDATE_FINISHED, NULL );

	thisApp()->setHaltStatus( FALSE );
	thisApp()->setThreadRunning( FALSE );


	return 0;
}

CFileIndex::CFileIndex()
	:
		m_nMinWordLength(3), m_nMaxWordLength( 16 ),
		m_pFiles( NULL ), m_pWords( NULL ), m_pSkipWords( NULL ),
		m_bIsUpdate( FALSE ), m_bDoZipped( FALSE ),	m_dwInternationalChar( CHAR_ENGLISH ),
		m_bUseRanking( TRUE ), m_bTrackAll( true ),
		m_bOEMTextChar( FALSE ), m_numberHandling( noNumbers ),
	    m_pTemporaryLinks( NULL ), m_pLinks( NULL ), m_sSkipFile( "skip" ),
		m_bSavePassword( false ), m_bEncryptNewZips( false), m_bRestrictCopies( false ),
		m_bPDF( false )
{
	setCharMasks();

	// consider everything under ascii space binary except for the tab
	// and line termination characters
	memset( m_bBinary, 0, 256 * sizeof( bool ) );

	for( int i = 1; i <= 8; i++ )
		m_bBinary[ i ] = true;
	m_bBinary[ 7] = false;

	for( i = 14; i <= 31; i++ )
		m_bBinary[ i ] = true;

	m_htmlCharMap.SetAt("quot", '"' );
    m_htmlCharMap.SetAt("amp", '&' );
    m_htmlCharMap.SetAt("lt", '<' );
    m_htmlCharMap.SetAt("gt", '>' );
	m_htmlCharMap.SetAt("apos", '\'');
    m_htmlCharMap.SetAt("Agrave", '' );
    m_htmlCharMap.SetAt("Aacute", '' );
    m_htmlCharMap.SetAt("Acirc", '' );
    m_htmlCharMap.SetAt("Atilde", '' );
    m_htmlCharMap.SetAt("Auml", '' );
    m_htmlCharMap.SetAt("Aring", '' );
    m_htmlCharMap.SetAt("AElig", '' );
    m_htmlCharMap.SetAt("Ccedil", '' );
    m_htmlCharMap.SetAt("Egrave", '' );
    m_htmlCharMap.SetAt("Eacute", '' );
    m_htmlCharMap.SetAt("Ecirc", '' );
    m_htmlCharMap.SetAt("Euml", '' );
    m_htmlCharMap.SetAt("Igrave", '' );
    m_htmlCharMap.SetAt("Iacute", '' );
    m_htmlCharMap.SetAt("Icirc", '' );
    m_htmlCharMap.SetAt("Iuml", '' );
    m_htmlCharMap.SetAt("ETH", '' );
    m_htmlCharMap.SetAt("Ntilde", '' );
    m_htmlCharMap.SetAt("Ograve", '' );
    m_htmlCharMap.SetAt("Oacute", '' );
    m_htmlCharMap.SetAt("Ocirc", '' );
    m_htmlCharMap.SetAt("Otilde", '' );
    m_htmlCharMap.SetAt("Ouml", '' );
    m_htmlCharMap.SetAt("Oslash", '' );
    m_htmlCharMap.SetAt("Ugrave", '' );
    m_htmlCharMap.SetAt("Uacute", '' );
    m_htmlCharMap.SetAt("Ucirc", '' );
    m_htmlCharMap.SetAt("Uuml", '' );
    m_htmlCharMap.SetAt("Yacute", '' );
    m_htmlCharMap.SetAt("THORN", '' );
    m_htmlCharMap.SetAt("szlig", '' );
    m_htmlCharMap.SetAt("agrave", '' );
    m_htmlCharMap.SetAt("aacute", '' );
    m_htmlCharMap.SetAt("acirc", '' );
    m_htmlCharMap.SetAt("atilde", '' );
    m_htmlCharMap.SetAt("auml", '' );
    m_htmlCharMap.SetAt("aring", '' );
    m_htmlCharMap.SetAt("aelig", '' );
    m_htmlCharMap.SetAt("ccedil", '' );
    m_htmlCharMap.SetAt("egrave", '' );
    m_htmlCharMap.SetAt("eacute", '' );
    m_htmlCharMap.SetAt("ecirc", '' );
    m_htmlCharMap.SetAt("euml", '' );
    m_htmlCharMap.SetAt("igrave", '' );
    m_htmlCharMap.SetAt("iacute", '' );
    m_htmlCharMap.SetAt("icirc", '' );
    m_htmlCharMap.SetAt("iuml", '' );
    m_htmlCharMap.SetAt("eth", '' );
    m_htmlCharMap.SetAt("ntilde", '' );
    m_htmlCharMap.SetAt("ograve", '' );
    m_htmlCharMap.SetAt("oacute", '' );
    m_htmlCharMap.SetAt("ocirc", '' );
    m_htmlCharMap.SetAt("otilde", '' );
    m_htmlCharMap.SetAt("ouml", '' );
    m_htmlCharMap.SetAt("oslash", '' );
    m_htmlCharMap.SetAt("ugrave", '' );
    m_htmlCharMap.SetAt("uacute", '' );
    m_htmlCharMap.SetAt("ucirc", '' );
    m_htmlCharMap.SetAt("uuml", '' );
    m_htmlCharMap.SetAt("yacute", '' );
    m_htmlCharMap.SetAt("thorn", '' );
    m_htmlCharMap.SetAt("yuml", '' );

	// see if pdftotext exists
	m_bPDF = thisApp()->hasPDFtoText();
}

CFileIndex::~CFileIndex()
{
	clear();
	thisApp()->closePDFCache();
}

void CFileIndex::clear()
{
	clearIncludes();
	clearExcludes();

	delete m_pFiles;            m_pFiles          = NULL;
	delete m_pWords;			m_pWords          = NULL;
	delete m_pSkipWords;		m_pSkipWords      = NULL;
	delete m_pTemporaryLinks;	m_pTemporaryLinks = NULL;
	delete m_pLinks;	   		m_pLinks	      = NULL;

	m_bIsUpdate = FALSE;
	m_bDoZipped = FALSE;
	m_bTrackAll = true;
	// m_bInternationalChar = 	( GetSystemDefaultLangID() != MAKELANGID( LANG_ENGLISH, SUBLANG_ENGLISH_US ));

	switch( PRIMARYLANGID( GetSystemDefaultLangID() ) )
	{
		case LANG_ENGLISH:
			m_dwInternationalChar = CHAR_ENGLISH;
			break;

		case LANG_DANISH:
		case LANG_NORWEGIAN:
			m_dwInternationalChar = CHAR_DANISH;
			break;

		case LANG_FINNISH:
			m_dwInternationalChar = CHAR_FINNISH;
			break;

		case LANG_FRENCH:
			m_dwInternationalChar = CHAR_FRENCH;
			break;

		case LANG_GERMAN:
			m_dwInternationalChar = CHAR_GERMAN;
			break;

		case LANG_SPANISH:
			m_dwInternationalChar = CHAR_SPANISH;
			break;

		case LANG_SWEDISH:
			m_dwInternationalChar = CHAR_SWEDISH;
			break;
	}

	m_bOEMTextChar = FALSE;
	m_numberHandling = noNumbers;
	m_sAddedChars.LoadString(IDS_DEFAULT_ADDED_CHARS);
	if( m_sAddedChars == "XXX" )
		m_sAddedChars.Empty();  // can't just have empty resource string :-(
	m_sAddedTrailingChars.Empty();
	m_nMinWordLength = 3;
	m_nMaxWordLength = 16;
	m_bUseRanking = TRUE;

	setCharMasks();
}

CString CFileIndex::encryptPassword( const CString sPassword )
{
	int nLength = sPassword.GetLength();
	ASSERT( nLength );

	int nLast = nLength * 2;
	CString sOut('.', nLast-- );
	srand( (unsigned)time( NULL ) );
	for( int i = 0; i < nLast; i++, nLast-- )
	{
		int nRandom = rand() % 256;
		sOut.SetAt( i, (char) nRandom );
		sOut.SetAt( nLast, (char) ( sPassword[ i ] ^ nRandom ) );
	}

	return sOut;
}

CString CFileIndex::decryptPassword( const CString sIn )
{
	int nLast = sIn.GetLength();
	ASSERT( nLast && !(nLast % 2 ) );

	int nLength = nLast-- / 2;

	CString sPassword( '.', nLength );

	for( int i = 0; i < nLast; i++, nLast-- )
	{
		sPassword.SetAt( i, (char) ( sIn[ nLast ] ^ sIn[ i ] ) );
	}

	return sPassword;
}

void CFileIndex::Serialize( CArchive& archive )
{
	ReleaseNumber nCurrentRelease = 17;
	ReleaseNumber nRelease;

	if( archive.IsStoring() )
		archive << nCurrentRelease;
	else
	{
		archive >> nRelease;
		if( nRelease < 1 && nRelease > nCurrentRelease )
			AfxThrowArchiveException( CArchiveException::badSchema );
	}
	
	CObject::Serialize( archive );
	m_includePatterns.Serialize( archive );
	m_excludePatterns.Serialize( archive );

	if( archive.IsStoring() )
	{
		archive << m_pFiles;
		archive << m_pWords;
		archive << m_pSkipWords;
		archive << m_pLinks;

		archive << m_timeBase << m_nFilesIndexed << m_nWordsIndexed << m_nBytesRead << m_nContentsIndexed;
		archive << (BYTE) m_bDoZipped;
		archive << (BYTE) m_numberHandling << m_dwInternationalChar;
		archive << m_sAddedChars << m_sAddedTrailingChars << m_bOEMTextChar;
		archive << m_nMinWordLength << m_nMaxWordLength << m_bUseRanking << m_bTrackAll;
		archive << m_sSkipFile;

		archive << (BYTE) m_bSavePassword << (BYTE) m_bEncryptNewZips << (BYTE) m_bRestrictCopies;

		if( !m_sPassword.IsEmpty() )
		{
			bool bPW = true;
			archive << (BYTE) bPW;

			if( m_bSavePassword )
			{
				CString sEncrypt = encryptPassword( m_sPassword );
				archive << sEncrypt;
			}
		}
		else
		{
			bool bPW = false;
			archive << (BYTE) bPW;
		}

		m_timeSaved = CTime::GetCurrentTime();
		thisApp()->savePDFCache();
	}
	else
	{
		DWORD nPos = archive.GetFile()->GetPosition();

		CFileStatus fsWil;
		if( archive.GetFile()->GetStatus( fsWil ) )
			m_timeSaved = fsWil.m_mtime;
		else
			m_timeSaved = 0;

		if( nRelease < 12 )
		{
			CTextStorage* pFiles;
			archive >> pFiles;
			m_pFiles = (CFileStorage*) pFiles; // let's hope this is never used
		}
		else
			archive >> m_pFiles;

		nPos = archive.GetFile()->GetPosition();
		
		archive >> m_pWords;
		nPos = archive.GetFile()->GetPosition();

		archive >> m_pSkipWords;

		nPos = archive.GetFile()->GetPosition();

		if( nRelease < 13 )
		{
			CFileLinks* pLinks;
			archive >> pLinks;
			delete pLinks;
		}

		nPos = archive.GetFile()->GetPosition();
		archive >> m_pLinks;
		nPos = archive.GetFile()->GetPosition();
		archive >> m_timeBase >> m_nFilesIndexed >> m_nWordsIndexed >> m_nBytesRead;

		if( nRelease > 13 )
			archive >> m_nContentsIndexed;
		else
			m_nContentsIndexed = m_nFilesIndexed;

		if( nRelease > 1 )
		{
			BYTE bFlag;
			archive >> bFlag;
			m_bDoZipped = bFlag;

			if( nRelease > 2 )
			{
				archive >> bFlag;
				m_numberHandling = (NumberInclusion) bFlag;
				
				if( nRelease > 3 )
				{
					if( nRelease < 11 )
					{
						archive >> bFlag;
						m_dwInternationalChar = (DWORD) bFlag;
					}
					else
						archive >> m_dwInternationalChar;

					if( nRelease > 4 )
					{
						archive >> m_sAddedChars;
						archive >> m_sAddedTrailingChars;

					}
				}
			}
		}

		if( nRelease > 5 )
		{
			BOOL bFlag;
			archive >> bFlag;
			m_bOEMTextChar = (BOOL) bFlag;
		}
		else
			m_bOEMTextChar = TRUE;

		nPos = archive.GetFile()->GetPosition();

		if( nRelease > 7 )
		{
			archive >> m_nMinWordLength >> m_nMaxWordLength;
			if( nRelease > 8 )
			{
				archive >> m_bUseRanking;
				if( nRelease > 14 )
				{
					BOOL bFlag;
					archive >> bFlag;
					m_bTrackAll = bFlag == TRUE;

					if( nRelease > 15 )
						archive >> m_sSkipFile;
				}
			}
		}
		else
		{
			m_nMaxWordLength = 0;
			m_nMinWordLength = 3;
		}

		if( nRelease < 16 )
		{
			emptyIndex();
			if( nRelease < 7 )
				m_nMaxWordLength = 0;

			simpleMessageBox( IDS_OLD_INDEX_WARN );
		}
		else if( nRelease > 16 )
		{
			BYTE bSave, bEncrypt, bRestrict;
			archive >> bSave >> bEncrypt >> bRestrict;
			m_bSavePassword =   (bSave != 0);
			m_bEncryptNewZips = (bEncrypt != 0);
			m_bRestrictCopies = (bRestrict != 0);

			BYTE bPW;
			archive >> bPW;
			if( bPW )
			{
				if( m_bSavePassword )
				{
					CString sEncrypted;
					archive >> sEncrypted;
					m_sPassword = decryptPassword( sEncrypted );
				}
				else if( !thisApp()->isBatch() )
				{
					CConfirmPassword passwordDlg;
					passwordDlg.m_sMessage.LoadString( IDS_GETPASSWORD );
					if( passwordDlg.DoModal() != IDCANCEL )
						m_sPassword = passwordDlg.m_sPassword;
				}
			}

			if( !thisApp()->isBatch() && mapsModified() )
				simpleMessageBox( IDS_FILEMODIFIED );
		}

		setCharMasks();
		thisApp()->openPDFCache();
	}
}

void CFileIndex::setCharMasks()
{
	// set the valid character masks

	memset( m_bValidChar, 0, 256 * sizeof( bool ) );
	memset( m_bValidTrailing, 0, 256 * sizeof( bool ) );

	for( int i = 'a'; i <= 'z'; i++ )
	{
		m_bValidChar[ i ] = m_bValidTrailing[ i ] = true;
		int j = i + 'A' - 'a';
		m_bValidChar[ j ] = m_bValidTrailing[ j ] = true;
	}

	m_bValidChar[ '_' ] = m_bValidTrailing[ '_' ] = true;

	if( m_numberHandling == allNumbers )
	{
		for( int i = '0'; i <= '9'; i++ )
			m_bValidChar[ i ] = m_bValidTrailing[ i ] = true;
	}

	if( m_numberHandling == trailingNumbers )
	{
		for( int i = '0'; i <= '9'; i++ )
			m_bValidTrailing[ i ] = true;
	}

	if( m_dwInternationalChar & CHAR_ALL)
	{
		for( int i = 191; i < 255; i++ )
			m_bValidChar[ i ] = m_bValidTrailing[ i ] = true;
		for(i = 140; i <= 143; i++)
		{
			m_bValidChar[ i ] = m_bValidTrailing[ i ] = true;
			m_bValidChar[ i + 16 ] = m_bValidTrailing[ i + 16 ] = true;
		}
		m_bValidChar[ 138 ] = m_bValidTrailing[ 138 ] = true;
		m_bValidChar[ 154 ] = m_bValidTrailing[ 154 ] = true;
		m_bValidChar[ 163 ] = m_bValidTrailing[ 163 ] = true;
		m_bValidChar[ 165 ] = m_bValidTrailing[ 165 ] = true;
		m_bValidChar[ 170 ] = m_bValidTrailing[ 170 ] = true;
		m_bValidChar[ 175 ] = m_bValidTrailing[ 175 ] = true;
		m_bValidChar[ 179 ] = m_bValidTrailing[ 179 ] = true;
		m_bValidChar[ 185 ] = m_bValidTrailing[ 185 ] = true;
		m_bValidChar[ 186 ] = m_bValidTrailing[ 186 ] = true;
		m_bValidChar[ 188 ] = m_bValidTrailing[ 188 ] = true;
	}
	else if( m_dwInternationalChar != CHAR_ENGLISH )
	{
		CString sLangChar = addedLangChars( m_dwInternationalChar );
		int nLength = sLangChar.GetLength();
		for( i = 0; i < nLength; i++ )
		{
			BYTE nChar = sLangChar[ i ];
			m_bValidChar[ nChar ] = m_bValidTrailing[ nChar ]= true;
		}
	}

	int nLength = m_sAddedChars.GetLength();
	for( i = 0; i < nLength; i++ )
	{
		BYTE nChar = m_sAddedChars[ i ];
		m_bValidChar[ nChar ] = true;
	}

	nLength = m_sAddedTrailingChars.GetLength();
	for( i = 0; i < nLength; i++ )
	{
		BYTE nChar = m_sAddedTrailingChars[ i ];
		m_bValidTrailing[ nChar ] = true;
	}
}

void CFileIndex::addToList( const char* pszPath, CList< CString, CString& >& m_list )
{
	m_timeBase = 0;  // force build rather than update

	while( *pszPath == ' ' ) pszPath++;	 //trim left blanks
	const char* pszChar = pszPath;

	for (;;)
	{
		char c = *pszChar;
		switch ( c )
		{
			case '"':
			{	// honor quotes
				pszChar++;
				pszPath = pszChar;
				while( *pszChar && *pszChar != c ) pszChar++;

				// fallthrough to separator handling - this is end
			}

			case '\t':
			case '\n':
			case '\r':
			{
				if( pszChar != pszPath )
				{
					CString sNext( pszPath, pszChar - pszPath );
					if( !m_list.Find( sNext ) )
						m_list.AddTail( sNext );
				}

				if( *pszChar ) pszChar++;   // have to check because of quote fall through

				while( *pszChar == ' ' ) pszChar++; // trim left blanks
				pszPath = pszChar;
				break;   // this will left trim as well
			}

			case '\0':
			{
				if( pszChar != pszPath )
				{
					CString sNext = pszPath;
					if( !m_list.Find( sNext ) )
						m_list.AddTail( sNext );
				}
				return;
			}

			default:
				pszChar++;
				break;
		}
	}
}
 
void CFileIndex::addInclude( const char* pszPath )
{
	addToList( pszPath, m_includePatterns );
}

void CFileIndex::addExclude( CString sPath )
{
	addToList( sPath, m_excludePatterns );
}

void CFileIndex::removeInclude( CString sPath )
{
	POSITION pos = m_includePatterns.Find( sPath );
	if( pos )
		m_includePatterns.RemoveAt( pos );	
}

void CFileIndex::removeExclude( CString sPath )
{
	POSITION pos = m_excludePatterns.Find( sPath );
	if( pos )
		m_excludePatterns.RemoveAt( pos );	
}

void CFileIndex::clearIncludes()
{
	m_includePatterns.RemoveAll();
}

void CFileIndex::clearExcludes()
{
	m_excludePatterns.RemoveAll();
}

POSITION CFileIndex::getFirstInclude()
{
	return m_includePatterns.GetHeadPosition();
}

CString& CFileIndex::getNextInclude( POSITION& pos )
{
	return m_includePatterns.GetNext( pos );
}

POSITION CFileIndex::getFirstExclude()
{
	return m_excludePatterns.GetHeadPosition();
}

CString& CFileIndex::getNextExclude( POSITION& pos )
{
	return m_excludePatterns.GetNext( pos );
}

void CFileIndex::createZipDirectory()
{
	const char* pszPath = thisApp()->getZipPath();

	CFileStatus status;
	if( CFile::GetStatus( pszPath, status ) )
	{
		if( status.m_attribute | _A_SUBDIR )
			return;
		else
		{
			simpleMessageBox( IDS_ZIP_DIR_NAME_USED );
			AfxThrowUserException();
		}
	}


	SECURITY_ATTRIBUTES security;
	security.nLength = sizeof( security );
	security.lpSecurityDescriptor = NULL;
	security.bInheritHandle = TRUE;

	char pszName[ MAX_PATH ];
	strcpy( pszName, pszPath );

	char* pszSlash = pszName;
	while( *pszSlash && *pszSlash != '\\' && *pszSlash != '/' ) pszSlash++;

	if( *pszSlash == '\0' )
	{
		simpleMessageBox( IDS_ILLEGAL_ZIP_DIR_NAME );
		AfxThrowUserException();
	}

	pszSlash++;
	while( *pszSlash )
	{
		while( *pszSlash && *pszSlash != '\\' && *pszSlash != '/' ) pszSlash++;

		char cSave = *pszSlash;
		*pszSlash = '\0';

		if( !CFile::GetStatus( pszName, status ) )
		{
			if( !CreateDirectory( pszName, &security ) )
			{
				simpleMessageBox( IDS_CREATE_ZIP_DIR_ERR );
				AfxThrowUserException();
			}
		}
		
		*pszSlash = cSave;

		if( *pszSlash )
			pszSlash++;
	}
}

void CFileIndex::setNumberInclusion( NumberInclusion newValue )
{
	if( newValue != m_numberHandling )
	{
		m_timeBase = 0;  // force build rather than update
		m_numberHandling = newValue;
		setCharMasks();
	}
}

void CFileIndex::setInternationalChar( DWORD dwValue)
{
	if( dwValue != m_dwInternationalChar )
	{
		m_timeBase = 0;  // force build rather than update
		m_dwInternationalChar = dwValue;
		setCharMasks();
	}
}

void CFileIndex::setOEMTextChar( BOOL bValue)
{
	if( bValue != m_bOEMTextChar )
	{
		m_timeBase = 0;  // force build rather than update
		m_bOEMTextChar = bValue;
		setCharMasks();  // probably don't need to do this, but can't hurt
	}
}

void CFileIndex::setAddedChars( const CString sAddedChars, const CString sAddedTrailing )
{
	if( sAddedChars != m_sAddedChars || sAddedTrailing != m_sAddedTrailingChars )
	{
		m_timeBase = 0;  // force build rather than update
		m_sAddedChars = sAddedChars;
		m_sAddedTrailingChars = sAddedTrailing;
		setCharMasks();
	}
}

CString CFileIndex::getAddedChars()
{
	return m_sAddedChars;
}

CString CFileIndex::getAddedTrailingChars()
{
	return m_sAddedTrailingChars;
}

void CFileIndex::build()
{
	CWaitCursor wait;

	if( m_bPDF ) thisApp()->makeNewPDFCache();

	createZipDirectory();
	
	indexExclusions();

	emptyIndex();
	
	int initialSize = regGetInt( HKEY_CURRENT_USER, "Settings", "off_initial_size", 0xffff );
	int maxIncrement = regGetInt( HKEY_CURRENT_USER, "Settings", "off_max_incr", 0x2fffff );
	m_pLinks = new CFileLinks( initialSize, maxIncrement,  "off", m_bUseRanking );
	m_pLinks->getSpace( 4 );  // just offset so first link is not 0

	initialSize = regGetInt( HKEY_CURRENT_USER, "Settings", "tmp_initial_size", 0xffff );
	maxIncrement = regGetInt( HKEY_CURRENT_USER, "Settings", "tmp_max_incr", 0xfffff );
	m_pTemporaryLinks  = new CWordLinks( initialSize, maxIncrement, "tmp", m_bUseRanking );
	buildLinks();

	// create empty update file so any old file gets truncated - crude but simple
	// m_pUpdateLinks = new CFileLinks( 0xffff, 0xfffff, "upd", m_bUseRanking );

	m_timeBase = CTime::GetCurrentTime();	
}

void CFileIndex::emptyIndex()
{
	m_bIsUpdate = FALSE;
	delete m_pFiles;
	delete m_pWords;
	delete m_pLinks;

	m_pFiles = NULL;
	m_pWords = NULL;
	m_pLinks = NULL;

	m_nFilesIndexed = 0;
	m_nContentsIndexed = 0;
	m_nWordsIndexed = 0;
	m_nBytesRead    = 0;
	m_timeBase      = 0;

	m_pFiles       = new CFileStorage(1000);

	int initialSize = regGetInt( HKEY_CURRENT_USER, "Settings", "wor_initial_size", 0x1ffff );
	int maxIncrement = regGetInt( HKEY_CURRENT_USER, "Settings", "wor_max_incr", 0xfffff );
	m_pWords       = new CTextStorage( initialSize, maxIncrement, "wor", sizeof(CWordInfo), 40000 );
}

bool CFileIndex::mapsModified()
{
	CFileStatus fsMap;

	if( !m_pFiles || !m_pFiles->getFileStatus( fsMap ) || fsMap.m_mtime > m_timeSaved )
		return true;

	if( !m_pLinks || !m_pLinks->getFileStatus( fsMap ) || fsMap.m_mtime > m_timeSaved )
		return true;

	if( !m_pWords || !m_pWords->getFileStatus( fsMap ) || fsMap.m_mtime > m_timeSaved )
		return true;

	return false;
}


void CFileIndex::update()
{
	CWaitCursor wait;

	if( !m_pFiles || !m_pWords || !m_pLinks || m_timeBase == 0 || mapsModified() )
	{
		build();
		return;
	}

	if( !m_pWords->enableWrite() )
	{
		simpleMessageBox("" );
		AfxThrowUserException();
	}

	if( !m_pFiles->enableWrite() )
	{
		simpleMessageBox( IDS_WORDS_NOT_WRITABLE );
		m_pWords->disableWrite();
		return;
	}

	if( !m_pLinks->enableWrite() )
	{
		simpleMessageBox( IDS_LINKS_NOT_WRITABLE );
		m_pFiles->disableWrite();
		m_pWords->disableWrite();
		return;
	}

	try
	{
		indexExclusions();

		m_bIsUpdate = TRUE;

		// get rid of the old update list indicies in the word list
		for( TextID nWord = m_pWords->getFirst(); nWord; nWord = m_pWords->getNext( nWord ) )
		{
			CWordInfo* pWord = (CWordInfo*) m_pWords->getData( nWord );

			if( !pWord )
			{
				simpleMessageBox( IDS_WORD_FILE_ERR );
				AfxThrowUserException();
			}

			pWord->m_nFileCount = 0;
		}

		int initialSize = regGetInt( HKEY_CURRENT_USER, "Settings", "tmp_initial_size", 0xffff );
		int maxIncrement = regGetInt( HKEY_CURRENT_USER, "Settings", "tmp_max_incr", 0xfffff );
		m_pTemporaryLinks  = new CWordLinks( initialSize, maxIncrement, "tmp", m_bUseRanking );
		
		prune();
		buildLinks();

		m_timeBase = CTime::GetCurrentTime();	
	}
	catch( ... )
	{
		if( m_pFiles )
			m_pFiles->disableWrite();

		if( m_pWords )
			m_pWords->disableWrite();

		if( m_pLinks )
			m_pLinks->disableWrite();

		throw;
	}

	m_pFiles->disableWrite();
	m_pWords->disableWrite();
	m_pLinks->disableWrite();
}

void CFileIndex::prune()
{
	int nStatusIncrement = 1;  // in percent
	int nStatusNumber = m_nFilesIndexed * nStatusIncrement / 100;
	int nStatusCount = nStatusNumber + 1;  // so it does 0
	int nStatusValue = -nStatusIncrement;
	bool bDoStatus;
	if( thisApp()->getMainFrame() )
		bDoStatus = true;
	else
		bDoStatus = false;

	bool bWinNT;

	DWORD dwVersion = GetVersion(); // Get major and minor version numbers of Windows
	bWinNT = (dwVersion < 0x80000000);                // Windows NT

	CString sStatus;

	FileID nLastFile = 0;  // keep track of any added files so we don't try updating them
	for( FileID nFile = m_pFiles->getFirst(); nFile; nFile = m_pFiles->getNext( nFile ) )
	{
		nStatusCount++;
		if( nStatusCount > nStatusNumber && thisApp()->getMainFrame() )
		{
			nStatusCount = 0;
			nStatusValue += nStatusIncrement;
			bDoStatus = true;
		}

		if( bDoStatus )
		{
			sStatus.Format( IDS_UPDATING_FMT, nStatusValue );
			if( thisApp()->getMainFrame() )
			{
				thisApp()->setStatus( sStatus );
				thisApp()->getMainFrame()->SetWindowText( sStatus );
			}
			bDoStatus = false;
		}

		CFileInfo* pInfo = (CFileInfo*) m_pFiles->getData( nFile );
		if( *(pInfo->m_pszName) == (char) 0xff )
			continue;  // already marked obsolete

		CFileInfo* pFolder = NULL;
		CFileInfo* pVolume = NULL;

		if( pInfo->m_nParent )
		{
			try {
			pFolder = (CFileInfo*) m_pFiles->getData( pInfo->m_nParent );
			if( pFolder->m_nParent )
				pVolume = (CFileInfo*) m_pFiles->getData( pFolder->m_nParent );
			else
			{
				pVolume = pFolder;
				pFolder = pInfo;
				pInfo = NULL;
			}
			}
			catch( ... )
			{
				FILE* f = fopen("deadwilbur.txt", "w");
				fprintf(f, "Name %s\nnFile %x\npInfo->m_nParent %x\npFolder %x\n", pInfo->m_pszName,
					nFile, pInfo->m_nParent, pFolder);
				fclose(f);
				throw;
			}

		}
		else
		{
			pVolume = pInfo;
			pInfo = NULL;
		}

		if( pVolume && ( pVolume->m_dwAttributes == DRIVE_FIXED || pVolume->m_dwAttributes == DRIVE_REMOTE ) )
		{
			WIN32_FIND_DATA findFileData;
			if( pFolder )
			{
				CString sPath = pFolder->m_pszName;
				sPath.MakeLower();
				int nZip = CFileIndex::getZipExtIndex(sPath);
				if( nZip == -1 )
				{
					if( pInfo )
						sPath += pInfo->m_pszName;
					else
						sPath = sPath.Left( sPath.GetLength() - 1 );

					int nLength = sPath.GetLength();

					if( (nLength == 1 && sPath[ 0 ] == '\\') ||
						(nLength == 2 && sPath[ 1 ] == ':' /* && (!bWinNT || tolower(sPath[ 0 ]) == 'c' ) */ ) ||
						(nLength > 1  && sPath[1] == '\\' && sPath[0] == '\\' ) )
					{
						// Win95 does not have GetFileAttribEx and FindFirstFile can't find root directories
						indexDirectory( pFolder->m_pszName, nFile, pFolder->m_nParent );
					}
					else
					{
						HANDLE hFind = FindFirstFile(sPath, &findFileData);
						if (hFind == INVALID_HANDLE_VALUE)
							m_pFiles->markObsolete( nFile );
						else
						{
							VERIFY(FindClose(hFind));
							CTime timeLastWritten( findFileData.ftLastWriteTime );
							if( m_timeBase < timeLastWritten )
							{
								if( pInfo )
								{
									CMaskList maskList;
									int nRoot = sPath.ReverseFind( '\\' );
									createMaskList( sPath.Left( nRoot + 1 ), maskList );

									bool bMatch = matchesMask( sPath.Mid( nRoot + 1 ), maskList ) &&
										!isExcluded( sPath );

									if( m_bTrackAll || bMatch )
									{
										pInfo->m_timeLastWritten = timeLastWritten;
										pInfo->m_dwAttributes = findFileData.dwFileAttributes;

										if( findFileData.nFileSizeHigh )
											pInfo->m_nSize = MAXDWORD;
										else
											pInfo->m_nSize = findFileData.nFileSizeLow;
									}
									
									if( bMatch && ( nLastFile == 0 || nFile < nLastFile ))
									{
										FileID nDummy = 1;
										FileID nNewFile = m_pFiles->addText( findFileData.cFileName, nDummy );
										if( !nLastFile )
											nLastFile = nNewFile;

										// pointers may now be bad due to addtext remapping, so get fresh copy
										pInfo = (CFileInfo*) m_pFiles->getData( nFile );
										m_pFiles->markObsolete( nFile );

										CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nNewFile );
										pFileInfo->m_nParent = pInfo->m_nParent;
										pFileInfo->m_timeLastWritten = timeLastWritten;
										pFileInfo->m_dwAttributes = findFileData.dwFileAttributes;

										if( findFileData.nFileSizeHigh )
											pFileInfo->m_nSize = MAXDWORD;
										else
											pFileInfo->m_nSize = findFileData.nFileSizeLow;

										indexFile( sPath, nNewFile );
									}
								}
								else // must be directory
									indexDirectory( pFolder->m_pszName, nFile, pFolder->m_nParent );

								bDoStatus = true;
							}
							else if( !bWinNT && !pInfo )
								indexDirectory( pFolder->m_pszName, nFile, pFolder->m_nParent );
								// Win95 doesn't update directory mod times when files added, so
								// we must do all directories :-(
						}
					}
				}
				else
				{
					// see if the zip has been modified
					sPath = sPath.Left( nZip + 4 );

					HANDLE hFind = FindFirstFile(sPath, &findFileData);
					if (hFind == INVALID_HANDLE_VALUE)
						m_pFiles->markObsolete( nFile );
					else
					{
						VERIFY(FindClose(hFind));
						CTime timeLastWritten( findFileData.ftLastWriteTime );
						if( m_timeBase < timeLastWritten  && !isExcluded( sPath ))
							if( !pInfo )
							{
								CMaskList maskList;
								int nRoot = sPath.ReverseFind( '\\' );
								createMaskList( sPath.Left( nRoot + 1 ), maskList );

								sStatus.Format(IDS_BYTES_INDEXING_FMT, m_nBytesRead, sPath );
								thisApp()->setStatus( sStatus );
								bDoStatus = true;

								indexZip( sPath, sPath, maskList, pFolder->m_nParent );
							}
					}
				}
			}
		}
	}
}

void CFileIndex::buildLinks()
{
	scanFiles();
	compressLinks();
}

void CFileIndex::scanFiles()
{
	try
	{
		CRootDirList dirList;
		createRootDirList( dirList );

		for( POSITION pos = dirList.GetHeadPosition(); pos; )
		{
			CString sRootPath = dirList.GetNext( pos );

			char pszVolumeName[ 256 ];
			DWORD dwCompLength, dwFlags;
			CString sRootDir;
			if( sRootPath.GetLength() >= 3 )
			{
				int nSlash = 1;
				char c = sRootPath[ nSlash++ ];
				if( c == '\\' )
				{
					for( int i = 0; c && i < 2; i++ )
					{
						c = sRootPath[ nSlash++ ];
						while( c && c != '\\' )
							c = sRootPath[ nSlash++ ];
					}
				}
				else nSlash++;

				sRootDir = sRootPath.Left(nSlash);
			}
			else
			{
				// no drive designation - prepend current drive
				char cDriveLetter = 'a' + _getdrive() - 1;
				sRootDir = cDriveLetter;
				sRootDir += ':';
			}

			if( !GetVolumeInformation( sRootDir, pszVolumeName, 255, NULL, &dwCompLength, &dwFlags, NULL, 0 ) )
				pszVolumeName[ 0 ] = '\0';

			FileID nVolume = m_pFiles->lookupFile( pszVolumeName, 0 );

			if( !nVolume )
				nVolume = m_pFiles->addText( pszVolumeName, m_nFilesIndexed );
			else if( m_bIsUpdate )
			{
				DWORD dwAttributes= ((CFileInfo*) m_pFiles->getData(nVolume))->m_dwAttributes;
				if( dwAttributes == DRIVE_FIXED || dwAttributes == DRIVE_REMOTE )
					continue;
			}

			CFileInfo* pVolumeInfo = (CFileInfo*) m_pFiles->getData( nVolume );
			pVolumeInfo->m_nParent = 0;
			pVolumeInfo->m_dwAttributes = GetDriveType( sRootDir );;

			int nLength = sRootPath.GetLength();

			if( nLength >= 1 )
			{
				WIN32_FIND_DATA fileData;

				if( (nLength == 1 && sRootPath[ 0 ] == '\\') ||
					(nLength == 3 && sRootPath[ 1 ] == ':') ||
					(nLength > 1  && sRootPath[1] == '\\' && sRootPath[0] == '\\' ) )
				{
					// Win95 does not have GetFileAttribEx and FindFirstFile can't find root directories
					
					FileID nParent = 0;
					if( m_bIsUpdate )
						nParent = m_pFiles->lookupFile( sRootPath, nVolume );

					if( !nParent )
					{
						nParent = m_pFiles->addText( sRootPath, m_nFilesIndexed );

						CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nParent );
						pFileInfo->m_nParent = nVolume;
						pFileInfo->m_timeLastWritten = CTime::GetCurrentTime();
						pFileInfo->m_dwAttributes = FILE_ATTRIBUTE_DIRECTORY;
						pFileInfo->m_nSize = 0;
					}

					indexDirectory( sRootPath, nParent, nVolume );
				}
				else
				{
					HANDLE hFind = FindFirstFile( (const char*) sRootPath.Left( nLength - 1 ), &fileData );

					if( hFind != INVALID_HANDLE_VALUE )
					{
						if( fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
						{
							FileID nParent = m_pFiles->lookupFile( sRootPath, nVolume );

							if( !nParent )
								nParent = m_pFiles->addText( sRootPath, m_nFilesIndexed );

							CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nParent );
							pFileInfo->m_nParent = nVolume;
							CTime timeLastWritten( fileData.ftLastWriteTime );
							pFileInfo->m_timeLastWritten = timeLastWritten;
							pFileInfo->m_dwAttributes = fileData.dwFileAttributes;

							if( fileData.nFileSizeHigh )
								pFileInfo->m_nSize = MAXDWORD;
							else
								pFileInfo->m_nSize = fileData.nFileSizeLow;

							indexDirectory( sRootPath, nParent, nVolume );
						}

						FindClose( hFind );
					}
				}
			}
		}

		if( m_nWordsIndexed == 0 )
		{
			// add at least one word to avoid zero length .wor file problems
			char pFakeword[10];
			strcpy( pFakeword, "xfakeword" );
			*pFakeword = (char) 0xff;
			indexWord( pFakeword, 0 );
		}

	}
	catch( ... )
	{
		delete m_pTemporaryLinks; 	m_pTemporaryLinks = NULL;

		delete m_pFiles;	m_pFiles = NULL;
		delete m_pWords;	m_pWords = NULL;

		throw;
	}		
}


void CFileIndex::compressLinks()
{
	// AfxMessageBox("About to compress");
	CFileLinks* pLinks = m_pLinks;
	try
	{
		// allocate all of the space for the links
		int nFileLinkSize = sizeof(FileID) + (( m_bUseRanking ) ?  sizeof( BYTE ) : 0);

		int nStatusIncrement = 5;  // in percent
		int nStatusNumber = m_nWordsIndexed * nStatusIncrement / 100;
		int nStatusCount = nStatusNumber + 1;  // so it does 0
		int nStatusValue = -nStatusIncrement;

		TextID nWord = m_pWords->getFirst();
		while( nWord )
		{
			nStatusCount++;
			if( nStatusCount > nStatusNumber && thisApp()->getMainFrame() )
			{
				nStatusValue += nStatusIncrement;
				CString sStatus;
				sStatus.Format( IDS_COUNTING_FMT, nStatusValue );
				thisApp()->setStatus( sStatus );
				thisApp()->getMainFrame()->SetWindowText( sStatus );

				nStatusCount = 0;
			}

			if( thisApp()->getHaltStatus() )
				AfxThrowUserException();

			CWordInfo* pWord = (CWordInfo*) m_pWords->getData( nWord );

			// add the last files to the list - I forget why this was necessary
			m_pTemporaryLinks->addWord( pWord->m_nLastFile, nWord, (BYTE) pWord->m_nLastRank );

			if( pWord->m_nFileCount > 0)
			{
				int nSize = ( 1 + pWord->m_nFileCount) * nFileLinkSize + sizeof(LinkID);

				// set the pointer to point to the null terminator
				// we will back through this later so it ends at beginning of list
				DWORD nList = pLinks->getSpace( nSize ) + nSize - sizeof( LinkID );
				if( pWord->m_nFiles )
					pLinks->setNextList( nList, pWord->m_nFiles );

				pWord->m_nFiles = nList - nFileLinkSize;
			}

			nWord = m_pWords->getNext( nWord );
		}

		// now run through the WordLinks
		nStatusIncrement = 5000;
		nStatusNumber = 0;
		nStatusCount = 0;  // so it does 0

		// temporary links now contain lists of words contained in files
		// each set of words begins with the FileID followed by
		// a 0 terminated list of Word IDs
		// getFile and getWord get the ID at nID and increment nID

		LinkID nID = 0;
		FileID nFile = m_pTemporaryLinks->getFile( nID );
		while( nFile )
		{
			nStatusCount++;
			if( nStatusCount >= nStatusNumber && thisApp()->getMainFrame() )
			{
				CString sStatus;
				sStatus.Format( IDS_COMP_LINKS_FMT, nStatusNumber, nStatusNumber + nStatusIncrement );
				nStatusNumber += nStatusIncrement;

				thisApp()->setStatus( sStatus );
				thisApp()->getMainFrame()->SetWindowText( sStatus );
			}

			if( thisApp()->getHaltStatus() )
				AfxThrowUserException();

			BYTE bCount;
			WordID nWord = m_pTemporaryLinks->getWord( nID, bCount );
			while( nWord )
			{
				nStatusCount++;

				CWordInfo* pWord = (CWordInfo*) m_pWords->getData( nWord );

				// one file link is maintained in the m_nLastFile member
				// it should not be placed in the list
				if( pWord->m_nFileCount > 0 && pWord->m_nLastFile != nFile )
				{
					pWord->m_nFiles -= nFileLinkSize;
					pLinks->addFile( pWord->m_nFiles, nFile, bCount );
				}

				nWord = m_pTemporaryLinks->getWord( nID, bCount );
			}

			nFile = m_pTemporaryLinks->getFile( nID );
		}
	}
	catch( ... )
	{
		delete pLinks;
		delete m_pTemporaryLinks; 	m_pTemporaryLinks = NULL;

		delete m_pFiles;		m_pFiles = NULL;
		delete m_pWords;		m_pWords = NULL;
		delete m_pLinks;		m_pLinks = NULL;

		simpleMessageBox( "" );

		throw;
	}

	delete m_pTemporaryLinks;	m_pTemporaryLinks = NULL;
	
	CString sDone;
	sDone.LoadString( IDS_DONE );
	thisApp()->setStatus( sDone );

	if( thisApp()->getMainFrame() )
		thisApp()->getMainFrame()->SetWindowText( sDone );
}

void CFileIndex::createRootDirList( CRootDirList& dirList )
{
	for( POSITION pos = m_includePatterns.GetHeadPosition(); pos; )
	{
		CString sInclude = makeLower( m_includePatterns.GetNext( pos ) );
		addPathToRootDirs( sInclude, dirList );
	}
}

void CFileIndex::addPathToRootDirs( const CString& sInclude, CRootDirList& dirList )
{
	if( ( sInclude[ 0 ] == '*' || sInclude[ 0 ] == '+' ) && sInclude[ 1 ] == ':' )
	{
		// wild carded drive designation

		CString sDriveInclude = sInclude;

		char* pszDriveBuffer = NULL;
		int nBufferLength = GetLogicalDriveStrings( 0, pszDriveBuffer );

		pszDriveBuffer = new char[ nBufferLength + 1 ];
		try
		{
			GetLogicalDriveStrings( nBufferLength, pszDriveBuffer );
			char* pszDrive = pszDriveBuffer;

			while( *pszDrive )
			{
				int nDriveType = GetDriveType( pszDrive );
				if( nDriveType == DRIVE_FIXED ||
					   (sInclude[ 0 ] == '+' && nDriveType == DRIVE_REMOTE ) )
				{
					char c = *pszDrive;
					if( c >= 'A' && c <= 'Z' )
						c = 'a' + (c - 'A');
					sDriveInclude.SetAt( 0, c );
					addPathToRootDirs( sDriveInclude, dirList );
				}

				while( *pszDrive ) pszDrive++;
				pszDrive++;
			}
		}
		catch( CMemoryException* )
		{
			throw;
		}
		catch( CException* e )
		{
			e->Delete();
		}

		delete pszDriveBuffer;

		return;
	}
	
	CString sRootDir = getRootDir( sInclude );

	// is sRootDir or one of it's parents already in list
	// or is this parent of member of list
	POSITION pos = dirList.GetHeadPosition();
	bool bAdd = true;
	while( pos )
	{
		POSITION lastPos = pos;
		CString sRootMember = dirList.GetNext( pos );
		int nFound = sRootDir.Find( sRootMember );

		if( nFound == 0 )
		{
			bAdd = false;
			break; //
		}

		nFound = sRootMember.Find( sRootDir );
		if( nFound == 0 )
		{
			// new entry is parent of existing - replace
			dirList.SetAt( lastPos, sRootDir );

			// must now check for other such members and delete
			while( pos )
			{
				lastPos = pos;
				sRootMember = dirList.GetNext( pos );
				if( sRootMember.Find( sRootDir ) == 0 )
					dirList.RemoveAt( lastPos );
			}

			bAdd = false;
			break;
		}
	}

	if( bAdd )
		dirList.AddTail( sRootDir );
}

CString CFileIndex::getRootDir( const CString& sInclude )
{
	CString sPath;

	if( sInclude.GetLength() < 2 )
		return sInclude;

	if( sInclude[1] == ':' || ( sInclude[1] == '\\' && sInclude[0] == '\\' ) )
		sPath = sInclude;
	else
	{
		// no drive designation - prepend current drive
/*		char cDriveLetter = 'a' + _getdrive() - 1;
		sPath = cDriveLetter;
		sPath += ':';
*/
		if( sInclude[0] != '\\' && sInclude[ 0 ] != '/' )
			sPath += '\\';

		sPath += sInclude;
	}

	CString sRootPath;

	int nEnd = sPath.FindOneOf( "*?" );

	if( nEnd == -1 )
		nEnd = sPath.GetLength();

	char c;
	for( int i = nEnd - 1; i >= 0; i-- )
	{
		c = sPath[ i ];
		if( c == '\\' || c == '/' || c == ':' )
			break;
	}

	ASSERT( i >= 0 );
	
	sRootPath = sPath.Left( i + 1 );
	if( c == ':' )
		sRootPath += '\\';

	return sRootPath;
}

void CFileIndex::createMaskList( const CString& sRootPath, CMaskList& maskList )
{
	for( POSITION pos = m_includePatterns.GetHeadPosition(); pos; )
	{
		CString sInclude = m_includePatterns.GetNext( pos );

		CString sFileMask;
		if( dirMatch( sRootPath, sInclude ) )
		{
			int nLength = sInclude.GetLength();
			char c;
			for( int i = nLength - 1; i >= 0; i-- )
			{
				c = sInclude[ i ];
				if( c == '\\' || c == '/' || c == ':' )
					break;
			}

			CString sFileMask;

			if( i >= 0 )
				sFileMask = sInclude.Right( nLength - i - 1 );
			else
				sFileMask = sInclude;

			maskList.AddTail( sFileMask );
		}
	}
}

bool CFileIndex::dirMatch( const char* pszRootPath, const char* pszInclude )
{
	// check if remote drive
	char pszDrive[ 2 ];
	if( *pszRootPath && pszRootPath[ 1 ] == ':' )
	{
		pszDrive[ 0 ] = _toupper( *pszRootPath );
		pszDrive[ 1 ] = '\0';

		int nDriveType = GetDriveType( pszDrive );
		if( nDriveType == DRIVE_REMOTE && ( *pszInclude != '+' || pszInclude[1] != ':' ) )
			return false;

		if( !*pszInclude || pszInclude[ 1 ] != ':' )
			pszRootPath += 2;
		else if( ( *pszInclude == '*' || *pszInclude == '+' ) && pszInclude[ 1 ] == ':' )
		{
			pszRootPath ++;
			pszInclude++;
		}
	}

	while( *pszRootPath )
	{
		if( *pszInclude == '*' )
		{
			// see if this is end of directory in include - if so then true
			const char *pszRemaining = pszInclude;
			while( *pszRemaining )
			{
				if( *pszRemaining == '\\' || *pszRemaining++ == '/' )
				{
					char c1, c2;
					c1 = makeLower( *++pszInclude );
					do
					{
						c2 = makeLower( *pszRootPath );
						if( c2 == c1  )
							return dirMatch( pszRootPath, pszInclude );

						pszRootPath++;
					} while( c2 );
					return true;
				}
			}

			return true;
		}
		else if( *pszInclude == '?' || makeLower( *pszRootPath ) == makeLower( *pszInclude ) )
		{
			*pszRootPath++;
			*pszInclude++;
		}
		else
			break;
	}

	// see if this is end of directory in include - if not fail
	while( *pszInclude )
	{
		if( *pszInclude == '\\' || *pszInclude++ == '/' )
			return false;
	}

	return true;
}

bool CFileIndex::matchesMask( const char* pszFile, CMaskList& maskList )
{
	for( POSITION pos = maskList.GetHeadPosition(); pos; )
	{
		CString sMask = maskList.GetNext( pos );
		if( CWilburDoc::fileNameMatch( pszFile, sMask ) )
			return true;
	}

	return false;
}

void CFileIndex::indexDirectory( const CString& sRootPath, FileID nParent, FileID nVolume )
{
	CString sStatus;

	sStatus.Format(IDS_BYTES_INDEXING_FMT, m_nBytesRead, sRootPath );
	thisApp()->setStatus( sStatus );

	CMaskList maskList;
	createMaskList( sRootPath, maskList );

	DWORD dwVolAttributes= ((CFileInfo*) m_pFiles->getData(nVolume))->m_dwAttributes;

	WIN32_FIND_DATA fileData;

	CString sSearchPath = sRootPath + "*";
	HANDLE hFind = FindFirstFile( (const char*) sSearchPath, &fileData );

	if( hFind != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( thisApp()->getHaltStatus() )
			{
				AfxThrowUserException();
			}

			if( fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
			{
				if( fileData.cFileName[0] != '.' || (fileData.cFileName[1] != '.' && fileData.cFileName[1] != '\0' ) )
				{
					CString sNewPath = sRootPath + fileData.cFileName + '\\';
					if( !isExcluded( sNewPath ) )
					{
						FileID nNewParent = m_pFiles->lookupFile( sNewPath, nVolume );
						
						if( !nNewParent )
							nNewParent = m_pFiles->addText( sNewPath, m_nFilesIndexed );
						else if( m_bIsUpdate )
						{
							if( dwVolAttributes == DRIVE_FIXED || dwVolAttributes == DRIVE_REMOTE )
								continue;
						}

						CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nNewParent );
						pFileInfo->m_nParent = nVolume;
						CTime timeLastWritten( fileData.ftLastWriteTime );
						pFileInfo->m_timeLastWritten = timeLastWritten;
						pFileInfo->m_dwAttributes = fileData.dwFileAttributes;

						if( fileData.nFileSizeHigh )
							pFileInfo->m_nSize = MAXDWORD;
						else
							pFileInfo->m_nSize = fileData.nFileSizeLow;

						indexDirectory( sNewPath, nNewParent, nVolume );
						thisApp()->setStatus( sStatus );
					}
				}
			}
			else
			{
				CTime timeLastWritten( fileData.ftLastWriteTime );

				// if build or file isn't in index or file has been modified since last build
				CString sFilePath = sRootPath + fileData.cFileName;
				FileID nFile = 0;
				if( !m_bIsUpdate ||
					!( nFile = m_pFiles->lookupFile( fileData.cFileName, nParent ) ) ||
					m_timeBase < timeLastWritten )
				{
					if( nFile )
						continue;  // must be update of known file if this is non zero

					if(  m_bDoZipped && isZip( fileData.cFileName ) )
					{
						if( (!m_bIsUpdate || m_timeBase < timeLastWritten ||
							  dwVolAttributes == DRIVE_REMOVABLE || dwVolAttributes == DRIVE_CDROM) &&
							  !isExcluded( sFilePath ) )
						{
							sStatus.Format(IDS_BYTES_INDEXING2_FMT, m_nBytesRead, sRootPath, fileData.cFileName);
							thisApp()->setStatus( sStatus );
							CString sZipPath = sFilePath;

							indexZip( sZipPath, sFilePath, maskList, nVolume );
						}
					}
					else
					{
						bool bMatch = matchesMask( fileData.cFileName, maskList ) &&
						              !isExcluded( sFilePath );

						if( m_bTrackAll || bMatch )
						{
							nFile = m_pFiles->lookupFile( fileData.cFileName, nParent );

							if( !nFile )
								nFile = m_pFiles->addText( fileData.cFileName, m_nFilesIndexed );

							CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nFile );
							pFileInfo->m_nParent = nParent;
							pFileInfo->m_timeLastWritten = timeLastWritten;
							pFileInfo->m_dwAttributes = fileData.dwFileAttributes;

							if( fileData.nFileSizeHigh )
								pFileInfo->m_nSize = MAXDWORD;
							else
								pFileInfo->m_nSize = fileData.nFileSizeLow;
						}

						if( bMatch )
							indexFile( sFilePath, nFile );
					}
				}
			}
		} while ( FindNextFile( hFind, &fileData ) );

		FindClose( hFind );
	}
}


void CFileIndex::removeTemporaryFile( const CString& sTempFile, int attr)
{
	try
	{
		if( attr & 7 )
		{
			CFileStatus fileStatus;
			CFile::GetStatus( sTempFile, fileStatus );

			// sanity checks for bad dates
			if(fileStatus.m_ctime == -1) fileStatus.m_ctime = time(NULL);
			if(fileStatus.m_atime == -1) fileStatus.m_atime = time(NULL);
			if(fileStatus.m_mtime == -1) fileStatus.m_mtime = time(NULL);

			fileStatus.m_attribute &= 0x38;  // clear read, sys and hidden
			CFile::SetStatus( sTempFile, fileStatus );
		}

		CFile::Remove( sTempFile );
	}
	catch( CException* e )
	{
		e->Delete();
	}
}

BOOL CFileIndex::isZip( const CString& sZipPath )
{
	CString sExt = makeLower( sZipPath.Right( 4 ) );
	return ( sExt == ".zip" || sExt == ".qwk" || sExt == ".cab" || sExt == ".sxw" || sExt == ".sxc" || sExt == ".sxi" ||
		sExt == ".odt" || sExt == ".ods" || sExt == ".odp" || sExt == ".odb" || sExt == "mmap" );
}

BOOL CFileIndex::getZipStatus()
{
	return m_bDoZipped;
}

void CFileIndex::setTrackAll( bool bNewStatus )
{
	if( bNewStatus != m_bTrackAll )
	{
		m_timeBase = 0;
		m_bTrackAll = bNewStatus;
	}
}

void CFileIndex::setZipStatus( BOOL bNewStatus )
{
	if( bNewStatus != m_bDoZipped )
	{
		m_timeBase = 0;  // force build rather than update
		m_bDoZipped = bNewStatus;
	}
}

BOOL CFileIndex::getUseRanking()
{
	return m_bUseRanking;
}

void CFileIndex::setUseRanking( BOOL bNewStatus )
{
	if( bNewStatus != m_bUseRanking )
	{
		m_timeBase = 0;  // force build rather than update
		m_bUseRanking = bNewStatus;
	}
}

void CFileIndex::setMinWordLength( int nLength )
{
	if( nLength != m_nMinWordLength )
	{
		m_timeBase = 0;  // force build rather than update
		m_nMinWordLength = nLength;
	}
}

void CFileIndex::setMaxWordLength( int nLength )
{
	if( nLength != m_nMaxWordLength )
	{
		m_timeBase = 0;  // force build rather than update
		m_nMaxWordLength = nLength;
	}
}

BOOL CFileIndex::nameMatches( const char* pFileName, const char* pFileMask )
{
	if( !lstrcmpi( pFileName, pFileMask ) )
		return TRUE;

	while( *pFileName )
	{
		if( *pFileMask == '*' )
		{
			while( *pFileName && *pFileName++ != '.' );
			while( *pFileMask && *pFileMask++ != '.' );
		}
		else if( *pFileMask == '?' || makeLower( *pFileName ) == makeLower( *pFileMask ) )
		{
			*pFileName++;
			*pFileMask++;
		}
		else
			return FALSE;
	}

	while( *pFileMask == '?' || *pFileMask == '*' )
		pFileMask++;

	if( *pFileMask )
		return FALSE;
	else
		return TRUE;
}

BOOL CFileIndex::isExcluded( const char* pszFilePath )
{
	for( POSITION pos = m_excludePatterns.GetHeadPosition(); pos; )
	{
		const char* pszExclude = m_excludePatterns.GetNext( pos );

		if( CWilburDoc::fileNameMatch( pszFilePath, pszExclude ) )
			return TRUE;
	}
	return FALSE;
}
	
void CFileIndex::indexFile( const CString& sPathName, FileID nFile, int nAttempt )
{
	try
	{
		CString sTempName = sPathName;
		CString sExt = sPathName.Right( 5 );
		sExt.MakeLower();

		bool bPDF = false;
		if (m_bPDF && sExt.Find(".pdf") != -1)
		{
			bPDF = true;
			sTempName = thisApp()->translatePDF(sTempName, true);
		}

		CFile file( sTempName, CFile::modeRead  | CFile::shareDenyNone );

		bool bHtml = false;
		// H.W. = changes for IExplorer multivolume files
		// H.W. begin --------------------------------------------
		bool bMht = false;
		bool bInsidePart = false;
		// H.W. end ----------------------------------------------

		if( sExt.Find(".htm") != -1 || sExt.Find(".xml") != -1 || sExt.Find(".php") != -1 )
			bHtml = true;
		// H.W. begin --------------------------------------------
		else if( sExt.Find(".mht") != -1 )
			// maybe do a better check here.
			// for this, the first buffer has to be read.
			bHtml = bMht = true;
		// H.W. end ----------------------------------------------

		const TextBlockSize BufferLength = 0xffff;
		TextBlockSize nBytesRead = 0;
		char buffer[ BufferLength ];
		m_sPartialWord.Empty();

		char* pStart;			// H.W. pointer to 'buffer' or pointer to start of html section
		char* pEnd = NULL;
		int nPartial = 0;	// H.W. for open html tags
		TextBlockSize nBytesBuffer;	// H.W. need if more but one sections
		int nBytes = 0;		// H.W. size of area to index

		while( (nBytesRead = file.Read( &(buffer[nPartial]), BufferLength - nPartial ) ) )
		{
			if( thisApp()->getHaltStatus() )
			{
				AfxThrowUserException();
			}

			m_nBytesRead += nBytesRead;
			// H.W. begin --------------------------------------------
			pStart = &buffer[0];
			nBytes = nBytesRead + nPartial;
			nBytesBuffer = nBytes;

			do
			{
				// there may be more but one html section in the buffer
				if ( true == bMht )
				{
					if (false == bInsidePart)
					{
						if ( NULL != pEnd )
						{
							pStart = pEnd;
							nBytes = (nBytesBuffer - (pEnd - &buffer[0]));
						}
						// adjust to first occurence of <HTML>
						do {
							if ( NULL != (pStart = scanBuffer(pStart, nBytes, "<html", 5)) )
								break;
							nPartial = 0;
							if ( 0 == (nBytesRead = file.Read( &(buffer[0]), BufferLength )) )
								break;
							pStart = &buffer[0];
							m_nBytesRead += nBytesRead;
							nBytes = nBytesRead;
							nBytesBuffer = nBytes;
						}while ( 0 < nBytesRead );

						if ( 0 == nBytesRead || NULL == pStart )
							break;

						nBytes = (nBytesBuffer - (pStart - &buffer[0]));
						bInsidePart = true;
					}
					
					if ( NULL != (pEnd = scanBuffer(pStart, nBytes, "</html>", 7)) )
					{
						nBytes = pEnd - pStart;
						bInsidePart = false;
					}
				}

				m_sPartialWord.Empty();
				// H.W. nPartial returns opened (<) but not closed (>) html tags
				nPartial = indexBuffer( pStart, nBytes , nFile, bHtml );				
			}while ( NULL != pEnd );

			// H.W. end ----------------------------------------------

			CString sStatus;
			sStatus.Format(IDS_BYTES_INDEXING_FMT, m_nBytesRead, sPathName );
			thisApp()->setStatus( sStatus );

			// H.W. begin --------------------------------------------
			int i;

			if( 0 < nPartial ) 
			{
				for( i = 0; i < nPartial; i++ )
					buffer[ i ] = pStart[ nBytes - nPartial + i ];
			}
			// H.W. end ----------------------------------------------
			else if( !m_sPartialWord.IsEmpty() )
			{
				nPartial = m_sPartialWord.GetLength();
				for( int i = 0; i < nPartial; i++)
					buffer[ i ] = m_sPartialWord[ i ];
			}
		}

		if( !m_sPartialWord.IsEmpty() && m_sPartialWord.GetLength() >= m_nMinWordLength )
			indexWord( m_sPartialWord, nFile );

		file.Close();
		if( bPDF )
		{
			if( !thisApp()->usePDFCache() )
				removeTemporaryFile(sTempName, 0);
		}
	}
	catch( CMemoryException* )
	{
		throw;
	}
	catch( CUserException* )
	{
		throw;
	}
	catch( CException* e )
	{
		e->Delete();

		/*
		if( nAttempt < 1 )
		{
			CString sMessage = "Could not read ";
			sMessage += sPathName + " waiting 10 seconds";
			thisApp()->setStatus( sMessage );
			Sleep( 10000L );
			indexFile( sPathName, nFile, nAttempt + 1 );
		}
		*/
		return;
	}

	m_nContentsIndexed++;
}

bool CFileIndex::indexExclusions()
{
	try
	{
		delete m_pSkipWords;
		m_pSkipWords   = new CTextStorage( 0x7fff, 0xffff, "ski", 0, 1000 );

		CString sStatus;
		sStatus.LoadString( IDS_INDEXING_EXCLUDED );
		thisApp()->setStatus( sStatus );

		CString sSkipFile( thisApp()->startDirectory() + "\\" + m_sSkipFile + ".txt" );
		CFile file( sSkipFile, CFile::modeRead  | CFile::shareDenyNone );

		const TextBlockSize BufferLength = 0xffff;
		TextBlockSize nBytesRead = 0;
		char buffer[ BufferLength ];
		m_sPartialWord.Empty();
		int nPartial = 0;

		while( (nBytesRead = file.Read( &(buffer[nPartial]), BufferLength - nPartial ) ) )
		{
			m_sPartialWord.Empty();
			indexExcludeBuffer( buffer, nBytesRead + nPartial );
			m_nBytesRead += nBytesRead;
			if( !m_sPartialWord.IsEmpty() )
			{
				nPartial = m_sPartialWord.GetLength();
				for( int i = 0; i < nPartial; i++)
					buffer[ i ] = m_sPartialWord[ i ];
			}
		}

		if( !m_sPartialWord.IsEmpty() && m_sPartialWord.GetLength() > 2 )
			indexExcludedWord( m_sPartialWord );
	}
	catch( CException* e )
	{
		simpleMessageBox( IDS_EXCLUDED_WORD_ERR );
		e->Delete();
		return false;
	}

	return true;
}

void CFileIndex::setSkipFile( CString sNewFile )
{
	if( m_sSkipFile != sNewFile )
	{
		m_sSkipFile = sNewFile;
		m_timeBase = 0;
		CString sSkipFile( thisApp()->startDirectory() + "\\" + m_sSkipFile + ".txt" );
		try
		{
			// create file if not there, but leave as is if it is
			CFile file( sSkipFile, CFile::modeCreate | CFile::modeNoTruncate );
		}
		catch( ... )
		{
			// if there was an error we will flag it on indexExclusions
		}
	}
}

bool CFileIndex::importSkipWords( CString sFile )
{
	// first get fresh copy of existing
	if( !indexExclusions() )
		return false;

	// now open new file
	try
	{
		CStdioFile importFile( sFile, CFile::modeRead );

		CString sLine;
		while( importFile.ReadString( sLine ) )
		{
			int nTab = sLine.FindOneOf( "\t\n ," );

			if( nTab != -1 )
				sLine = sLine.Left( nTab );

			if( !sLine.IsEmpty() )
				indexExcludedWord( sLine );
		}
	}
	catch( ... )
	{
		simpleMessageBox( IDS_SKIP_FILE_IMPORT_ERR );
		return false;
	}

	try
	{
		CString sSkipFile( thisApp()->startDirectory() + "\\" + m_sSkipFile + ".txt" );
		CStdioFile skipFile( sSkipFile, CFile::modeCreate | CFile::modeWrite );

		for( TextID nID = m_pSkipWords->getFirst(); nID; nID = m_pSkipWords->getNext( nID ) )
		{
			CString sLine = m_pSkipWords->getText( nID );
			sLine += '\n';

			skipFile.WriteString( sLine );
		}
	}
	catch( ... )
	{
		simpleMessageBox( IDS_SKIP_FILE_WRITE_ERR );
		return false;
	}

	simpleMessageBox( IDS_SKIP_IMPORT_SUCCESS );
	return true;
}

unsigned char CFileIndex::htmlChar( char* pszIn )
{
	unsigned char chIn;

	if( *pszIn == '#' )
	{
		pszIn++;
		int nChar = 0;
		while( (chIn = *pszIn ) && chIn >= '0' && chIn <= '9' )
		{
			nChar = nChar * 10 + chIn - '0';
			pszIn++;
		}

		return (unsigned char) nChar;
	}

	if( m_htmlCharMap.Lookup( pszIn, chIn ) != 0 )
		return chIn;
	else
		return ' ';
}

// H.W. changed from void to int ( nPartial ), need this for partial tag
int CFileIndex::indexBuffer( char* pBuffer, ByteID nBytes, FileID nFile, bool bHtml )
{
	char* pWordStart = pBuffer;
	char* pWordEnd = pBuffer;
	char* pNextChar = pBuffer;

	bool bText = true;
	bool bRtf = false;

	int nPartial = 0;	// H.W.

	if( !bHtml )
	{
		if( pBuffer[ 0 ] == '{' && pBuffer[ 1 ] == '\\' && pBuffer[ 2 ] == 'r' && pBuffer[ 3 ] == 't' && pBuffer[ 4 ] == 'f' )
			bRtf = true;
		else
		{
			for( int i = 0; i < 12; i++ )
			{
				unsigned char chIn = pBuffer[ i ];
				if( ( chIn < ' ' || chIn > 0xa9 ) && (chIn < '\t' || chIn > '\r' ))
				{
					bText = false;
					break;
				}
			}
		}
	}

	int nMaxWordSize = m_nMaxWordLength;	  // used to limit indexing of gibberish in non text files
	if( nMaxWordSize == 0 )
	{
		if( bText )
			nMaxWordSize = 100;
		else
			nMaxWordSize = 20;
	}

	nMaxWordSize++; // to allow for EOS

	if( m_bOEMTextChar && bText )
		OemToCharBuff( pBuffer, pBuffer, nBytes );

	BOOL bBinaryBefore = FALSE;
	bool bSkippedZero = true;

	while( pNextChar - pBuffer < (int) nBytes )
	{
		char cEnd = *pNextChar;

		// H.W. begin ------------------------------------
		// need next for opened but not closed html tags if e.g. the
		// </html> tag is not completely withing the current buffer, the
		// end of an htlm section withing a mht file would not be found.
		if ( true == bHtml )
		{
			// does not test for clean html (e.g. <= or => in queries)
			// to make it simple, test at most for what we need, what is
			// the size of "</html>
			if ( '<' == cEnd )
			{
				nPartial = 1;
				pNextChar++;
				continue;
			}
			else if ( 7 < nPartial )
				nPartial = 0;
			else if ( 0 < nPartial )
				nPartial++;
		}
		// H.W. end --------------------------------------

		if( cEnd == '\0' && !bSkippedZero && pWordEnd != pWordStart )
		{
			pNextChar++;
			bSkippedZero = true;
			continue;
		}
		else
			bSkippedZero = false;

		if( bHtml )
		{
			if( cEnd == '&' )
			{
				// H.W. attention !!! there may be a special char within a tag !!!
				char* pszSpecial = pNextChar;
				pNextChar++;
				if( pNextChar - pBuffer >= (int) nBytes )
					continue;

				const nStringMax = 32;
				char pszHtmlString[ nStringMax ];
								
				for( int i = 0; i < nStringMax - 1 && *pNextChar != ';'; i++, pNextChar++ )
				{
					if( pNextChar - pBuffer >= (int) nBytes )
					{
						pszHtmlString[ i ] = '\0';
						m_sPartialWord = pszHtmlString;
						return (nPartial + i);	// H.W.: must not become -1
					}

					pszHtmlString[ i ] = *pNextChar;
				}
				
				if ( 0 < nPartial )	// H.W.
					nPartial += (i - 1);
				
				if( i < nStringMax )
				{
					pszHtmlString[ i ] = '\0';
					cEnd = *pWordEnd = htmlChar( pszHtmlString );
				}
			}
			else if( (cEnd & 0xe0) == 0xc0 )  // check for UTF-8 encoding of latin
			{
				pNextChar++;
				if( pNextChar - pBuffer >= (int) nBytes )
					continue;

				cEnd = ((cEnd & 0x3) << 6) | (*pNextChar & 0x3f);
			}
		}
		else if( bRtf )
		{
			if( cEnd == '\\')
			{
				pNextChar++;
				if( pNextChar - pBuffer >= (int) nBytes )
				{
					m_sPartialWord = '\\';
					return nPartial;	// H.W.
				}

				if( *pNextChar == '\'' )
				{	// character code
					pNextChar++;
					if( pNextChar - pBuffer >= (int) nBytes )
					{
						m_sPartialWord = "\\\'";
						return nPartial;	// H.W.
					}
					char pHex[3];
					pHex[0]= *pNextChar;
					pNextChar++;
					if( pNextChar - pBuffer >= (int) nBytes )
					{
						m_sPartialWord = "\\\'";
						m_sPartialWord += pHex[0];
						return nPartial;	// H.W.
					}

					pHex[1] = *pNextChar;
					pHex[2] = '\0';
					cEnd = (char) strtol( pHex, NULL, 16);
					*pNextChar = cEnd;
				}

				else if( *pNextChar >= 'a' && *pNextChar <= 'z' )
				{	// delimited control
					while( *pNextChar >= 'a' && *pNextChar <= 'z' )
					{
						pNextChar++;
						if( pNextChar - pBuffer >= (int) nBytes )
						{
							m_sPartialWord = '\\';
							return nPartial;	// H.W.
						}
					}

					// now figure out delimiter
					if( *pNextChar == ' ')
					{
						pNextChar++;
						if( pNextChar - pBuffer >= (int) nBytes )
						{
							m_sPartialWord.Empty();
							return nPartial;	// H.W.
						}
					}

					while( ( *pNextChar >= '0' && *pNextChar <= '9' ) || *pNextChar == '-' )
					{
						pNextChar++;
						if( pNextChar - pBuffer >= (int) nBytes )
						{
							m_sPartialWord.Empty();
							return nPartial;	// H.W.
						}
					}
				}
				else if( *pNextChar != '\\')
				{ // not just escaped backslash
					*pNextChar = ' ';  // non delimited control - just replace with space
				}

				if( pWordStart == pWordEnd )
				{
					pWordStart = pWordEnd = pNextChar;
				}
				else
				{
					int nLength = pWordEnd - pWordStart;
					char* pDest = pNextChar - nLength;
					memmove( pDest, pWordStart, nLength );
					pWordStart = pDest;
					pWordEnd = pNextChar;
				}

				continue;
			}
		}
		if( isValidAlpha( cEnd ) ||
			( pWordEnd != pWordStart && isValidTrailing( cEnd ) ) )
		{
			*pWordEnd = makeLower( cEnd );
			pWordEnd++; pNextChar++;
		}
		else if( pWordEnd != pWordStart )
		{
			BOOL bBinaryAfter = isBinary( cEnd );

			if( ( !bBinaryBefore || !bBinaryAfter ) &&
			      pWordEnd - pWordStart >= m_nMinWordLength &&
				  pWordEnd - pWordStart <= nMaxWordSize )
			{
				*pWordEnd++ = '\0';
				indexWord( pWordStart, nFile );
			}

			pWordStart = pWordEnd = pNextChar;
			bBinaryBefore = bBinaryAfter;
		}
		else
		{
			if( !bBinaryBefore && isBinary( cEnd ) )
				bBinaryBefore = TRUE;

			pNextChar++;

			pWordEnd = pWordStart = pNextChar;
		}
	}

	if( pWordEnd != pWordStart )
	{
		pWordEnd--;
		char cLast = *pWordEnd;  // want to put in EOS, but not beyond end of buffer
		*pWordEnd = '\0';
		m_sPartialWord = pWordStart;
		m_sPartialWord += cLast;
	}

	return nPartial;	// H.W.
}

void CFileIndex::indexExcludeBuffer( char* pBuffer, ByteID nBytes )
{
	char* pWordStart = pBuffer;
	char* pWordEnd = pBuffer;

	while( nBytes )
	{
		if( isValidAlpha( *pWordEnd ) ||
			( pWordEnd != pWordStart && isValidTrailing( *pWordEnd ) ) )
		{
			*pWordEnd = makeLower( *pWordEnd );
			pWordEnd++;
		}
		else if( pWordEnd != pWordStart )
		{
			*pWordEnd++ = '\0';
			// if( pWordEnd - pWordStart > m_nMinWordLength )
				indexExcludedWord( pWordStart );

			pWordStart = pWordEnd;
		}
		else
		{
			pWordEnd++; pWordStart++;
		}

		nBytes--;
	}

	if( pWordEnd != pWordStart )
	{
		pWordEnd--;
		char cLast = *pWordEnd;  // want to put in EOS, but not beyond end of buffer
		*pWordEnd = '\0';
		m_sPartialWord = pWordStart;
		m_sPartialWord += cLast;
	}
}

void CFileIndex::indexWord( const char* pWord, FileID nFile )
{
	if( m_pSkipWords->lookup( pWord ) )
		return;

	WordID nWord = m_pWords->lookup( pWord );
	if( !nWord )
		nWord = m_pWords->addText( pWord, m_nWordsIndexed );

	CWordInfo* pWordInfo = (CWordInfo*) m_pWords->getData( nWord );

	if( pWordInfo->m_nLastFile == 0 )
	{
		pWordInfo->m_nLastFile = nFile;
		pWordInfo->m_nLastRank = 0;
		pWordInfo->m_nFileCount = 0;
	}
	else if( nFile == pWordInfo->m_nLastFile )
	{
		if( m_bUseRanking )
		{
			if( pWordInfo->m_nLastRank != 0xff )
				pWordInfo->m_nLastRank++;
		}
	}
	else
	{
		// new file - save the old last values and replace them with the new file
		m_pTemporaryLinks->addWord( pWordInfo->m_nLastFile, nWord, (BYTE) pWordInfo->m_nLastRank );
		pWordInfo->m_nLastFile = nFile;
		pWordInfo->m_nLastRank = 0;

		pWordInfo->m_nFileCount++;
	}
}

void CFileIndex::indexExcludedWord( const char* pWord )
{
	TextID nDummy = 1;
	if( !m_pSkipWords->lookup( pWord ) )
		m_pSkipWords->addText( pWord, nDummy );
}

void CFileIndex::getSoundex( const char* pszWord, char sxCode[4] )
{
	static char letterVals[ 26 ] = {
		0x00, 0x01, 0x02, 0x03, 0x00, 0x01, 0x02, 0x00, 0x00,
		0x02, 0x02, 0x04, 0x05, 0x05, 0x00, 0x01, 0x02, 0x06,
		0x02, 0x03, 0x00, 0x01, 0x00, 0x02, 0x00, 0x02 };

	while( *pszWord && *pszWord == '_' )
		pszWord++;

	sxCode[ 0 ] = makeLower( *pszWord++ );
	sxCode[ 1 ] = sxCode[ 2 ] = sxCode[ 3 ] = 0;
	if( !*pszWord) return;

	char cLastCode = letterVals[ sxCode[ 0 ] - 'a' ];
	int nCode = 1;
	while( nCode < 4 && isalpha( *pszWord ) )
	{
		char cCode = letterVals[ makeLower( *pszWord++ ) - 'a' ];
		if( cCode && cCode != cLastCode )   // ignore duplicates
			sxCode[ nCode++ ] = cCode;

		cLastCode = cCode;
	}
}

CString CFileIndex::parenContents( const CString& sText, int& nEnd )
{
	if( sText[nEnd] != '(' )
		return sText;

	nEnd++;
	int nStart = nEnd;

	int nParen = 1;
	char c;
	while( (c = sText[ nEnd ]) )
	{
		if( c == ')' )
		{
			nParen--;
			if( nParen == 0 )
			{
				nEnd++;
				return sText.Mid( nStart, nEnd - 2);
			}
		}

		if( c == '(' )
			nParen++;

		nEnd++;
	}

	ASSERT(0);  // should be checking for this in WilburDoc
	return sText;
}

void CFileIndex::andResults( CSearchResults& results, CSearchResults& secondResults )
{
	int nInterruptCheck = 200;  // check for halt every n files
	int nCount = 0;

	for( POSITION pos = results.GetStartPosition(); pos; )
	{
		nCount++;
		if( nCount > nInterruptCheck )
		{
			nCount = 0;

			if( thisApp()->getHaltStatus() )
				AfxThrowUserException();
		}

		POSITION lastPos = pos;
		CFileInfo* pFile;
		CFileResult fileResult, secondResult;
		results.GetNextAssoc( pos, pFile, fileResult );

		if( secondResults.Lookup( pFile, secondResult ) )
		{
			fileResult.m_nRank += ( secondResult.m_nRank + 1 );
			results.SetAt( pFile, fileResult );
		}
		else
			results.RemoveKey( pFile );
	}
}

void CFileIndex::notResults( CSearchResults& results, CSearchResults& secondResults )
{
	int nInterruptCheck = 200;  // check for halt every n files
	int nCount = 0;

	for( POSITION pos = results.GetStartPosition(); pos; )
	{
		nCount++;
		if( nCount > nInterruptCheck )
		{
			nCount = 0;

			if( thisApp()->getHaltStatus() )
				AfxThrowUserException();
		}

		CFileResult fileResult, secondResult;
		CFileInfo* pFile;
		results.GetNextAssoc( pos, pFile, fileResult );

		if( secondResults.Lookup( pFile, secondResult ) )
			results.RemoveKey( pFile );
	}
}

void CFileIndex::orResults( CSearchResults& results, CSearchResults& secondResults )
{
	int nInterruptCheck = 200;  // check for halt every n files
	int nCount = 0;

	for( POSITION pos = secondResults.GetStartPosition(); pos; )
	{
		nCount++;
		if( nCount > nInterruptCheck )
		{
			nCount = 0;

			if( thisApp()->getHaltStatus() )
				AfxThrowUserException();
		}

		CFileResult fileResult, secondResult;
		CFileInfo* pFile;
		
		secondResults.GetNextAssoc( pos, pFile, secondResult );

		if( results.Lookup( pFile, fileResult ) )
		{
			fileResult.m_nRank += (secondResult.m_nRank + 1 );
			results.SetAt( pFile, fileResult );
		}
		else
			results.SetAt( pFile, secondResult );
	}
}

void CFileIndex::find( const CString& sText, CSearchCriteria& criteria, CSearchResults& results )
{
	m_sSkippedWords.Empty();

	CString sRemaining = sText;
	sRemaining.TrimLeft();

//	int nEnd;
//	if( ( nEnd = sRemaining.FindOneOf( sTerminator ) ) == -1 )

	int nEnd = 0;
	int nMax = sRemaining.GetLength() - 1;
	while( nEnd < nMax && (isValidAlpha(sRemaining[nEnd]) ||
		(nEnd && isValidTrailing(sRemaining[nEnd]) ) ||
		sRemaining[nEnd] == '"' ||
		sRemaining[nEnd] == '*' || sRemaining[nEnd] == '?' || sRemaining[nEnd] == '+' ) )
		nEnd++;

	if( nEnd >= nMax )
		findToken( sRemaining, criteria, results, TRUE );
	else
	{
		if( sRemaining[0] == '(' )
		{
			CString sContents = parenContents( sRemaining, nEnd );
			find( sContents, criteria, results );
		}
		else
		{
			CString sToken = sRemaining.Left( nEnd );
			if( !sToken.IsEmpty() )
				findToken( sToken, criteria, results, FALSE );
		}

//		int nMax = sRemaining.GetLength() - 1;

		while( nEnd != -1 && nEnd <= nMax )
		{
			char c = sRemaining[ nEnd ];
			// while( c == ' ' || c == '\t' ||	c == ',' || c == '&' || c== '<' || c == '>' || c == ':' )
			while( !isValidAlpha(c) && c != '|' && c != '(' && c != ')' && c != '^' && c != '*' && c != '?')
			{
				if( c == '<' || c == '>' || c == ':' )
					while( nEnd < nMax && ( c = sRemaining[ nEnd + 1 ] ) >= '0' && c <= '9' )
						nEnd++;

				if( nEnd >= nMax )
					return;

				c = sRemaining[ ++nEnd ];
			}


			if( nEnd >= nMax )
				return;

			if( c == '|' || c == '^' )
				nEnd++;

			if( nEnd >= nMax )
				return;

			ASSERT( c != ')'); // should never happen

			sRemaining = sRemaining.Mid( nEnd );
			sRemaining.TrimLeft();
			nMax = sRemaining.GetLength() - 1;

			if( sRemaining[0] == '(' )
			{
				CSearchResults tempResults;
				tempResults.InitHashTable( m_nFilesIndexed * 13 / 10 );

				nEnd = 0;
				CString sContents = parenContents( sRemaining, nEnd );
				find( sContents, criteria, tempResults );

				if( c == '|' )
					orResults( results, tempResults );
				else if( c == '^' )
					notResults( results, tempResults );
				else
					andResults( results, tempResults );
			}
			else
			{
				CString sToken;

//				if( ( nEnd = sRemaining.FindOneOf( sTerminator ) ) == -1 )

				nEnd = 0;
				bool bStarValid     = isValidAlpha('*') || isValidTrailing('*');
				bool bQuestionValid = isValidAlpha('?') || isValidTrailing('?');

				if( (isValidAlpha(sRemaining[nEnd]) ||
							sRemaining[nEnd] == '"' ||
							sRemaining[nEnd] == '*' ||
							sRemaining[nEnd] == '?' ) )
				{
					nEnd++;
					while( nEnd < nMax &&
							(isValidTrailing(sRemaining[nEnd]) ||
								sRemaining[nEnd] == '"' ||
								sRemaining[nEnd] == '*' ||
								sRemaining[nEnd] == '?' ) )
						nEnd++;
				}
				
				if( nEnd >= nMax )
					sToken = sRemaining;
				else
					sToken = sRemaining.Left( nEnd );

				if( c == '|' )
				{
					findToken( sToken, criteria, results, TRUE ); // TRUE so common word doesn't add everything
				}
				else
				{
					CSearchResults secondResults;
					secondResults.InitHashTable( m_nFilesIndexed * 13 / 10 );
					findToken( sToken, criteria, secondResults, FALSE );
					if( c == '^' )
						notResults( results, secondResults );
					else
						andResults( results, secondResults );

				}
			}
		}
	}
}

void CFileIndex::addAll( CSearchCriteria& criteria, CSearchResults& results )
{
	for( FileID nFile = m_pFiles->getFirst(); nFile; nFile = m_pFiles->getNext( nFile ) )
	{	
		CFileInfo* pInfo = (CFileInfo*) m_pFiles->getData( nFile );
		if( matchesCriteria( pInfo, criteria ) )
		{
			CFileResult fileResult;
			fileResult.m_pFileInfo = pInfo;
			fileResult.m_pDirInfo = NULL;
			if( pInfo->m_nParent )
			{
				// don't bother with Volume records
				CFileInfo* pDirInfo = (CFileInfo*) m_pFiles->getData( pInfo->m_nParent );
				fileResult.m_pDirInfo = pDirInfo;

				fileResult.m_nRank = 0;

				results.SetAt( pInfo, fileResult );
			}
		}
	}
}

void CFileIndex::findToken( const CString& sText, CSearchCriteria& criteria,
						   CSearchResults& results, BOOL bWholeSearch )
{
	if( thisApp()->getHaltStatus() )
		AfxThrowUserException();

	if( !m_pWords )
		return;

	if( sText.IsEmpty()  )
	{
		// for empty search string, show all files
		addAll( criteria, results );
		return;
	}

	if( sText[0] == '+' || sText[ 0 ] == '\"' )
	{
		// if prepended + sign skip; it is flag to local search to skip
		findToken( sText.Mid(1), criteria, results, bWholeSearch );
		return;
	}
	else
	{
		int nQuote = sText.Find( '\"' );
		if( nQuote > 0 )
		{
			findToken( sText.Left( nQuote ), criteria, results, bWholeSearch );
			return;
		}
	}
	
	if( m_pSkipWords->lookup( sText ) || ( sText.GetLength() < m_nMinWordLength && sText.FindOneOf( "*?" ) == -1) )
	{
		m_sSkippedWords += " " + sText;

		// if just one term in search then add all otherwise none
		if( !bWholeSearch )
			addAll( criteria, results );

		return;
	}
			

	if( m_numberHandling != allNumbers )
	{
		int nNumber = sText.FindOneOf( "0123456789" );
		if( nNumber != -1 )
		{
			if( m_numberHandling == noNumbers || nNumber == 0 )
			{
				simpleMessageBox( IDS_INCONSISTENT_SEARCH_PHRASE );
				return;
			}
		}
	}

	if( criteria.getSoundex() )
	{
		DWORD sxWord, sxMask;
		getSoundex( sText, (char*) &sxMask );
		for( TextID nWord = m_pWords->getFirst(); nWord; nWord = m_pWords->getNext( nWord ) )
		{
			const char* pszWord = m_pWords->getText( nWord );
			getSoundex( pszWord, (char*) &sxWord );
			if( sxMask == sxWord )
				addWordFiles( nWord, criteria, results );
		}
	}
	else if( sText.FindOneOf( "*?" ) != -1 )
	{
		int nInterruptCheck = 200;  // check for halt every n files
		int nFileCount = 0;

		for( TextID nWord = m_pWords->getFirst(); nWord; nWord = m_pWords->getNext( nWord ) )
		{
			nFileCount++;
			if( nFileCount > nInterruptCheck )
			{
				nFileCount = 0;

				if( thisApp()->getHaltStatus() )
					AfxThrowUserException();
			}

			const char* pszMask = sText;
			const char* pszWord = m_pWords->getText( nWord );

			if( isWordMatch( pszWord, pszMask ) )
				addWordFiles( nWord, criteria, results );
		}
	}
	else
	{			
		TextID nWord = m_pWords->lookup( sText );
		if( !nWord )
			return;

		addWordFiles( nWord, criteria, results );
	}

}

bool CFileIndex::isWordMatch( const char* pszWord, const char* pszMask )
{
	bool bStarValid     = isValidAlpha('*') || isValidTrailing('*');
	bool bQuestionValid = isValidAlpha('?') || isValidTrailing('?');

	while( *pszMask && *pszWord )
	{
		if( *pszMask == '*' && !bStarValid )
		{
			pszMask++;
			if( *pszMask == '\0' )
				return true;

			while( *pszWord )
			{
				if( *pszWord == *pszMask && isWordMatch( pszWord, pszMask ) )
					return true;
				
				pszWord++;
			}

			return false;
		}
		
		if( *pszMask == *pszWord ||
			( *pszMask == '?' && !bQuestionValid ) )
		{
			pszMask++; pszWord++;
		}
		else
			break;
	}

	while( ( *pszMask == '*' && !bStarValid ) || ( *pszMask == '?' && !bQuestionValid ) )
		pszMask++;

	if( !*pszMask && !*pszWord )
		return true;
	else
		return false;
}

void CFileIndex::addWordFiles( TextID nWord, CSearchCriteria& criteria, CSearchResults& results )
{
	CWordInfo* pWordInfo = (CWordInfo*) m_pWords->getData( nWord ); 

	ASSERT( pWordInfo );

	addToResults( pWordInfo->m_nLastFile, criteria, (BYTE) pWordInfo->m_nLastRank, results );
	
	IndexPosition nIndex = pWordInfo->m_nFiles;
	if( nIndex )
		addWordList( nIndex, criteria, results );
}

void CFileIndex::addWordList( IndexPosition nIndex, CSearchCriteria& criteria, CSearchResults& results )
{
	FileID nFile;
	BYTE bCount;

	while( ( nFile = m_pLinks->getFile( nIndex, bCount ) ) )
	{
		addToResults( nFile, criteria, bCount, results );
	}

	nIndex = m_pLinks->getNextList( nIndex );

	if( nIndex )
		addWordList( nIndex, criteria, results );
}

bool CFileIndex::matchesCriteria( CFileInfo* pInfo, CSearchCriteria& criteria )
{
	long nSize = criteria.getMinSize();
	if( nSize && (long) pInfo->m_nSize < nSize)
		return false;

	nSize = criteria.getMaxSize();
	if( nSize && (long) pInfo->m_nSize >= nSize )
		return false;

	COleDateTime modTime = criteria.getBeforeTime();
	if( (DATE) modTime != 0.0 )
	{
		COleDateTime fileTime( pInfo->m_timeLastWritten.GetTime() );
		if( fileTime < modTime )
			return false;
	}

	modTime = criteria.getAfterTime();
	if( (DATE) modTime != 0.0 )
	{
		COleDateTime fileTime( pInfo->m_timeLastWritten.GetTime() );
		if( fileTime > modTime )
			return false;
	}

	DWORD dwAttrib = criteria.getAttributes();
	if( dwAttrib )
	{
		if( dwAttrib & 0x10000000 )
		{
			dwAttrib ^= 0x10000000;
			if( ( dwAttrib & pInfo->m_dwAttributes ) == dwAttrib )
				return false;
		}
		else if( (dwAttrib & pInfo->m_dwAttributes) != dwAttrib )
			return false;
	}

	CString sName = criteria.getFileName();
	if( !sName.IsEmpty() && !CWilburDoc::fileNameMatch( pInfo->m_pszName, sName ) )
		return false;

	sName = criteria.getFolderName();
	if( !sName.IsEmpty() )
	{
		CFileInfo* pFolder;
		if( pInfo->m_nParent )
		{
			pFolder = (CFileInfo*) m_pFiles->getData( pInfo->m_nParent );
			if( pFolder->m_nParent == 0 )
				pFolder = pInfo;
		}
		else
			pFolder = pInfo;

		if( !CWilburDoc::fileNameMatch( pFolder->m_pszName, sName ) )
			return false;
	}

	return true;
}

void CFileIndex::addToResults( FileID nFile, CSearchCriteria& criteria, BYTE bCount, CSearchResults& results )
{
	if( nFile != RemovedFileID )
	{
		CFileInfo* pInfo = (CFileInfo*) m_pFiles->getData( nFile );
		ASSERT( pInfo );

		if( matchesCriteria( pInfo, criteria ) )
		{
			CFileResult fileResult, secondResult;
			fileResult.m_pFileInfo = pInfo;
			fileResult.m_pDirInfo = NULL;

			if( pInfo->m_nParent )
			{
				CFileInfo* pDirInfo = (CFileInfo*) m_pFiles->getData( pInfo->m_nParent );
				if( pDirInfo->m_nParent )
					fileResult.m_pDirInfo = pDirInfo;
			}
				

			fileResult.m_nRank = bCount;

			if( results.Lookup( pInfo, secondResult ) )
				fileResult.m_nRank += ( secondResult.m_nRank + 1 );

			results.SetAt( pInfo, fileResult );
		}
	}
}

void CFileIndex::markFileObsolete( FileID nFile )
{
	if( nFile )
	{
		if( m_pFiles->enableWrite() )
			m_pFiles->markObsolete( nFile );

		m_pFiles->disableWrite();
	}
}

FileID CFileIndex::getFileIDFromPath( const CString& sPath )
{
	return m_pFiles->lookupPath( sPath );
}

void CFileIndex::wordCount( const CString sListName, double dPercent, bool bShowMessage )
{
	int nInterruptCheck = 200;  // check for halt every n files
	int nCheck = 0;

	CStdioFile listFile( sListName, CFile::modeCreate | CFile::modeWrite );

	int nMinFiles = (int) (m_nContentsIndexed * dPercent / 100.0);

	for( TextID nWord = m_pWords->getFirst(); nWord; nWord = m_pWords->getNext( nWord ) )
	{
		int nFileCount = 1;

		nCheck++;
		if( nCheck > nInterruptCheck )
		{
			nCheck = 0;

			if( thisApp()->getHaltStatus() )
				AfxThrowUserException();
		}

		CWordInfo* pWordInfo = (CWordInfo*) m_pWords->getData( nWord );

		ASSERT( pWordInfo );
		
		int nWordCount = pWordInfo->m_nLastRank + 1;

		IndexPosition nIndex = pWordInfo->m_nFiles;

		FileID nFile;
		BYTE bCount;

		while( nIndex )
		{
			while( ( nFile = m_pLinks->getFile( nIndex, bCount ) ) )
			{
				if( nFile != RemovedFileID )
				{
					nFileCount++;
					nWordCount += bCount + 1;
				}
			}

			nIndex = m_pLinks->getNextList( nIndex );
		}

		if( nFileCount > nMinFiles )
		{
			CString sLine;
			sLine.Format("%s,%d,%d\n", m_pWords->getText( nWord ), nFileCount, nWordCount );
			
			listFile.WriteString( sLine );
		}
	}
	if( bShowMessage)
		simpleMessageBox( IDS_COUNT_FILE_CREATED );
}

void CFileIndex::setPassword( const CString sPassword, bool bSave, bool bEncryptNew, bool bRestrictCopies )
{
	m_sPassword = sPassword;
	m_bSavePassword = bSave;
	m_bEncryptNewZips = bEncryptNew;
	m_bRestrictCopies = bRestrictCopies;
}

CString CFileIndex::getPassword( bool& bSave, bool& bEncryptNew, bool& bRestrictCopies )
{
	bSave = m_bSavePassword;
	bEncryptNew = m_bEncryptNewZips;
	bRestrictCopies = m_bRestrictCopies;

	return m_sPassword;
}

CTextStorage::CTextStorage()
{
	m_nDataSize = 0;
	m_pHashEntries = NULL;
	m_nHashEntries = 0;
}

CTextStorage::CTextStorage( TextID nInitialSize, TextID nMaxIncrement, CString sExtension,
						    size_t nDataSize, TextID nHashEntries )
							 :
		m_fileMemory( nInitialSize, nMaxIncrement, sExtension ) 
{
	m_nDataSize = nDataSize + sizeof( TextID );

	m_pHashEntries = new TextID[ nHashEntries ];
	memset( m_pHashEntries, 0, nHashEntries * sizeof( TextID ) );
	m_nHashEntries = nHashEntries;
}

CTextStorage::~CTextStorage()
{
	delete [] m_pHashEntries;
}

void CTextStorage::Serialize( CArchive& archive )
{
	CObject::Serialize( archive );

	// because release numbers were forgotten in the first version
	// the version numbers are negative to allow the first version to
	// be identified
	int nVersion;

	if( archive.IsStoring() )
	{
		nVersion = -1;
		archive << nVersion;

		m_fileMemory.Serialize( archive );
		archive << m_nHashEntries << (DWORD) m_nDataSize;

		archive.Write( m_pHashEntries, m_nHashEntries * sizeof( TextID ) );
	}
	else
	{
		archive >> nVersion;
		if( nVersion < 0 )
		{
			m_fileMemory.Serialize( archive );
			TextBlockSize nDataSize;
			archive >> m_nHashEntries >> nDataSize;
			m_nDataSize = nDataSize;
		}
		else
		{
			TextID nBlocksAllocated, nSizeOfBlock, nNextFree;

			nBlocksAllocated = nVersion;
			archive >> nSizeOfBlock >> nNextFree;

			TextBlockSize nDataSize;
			archive >> m_nHashEntries >> nDataSize;
			m_nDataSize = nDataSize;

			TextID nID = m_fileMemory.getNewSpace( nNextFree );
			BYTE* pBytes= m_fileMemory.getPointer( 0 );

			TextID nLastBlock = nNextFree / nSizeOfBlock;
			for( TextID i = 0; i <= nLastBlock; i++ )
			{
				TextBlockSize nBytes = nSizeOfBlock;
				if( i == nLastBlock )
					nBytes = nNextFree - i * nSizeOfBlock;

				if( nBytes > 1 + m_nDataSize)
					archive.Read( pBytes + i * nSizeOfBlock, nBytes * sizeof( char ) );
			}
		}

		m_pHashEntries = new TextID[ m_nHashEntries ];

		archive.Read( m_pHashEntries, m_nHashEntries * sizeof( TextID ) );
	}
}

TextID CTextStorage::addText( const char* pText, TextID& nCount )
{
	nCount++;
	if( nCount > m_nHashEntries )
		reHash( nCount );

	TextID nLength = strlen( pText ) + 1;
	TextID nReturn = m_fileMemory.getNewSpace( nLength + m_nDataSize) + m_nDataSize;

	char* pMemText = (char*) m_fileMemory.getPointer( nReturn );


	strncpy( pMemText, pText, nLength - 1 );
	*(pMemText + nLength - 1 ) = '\0';

	addHash( pText, nReturn );

	return nReturn;
}


void CTextStorage::addHash( const char* pText, TextID nReturn)
{
	TextID nHash = getHash( pText );

	if( m_pHashEntries[ nHash ] )
	{
		// already hash entry add to chain
		const char* pNextText = getText( m_pHashEntries[ nHash ] );
		ASSERT( pNextText );

		TextID* pNext = (TextID*) ( pNextText - m_nDataSize );
		while( *pNext )
		{
			pNextText = getText( *pNext );
			pNext = (TextID*) ( pNextText - m_nDataSize );
		}

		*pNext = nReturn;
	}
	else
		m_pHashEntries[ nHash ] = nReturn;
}

const char* CTextStorage::getText( TextID nText )
{
	return (const char*) m_fileMemory.getPointer( nText );
}

void* CTextStorage::getData( TextID nText )
{
	return (void*) ( getText( nText ) - m_nDataSize + sizeof( TextID ) );
}

void CTextStorage::markObsolete( TextID nText )
{
	char* pszText = (char*) getText( nText );
	if( pszText && *pszText )
		*pszText = (char) 0xff;
}

TextID CTextStorage::lookup( const char* pText )
{
	TextID nHash = getHash( pText );

	
	ASSERT( nHash < m_nHashEntries );
	TextID nNext = m_pHashEntries[ nHash ];

	if( nNext )
	{
		const char* pChar = getText( nNext );
		ASSERT( pChar );

		while( stricmp( pChar, pText ) )
		{
			TextID* pNext = (TextID*) ( pChar - m_nDataSize );
			nNext = *pNext;
			if( nNext )
			{
				pChar = getText( nNext );
				ASSERT( pChar );
			}
			else
				return 0;
		}

		return nNext;
	}
	else
		return 0;
}

TextID CTextStorage::getHash( const char* pszKey )
{
	TextID nHash = 0;
	char c;
	while( ( c = makeLower( *pszKey++ ) ) )
		nHash = (nHash<<5) + nHash + c;

	return nHash % m_nHashEntries;
}

void CTextStorage::reHash( TextID nCount )
{
	while( nCount > m_nHashEntries ) m_nHashEntries *= 2;

	delete m_pHashEntries;
	
	m_pHashEntries = new TextID[ m_nHashEntries ];
	memset( m_pHashEntries, 0, m_nHashEntries * sizeof( TextID ) );

	// clear hash chains
	for( TextID id = getFirst(); id; id = getNext(id))
	{
		const char* pText = getText(id);
		TextID* pNext = (TextID*) ( pText - m_nDataSize );
		*pNext = 0;
	}

	for( id = getFirst(); id; id = getNext(id))
	{
		const char* pText = getText(id);
		addHash( pText, id );
	}
}

TextID CTextStorage::getFirst()
{
	TextID nReturn = m_nDataSize;
	if( !m_fileMemory.getPointer( nReturn ) )
		nReturn = 0;

	return nReturn;
}

TextID CTextStorage::getNext( TextID nText )
{
	const char* pText = getText( nText );

	TextID nReturn = 0;
	if( pText )
	{
		nReturn = nText + strlen( pText ) + 1 + m_nDataSize;

		if( !m_fileMemory.getPointer( nReturn ) )
			nReturn = 0;
	}

	return nReturn;
}

CFileStorage::CFileStorage( TextID nHashEntries )
	:
	CTextStorage(
		regGetInt(HKEY_CURRENT_USER, "Settings", "fil_initial_size", 0xffff ),
		regGetInt( HKEY_CURRENT_USER, "Settings", "fil_max_incr", 0xfffff ),
		"fil",
		sizeof( CTime ) + sizeof( FileID ) + sizeof(DWORD) + sizeof( unsigned int),
		nHashEntries )
{}

void CFileStorage::Serialize( CArchive& archive )
{
	CTextStorage::Serialize( archive );
}

TextID CFileStorage::lookupFile( const char* pText, FileID nParent )
{
	TextID nHash = getHash( pText );

	
	ASSERT( nHash < m_nHashEntries );
	TextID nNext = m_pHashEntries[ nHash ];

	if( nNext )
	{
		CFileInfo* pInfo = (CFileInfo*) getData( nNext );

		ASSERT( pInfo );

		while( stricmp( pInfo->m_pszName, pText ) || pInfo->m_nParent != nParent )
		{
			TextID* pNext = (TextID*) ( ( (BYTE*) pInfo ) - sizeof( TextID ) );
			nNext = *pNext;
			if( nNext )
			{
				pInfo = (CFileInfo*) getData( nNext );
				ASSERT( pInfo );
			}
			else
				return 0;
		}

		return nNext;
	}
	else
		return 0;
}

TextID CFileStorage::lookupPath( const char* pPathName )
{
	int nLength = strlen( pPathName );

	while( nLength )
	{
		const char* pName = pPathName + nLength;
		if( *pName == '/' || *pName == '\\' || *pName == ':' )
		{
			nLength += 1;
			break;
		}

		nLength--;
	}

	const char* pText = pPathName + nLength;

	TextID nHash = getHash( pText );

	
	ASSERT( nHash < m_nHashEntries );
	TextID nNext = m_pHashEntries[ nHash ];

	if( nNext )
	{
		CString sChar = getPath( nNext );
		const char* pChar = sChar;
		ASSERT( pChar );

		while( stricmp( pChar, pPathName ) )
		{
			CFileInfo* pInfo = (CFileInfo*) getData( nNext );
			ASSERT( pInfo );

			TextID* pNext = (TextID*) ( ( (BYTE*) pInfo ) - sizeof( TextID ) );
			nNext = *pNext;
			if( nNext )
			{
				sChar = getPath( nNext );
				pChar = sChar;
				ASSERT( pChar );
			}
			else
				return 0;
		}

		return nNext;
	}
	else
		return 0;
}

CString CFileStorage::getPath( TextID nFile )
{
	CFileInfo* pLeaf = (CFileInfo*) getData( nFile );
	ASSERT( pLeaf );

	FileID nParent = pLeaf->m_nParent;
	CString sReturn;

	if( nParent )
	{
		CFileInfo* pParent = (CFileInfo*) getData( nParent );
		if( pParent->m_nParent == 0 )
		{
			// it was a folder since parent is volume
			sReturn = pLeaf->m_pszName;
		}
		else
		{
			sReturn = pParent->m_pszName;
			sReturn += pLeaf->m_pszName;
		}
	}
	else
		sReturn = pLeaf->m_pszName;  // must be volume
/*
	CString sReturn = pLeaf->m_pszName;
	while( pLeaf->m_nParent )
	{
		pLeaf = (CFileInfo*) getData( pLeaf->m_nParent );
		ASSERT( pLeaf );
		sReturn = pLeaf->m_pszName + sReturn;
	}
*/
	return sReturn;
}

CBuildLinksList::CBuildLinksList()
	:
		m_pLinkBlocks( NULL ), m_nBlocksAllocated( 0 ),  m_nSizeOfBlock( 0 ),
		m_nNextFree( 0 )
{
}

CBuildLinksList::CBuildLinksList( LinkBlockSize nSizeOfBlock, LinkID nInitBlocks )
	:
		m_nSizeOfBlock( nSizeOfBlock )
	
{
	m_pLinkBlocks = new CBuildLink*[ nInitBlocks ];
	memset( m_pLinkBlocks, 0, nInitBlocks * sizeof( CBuildLink* ) );

	m_nBlocksAllocated = nInitBlocks;
	m_nNextFree = 1;  // leave the first one free for empty
}

CBuildLinksList::~CBuildLinksList()
{
	for( LinkID i = 0; i < m_nBlocksAllocated && m_pLinkBlocks[ i ]; i++ )
		delete m_pLinkBlocks[ i ];

	delete [] m_pLinkBlocks;
}

LinkID CBuildLinksList::addLink( FileID nFile, LinkID nNextLink )
{
	if( nNextLink )
	{
		CBuildLink* pLink = getLink( nNextLink );
		if( pLink && pLink->m_nFile == nFile )
		{
			if( pLink->m_bCount != 0xff )
				pLink->m_bCount++;

			return nNextLink;
		}
	}

	LinkID nReturn = m_nNextFree;
	CBuildLink* pLink = makeNew();
	pLink->m_nFile  = nFile;
	pLink->m_bCount = 0;  // use 0 as base rather than 1
	pLink->m_nNextLink = nNextLink;

	return nReturn;
}

CBuildLink* CBuildLinksList::makeNew()
{
	LinkID nBlock = m_nNextFree / m_nSizeOfBlock;
	LinkID nOffset = m_nNextFree - m_nSizeOfBlock * nBlock;

	if( nBlock >= m_nBlocksAllocated )
	{
		LinkID nNewBlocks = m_nBlocksAllocated * 2;
		
		CBuildLink** pNewBlocks = new CBuildLink*[ nNewBlocks ];

		memset( pNewBlocks, 0, nNewBlocks * sizeof( CBuildLink* ) );

		for( LinkID i = 0; i < m_nBlocksAllocated; i++ )
			pNewBlocks[ i ] = m_pLinkBlocks[ i ];

		delete m_pLinkBlocks;
		m_pLinkBlocks = pNewBlocks;
		m_nBlocksAllocated = nNewBlocks;
	}

	if( !m_pLinkBlocks[ nBlock ] )
	{
		m_pLinkBlocks[ nBlock ] = new CBuildLink[ m_nSizeOfBlock ];
		memset( m_pLinkBlocks[ nBlock ], 0, m_nSizeOfBlock * sizeof( CBuildLink ) );
	}

	m_nNextFree++;
	return m_pLinkBlocks[ nBlock ] + nOffset;
}

CBuildLink* CBuildLinksList::getLink( LinkID nID )
{
	LinkID nBlock = nID / m_nSizeOfBlock;
	LinkID nOffset = nID - m_nSizeOfBlock * nBlock;

	if( nBlock < m_nBlocksAllocated && m_pLinkBlocks[ nBlock ] )
		return m_pLinkBlocks[ nBlock ] + nOffset;
	else
		return NULL;
}


CFileLinks::CFileLinks() : m_bUseRanking( TRUE )

{
}

CFileLinks::CFileLinks( LinkID nInitialSize, LinkID nMaxIncrement,
					      CString sExtension, BOOL bUseRanking )
	:
		m_fileMemory( nInitialSize, nMaxIncrement, sExtension ),
		m_bUseRanking( bUseRanking )
	
{
}

CFileLinks::~CFileLinks()
{
}


void CFileLinks::Serialize( CArchive& archive )
{
	CObject::Serialize( archive );

	// because release numbers were forgotten in the first version
	// the version numbers are negative to allow the first version to
	// be identified
	int nVersion;

	if( archive.IsStoring() )
	{
		nVersion = -2;
		archive << nVersion << m_bUseRanking;
		m_fileMemory.Serialize( archive );
	}
	else
	{
		archive >> nVersion;

		if( nVersion < 0 )
		{
			if( nVersion < -1 )
				archive >> m_bUseRanking;

			m_fileMemory.Serialize( archive );
		}
		else
		{
			LinkID nBlocksAllocated, nSizeOfBlock, nNextFree;

			nBlocksAllocated = nVersion;
			archive >> nSizeOfBlock >> nNextFree;

			LinkID nID = m_fileMemory.getNewSpace( nNextFree * sizeof( ShortFileID ) );
			BYTE* pBytes= m_fileMemory.getPointer( 0 );

			LinkID nLastBlock = nNextFree / nSizeOfBlock;
			for( LinkID i = 0; i <= nLastBlock; i++ )
			{
				LinkID nItems = nSizeOfBlock;
				if( i == nLastBlock )
					nItems = nNextFree - i * nSizeOfBlock - 1;

				archive.Read( pBytes + i * nSizeOfBlock * sizeof( ShortFileID ),
							  nItems * sizeof( ShortFileID ) );
			}
		}
	}
}

LinkID CFileLinks::getSpace( int nSize )
{
	return m_fileMemory.getNewSpace( nSize );
}

void CFileLinks::addFile( LinkID nPosition, FileID nFile, BYTE bCount )
{
	int nRankSize = ( m_bUseRanking ) ?  sizeof( BYTE ) : 0;

	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nPosition );
	ASSERT( pBytes );
	FileID* pLinks = (FileID*) pBytes;

	*pLinks = (FileID) nFile;

	if( m_bUseRanking )
		*(pBytes + sizeof( FileID ) ) = bCount;
}	

LinkID CFileLinks::getNextList( LinkID nID )
{
	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nID );
	LinkID* pLinks = (LinkID*) pBytes;

	ASSERT( pLinks );

	return *pLinks;
}

void CFileLinks::setNextList( LinkID nID, LinkID nNext )
{
	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nID );
	LinkID* pLinks = (LinkID*) pBytes;

	ASSERT( pLinks );

	*pLinks = nNext;
}

FileID CFileLinks::getFile( LinkID& nID, BYTE& bCount )
{
	int nRankSize = ( m_bUseRanking ) ?  sizeof( BYTE ) : 0;

	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nID );
	FileID* pLinks = (FileID*) pBytes;

	nID += ( sizeof( FileID ) + nRankSize );

	if( pLinks )
	{
		bCount = ( m_bUseRanking ) ? *(pBytes + sizeof( FileID ) ) : 255;
		return *pLinks;
	}
	else
	{
		bCount = 0;
		return 0;
	}
}

void CFileLinks::removeFile( FileID nFile )
{
	FileID* pLinks;
	int nLinkSize = sizeof( FileID ) + ( ( m_bUseRanking ) ?  sizeof( BYTE ) : 0 );
	for( LinkID nLink = 0; (pLinks = (FileID*) m_fileMemory.getPointer( nLink )); nLink += nLinkSize )
		if( *pLinks == nFile )
			*pLinks = RemovedFileID;
}


CWordLinks::CWordLinks() : m_bUseRanking( TRUE )

{
}

CWordLinks::CWordLinks( LinkID nInitialSize, LinkID nMaxIncrement,
					      CString sExtension, BOOL bUseRanking )
	:
		m_fileMemory( nInitialSize, nMaxIncrement, sExtension ),
		m_bUseRanking( bUseRanking ), m_nCurrentFile( 0 ), m_nCurrentID( 0 )
	
{
}

CWordLinks::~CWordLinks()
{
}

void CWordLinks::addWord( FileID nFile, WordID nWord, BYTE nRank )
{
	LinkID nLink;

	int nRankSize = ( m_bUseRanking ) ?  sizeof( BYTE ) : 0;

	if( nFile != m_nCurrentFile )
	{
		// each set of words in a file begins with the FileID followed by
		// a 0 terminated list of Word IDs

		if( m_nCurrentFile )
		{
			// not first file so add terminator for previous list
			nLink = m_fileMemory.getNewSpace( sizeof( WordID ) + nRankSize );
			BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nLink );
			WordID* pWord = (WordID*) pBytes;

			*pWord = 0;
		}

		m_nCurrentFile = nFile;

		nLink = m_fileMemory.getNewSpace( sizeof( FileID ) );
		BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nLink );
		FileID* pFileID = (FileID*) pBytes;

		*pFileID = (FileID) nFile;
		m_nCurrentID = nLink;
	}

	nLink = m_fileMemory.getNewSpace( sizeof( WordID ) + nRankSize );
	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nLink );
	WordID* pLinks = (WordID*) pBytes;

	*pLinks = (WordID) nWord;

	if( m_bUseRanking )
	{
		pBytes += sizeof( WordID );
		*pBytes = nRank;
	}
}

FileID CWordLinks::getFile( LinkID& nID )
{
	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nID );
	FileID* pLinks = (FileID*) pBytes;

	nID +=  sizeof( FileID );

	if( pLinks )
		return *pLinks;
	else
		return 0;
}


WordID CWordLinks::getWord( LinkID& nID, BYTE& bCount )
{
	int nRankSize = ( m_bUseRanking ) ?  sizeof( BYTE ) : 0;

	BYTE* pBytes = (BYTE*) m_fileMemory.getPointer( nID );
	WordID* pLinks = (WordID*) pBytes;

	nID += ( sizeof( WordID ) + nRankSize );

	if( pLinks )
	{
		bCount = ( m_bUseRanking ) ? *(pBytes + sizeof( WordID ) ) : 255;
		return *pLinks;

	}
	else
	{
		bCount = 0;
		return 0;
	}
}
