/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// fileindx.h : Interface for the CFileIndex class

#ifndef __FILEINDX_H__
#define __FILEINDX_H__ 1

#ifndef __AFXTEMPL_H__
#include "afxtempl.h"
#endif

#ifndef __FILEMEMORY_H__
#include "FileMemory.h"
#endif

#if !defined(AFX_SEARCHCRITERIA_H__8C5A2193_ED0C_11D1_B2D4_006008A8E2D0__INCLUDED_)
#include "searchcriteria.h"
#endif

#if _DEBUG
void debugMsg( CString sMsg );
void debugErr();
#endif


typedef DWORD ReleaseNumber;

typedef DWORD LinkID;
typedef DWORD WordID;
typedef WORD  ShortWordID;

struct CWordInfo
{
	DWORD m_nLastFile;
	WORD  m_nLastRank;
	DWORD m_nFiles;
	int   m_nFileCount;
};

typedef DWORD FileID;
typedef WORD  ShortFileID;

struct CFileInfo
{
	FileID			m_nParent;
	CTime			m_timeLastWritten;
	unsigned int	m_nSize;
	DWORD           m_dwAttributes;
	char			m_pszName[2];	   // dummy to represent name
};

struct CFileResult
{
	CFileInfo* m_pFileInfo;
	CFileInfo* m_pDirInfo;
	int        m_nRank;
	// CFileInfo* operator= (CFileInfo*) { return m_pFileInfo; }
};

inline int operator==( CFileResult Element1, CFileResult Element2 )
{
	return ( Element1.m_pFileInfo == Element2.m_pFileInfo );
}

typedef CMap< CFileInfo*, CFileInfo*&, CFileResult, CFileResult& > CSearchResults;
typedef CList< CString, CString& > CRootDirList;
typedef CList< CString, CString& > CMaskList;

/*
BOOL CompareElements( const CFileResult* pElement1, const CFileResult* pElement2 )
{
	return ( pElement1->m_pFileInfo == pElement2->m_pFileInfo );
}*/

struct CBuildLink
{
	FileID m_nFile;   // an index into a CFileStorage
	BYTE   m_bCount;  
	LinkID m_nNextLink; // an index into a CBuildLinksList
};

typedef DWORD TextID;
typedef DWORD TextBlockSize;

class CTextStorage : public CObject
{
	DECLARE_SERIAL( CTextStorage )

	public:
		CTextStorage();
		CTextStorage( TextID nInitialSize, TextID nMaxIncrement, CString sExtesion,
			          size_t nDataSize, TextID nHashEntries );

		virtual ~CTextStorage();

		virtual void Serialize( CArchive& archive );

		TextID addText( const char* pText, TextID& nCount );
		const char* getText( TextID nTextID );
		void* getData( TextID nTextID );
		void  markObsolete( TextID nTextID );  // used to erase a name

		TextID lookup( const char* pText );  // returns NULL if not found
		TextID getFirst();
		TextID getNext( TextID nText );
		BOOL enableWrite() { return m_fileMemory.enableWrite(); }
		BOOL disableWrite() { return m_fileMemory.disableWrite(); }
		BOOL getFileStatus( CFileStatus& fileStatus ) { return m_fileMemory.getFileStatus( fileStatus ); }

	protected:
		CFileMemory m_fileMemory;
		size_t  m_nDataSize;
		TextID* m_pHashEntries;
		TextID  m_nHashEntries;

		TextID getHash( const char* pText );  // return hash index
		void addHash( const char* pText, TextID nID);
		void reHash( TextID nCount );  // rebuild hash bash on nCount entries

		// Each hash entry is an offset to a text entry.  If during indexing the hash slot
		// is already is use, then sizeof(TextID) is subtracted from hash entry.  If this address
		// is 0 then the current offset is put there, otherwise the chain is followed until a
		// free spot is found.  Thus every text string is preceded by a Next hash pointer (offset)
};

class CFileStorage : public CTextStorage
{
	DECLARE_SERIAL( CFileStorage )

	public:
		CFileStorage() {};
		CFileStorage( TextID nHashEntries );
		void CFileStorage::Serialize( CArchive& archive );

		TextID lookupFile( const char* pText, FileID nParent );
		TextID lookupPath( const char* pPathName );
		CString getPath( TextID nFile );
};

typedef DWORD LinkBlockSize;
class CBuildLinksList : public CObject
{
	DECLARE_SERIAL( CBuildLinksList )

	public:
		CBuildLinksList();
		CBuildLinksList( LinkBlockSize nSizeOfBlock, LinkID nInitialBlocks );
		virtual ~CBuildLinksList();

		LinkID addLink( FileID nFile, LinkID nFirstLink /* 0 if none */ );
		CBuildLink*  getLink( LinkID nID );

		LinkID getNumberOfLinks() { return m_nNextFree; }

	private:
		LinkID        m_nBlocksAllocated;
		LinkBlockSize m_nSizeOfBlock;
		LinkID        m_nNextFree;

		CBuildLink** m_pLinkBlocks;

		CBuildLink* makeNew();
};

class CFileLinks : public CObject
{
	DECLARE_SERIAL( CFileLinks )

	public:
		CFileLinks();
		CFileLinks( LinkID nInitialSize, LinkID nMaxIncrement, CString sExtension,
			        BOOL bUseRanking );
		virtual ~CFileLinks();
		void   Serialize( CArchive& archive );

		LinkID getSpace( int nSize );
		LinkID getNextList( LinkID nID) ;
		void   setNextList( LinkID nID, LinkID nNext );

		void addFile( LinkID nPosition, FileID nFile, BYTE bCount );

		FileID getFile( LinkID& nID, BYTE& bCount );
		void   removeFile( FileID nFile );  // replaced everywhere with FileRemovedID
		BOOL   enableWrite() { return m_fileMemory.enableWrite(); }
		BOOL   disableWrite() { return m_fileMemory.disableWrite(); }
		BOOL getFileStatus( CFileStatus& fileStatus ) { return m_fileMemory.getFileStatus( fileStatus ); }

	private:
		CFileMemory m_fileMemory;
		BOOL        m_bUseRanking;
};

class CWordLinks : public CObject
{
	DECLARE_DYNAMIC( CWordLinks )

	public:
		CWordLinks();
		CWordLinks( LinkID nInitialSize, LinkID nMaxIncrement, CString sExtension,
			        BOOL bUseRanking );
		virtual ~CWordLinks();

		void addWord( FileID nFile, WordID nWord, BYTE nRank );  // must be done serially

		FileID getFile( LinkID& nID );
		WordID getWord( LinkID& nID, BYTE& bCount );
		BOOL   enableWrite() { return m_fileMemory.enableWrite(); }
		BOOL   disableWrite() { return m_fileMemory.disableWrite(); }
		BOOL getFileStatus( CFileStatus& fileStatus ) { return m_fileMemory.getFileStatus( fileStatus ); }

	private:
		CFileMemory m_fileMemory;
		FileID      m_nCurrentFile;
		LinkID      m_nCurrentID;
		BOOL        m_bUseRanking;
};


typedef DWORD WordID;
typedef DWORD ByteID;
typedef DWORD IndexPosition;
const FileID RemovedFileID = 1;  // must be smaller than any possible valid ID

class CFileIndex : public CObject
{
	DECLARE_SERIAL( CFileIndex );

	public:
		static void getSoundex( const char* pszWord, char sxCode[4] );
		static CString getTempZip( const CString& sFileName, int nZip, bool bCheckRestriction = false );
		static int getZipExtIndex( const CString& sPath);
		static void removeTemporaryFile( const CString& sTempFile, int attr );
		static BOOL nameMatches( const char* pFileName, const char* pFileMask );
		static void orResults( CSearchResults& results, CSearchResults& secondResults );
		static void andResults( CSearchResults& results, CSearchResults& secondResults );
		static void notResults( CSearchResults& results, CSearchResults& secondResults );

		enum NumberInclusion { noNumbers, trailingNumbers, allNumbers };

		inline bool isValidAlpha( unsigned char c );
		inline bool isValidTrailing( unsigned char c );
		inline bool isBinary( unsigned char c );

		CFileIndex();
		~CFileIndex();

		virtual void Serialize( CArchive& archive );

		void clear();

		void addInclude( const char* sPath );
		void addExclude( CString sPath );

		void removeInclude( CString sPath );
		void removeExclude( CString sPath );

		void clearIncludes();
		void clearExcludes();

		void setNumberInclusion( NumberInclusion newValue );
		NumberInclusion getNumberInclusion() { return m_numberHandling; }

		void setInternationalChar( DWORD dwValue);
		DWORD getInternationalChar() { return m_dwInternationalChar; }

		void setOEMTextChar( BOOL bValue);
		BOOL getOEMTextChar() { return m_bOEMTextChar; }

		void setAddedChars( const CString sAddedChar, const CString sAddedTrailing );
		CString getAddedChars();
		CString getAddedTrailingChars();

		POSITION getFirstInclude();
		CString& getNextInclude( POSITION& pos );

		BOOL getZipStatus();
		void setZipStatus( BOOL bNewStatus );

		bool getTrackAll() { return m_bTrackAll; }
		void setTrackAll( bool bNewStatus );

		BOOL getUseRanking();
		void setUseRanking( BOOL bNewStatus );

		POSITION getFirstExclude();
		CString& getNextExclude( POSITION& pos );

		int  getMinWordLength() { return m_nMinWordLength; }
		void setMinWordLength( int nLength );

		int  getMaxWordLength() { return m_nMaxWordLength; }
		void setMaxWordLength( int nLength );

		CString getSkipFile() { return m_sSkipFile; }
		void    setSkipFile( CString sNewFile );

		void setPassword( const CString sPassword, bool bSave, bool bEncryptNew, bool bRestrictCopies );
		CString CFileIndex::getPassword( bool& bSave, bool& bEncryptNew, bool& bRestrictCopies );

		FileID  numberOfFiles() { return m_nFilesIndexed; }
		FileID	numberOfContents() { return m_nContentsIndexed; }
		WordID  numberOfWords() { return m_nWordsIndexed; }
		ByteID  numberOfBytes() { return m_nBytesRead; }
		CTime   timeCreated()   { return m_timeBase; }
		void    setTimeCreated( CTime newValue ) { m_timeBase = newValue; }
		FileID  getFileIDFromPath( const CString& sPath );

		bool	importSkipWords( CString sFile );

		void build();
		void update();
		void prune();
		bool mapsModified();

		void find( const CString& sText, CSearchCriteria& criteria, CSearchResults& resultsList );
		void markFileObsolete( FileID nID );
		// BOOL setFileTime( const CString& sPath, const CTime timeLastWritten );
		void wordCount( const CString sListName, double percent, bool bShowMessage = true );
		CFileInfo* getFileInfo( FileID nID ) { return (CFileInfo*) m_pFiles->getData( nID ); }
		CString skippedWords() { return m_sSkippedWords; };
		void indexZipEntry(char* fileName,  CString sZipPath, CString sIndexPath,
			CMaskList& maskList, FileID nVolume, CTime& timeLastWritten, unsigned int nSize);

	private:
		int            m_nMinWordLength;
		int            m_nMaxWordLength;
		CTime          m_timeBase;  // time base index was created
		CTime          m_timeSaved; // last mod time of the .WIL file
		BOOL           m_bIsUpdate;
		BOOL           m_bUseRanking;
		BOOL           m_bDoZipped;
		DWORD          m_dwInternationalChar;
		BOOL           m_bOEMTextChar;
		CFileStorage*  m_pFiles;
		CTextStorage*  m_pWords;
		CTextStorage*  m_pSkipWords;
		CString        m_sSkipFile;
		CWordLinks*    m_pTemporaryLinks;   // only used during build and update
		CFileLinks*       m_pLinks;		// constructed from temporary links during compress

		bool    m_bSavePassword;
		bool    m_bEncryptNewZips;
		bool    m_bRestrictCopies;  // don't copy from encrypted zips
		CString m_sPassword;

		CList< CString, CString& > m_includePatterns;
		CList< CString, CString& > m_excludePatterns;

		FileID m_nFilesIndexed;
		FileID m_nContentsIndexed;
		WordID m_nWordsIndexed;
		ByteID m_nBytesRead;

		CString m_sPartialWord; 

		NumberInclusion   m_numberHandling;
		bool            m_bValidChar[ 256 ];
		bool            m_bValidTrailing[ 256 ];
		bool            m_bBinary[ 256 ];
		bool			m_bTrackAll;
		bool			m_bPDF;
		CString         m_sAddedChars;
		CString         m_sAddedTrailingChars;
		CMap< CString, LPCSTR, unsigned char, unsigned char > m_htmlCharMap;

		CString m_sSkippedWords;

		void buildLinks();
		void indexPattern( const CString& sInclude );
		void indexDirectory( const CString& sRootPath, FileID nParent, FileID nVolume );
		void indexZip( CString sZipPath, const CString& sIndexPath, CMaskList& maskList, FileID nVolume );
		void createZipDirectory();
		void emptyIndex();

        void setCharMasks();

		BOOL isExcluded( const char* pszFilePath );
		BOOL isZip( const CString& sZipPath );

		void indexFile( const CString& sPathName, FileID nFile, int nAttempt = 0 );
		// H.W. need return value for opened but not closed html tags
		int indexBuffer( char* pBuffer, ByteID nBytesRead, FileID nFile, bool bHtml );
		  // note buffer may be changed

		void indexWord( const char* pWord, FileID nFile );

		void indexExcludedWord( const char* pWord );
		void indexExcludeBuffer( char* pBuffer, ByteID nBytes );
		bool indexExclusions();
		void addToList( const char* pszPath, CList< CString, CString& >& m_list );

		void addWordFiles( TextID nWord, CSearchCriteria& criteria, CSearchResults& results );
		void addWordList( IndexPosition nIndex, CSearchCriteria& criteria, CSearchResults& results );
		void addToResults( FileID nFile, CSearchCriteria& criteria, BYTE bRank, CSearchResults& results );

		void findToken( const CString& sText, CSearchCriteria& criteria, CSearchResults& resultsList, BOOL bWholeSearch );
		bool isWordMatch( const char* pszWord, const char* pszMask );
		void addAll( CSearchCriteria& criteria, CSearchResults& results );
		CString parenContents( const CString& sText, int& nEnd );

		void scanFiles();
		void CFileIndex::createRootDirList( CRootDirList& dirList );
		void addPathToRootDirs( const CString& sInclude, CRootDirList& dirList );
		CString getRootDir( const CString& sPath );
		bool dirMatch( const char* pszRootPath, const char* pszInclude );
		void createMaskList( const CString& sRootPath, CMaskList& maskList );
		bool matchesMask( const char* pszFile, CMaskList& maskList );
		bool matchesCriteria( CFileInfo* pInfo, CSearchCriteria& criteria );

		void compressLinks();
		unsigned char htmlChar( char* pszIn );

		CString encryptPassword( const CString sPassword );
		CString decryptPassword( const CString sIn );
};

inline bool CFileIndex::isValidAlpha( unsigned char c )
{
	
	return m_bValidChar[ c ];
}

inline bool CFileIndex::isValidTrailing( unsigned char c )
{
	return m_bValidTrailing[ c ];
}

inline bool CFileIndex::isBinary( unsigned char c )
{
	return m_bBinary[ c ];
}

UINT indexThread( LPVOID pParam );
UINT updateThread( LPVOID pParam );
CString registerCode( const CString& sName );

#endif
