/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
//
//  mmfile.cpp
// 
//
//      Implementation file for CMemoryMappedFile. Class supports memory mapped files
//

#include "stdafx.h"
#include "mmfile.h"
#include "wilbur.h"


CMemoryMappedFile::CMemoryMappedFile()
{
	m_hFile = CFile::hFileNull;
	m_bAutoDelete = TRUE;
}

CMemoryMappedFile::~CMemoryMappedFile()
{
	if (m_hFile)
		Close();
}


BOOL CMemoryMappedFile::Open(LPCTSTR szName, 
	HANDLE hFile				/* = INVALID_HANDLE_VALUE */,
	DWORD  flProtect			/* = PAGE_READWRITE */, 
	DWORD  dwMaximumSizeHigh	/* = 0*/, 
	DWORD  dwMaximumSizeLow		/* = 1024 * 4*/,
	DWORD  dwNumberOfBytesToMap /* = 0 */,
	DWORD  dwDesiredAccess		/* = FILE_MAP_WRITE | FILE_MAP_READ */, 
	DWORD  dwFileOffsetHigh		/* = 0*/, 
	DWORD  dwFileOffsetLow		/* = 0*/)
{
	ASSERT(m_hFile == CFile::hFileNull);

	m_hFile = (UINT) CreateFileMapping(hFile,
		NULL,
		flProtect,
		dwMaximumSizeHigh,
		dwMaximumSizeLow,
		szName);

	if (!m_hFile)
	{
//		simpleMessageBox("Unable to open shared file\n");
		DWORD nErr = GetLastError();

		char* pszMessage;
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
					   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );
		simpleMessageBox( pszMessage );

		return FALSE;
	}

	if (!MapViewOfFile(dwDesiredAccess, dwFileOffsetHigh, dwFileOffsetLow, dwMaximumSizeLow))
		return FALSE;
		
	return TRUE;
}

BOOL CMemoryMappedFile::Attach(HANDLE *hFile, 
	DWORD  dwNumberOfBytesToMap /* = 0 */,
	DWORD  dwDesiredAccess		/* = FILE_MAP_WRITE | FILE_MAP_READ */, 
	DWORD  dwFileOffsetHigh		/* = 0*/, 
	DWORD  dwFileOffsetLow		/* = 0*/)
{
	ASSERT(m_hFile == CFile::hFileNull);
	ASSERT(hFile != INVALID_HANDLE_VALUE);

	m_hFile = (UINT) hFile;
	return MapViewOfFile(dwDesiredAccess, 
		dwFileOffsetHigh, 
		dwFileOffsetLow,
		dwNumberOfBytesToMap);
}


BOOL CMemoryMappedFile::MapViewOfFile(DWORD  dwDesiredAccess, 
		DWORD  dwFileOffsetHigh		/*= 0*/, 
		DWORD  dwFileOffsetLow		/*= 0*/,
		DWORD  dwNumberOfBytesToMap /*= 0*/)
{
	if (!UnmapViewOfFile())
		return FALSE;

	BYTE *pBuffer = (BYTE* ) ::MapViewOfFile((HANDLE) m_hFile,
		dwDesiredAccess,
		dwFileOffsetHigh, 
		dwFileOffsetLow,
		dwNumberOfBytesToMap);

	if (!pBuffer)
	{
		DWORD nErr = GetLastError();
		char* pszMessage;
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
					   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );
		simpleMessageBox( pszMessage );
		//simpleMessageBox( IDS_MEM_FILE_MAP_ERR );

		MEMORY_BASIC_INFORMATION mbi;
		LPBYTE p = NULL;
		FILE* f = fopen("deadwilbur.txt", "w");
		while (true)
		{
			SIZE_T size = VirtualQuery(p, &mbi, sizeof(mbi));
			if (size == 0) break;
			fprintf(f, "%x\t%lu\t%x\t%x\n", mbi.AllocationBase, mbi.RegionSize, mbi.State, mbi.Type);
			p += mbi.RegionSize;
		}
		fclose(f);
		return FALSE;
	}

	BASEATTACH(pBuffer, dwNumberOfBytesToMap, 0);

	m_bAutoDelete = TRUE;

	return TRUE;
}

BOOL CMemoryMappedFile::UnmapViewOfFile()
{
	BYTE *p = BASEDETACH();

	if (!p)
		return TRUE;

	if (!::UnmapViewOfFile(p))
	{
		simpleMessageBox( IDS_MEM_FILE_MAP_VIEW_ERR );
		return FALSE;
	}

	return TRUE;
}

	
HANDLE CMemoryMappedFile::Detach()
{
	HANDLE hTemp = (HANDLE)m_hFile;
	m_bAutoDelete = FALSE;
	Close();
	m_hFile = CFile::hFileNull;
	m_bAutoDelete = TRUE;
	
	return hTemp;
}


void CMemoryMappedFile::Close(void)
{
	BOOL bAutoDelete = m_bAutoDelete;
	if (m_hFile)
	{
		UnmapViewOfFile();
        BASEDETACH();
    }

	if (bAutoDelete && m_hFile)
	{
		CloseHandle((HANDLE)m_hFile);
		m_hFile = CFile::hFileNull;
	}
}

void CMemoryMappedFile::Flush(DWORD startBytePosition /* = 0 */, DWORD  dwNumberOfBytesToFlush /* = 0 */)
{
	if (m_lpBuffer)
	{
		if (startBytePosition < GetLength() / sizeof(BYTE))
			FlushViewOfFile(m_lpBuffer + startBytePosition, dwNumberOfBytesToFlush);
	}
}


