/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// SearchCriteria.cpp : implementation file
//

#include "stdafx.h"
#include "wilbur.h"
#include "SearchCriteria.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSearchCriteria

IMPLEMENT_SERIAL(CSearchCriteria, CCmdTarget, 0)

CSearchCriteria::CSearchCriteria() : m_minSize( 0 ), m_maxSize( 0 ),
									 m_onOrBefore( 0.0 ), m_onOrAfter( 0.0 ),
									 m_attributes( 0 ),
									 m_reverse( false ), m_soundex( false )
{
	m_bAutomated = true;
	EnableAutomation();
	
	// To keep the application running as long as an OLE automation 
	//	object is active, the constructor calls AfxOleLockApp.
	
	AfxOleLockApp();
}

CSearchCriteria::CSearchCriteria( bool bAutomated )  :	m_minSize( 0 ), m_maxSize( 0 ),
														m_onOrBefore( 0.0 ), m_onOrAfter( 0.0 ),
														m_attributes( 0 ),
														m_reverse( false ), m_soundex( false )
{
	if( bAutomated )
	{
		m_bAutomated = true;
		EnableAutomation();
		
		// To keep the application running as long as an OLE automation 
		//	object is active, the constructor calls AfxOleLockApp.
		
		AfxOleLockApp();
	}
	else
		m_bAutomated = false;
}

CSearchCriteria::~CSearchCriteria()
{
	// To terminate the application when all objects created with
	// 	with OLE automation, the destructor calls AfxOleUnlockApp.
	
	if( m_bAutomated )
		AfxOleUnlockApp();
}

void CSearchCriteria::clear()
{
	m_minSize = 0;
	m_maxSize = 0;
	m_onOrBefore = 0.0;
	m_onOrAfter = 0.0;
	m_attributes = 0;
	m_reverse = false;
	m_soundex = false;
	m_fileName.Empty();
	m_folderName.Empty();
}

void CSearchCriteria::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CCmdTarget::OnFinalRelease();
}

void CSearchCriteria::setFolderName( CString sNewValue )
{
	if( !sNewValue.IsEmpty() )
	{
		int nLength = sNewValue.GetLength();

		if( nLength < 2 || ( sNewValue[1] != ':' && sNewValue[1] != '\\' && sNewValue[1] != '/' ) )
		{
			if( sNewValue[0] != '*' )
			{
				sNewValue = "*\\" + sNewValue;
				nLength += 2;
			}
		}

		char cLast = sNewValue[ nLength - 1 ];
		if( cLast != '*' && cLast != '\\' && cLast != '/' )
			sNewValue += '\\';
	}

	m_folderName = sNewValue;
}

bool CSearchCriteria::setBeforeTime( CString sBefore )
{
	try
	{
		COleDateTime tBefore;
		tBefore.ParseDateTime( sBefore );

		m_onOrBefore = tBefore;
	}
	catch( ... )
	{
		return false;
	}

	return true;
}

bool CSearchCriteria::setAfterTime( CString sAfter )
{
	try
	{
		COleDateTime tAfter;
		tAfter.ParseDateTime( sAfter );

		m_onOrAfter = tAfter;
	}
	catch( ... )
	{
		return false;
	}

	return true;
}

void CSearchCriteria::setAttributes( CString sAttrib )
{
	DWORD dwAttrib = 0;

	const char* pszAttrib = sAttrib;
	while( *pszAttrib )
	{
		switch( *pszAttrib++ )
		{
		case 'a':
			dwAttrib |= FILE_ATTRIBUTE_ARCHIVE;
			break;

		case 'c':
			dwAttrib |= FILE_ATTRIBUTE_COMPRESSED;
			break;

		case 'd':
			dwAttrib |= FILE_ATTRIBUTE_DIRECTORY;
			break;

		case 'h':
			dwAttrib |= FILE_ATTRIBUTE_HIDDEN;
			break;

		case 'r':
			dwAttrib |= FILE_ATTRIBUTE_READONLY;
			break;

		case 's':
			dwAttrib |= FILE_ATTRIBUTE_SYSTEM;
			break;

		case '^':
			dwAttrib |= 0x10000000;  // used to indicate NOT
			break;
		}
	}

	m_attributes = dwAttrib;
}

void CSearchCriteria::storeInRegistry()
{
	regPutInt( HKEY_CURRENT_USER, "Last Criteria", "MinSize", m_minSize );
	regPutInt( HKEY_CURRENT_USER, "Last Criteria", "MaxSize", m_maxSize );
	regPutBinary( HKEY_CURRENT_USER, "Last Criteria", "Before", &m_onOrBefore, sizeof(DATE) );
	regPutBinary( HKEY_CURRENT_USER, "Last Criteria", "After", &m_onOrAfter, sizeof(DATE) );
	regPutInt( HKEY_CURRENT_USER, "Last Criteria", "Reverse", m_reverse );
	regPutInt( HKEY_CURRENT_USER, "Last Criteria", "Soundex", m_soundex );
	regPutInt( HKEY_CURRENT_USER, "Last Criteria", "Attributes", m_attributes );
	regPutString( HKEY_CURRENT_USER, "Last Criteria", "FileName", m_fileName );
	regPutString( HKEY_CURRENT_USER, "Last Criteria", "FolderName", m_folderName );
}

void CSearchCriteria::recallFromRegistry()
{
	m_minSize = regGetInt( HKEY_CURRENT_USER, "Last Criteria", "MinSize", 0 );
	m_maxSize = regGetInt( HKEY_CURRENT_USER, "Last Criteria", "MaxSize", 0 );
	regGetBinary( HKEY_CURRENT_USER, "Last Criteria", "Before", &m_onOrBefore, sizeof(DATE) );
	regGetBinary( HKEY_CURRENT_USER, "Last Criteria", "After", &m_onOrAfter, sizeof(DATE) );

	int nBool;
	nBool = regGetInt( HKEY_CURRENT_USER, "Last Criteria", "Reverse", 0 );
	m_reverse = ( nBool != 0);
	nBool = regGetInt( HKEY_CURRENT_USER, "Last Criteria", "Soundex", 0 );
	m_soundex = (nBool != 0 );

	m_attributes = regGetInt( HKEY_CURRENT_USER, "Last Criteria", "Attributes", 0 );

	m_fileName = regGetString( HKEY_CURRENT_USER, "Last Criteria", "FileName", "" );
	m_folderName = regGetString( HKEY_CURRENT_USER, "Last Criteria", "FolderName", "" );
}

BEGIN_MESSAGE_MAP(CSearchCriteria, CCmdTarget)
	//{{AFX_MSG_MAP(CSearchCriteria)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CSearchCriteria, CCmdTarget)
	//{{AFX_DISPATCH_MAP(CSearchCriteria)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "FileName", m_fileName, OnFileNameChanged, VT_BSTR)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "OnOrBefore", m_onOrBefore, OnOnOrBeforeChanged, VT_DATE)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "OnOrAfter", m_onOrAfter, OnOnOrAfterChanged, VT_DATE)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "FolderName", m_folderName, OnFolderNameChanged, VT_BSTR)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "MinSize", m_minSize, OnMinSizeChanged, VT_I4)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "MaxSize", m_maxSize, OnMaxSizeChanged, VT_I4)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "Reverse", m_reverse, OnReverseChanged, VT_BOOL)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "Soundex", m_soundex, OnSoundexChanged, VT_BOOL)
	DISP_PROPERTY_NOTIFY(CSearchCriteria, "Attributes", m_attributes, OnAttributesChanged, VT_I4)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_ISearchCriteria to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {8C5A2191-ED0C-11D1-B2D4-006008A8E2D0}
static const IID IID_ISearchCriteria =
{ 0x8c5a2191, 0xed0c, 0x11d1, { 0xb2, 0xd4, 0x0, 0x60, 0x8, 0xa8, 0xe2, 0xd0 } };

BEGIN_INTERFACE_MAP(CSearchCriteria, CCmdTarget)
	INTERFACE_PART(CSearchCriteria, IID_ISearchCriteria, Dispatch)
END_INTERFACE_MAP()

// {8C5A2192-ED0C-11D1-B2D4-006008A8E2D0}
IMPLEMENT_OLECREATE(CSearchCriteria, "Wilbur.SearchCriteria", 0x8c5a2192, 0xed0c, 0x11d1, 0xb2, 0xd4, 0x0, 0x60, 0x8, 0xa8, 0xe2, 0xd0)

/////////////////////////////////////////////////////////////////////////////
// CSearchCriteria message handlers

void CSearchCriteria::OnFileNameChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnOnOrBeforeChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnOnOrAfterChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnFolderNameChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnMinSizeChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnMaxSizeChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnReverseChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnSoundexChanged() 
{
	// TODO: Add notification handler code

}

void CSearchCriteria::OnAttributesChanged() 
{
	// TODO: Add notification handler code

}
