/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// splitter.cpp : implementation file
//

#include "stdafx.h"

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __SPLITTER_H__
#include "splitter.h"
#endif

#ifndef __CONTENTS_H__
#include "contents.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSplitterFrame

IMPLEMENT_DYNCREATE(CSplitterFrame, CMainFrame)

CSplitterFrame::CSplitterFrame()
{
}

CSplitterFrame::~CSplitterFrame()
{
}

BOOL CSplitterFrame::OnCreateClient(LPCREATESTRUCT /*lpcs*/, CCreateContext* pContext)
{
	m_nOrientation = regGetInt( HKEY_CURRENT_USER, "List View", "Orientation", 1);
	if( m_nOrientation )
	{
		// create a splitter with 2 row, 1 column
		if (!m_wndSplitter.CreateStatic(this, 2, 1))
		{
			TRACE0("Failed to CreateStaticSplitter\n");
			return FALSE;
		}

		// add the first splitter pane - the default (List) view in row 0
		CRect rtFrame;
		GetWindowRect( &rtFrame );
		int cy = regGetInt( HKEY_CURRENT_USER,"List View", "Height",
								  ( rtFrame.bottom - rtFrame.top ) / 4 );
		if (!m_wndSplitter.CreateView(0, 0,
			pContext->m_pNewViewClass, CSize(rtFrame.right - rtFrame.left, cy), pContext))
		{
			TRACE0("Failed to create first pane\n");
			return FALSE;
		}

		// add the second splitter pane - an input view in column 1
		if (!m_wndSplitter.CreateView(1, 0,
			RUNTIME_CLASS(CContentsView), CSize(0, 0), pContext))
		{
			TRACE0("Failed to create second pane\n");
			return FALSE;
		}
	}
	else
	{
		// create a splitter with 2 row, 1 column
		if (!m_wndSplitter.CreateStatic(this, 1, 2))
		{
			TRACE0("Failed to CreateStaticSplitter\n");
			return FALSE;
		}

		// add the first splitter pane - the default (List) view in row 0
		CRect rtFrame;
		GetWindowRect( &rtFrame );
		int cx = regGetInt( HKEY_CURRENT_USER,"List View", "Width",
								  ( rtFrame.right - rtFrame.left ) / 4 );
		if (!m_wndSplitter.CreateView(0, 0,
			pContext->m_pNewViewClass, CSize(cx, rtFrame.bottom - rtFrame.top), pContext))
		{
			TRACE0("Failed to create first pane\n");
			return FALSE;
		}

		// add the second splitter pane - an input view in column 1
		if (!m_wndSplitter.CreateView(0, 1,
			RUNTIME_CLASS(CContentsView), CSize(0, 0), pContext))
		{
			TRACE0("Failed to create second pane\n");
			return FALSE;
		}
	}

	// activate the input view
	SetActiveView((CView*)m_wndSplitter.GetPane(0,0));

	return TRUE;
}


BEGIN_MESSAGE_MAP(CSplitterFrame, CMainFrame)
	//{{AFX_MSG_MAP(CSplitterFrame)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CSplitterFrame message handlers


void CSplitterFrame::OnDestroy() 
{
	CMainFrame::OnDestroy();
	
	if( m_nOrientation )
	{
		int cy, cyMin;
		m_wndSplitter.GetRowInfo( 0, cy, cyMin );
		regPutInt( HKEY_CURRENT_USER, "List View", "Height", cy );
	}
	else
	{
		int cx, cxMin;
		m_wndSplitter.GetColumnInfo( 0, cx, cxMin );
		regPutInt( HKEY_CURRENT_USER, "List View", "Width", cx );
	}
}
