/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// wilbur.cpp : Defines the class behaviors for the application.
//

#include <process.h>
#include <locale.h>
#include "stdafx.h"

#include <direct.h>
#include <afxadv.h>

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __MAINFRM_H__
#include "mainfrm.h"
#endif

#ifndef __WILBUDOC_H__
#include "wilbudoc.h"
#endif

#ifndef __FILELISTVIEW_H__
#include "FileListView.h"
#endif

#ifndef __SPLITTER_H__
#include "splitter.h"
#endif

#ifndef __DIALOGS_H__
#include "dialogs.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

static const CString sExtension("pdftext");

static BOOL removeDirectories( CString sRootPath )
{
	BOOL bEmpty = TRUE;

	WIN32_FIND_DATA fileData;

	CString sSearchPath = sRootPath + "*";
	HANDLE hFind = FindFirstFile( (const char*) sSearchPath, &fileData );

	if( hFind != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
			{
				if( fileData.cFileName[0] != '.' || (fileData.cFileName[1] != '.' && fileData.cFileName[1] != '\0' ) )
				{
					CString sNewPath = sRootPath + fileData.cFileName;
					if( removeDirectories( sNewPath + '\\' ) )
					{
						if( RemoveDirectory( sNewPath ) )
							return FALSE;  // must leave or this will be found again
					}
					else
						bEmpty = FALSE;
				}
			}
			else
				bEmpty = FALSE;
		} while ( FindNextFile( hFind, &fileData ) );

		FindClose( hFind );
	}

	return bEmpty;
}

/////////////////////////////////////////////////////////////////////////////
// CWilburApp

BEGIN_MESSAGE_MAP(CWilburApp, CWinApp)
	//{{AFX_MSG_MAP(CWilburApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_INDEX_STOP, OnIndexStop)
	ON_UPDATE_COMMAND_UI(ID_INDEX_STOP, OnUpdateIndexStop)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_COMMAND(ID_FILE_PREFERENCES, OnFilePreferences)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
	ON_COMMAND(ID_INDEX_FINISHED, OnIndexFinished)
	ON_COMMAND(ID_UPDATE_FINISHED, OnUpdateFinished)
	ON_COMMAND(ID_SET_STATUS, OnSetStatus )
	ON_COMMAND(ID_SECOND_INVOKE, OnSecondInvocation )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWilburApp construction

CWilburApp::CWilburApp() : m_bIsBatch( FALSE ), m_bRunAutomated( FALSE ),
						   m_searchCriteria( false ),
						   m_bPDF( false ), m_bUsePDFCache(false),
						   m_sPDFPath(""), m_bTemporaryPDFCache(false),
						   m_sPDFargs("")
{
	*m_pszCurrentDir = '\0';
	setlocale(LC_ALL, "");

	// see if pdftotext exists
	HINSTANCE rc = ShellExecute(NULL,NULL,"pdftotext","",NULL,SW_HIDE);
	if( int(rc) > 32 )
	{
		m_bPDF = true;
		m_sPDFargs = regGetString( HKEY_CURRENT_USER, "options", "pdftotextArgs", "");
	}

	mapLowerCaseChars();
}

CString CWilburApp::translatePDF( const CString sPath, bool bIsIndexing )
{

	CString sTempPath;
	if( m_bUsePDFCache ) {
		unsigned int nIndex = getDocument()->getFileIDFromPath(sPath);
		sTempPath.Format("%s\\pdftext%d", m_sPDFPath, nIndex);
		if( !bIsIndexing ) {
			CFileStatus stat;
			if( CFile::GetStatus(sTempPath, stat) )
				return sTempPath;  // cached copy already exists
		}
    }
	else {
		int nSlash = sPath.ReverseFind('\\');
		sTempPath = getZipPath() + sPath.Mid(nSlash+1) + '_';
	}

	char cmd[1024];
	sprintf(cmd, "pdftotext -q -eol unix %s \"%s\" \"%s\"", m_sPDFargs, sPath, sTempPath);

	STARTUPINFO startInfo;
	memset(&startInfo, 0, sizeof(startInfo));
	startInfo.cb = sizeof(STARTUPINFO);
	startInfo.dwFlags = STARTF_USESHOWWINDOW;
	startInfo.wShowWindow = SW_HIDE;
	PROCESS_INFORMATION procInfo;

	if( CreateProcess(NULL, cmd, NULL, NULL, 0, NORMAL_PRIORITY_CLASS, NULL,
				NULL, &startInfo, &procInfo))
	{
		WaitForSingleObject(procInfo.hThread, INFINITE);
		CloseHandle(procInfo.hThread);
		CloseHandle(procInfo.hProcess);
	}

	return sTempPath;
}

void CWilburApp::makeNewPDFCache()
{
	m_bUsePDFCache = false;  // turn off if can't make directory
	m_bTemporaryPDFCache = false;

	BOOL bUseCache = regGetInt( HKEY_CURRENT_USER, "options", "Cache PDF", 0 );
	if( !bUseCache ) return;

	char pszTempFileName[ MAX_PATH ];;

	if( !GetTempFileName( thisApp()->getZipPath(), sExtension, 0, pszTempFileName ) )
	{
		DWORD nErr = GetLastError();

		char* pszMessage;
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
					   NULL, nErr, 0, (char*) &pszMessage, 10, NULL );
		simpleMessageBox( pszMessage );

		AfxThrowUserException();
	}

	CString sPath = pszTempFileName;
	CFile::Remove(sPath);  // don't want file, just name
	try {
		if( _mkdir(sPath) != -1 ) {
			m_sPDFPath = sPath;
			m_bUsePDFCache = true;
			m_bTemporaryPDFCache = true;
			CFile dummy(sPath + "\\pdftext0", CFile::modeCreate);  // create dummy file so zip clean up doesn't zap dir
			dummy.Close();
		}
	}
	catch(CFileException* e) {
		e->Delete();
	}
}

void CWilburApp::savePDFCache()
{
	// if necessary, move the pdf cache directory to an appropriate name
	if( !m_bUsePDFCache ) return;

	CString sPath = getDocument()->getDocFileName();
	int nDot = sPath.ReverseFind( '.' );
	if( nDot == -1 )
		sPath += '.' + sExtension;
	else
		sPath = sPath.Left( nDot + 1) + sExtension;

	if( sPath == m_sPDFPath ) return;  // already in right place

	deletePDFCache(sPath);         // delete destination if it exists
	MoveFile(m_sPDFPath, sPath);  // not end of world if it fails
	m_bTemporaryPDFCache = false;
	m_sPDFPath = sPath;
}

void CWilburApp::openPDFCache()
{
	// set the pdf cache directory name
	CString sPath = getDocument()->getDocFileName();
	int nDot = sPath.ReverseFind( '.' );
	if( nDot == -1 )
		sPath += '.' + sExtension;
	else
		sPath = sPath.Left( nDot + 1) + sExtension;

	m_bTemporaryPDFCache = false;
	CFileStatus status;
	if( CFile::GetStatus( sPath, status) && status.m_attribute & CFile::Attribute::directory ) {
		m_sPDFPath = sPath;
		m_bUsePDFCache = true;
	}
	else {
		m_bUsePDFCache = false;

		// but see if a cache directory should be created
		BOOL bUseCache = regGetInt( HKEY_CURRENT_USER, "options", "Cache PDF", 0 );
		if( bUseCache ) {
			try {
				if( _mkdir(sPath) != -1 ) {
					m_sPDFPath = sPath;
					m_bUsePDFCache = true;
				}
			}
			catch(CFileException* e) {
				e->Delete();
			}
		}
	}
}

void CWilburApp::closePDFCache()
{
	if( !m_bTemporaryPDFCache ) return;
	m_bTemporaryPDFCache = false;
	deletePDFCache(m_sPDFPath);
	m_sPDFPath = "";
}

void CWilburApp::deletePDFCache(CString sPDFPath)
{
	CString sSearchPath = sPDFPath + "\\pdftext*";

	WIN32_FIND_DATA fileData;

	for(;;) {
		HANDLE hFind = FindFirstFile( (const char*) sSearchPath, &fileData );
		FindClose(hFind);

		if( hFind == INVALID_HANDLE_VALUE ) break;

		CString sFilePath = sPDFPath + '\\' + fileData.cFileName;
		try {
			CFile::Remove(sFilePath);
		}
		catch( CFileException* e)
		{
			e->Delete();
			break;  // must exit to avoid infinite loop
		}
	}
		

	RemoveDirectory(sPDFPath);
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CWilburApp object

CWilburApp theApp;
CWilburApp* thisApp() { return &theApp; }

// This identifier was generated to be statistically unique for your app.
// You may change it if you prefer to choose a specific identifier.

// {28FEADE0-DD82-11CF-9B6F-00006E23CF57}
static const CLSID clsid =
{ 0x28feade0, 0xdd82, 0x11cf, { 0x9b, 0x6f, 0x0, 0x0, 0x6e, 0x23, 0xcf, 0x57 } };

/////////////////////////////////////////////////////////////////////////////
// CWilburApp initialization

BOOL CWilburApp::InitInstance()
{
	// Initialize OLE libraries
	if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}

	
	CString sCmd = m_lpCmdLine;
	sCmd.TrimLeft();
	if( !sCmd.IsEmpty() && sCmd.GetLength() == 2 && sCmd[0] == '-' && sCmd[1] == 'r' )
		return FALSE;

	// AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.


	CString sHelpFile = m_pszHelpFilePath;
	int nIndex = sHelpFile.ReverseFind( '\\' );
	if( nIndex != -1 )
	{
		m_sStartDirectory = sHelpFile.Left( nIndex );
#ifndef _DEBUG
		regPutString( HKEY_CURRENT_USER, "Settings", "Path", m_sStartDirectory );
#endif
	}

	m_bThreadRunning = FALSE;
	m_bHaltThread    = FALSE;

#ifdef _AFXDLL
    Enable3dControls( );   //Call Enable3dControls
#else
    Enable3dControlsStatic( );
#endif

	int nFiles = regGetInt( HKEY_CURRENT_USER, "options", "Recent File Count", 16 );
	if( nFiles < 1 ) nFiles = 1;
	LoadStdProfileSettings( nFiles );  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CWilburDoc),
		RUNTIME_CLASS(CSplitterFrame),       // main SDI frame window
		RUNTIME_CLASS(CFileListView));

	AddDocTemplate(pDocTemplate);

	// Connect the COleTemplateServer to the document template.
	//  The COleTemplateServer creates new documents on behalf
	//  of requesting OLE containers by using information
	//  specified in the document template.
	m_server.ConnectTemplate(clsid, pDocTemplate, TRUE);
	// Note: SDI applications register server objects only if /Embedding
	//   or /Automation is present on the command line.

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	BOOL bFirstTime = FALSE;
	if( regGetString( HKEY_CURRENT_USER, "Recent File List", "File1", "" ).IsEmpty() )
		bFirstTime = TRUE;

	CString sVersion;
	sVersion.LoadString( IDS_VERSION );
	BOOL bAdmin = regPutString( HKEY_LOCAL_MACHINE, "Settings", "Version", sVersion );

	CString sDefault = m_sStartDirectory + "\\tempzip\\";
	m_sZipPath = regGetFixString( "Zip Options", "TempDir", sDefault );

	sDefault = m_sStartDirectory + "\\indexes";
	m_sIndexPath = regGetString( HKEY_CURRENT_USER, "Options", "IndexPath", sDefault );

	char pszSystemDir[255];
	GetSystemDirectory( pszSystemDir, 255 );
	CString sViewPath = pszSystemDir;
	sViewPath = '"' + sViewPath + "\\viewers\\quikview.exe\" -f:";

	m_sQuickViewPath = regGetString( HKEY_CURRENT_USER, "Quick View", "Path", sViewPath );
	
	// Check to see if launched as OLE server
	if (cmdInfo.m_bRunEmbedded || cmdInfo.m_bRunAutomated)
	{
		// Register all OLE server (factories) as running.  This enables the
		//  OLE libraries to create objects from other applications.
		COleTemplateServer::RegisterAll();
		// Application was run with /Embedding or /Automation.  Don't show the
		//  main window in this case.
		m_pMainWnd = NULL;
		return TRUE;
	}

	// When a server application is launched stand-alone, it is a good idea
	// to update the system registry in case it has been damaged.

	if( bAdmin ) {
		m_server.UpdateRegistry(OAT_DISPATCH_OBJECT);
		COleObjectFactory::UpdateRegistryAll();
	}

	// Dispatch commands specified on the command line
	//if (!ProcessShellCommand(cmdInfo))
	//	return FALSE;

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes();

	if( !processCommandLine( m_lpCmdLine ) )
		return FALSE;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	if( bFirstTime )
		m_pMainWnd->OnHelpUsing();  // actually spawn getting started

	return TRUE;
}

BOOL CWilburApp::createMainFrame()
{
	if( m_pMainWnd )
		return FALSE;  // already has one

	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	return TRUE;  // everything seems to have worked
}

BOOL CWilburApp::processCommandLine( CString sCmd )
{
	m_sSearch.Empty();  // make sure the search text is empty
	m_bUseSearchDialog = TRUE; // if -s flag then first search occurs with out dialog

	// simple command line parsing
	if( !sCmd.IsEmpty() && sCmd.GetLength() >= 2 && sCmd[0] == '-' && sCmd[1] == '2' )
	{
		sCmd = sCmd.Mid(2);
		sCmd.TrimLeft();
	}

	m_searchCriteria.recallFromRegistry();

	if( sCmd.IsEmpty() )
	{
		CString sFile = regGetString( HKEY_CURRENT_USER, "Recent File List", "File1", "" );

		if( sFile.IsEmpty() || isAutomated() )
		{
			// create a new (empty) document
			OnFileNew();
		}
		else
		{
			// open last index
			OpenDocumentFile( sFile );
			if( !m_pMainWnd )
				OnFileNew();  // in case of file failure
		}
	}
	else
	{
		sCmd.TrimLeft();
		int nLength = sCmd.GetLength();
		if( nLength > 1 && sCmd[0] == '-' )
		{
			char c = makeLower( sCmd[1] );
			if( c == 'u' || c == 'b' )
			{
				m_bIsBatch = TRUE;

				sCmd = sCmd.Mid(2);
				sCmd.TrimLeft();
				OpenDocumentFile( sCmd );

				if( !m_pMainWnd )
					return FALSE;  // in case of file failure
				else
				{
					ASSERT_VALID( m_pDocument );
					if( c == 'u' )
						m_pDocument->runUpdate();
					else if( c == 'b' )
						m_pDocument->runBuild();

					return TRUE;
				}
			}
			else if( c== 'c' )
			{
				m_bIsBatch = TRUE;

				sCmd = sCmd.Mid(2);
				sCmd.TrimLeft();
				OpenDocumentFile( sCmd );

				if( !m_pMainWnd )
					return FALSE;  // in case of file failure

				ASSERT_VALID( m_pDocument );
				
				changeToIndexDirectory();
				try
				{
					m_pDocument->wordCount( "wilCount.txt" );
				}
				catch( ... ) {}

				return FALSE;
			}
			else
			{
				while( sCmd.GetLength() && sCmd[0] == '-' )
				{
					if( sCmd.GetLength() == 1 )
						return FALSE;

					char c = makeLower( sCmd[1] );
					if( c == 's' )
					{
						m_bUseSearchDialog = FALSE;
						sCmd = sCmd.Mid(2);
						sCmd.TrimLeft();
					}
					else if( c == 'f' )
					{
						CString sFolder;
						sCmd = sCmd.Mid(2);
						sCmd.TrimLeft();
						if( sCmd[0] == '"' || sCmd[0] == '\'' )
						{
							sFolder = sCmd.Mid(1);
							int nMatch = sFolder.Find( sCmd[0] );
							if( nMatch != -1 )
							{
								sCmd = sFolder.Mid( nMatch + 1 );
								sCmd.TrimLeft();
								sFolder = sFolder.Left( nMatch );
							}
						}
						else
						{
							int nSpace = sCmd.Find( ' ' );
							if( nSpace != -1 )
							{
								sFolder = sCmd.Left( nSpace );
								sCmd = sCmd.Mid( nSpace + 1 );
								sCmd.TrimLeft();
							}
							else
							{
								sFolder = sCmd;
								sCmd.Empty();
							}
						}

						m_searchCriteria.setFolderName( sFolder );
					}
				}
			}
		}

		CString sName;

		if( sCmd[0] == '"' || sCmd[0] == '\'' )
		{
			sName = sCmd.Mid(1);
			int nMatch = sName.Find( sCmd[0] );
			if( nMatch != -1 )
			{
				m_sSearch = sName.Mid( nMatch + 1 );
				sName = sName.Left( nMatch );
			}
		}
		else
		{
			int nSpace = sCmd.Find( ' ' );
			if( nSpace != -1 )
			{
				sName = sCmd.Left( nSpace );
				m_sSearch = sCmd.Mid( nSpace + 1 );
			}
			else
				sName = sCmd;
		}

		m_sSearch.TrimLeft();
		if( !m_sSearch.IsEmpty() && m_sSearch[0] == '-' && tolower( m_sSearch[1] ) == 's' )
		{
			m_bUseSearchDialog = FALSE;
			m_sSearch = m_sSearch.Mid(2);
		}

		// open an existing document
		OpenDocumentFile( sName );
		if( !m_pMainWnd )
			OnFileNew();  // in case of file failure
	}

	return TRUE;
}

int CWilburApp::ExitInstance()
{
	// WriteProfileString( "Zip Options", "TempDir", m_sZipPath );
	// WriteProfileString( "Quick View", "Path", m_sQuickViewPath );
	if( !isBatch() && !isAutomated() ) m_searchCriteria.storeInRegistry();
	return CWinApp::ExitInstance();
}

void CWilburApp::setMainFrame( CMainFrame* pFrame )
{
	m_pMainFrame = pFrame;
}

CMainFrame* CWilburApp::getMainFrame()
{
	return m_pMainFrame;
}

void CWilburApp::setStatus( const CString& sStatus )
{
	if( m_pMainFrame )
	{
		m_sStatus = sStatus;

		if( isThreadRunning() )
			getMainFrame()->PostMessage( WM_COMMAND, ID_SET_STATUS, NULL );
		else
			getMainFrame()->setStatus( sStatus );
	}
}

void CWilburApp::setHaltStatus( BOOL bHaltThread )
{
	m_bHaltThread = bHaltThread;
}

BOOL CWilburApp::getHaltStatus()
{
	if( m_bThreadRunning )
		return m_bHaltThread;

	else if( isAutomated() )
		return FALSE;

	else
		return m_pDocument->getHaltStatus();
}

void CWilburApp::setThreadRunning( BOOL bThreadIsRunning )
{
	m_bThreadRunning = bThreadIsRunning;
}

BOOL CWilburApp::isThreadRunning()
{
	return m_bThreadRunning;
}

const CString& CWilburApp::getZipPath()
{
	return m_sZipPath;
}

const CString& CWilburApp::getIndexPath()
{
	return m_sIndexPath;
}

void CWilburApp::setZipPath( const CString& sZipPath )
{
	m_sZipPath = sZipPath;
	int nIndex;
	while( (nIndex = m_sZipPath.Find('/')) != -1 )
		m_sZipPath.SetAt( nIndex, '\\' );

	int nLength = m_sZipPath.GetLength();
	if( nLength && m_sZipPath[ nLength - 1] != '\\' )
		m_sZipPath += '\\';

	regPutString( HKEY_LOCAL_MACHINE, "Zip Options", "TempDir", m_sZipPath );
}

void CWilburApp::cleanUpTempDir()
{
	// delete any temporary files that might be left over
	WIN32_FIND_DATA fileData;

	CString sSearchPath = m_sZipPath + "*.tmp";
	HANDLE hFind = FindFirstFile( (const char*) sSearchPath, &fileData );

	if( hFind != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( !(fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) )
			{
				CString sNewPath = m_sZipPath + fileData.cFileName;
				DeleteFile(sNewPath);
			}
		} while ( FindNextFile( hFind, &fileData ) );

		FindClose( hFind );
	}

	sSearchPath = m_sZipPath + "cmd*.bat";
	hFind = FindFirstFile( (const char*) sSearchPath, &fileData );

	if( hFind != INVALID_HANDLE_VALUE )
	{
		do
		{
			if( !(fileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) )
			{
				CString sNewPath = m_sZipPath + fileData.cFileName;
				DeleteFile(sNewPath);
			}
		} while ( FindNextFile( hFind, &fileData ) );

		FindClose( hFind );
	}

	removeZipDirectories();
}

void CWilburApp::removeZipDirectories()
{
	removeDirectories( m_sZipPath );
}

const CString& CWilburApp::getQuickViewPath()
{
	return m_sQuickViewPath;
}

void CWilburApp::setQuickViewPath( const CString& sQuickViewPath )
{
	m_sQuickViewPath = sQuickViewPath;

	regPutString( HKEY_CURRENT_USER, "Quick View", "Path", m_sQuickViewPath );
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString	m_sBytes;
	CString	m_sFiles;
	CString	m_sWords;
	CString	m_sVersion;
	//}}AFX_DATA

// Implementation
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_sBytes = _T("");
	m_sFiles = _T("");
	m_sWords = _T("");
	m_sVersion = _T("");
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_ABOUT_BYTES, m_sBytes);
	DDX_Text(pDX, IDC_ABOUT_FILES, m_sFiles);
	DDX_Text(pDX, IDC_ABOUT_WORDS, m_sWords);
	DDX_Text(pDX, IDC_VERSION, m_sVersion);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CWilburApp::OnAppAbout()
{
	CAboutDlg aboutDlg;

	aboutDlg.m_sVersion.LoadString( IDS_VERSION );

#ifdef _DEBUG	
	aboutDlg.m_sVersion += " DB";
#endif

	CWilburDoc* pDoc = getDocument();
	if( pDoc )
	{
		aboutDlg.m_sBytes.Format("%d", pDoc->oleGetNumberOfBytes() );
		aboutDlg.m_sWords.Format("%d", pDoc->oleGetNumberOfWords() );
		aboutDlg.m_sFiles.Format("%d", pDoc->oleGetNumberOfFiles() );
	}

	aboutDlg.DoModal();
}

void CWilburApp::OnIndexFinished()
{
	if( isBatch() )
	{
		m_pDocument->DoFileSave();
		getMainFrame()->PostMessage( WM_COMMAND, ID_APP_EXIT );
	}
	else if( !thisApp()->isAutomated() )
		m_pDocument->indexingFinished();
}

void CWilburApp::OnUpdateFinished()
{
	if( !isBatch() )
		m_pDocument->DoFileSave();  // done by OnIndexFinished if not batch

	OnIndexFinished();
}

void CWilburApp::OnSetStatus()
{
	m_pMainFrame->setStatus( m_sStatus );
}

/////////////////////////////////////////////////////////////////////////////
// CWilburApp commands


/////////////////////////////////////////////////////////////////////////////
// Registery functions

BOOL regGetBinary( HKEY hBaseKey, CString sSubKey, CString sVarName, void* pBuffer, DWORD lSize )
{ 
	HKEY hKeySoftware;
	long lResult = RegOpenKeyEx( hBaseKey, "Software", 0, KEY_READ, &hKeySoftware );
	if( lResult == ERROR_SUCCESS )
	{
		HKEY hSubKey;
		sSubKey = "RedTree\\Wilbur\\" + sSubKey;

		lResult = RegOpenKeyEx( hKeySoftware, sSubKey,
		                        0, KEY_READ, &hSubKey );

		if( lResult == ERROR_SUCCESS )
		{
			lResult = RegQueryValueEx( hSubKey, sVarName, NULL, NULL,
			                           (LPBYTE) pBuffer, &lSize );


			RegCloseKey( hSubKey );
		}

		RegCloseKey( hKeySoftware );
	}

	return (lResult == ERROR_SUCCESS );
}

int regGetInt( HKEY hBaseKey, CString sSubKey, CString sVarName, int nDefault )
{ 
	HKEY hKeySoftware;

	long lResult = RegOpenKeyEx( hBaseKey, "Software", 0, KEY_READ, &hKeySoftware );
	if( lResult == ERROR_SUCCESS )
	{
		HKEY hSubKey;
		sSubKey = "RedTree\\Wilbur\\" + sSubKey;

		lResult = RegOpenKeyEx( hKeySoftware, sSubKey,
		                        0, KEY_READ, &hSubKey );

		if( lResult == ERROR_SUCCESS )
		{
			unsigned long nSize = sizeof( DWORD );
			int nReturn;

			lResult = RegQueryValueEx( hSubKey, sVarName, NULL, NULL,
			                           (LPBYTE) &nReturn, &nSize );

			if( lResult == ERROR_SUCCESS )
				nDefault = nReturn;

			RegCloseKey( hSubKey );
		}

		RegCloseKey( hKeySoftware );
	}

	return nDefault;
}

CString regGetString( HKEY hBaseKey, CString sSubKey, CString sVarName, CString sDefault )
{ 
	HKEY hKeySoftware;

	long lResult = RegOpenKeyEx( hBaseKey, "Software", 0, KEY_READ, &hKeySoftware );
	if( lResult == ERROR_SUCCESS )
	{
		HKEY hSubKey;
		sSubKey = "RedTree\\Wilbur\\" + sSubKey;

		lResult = RegOpenKeyEx( hKeySoftware, sSubKey,
		                        0, KEY_READ, &hSubKey );

		if( lResult == ERROR_SUCCESS )
		{
			unsigned long nSize;

			lResult = RegQueryValueEx( hSubKey, sVarName, NULL, NULL,
			                           NULL, &nSize );

			if( lResult == ERROR_SUCCESS )
			{
				CString sVar;
				lResult = RegQueryValueEx( hSubKey, sVarName, NULL, NULL,
										   (LPBYTE) sVar.GetBuffer( nSize ), &nSize );
				sVar.ReleaseBuffer();

				if( lResult == ERROR_SUCCESS )
					sDefault = sVar;
			}

			RegCloseKey( hSubKey );
		}

		RegCloseKey( hKeySoftware );
	}

	return sDefault;
}

BOOL regPutBinary( HKEY hBaseKey, CString sSubKey, CString sVarName, void* pBuffer, DWORD lSize )
{ 
	HKEY hKeySoftware;
	long lResult = RegOpenKeyEx( hBaseKey, "Software", 0, KEY_READ, &hKeySoftware );
	if( lResult == ERROR_SUCCESS )
	{
		HKEY hSubKey;
		DWORD dwDisp;

		sSubKey = "RedTree\\Wilbur\\" + sSubKey;

		lResult = RegCreateKeyEx( hKeySoftware, sSubKey,
		                        0, "Per-User Data", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
		                        NULL, &hSubKey, &dwDisp );

		if( lResult == ERROR_SUCCESS )
		{
			lResult = RegSetValueEx( hSubKey, sVarName, 0, REG_BINARY,
			                           (LPBYTE) pBuffer, lSize );


			RegCloseKey( hSubKey );
		}

		RegCloseKey( hKeySoftware );
	}

	return (lResult == ERROR_SUCCESS );
}

BOOL regPutInt( HKEY hBaseKey, CString sSubKey, CString sVarName, int nVar )
{ 
	HKEY hKeySoftware;
	long lResult = RegOpenKeyEx( hBaseKey, "Software", 0, KEY_READ, &hKeySoftware );
	if( lResult == ERROR_SUCCESS )
	{
		HKEY hSubKey;
		DWORD dwDisp;

		sSubKey = "RedTree\\Wilbur\\" + sSubKey;
		lResult = RegCreateKeyEx( hKeySoftware, sSubKey,
		                        0, "Per-User Data", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
		                        NULL, &hSubKey, &dwDisp );

		if( lResult == ERROR_SUCCESS )
		{
			lResult = RegSetValueEx( hSubKey, sVarName, 0, REG_DWORD,
			                           (LPBYTE) &nVar, sizeof( DWORD ) );


			RegCloseKey( hSubKey );
		}

		RegCloseKey( hKeySoftware );
	}

	return (lResult == ERROR_SUCCESS );
}

BOOL regPutString( HKEY hBaseKey, CString sSubKey, CString sVarName, CString sVar )
{
	HKEY hKeySoftware;
	long lResult = RegOpenKeyEx( hBaseKey, "Software", 0, KEY_READ, &hKeySoftware );
	if( lResult == ERROR_SUCCESS )
	{
		HKEY hSubKey;
		DWORD dwDisp;

		sSubKey = "RedTree\\Wilbur\\" + sSubKey;

		lResult = RegCreateKeyEx( hKeySoftware, sSubKey,
		                        0, "Per-User Data", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
		                        NULL, &hSubKey, &dwDisp );

		if( lResult == ERROR_SUCCESS )
		{
			lResult = RegSetValueEx( hSubKey, sVarName, 0, REG_SZ,
			                           (LPBYTE) ( (LPCSTR) sVar), sVar.GetLength() + 1 );


			RegCloseKey( hSubKey );
		}

		RegCloseKey( hKeySoftware );
	}

	return (lResult == ERROR_SUCCESS );
}

CString regGetFixString( CString sSubKey, CString sVarName, CString sDefault )
{
	CString sReturn = regGetString( HKEY_LOCAL_MACHINE, sSubKey, sVarName, "" );
	if( sReturn.IsEmpty() )
	{
		sReturn = regGetString( HKEY_CURRENT_USER,  sSubKey, sVarName, sDefault );
		regPutString( HKEY_LOCAL_MACHINE,  sSubKey, sVarName, sReturn );
	}

	return sReturn;
}

int regGetFixInt( CString sSubKey, CString sVarName, int nDefault )
{
	int nReturn = regGetInt( HKEY_LOCAL_MACHINE, sSubKey, sVarName, -32767 );
	if( nReturn == -32767 )
	{
		nReturn = regGetInt( HKEY_CURRENT_USER,  sSubKey, sVarName, nDefault );
		regPutInt( HKEY_LOCAL_MACHINE,  sSubKey, sVarName, nReturn );
	}

	return nReturn;
}

void CWilburApp::OnIndexStop() 
{
	m_bHaltThread = TRUE;
}

void CWilburApp::OnUpdateIndexStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( m_bThreadRunning );	
}


void CWilburApp::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( !m_bThreadRunning );	
}

void CWilburApp::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( !m_bThreadRunning );
}

void CWilburApp::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( !m_bThreadRunning );	
}

void CWilburApp::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( !m_bThreadRunning );	
}


void CWilburApp::OnFilePreferences() 
{
	CPreferencesDialog dlg;
	
	dlg.m_sEditor = regGetString( HKEY_CURRENT_USER, "options", "editor", "notepad.exe" );
	dlg.m_sZipPath = m_sZipPath;
	dlg.m_nNear = m_pDocument->getNear();
	dlg.m_nTabSize = regGetInt( HKEY_CURRENT_USER, "Contents View", "TabWidth", 4 );
	dlg.m_nWordWrap = regGetInt( HKEY_CURRENT_USER, "Contents View", "WordWrap", 80 );
	dlg.m_sIndexPath = m_sIndexPath;
	int nOutline = regGetInt( HKEY_CURRENT_USER,"Contents View", "Outline", 1 );
	dlg.m_nFileHistory = regGetInt( HKEY_CURRENT_USER, "options", "Recent File Count", 16 );
	dlg.m_nMaxBuffer = regGetInt( HKEY_CURRENT_USER, "Contents View", "MaxBuffer", 1000000 );
	dlg.m_bCachePDF = regGetInt( HKEY_CURRENT_USER, "options", "Cache PDF", 0 );
	dlg.m_spdftotextArgs = regGetString( HKEY_CURRENT_USER, "options", "pdftotextArgs", "");
	dlg.m_bOutline = (nOutline != 0);
	
	if( regGetInt( HKEY_CURRENT_USER, "List View", "Orientation", 1) )
	{
		dlg.m_nVertical = -1;
		dlg.m_nHorizontal = 0;
	}
	else
	{
		dlg.m_nVertical = 0;
		dlg.m_nHorizontal = -1;
	}
	
	if( dlg.DoModal() == IDOK )
	{
		setZipPath( dlg.m_sZipPath );

		m_sIndexPath = dlg.m_sIndexPath;
		regPutString( HKEY_CURRENT_USER, "options", "IndexPath", m_sIndexPath );

		regPutString( HKEY_CURRENT_USER, "options", "editor", dlg.m_sEditor );

		regPutInt( HKEY_CURRENT_USER, "Contents View", "TabWidth", dlg.m_nTabSize );
		m_pDocument->setTabSize( dlg.m_nTabSize );

		regPutInt( HKEY_CURRENT_USER, "Contents View", "WordWrap", dlg.m_nWordWrap );
		m_pDocument->setWordWrap( dlg.m_nWordWrap );

		m_pDocument->setNear( dlg.m_nNear );

		regPutInt( HKEY_CURRENT_USER, "Contents View", "Outline", dlg.m_bOutline );
		regPutInt( HKEY_CURRENT_USER, "Contents View", "MaxBuffer", dlg.m_nMaxBuffer );
		regPutInt( HKEY_CURRENT_USER, "options", "Recent File Count", dlg.m_nFileHistory );
		regPutInt( HKEY_CURRENT_USER, "options", "Cache PDF", dlg.m_bCachePDF );
		regPutString( HKEY_CURRENT_USER, "options", "pdftotextArgs", dlg.m_spdftotextArgs );
		regPutInt( HKEY_CURRENT_USER, "List View", "Orientation", dlg.m_nHorizontal + 1);
		m_sPDFargs = dlg.m_spdftotextArgs;
	}
}

CDocument* CWilburApp::OpenDocumentFile(LPCTSTR lpszFileName) 
{
	CDocument* pDoc = CWinApp::OpenDocumentFile(lpszFileName);
	if( *m_pszCurrentDir )
		SetCurrentDirectory( m_pszCurrentDir );

	if( !isBatch() && !thisApp()->isAutomated() && m_pMainWnd )
	{
		m_sSearch.TrimLeft();
		if( !m_sSearch.IsEmpty() )
			m_pDocument->setSearchText( m_sSearch );

		m_sSearch.Empty();  // make sure it isn't used again

		m_pDocument->doSearch( m_bUseSearchDialog );
		m_bUseSearchDialog = TRUE;
	}
	
	return pDoc;
}

BOOL CWilburApp::InitApplication() 
{
	SetRegistryKey( "RedTree" );

	CString sCmd = m_lpCmdLine;
	sCmd.TrimLeft();

	if( sCmd.IsEmpty() || sCmd.GetLength() < 2 || sCmd[0] != '-' || sCmd[1] != '2' )
	{
		m_bCanReplace = TRUE;

		HWND      hwnd;

		// Win32 will always set hPrevInstance to NULL, so lets check
		// things a little closer. This is because we only want a single
		// version of this app to run at a time
		hwnd = FindWindow (szWilburClass, NULL);
		if (hwnd)
		{
			// We found another version of ourself. Lets defer to it:
			if (IsIconic(hwnd))
			{
				ShowWindow(hwnd, SW_RESTORE);
			}
			SetForegroundWindow (hwnd);

			// If this app actually had any functionality, we would
			// also want to communicate any action that our 'twin'
			// should now perform based on how the user tried to
			// execute us.

			// stash cmdline in registry as a quick transfer mechanism
			regPutString( HKEY_CURRENT_USER, "Settings", "CmdLine", m_lpCmdLine );
			::SendMessage( hwnd, WM_COMMAND, ID_SECOND_INVOKE, NULL );
			return FALSE;
		}
	}
	else
		m_bCanReplace = FALSE;

	return CWinApp::InitApplication();
}

void CWilburApp::OnSecondInvocation()
{
	CString sCmd = regGetString( HKEY_CURRENT_USER, "Settings", "CmdLine", "" );
	regPutString( HKEY_CURRENT_USER, "Settings", "CmdLine", "" );

	if( !isBatch() && !isThreadRunning() && !sCmd.IsEmpty() )
		processCommandLine( sCmd );
}

void CWilburApp::changeToIndexDirectory()
{
	if( !SetCurrentDirectory( m_sIndexPath ) )
	{
		if( CreateDirectory( m_sIndexPath, NULL ) )
			SetCurrentDirectory( m_sIndexPath );
		else
		{
			CString sMsg;
			sMsg.LoadString( IDS_CREATE_INDEX_FOLDER_ERR );
			simpleMessageBox(sMsg + m_sIndexPath );
		}
	}
}

void CWilburApp::OnFileOpen() 
{
	GetCurrentDirectory( MAX_PATH, m_pszCurrentDir );

	changeToIndexDirectory();
	CWinApp::OnFileOpen();
	*m_pszCurrentDir = '\0';
}

void CWilburApp::fillFileList( CComboBox& cbIndexes )
{
	cbIndexes.AddString( "" );
	int nFiles = m_pRecentFileList->GetSize();

	for( int i = 0; i < nFiles; i++ )
	{
		CString sFile = m_pRecentFileList->operator[]( i );
		if( !sFile.IsEmpty() && cbIndexes.FindString( -1, sFile ) == CB_ERR )
			cbIndexes.AddString( sFile );
	}
}

void CWilburApp::searchNewIndex( CString sIndex, CString sSearchText )
{
	m_sSearch = sSearchText;
	m_bUseSearchDialog = FALSE;
	OpenDocumentFile(sIndex);
}

int simpleMessageBox( LPCTSTR lpszText, UINT nType, UINT nIDHelp )
{
	if( thisApp()->isAutomated() )
	{
		thisApp()->getDocument()->oleAddMessage( lpszText );
		return IDNO;
	}
	else
		return AfxMessageBox( lpszText, nType, nIDHelp );
}

int simpleMessageBox( UINT nIDPrompt, UINT nType, UINT nIDHelp )
{
	if( thisApp()->isAutomated() )
	{
		CString sPrompt;
		sPrompt.LoadString( nIDPrompt );
		thisApp()->getDocument()->oleAddMessage( sPrompt );
		return IDNO;
	}
	else
		return AfxMessageBox( nIDPrompt, nType, nIDHelp );
}

char lowerCaseTable[256];

void mapLowerCaseChars()
{
	// make standard lower case
	for(int i = -128; i < 127; i++)
		lowerCaseTable[i + 128] = i;
	for(i = 65; i <= 90; i++)
		lowerCaseTable[i + 128] = i + 32;
	for(i = -63; i <= -34; i++)
		lowerCaseTable[i + 128] = i + 32;
	for(i = -118; i <= -113; i++)
		lowerCaseTable[i+128] = i + 16;
	lowerCaseTable[128 - 93] = -77;
	lowerCaseTable[128 - 81] = -65;
	lowerCaseTable[128 - 91] = -71;
}

CString addedLangChars( DWORD dwInternational )
{
	CString sReturn;
	if( dwInternational & CHAR_DANISH )
		sReturn += "";

	if( dwInternational & CHAR_FINNISH )
		sReturn += "";

	if( dwInternational & CHAR_FRENCH )
		sReturn += "";
		sReturn += char(0x9c);

	if( dwInternational & CHAR_GERMAN )
		sReturn += "";

	if( dwInternational & CHAR_SPANISH )
		sReturn += "";

	if( dwInternational & CHAR_SWEDISH )
		sReturn += "";

	if( dwInternational & CHAR_POLISH ) {
		sReturn += "柿";
		sReturn += "ʌƏ";
	}

	return sReturn;
}

CString makeLower( const CString& sText )
{
	CString sLower = sText;

	int nLength = sLower.GetLength();
	for( int i = 0; i < nLength ; i++ )
		sLower.SetAt( i, makeLower( sLower[ i ] ) );

	return sLower;
}


