/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// wilbur.h : main header file for the WILBUR application
//

#ifndef __WILBUR_H__
#define __WILBUR_H__

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols

#ifndef __AFXTEMPL_H__
#include "afxtempl.h"
#endif

#if !defined(AFX_SEARCHCRITERIA_H__8C5A2193_ED0C_11D1_B2D4_006008A8E2D0__INCLUDED_)
#include "searchcriteria.h"
#endif

/////////////////////////////////////////////////////////////////////////////
// CWilburApp:
// See wilbur.cpp for the implementation of this class
//

class CMainFrame;
class CWilburDoc;

class CWilburApp : public CWinApp
{
	public:
		CWilburApp();

 		void setStatus( const CString& sStatus );

		void setMainFrame( CMainFrame* pFrame );
		CMainFrame* getMainFrame();
		BOOL createMainFrame();

		void setDocument( CWilburDoc* pDoc ) { m_pDocument = pDoc; }
		CWilburDoc* getDocument() { return m_pDocument; }

		void setHaltStatus( BOOL bHalt );
		BOOL getHaltStatus();

		void setThreadRunning( BOOL bThreadIsRunning );
		BOOL isThreadRunning();

		const CString& getZipPath();
		void  setZipPath( const CString& sZipPath );
		void  cleanUpTempDir();
		void  removeZipDirectories();

		const CString& getIndexPath();

		const CString& getQuickViewPath();
		void  setQuickViewPath( const CString& sQuickViewPath );

		BOOL isBatch()     { return m_bIsBatch; }
		BOOL isAutomated() { return m_bRunAutomated; }
		void setAutomated( BOOL bValue ) { m_bRunAutomated = bValue; }
		BOOL canReplace() { return m_bCanReplace; }  // can this instance be usurped
		void changeToIndexDirectory();

		bool hasPDFtoText() { return m_bPDF; }

		BOOL processCommandLine( CString sCmd );

		CString startDirectory() { return m_sStartDirectory; }

		CSearchCriteria& searchCriteria() { return m_searchCriteria; }

		void fillFileList( CComboBox& cbIndexes );
		void searchNewIndex( CString sIndex, CString sSearchText );

		CString translatePDF( const CString sPath, bool bIsIndexing );
		bool usePDFCache() { return m_bUsePDFCache; }
		CString pdfCachePath() { return m_sPDFPath; }
		void makeNewPDFCache();
		void savePDFCache();
		void openPDFCache();
		void closePDFCache();
		void deletePDFCache(CString sPDFPath);

	// Overrides
		// ClassWizard generated virtual function overrides
		//{{AFX_VIRTUAL(CWilburApp)
	public:
		virtual BOOL InitInstance();
		virtual int  ExitInstance();
	virtual CDocument* OpenDocumentFile(LPCTSTR lpszFileName);
	virtual BOOL InitApplication();
	//}}AFX_VIRTUAL

	// Implementation
		COleTemplateServer m_server;
			// Server object for document creation

		//{{AFX_MSG(CWilburApp)
		afx_msg void OnAppAbout();
		afx_msg void OnIndexFinished();
		afx_msg void OnUpdateFinished();
		afx_msg void OnSetStatus();
		afx_msg void OnIndexStop();
		afx_msg void OnSecondInvocation();
		afx_msg void OnUpdateIndexStop(CCmdUI* pCmdUI);
		afx_msg void OnUpdateFileNew(CCmdUI* pCmdUI);
		afx_msg void OnUpdateFileOpen(CCmdUI* pCmdUI);
		afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
		afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
		afx_msg void OnFilePreferences();
	afx_msg void OnFileOpen();
	//}}AFX_MSG
		DECLARE_MESSAGE_MAP()

	private:
		CMainFrame* m_pMainFrame;
		CWilburDoc* m_pDocument;
		CSearchCriteria m_searchCriteria;
		CString m_sZipPath;
		CString m_sStatus;
		CString m_sSearch;  // from command line
		CString m_sQuickViewPath;

		CString m_sStartDirectory;
		CString m_sIndexPath;
		char    m_pszCurrentDir[ MAX_PATH ];


		BOOL m_bThreadRunning;
		BOOL m_bHaltThread;
		BOOL m_bIsBatch;      // TRUE when command line calls for batch build
		BOOL m_bRunAutomated; // TRUE when created with OLE automation
		BOOL m_bCanReplace;   // FALSE when command line calls for seperate invocation
		BOOL m_bUseSearchDialog; // FALSE when command line calls for immediate search


		bool m_bPDF;   // true if pdftotext program is available
		bool m_bUsePDFCache;
		bool m_bTemporaryPDFCache;
		CString m_sPDFPath;  // if not blank, use this as cache directory for PDFs
							 // NOTE - everything in this path will be deleted when a new index is built!!!
		CString m_sPDFargs;  // extra arguments to pass to pdftotext
};

CWilburApp* thisApp();

/////////////////////////////////////////////////////////////////////////////
// Registery functions

BOOL regGetBinary( HKEY hBaseKey, CString sSubKey, CString sVarName, void* pBuffer, DWORD lSize );
int  regGetInt( HKEY hBaseKey, CString sSubKey, CString sVarName, int nDefault );
CString regGetString( HKEY hBaseKey, CString sSubKey, CString sVarName, CString sDefault );
BOOL regPutBinary( HKEY hBaseKey, CString sSubKey, CString sVarName, void* pBuffer, DWORD lSize );
BOOL regPutInt( HKEY hBaseKey, CString sSubKey, CString sVarName, int nVar );
BOOL regPutString( HKEY hBaseKey, CString sSubKey, CString sVarName, CString sVar );

CString regGetFixString( CString sSubKey, CString sVarName, CString sDefault );
int regGetFixInt( CString sSubKey, CString sVarName, int nDefault );

/////////////////////////////////////////////////////////////////////////////
// General utilities

int simpleMessageBox( LPCTSTR lpszText, UINT nType = MB_OK, UINT nIDHelp = 0 );
int simpleMessageBox( UINT nIDPrompt, UINT nType = MB_OK, UINT nIDHelp = 0 );

extern char lowerCaseTable[256];

inline int makeLower( int c)
{
	return lowerCaseTable[128 + c];
}

/*
inline int makeLower( int c )
{
	if( ( c >= 'A' && c <= 'Z' ) || ( c >= -63 && c <= -34 ) )
		c += 32;
	else if( (c >= -118 && C <= -113) || c == -93 || c == -81)
		c += 16;
	else if( c == -91 )
		c = -71;

	return c;
}
*/

void mapLowerCaseChars();
CString makeLower( const CString& sText );

#define CHAR_ENGLISH 0
#define CHAR_ALL     1
#define CHAR_DANISH  2
#define CHAR_FINNISH 4
#define CHAR_FRENCH  8
#define CHAR_GERMAN  16
#define CHAR_SPANISH 32
#define CHAR_SWEDISH 64
#define CHAR_POLISH 128

CString addedLangChars( DWORD dwInternational );

#endif
