/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// WilburResults.cpp : implementation file
//

#include "stdafx.h"
#include "wilbur.h"

#ifndef __FILEINDX_H__
#include "fileindx.h"
#endif

#ifndef __WILBURRESULTS_H__
#include "WilburResults.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWilburResults

IMPLEMENT_DYNCREATE(CWilburResults, CCmdTarget)

CWilburResults::CWilburResults()
{
	m_pResults = NULL;
	m_pDoc = NULL;
	m_position = NULL;
	m_pContents = NULL;

	EnableAutomation();
}

CWilburResults::~CWilburResults()
{
	if( m_pResults )
		delete m_pResults;

	if( m_pContents )
		delete m_pContents;
}

void CWilburResults::setSearchResults( CSearchResults* pResults, CWilburDoc* pDoc )
{
	if( m_pResults )
		delete m_pResults;

	m_pResults = pResults;
	m_pDoc = pDoc;

	m_pContents = new CAutoContents( pDoc );
	m_sSearchText = pDoc->getSearchText();
	m_pContents->parseSearchText( m_sSearchText );
	m_bUseSoundex = pDoc->isSoundex() != 0;
}

void CWilburResults::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CCmdTarget::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(CWilburResults, CCmdTarget)
	//{{AFX_MSG_MAP(CWilburResults)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CWilburResults, CCmdTarget)
	//{{AFX_DISPATCH_MAP(CWilburResults)
	DISP_FUNCTION(CWilburResults, "setToStart", setToStart, VT_BOOL, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "setToNext", setToNext, VT_BOOL, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "getPath", getPath, VT_BSTR, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "getModifiedDate", getModifiedDate, VT_DATE, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "getRank", getRank, VT_I4, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "isNearOK", isNearOK, VT_BOOL, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "getCount", getCount, VT_I4, VTS_NONE)
	DISP_FUNCTION(CWilburResults, "getContents", getContents, VT_BSTR, VTS_BOOL VTS_I4)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IWilburResults to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {2438A580-FF5C-11CF-9B6F-00006E23CF57}
static const IID IID_IWilburResults =
{ 0x2438a580, 0xff5c, 0x11cf, { 0x9b, 0x6f, 0x0, 0x0, 0x6e, 0x23, 0xcf, 0x57 } };

BEGIN_INTERFACE_MAP(CWilburResults, CCmdTarget)
	INTERFACE_PART(CWilburResults, IID_IWilburResults, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWilburResults message handlers

BOOL CWilburResults::setToStart() 
{
	if( m_pResults )
	{
		m_position = m_pResults->GetStartPosition();
		if( m_position == NULL )
			return FALSE;

		m_pResults->GetNextAssoc( m_position, m_pLastFile, m_lastResult );

		return TRUE;
	}
	else
		return FALSE;
}


BOOL CWilburResults::setToNext() 
{
	if( m_pResults && m_position )
	{
		m_pResults->GetNextAssoc( m_position, m_pLastFile, m_lastResult );

		return TRUE;
	}
	else
		return FALSE;

}

BSTR CWilburResults::getPath() 
{
	CString strResult;

	if( m_pResults && m_pDoc )
	{
		CFileInfo* pDir = m_pDoc->getFileInfo( m_pLastFile->m_nParent ); 

		if( pDir && pDir->m_nParent )
		{
			strResult = pDir->m_pszName;
			strResult += m_pLastFile->m_pszName;
		}
		else
			strResult = m_pLastFile->m_pszName;
       
	}

	return strResult.AllocSysString();
}

DATE CWilburResults::getModifiedDate() 
{
	if( m_pResults )
	{
		COleDateTime timeMod( m_pLastFile->m_timeLastWritten.GetTime() );
		return timeMod;
	}

	return (DATE) 0;
}

long CWilburResults::getRank() 
{
	if( m_pResults )
	{
		return m_lastResult.m_nRank;
	}

	return 0;
}


BOOL CWilburResults::isNearOK() 
{
	thisApp()->setAutomated( TRUE );

	BOOL bResult = TRUE;
	if( m_pResults && m_sSearchText.FindOneOf("<>:") != -1 )
	{
		ASSERT( m_pContents );

		CString sPath;
		if( m_lastResult.m_pDirInfo )
			sPath = m_lastResult.m_pDirInfo->m_pszName;

		sPath += m_pLastFile->m_pszName;

		bResult = m_pContents->containsHit( sPath, m_sSearchText, m_bUseSoundex );
	}

	thisApp()->setAutomated( FALSE );
	return bResult;
}

long CWilburResults::getCount() 
{
	if( m_pResults )
		return m_pResults->GetCount();
	else
		return 0;
}

BSTR CWilburResults::getContents(BOOL bOutline, long nMaxChars) 
{
	thisApp()->setAutomated( TRUE );

	CString strResult;
	BOOL bResult = TRUE;
	if( m_pResults )
	{
		ASSERT( m_pContents );

		CString sPath;
		if( m_lastResult.m_pDirInfo )
			sPath = m_lastResult.m_pDirInfo->m_pszName;

		sPath += m_pLastFile->m_pszName;

		strResult = m_pContents->getContentsOf( sPath, bOutline, nMaxChars );

		strResult = strResult.Left( nMaxChars );
	}

	thisApp()->setAutomated( FALSE );
	return strResult.AllocSysString();
}
