/*  Wilbur - file indexing and search utility - WilburUtil launcher and scheduler
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// ScheduleListDialog.cpp : implementation file
//

#include "stdafx.h"
#include "wilburutil.h"

#if !defined(AFX_SCHEDULELISTDIALOG_H__4863C252_F97C_11D1_B2D9_006008A8E2D0__INCLUDED_)
#include "ScheduleListDialog.h"
#endif

#if !defined(AFX_SCHEDULEDIALOG_H__4863C253_F97C_11D1_B2D9_006008A8E2D0__INCLUDED_)
#include "ScheduleDialog.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScheduleListDialog dialog


CScheduleListDialog::CScheduleListDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CScheduleListDialog::IDD, pParent), m_pScheduleList( NULL )
{
	//{{AFX_DATA_INIT(CScheduleListDialog)
	//}}AFX_DATA_INIT
}


void CScheduleListDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CScheduleListDialog)
	DDX_Control(pDX, IDC_SCHEDULE_LIST, m_scheduleListBox);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CScheduleListDialog, CDialog)
	//{{AFX_MSG_MAP(CScheduleListDialog)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	ON_BN_CLICKED(IDC_EDIT, OnEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScheduleListDialog message handlers

void CScheduleListDialog::OnAdd() 
{
	ASSERT_VALID( m_pScheduleList );

	CScheduleDialog dlg;
	dlg.m_sStartTime = "03:00";
	dlg.m_nInterval = 1;
	dlg.m_nUnit = 0;
	if( dlg.DoModal() == IDOK )
	{
		COleDateTime timeStart;
		while( !timeStart.ParseDateTime( dlg.m_sStartTime ) )
		{
			AfxMessageBox("Invalid start time!");
			if( dlg.DoModal() != IDOK )
				return;
		}

		CSchedule* pSchedule = new CSchedule(
								dlg.m_sIndex,
								timeStart,
								dlg.m_nInterval,
								(CSchedule::IntervalType) (dlg.m_nUnit + 1),
								(dlg.m_bUpdate == TRUE),
								TRUE 
							);

		addToListBox( pSchedule );

		UpdateData( FALSE );
	}
}

void CScheduleListDialog::addToListBox( CSchedule* pSchedule )
{
	ASSERT_VALID( pSchedule );
	CString sItem = pSchedule->getNextStart().Format();
	sItem += '\t';

	sItem += pSchedule->getIndexPath();

	int nItem = m_scheduleListBox.AddString( sItem );

	if( nItem != LB_ERR && nItem != LB_ERRSPACE )
		m_scheduleListBox.SetItemDataPtr( nItem, pSchedule );
	else
		delete pSchedule;
}

void CScheduleListDialog::processList( bool bAdd )
{
	int nCount = m_scheduleListBox.GetCount();

	for( int nItem = 0; nItem < nCount; nItem++ )
	{
		CSchedule* pSchedule = (CSchedule*) m_scheduleListBox.GetItemDataPtr( nItem );
		if( (int) pSchedule != -1 )
		{
			ASSERT_VALID( pSchedule );
			if( bAdd )
				m_pScheduleList->AddTail( *pSchedule );

			delete pSchedule;
		}
	}
}

BOOL CScheduleListDialog::OnInitDialog()
{
	CDialog::OnInitDialog(); 
	m_scheduleListBox.SetTabStops( 72 );

	ASSERT_VALID( m_pScheduleList );

	POSITION pos = m_pScheduleList->GetHeadPosition();
	while( pos )
	{
		CSchedule* pSchedule = new CSchedule;
		*pSchedule = m_pScheduleList->GetNext( pos );
		addToListBox( pSchedule );
	}

	return TRUE;
}

void CScheduleListDialog::OnOK() 
{
	m_pScheduleList->RemoveAll();
	processList( true );
	
	CDialog::OnOK();
}

void CScheduleListDialog::OnCancel() 
{
	processList( false );
	
	CDialog::OnCancel();
}


void CScheduleListDialog::OnRemove() 
{
	UpdateData();
	int nCount = m_scheduleListBox.GetCount();
	for( int nItem = nCount - 1; nItem >= 0; nItem-- )
	{
		// reverse order so index numbers don't get screwed up
		if( m_scheduleListBox.GetSel( nItem ) > 0 )
		{	
			CSchedule* pSchedule = (CSchedule*) m_scheduleListBox.GetItemDataPtr( nItem );
			if( (int) pSchedule != -1 )
			{
				ASSERT_VALID( pSchedule );
				delete pSchedule;
			}
			
			m_scheduleListBox.DeleteString( nItem );
		}
	}

	UpdateData( FALSE );
}

void CScheduleListDialog::OnEdit() 
{
	int nItem = m_scheduleListBox.GetCaretIndex();

	if( nItem < 0 )
		return;

	CSchedule* pSchedule = (CSchedule*) m_scheduleListBox.GetItemDataPtr( nItem );
	if( (int) pSchedule == -1 )
		return;

	CScheduleDialog dlg;
	dlg.m_sStartTime = pSchedule->getStartTime().Format();
	dlg.m_nInterval = pSchedule->getInterval();
	dlg.m_nUnit = (int) pSchedule->getUnit() - 1;
	dlg.m_sIndex = pSchedule->getIndexPath();
	dlg.m_bUpdate = pSchedule->isUpdate();

	if( dlg.DoModal() == IDOK )
	{
		delete pSchedule;  // remove existing entry
		pSchedule = NULL;
		m_scheduleListBox.DeleteString( nItem );

		COleDateTime timeStart;
		while( !timeStart.ParseDateTime( dlg.m_sStartTime ) )
		{
			AfxMessageBox("Invalid start time!");
			if( dlg.DoModal() != IDOK )
				return;
		}

		pSchedule = new CSchedule(
						dlg.m_sIndex,
						timeStart,
						dlg.m_nInterval,
						(CSchedule::IntervalType) (dlg.m_nUnit + 1),
						(dlg.m_bUpdate == TRUE),
						TRUE
					);

		addToListBox( pSchedule );

		UpdateData( FALSE );
	}
}
