/*  Wilbur - file indexing and search utility - WilburUtil launcher and scheduler
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// Icon Frame.cpp : implementation file
//
// Icon Frame core design by Andy Stacy 1997.
// This Code may be used freely and modified as required providing a reference to the author is left
// in the file header.
//

#include "stdafx.h"

#if !defined(AFX_WILBURUTIL_H__B0D554F4_F5D3_11D1_B2D8_006008A8E2D0__INCLUDED_)
#include "WilburUtil.h"
#endif

#if !defined(__ICONFRAME_H__)
#include "IconFrame.h"
#endif

#if !defined(AFX_SCHEDULELISTDIALOG_H__4863C252_F97C_11D1_B2D9_006008A8E2D0__INCLUDED_)
#include "ScheduleListDialog.h"
#endif

#if !defined(AFX_SCHEDULEDIALOG_H__4863C253_F97C_11D1_B2D9_006008A8E2D0__INCLUDED_)
#include "ScheduleDialog.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////

// CIconFrame

IMPLEMENT_DYNCREATE(CIconFrame, CFrameWnd)

CIconFrame::CIconFrame()
{
	// Frame creation, you'll never see it so ...
	Create(NULL, _T("IconFrame - You should never see this ..."),
		WS_OVERLAPPEDWINDOW, rectDefault);

	m_idTimer = 0;

	CFile scheduleFile;
	if( scheduleFile.Open( "schedule.dat", CFile::modeRead ) )
	{
		try
		{
			CArchive archive( &scheduleFile, CArchive::load );

			int nRel;
			archive >> nRel;
			if( nRel < 1 || nRel > 1 )
				return;

			int nCount;
			archive >> nCount;

			while( nCount-- )
			{
				CSchedule sched;
				sched.Serialize( archive );
				m_scheduleList.AddTail( sched );
			}

			setUpTimer();
		}
		catch( ... )
		{}
	}
}

CIconFrame::~CIconFrame()
{
}

BEGIN_MESSAGE_MAP(CIconFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CIconFrame)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_MESSAGE(ICON_CALLBACK,IconCallback)
	ON_COMMAND(CLOSE_MESSAGE,IconMenuClose)
	ON_COMMAND(IDCANCEL, IconClosePopup)
	ON_COMMAND(LAUNCH_WILBUR, launchWilbur)
	ON_COMMAND(LAUNCH_WILBUR2, launch2ndWilbur)
	ON_COMMAND(SCHEDULE_DIALOG, scheduleDialog)
	ON_COMMAND(LAUNCH_NOTE, launchNote)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////

// CIconFrame message handlers

BOOL CIconFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Change the extended style so that this app does not
	// appear on the TaskBar ...
	cs.dwExStyle = cs.dwExStyle | WS_EX_TOOLWINDOW;

	return CFrameWnd::PreCreateWindow(cs);
}

BOOL CIconFrame::DestroyWindow()
{
	// End of the Apps life so delete the icon from the system tray...
	if( m_idTimer )
		KillTimer( m_idTimer );

	IconMessage(NIM_DELETE);
	delete m_pIconPopup;

	return CFrameWnd::DestroyWindow();
}

	BOOL CIconFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext *pContext)
{
	// Load any icons to be displayed in the system tray
	m_okicon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	// Create the pop-up menu
	m_pIconPopup = new CMenu();
	m_pIconPopup->CreatePopupMenu();

	static CString sNote;
	sNote.LoadString(IDS_NOTE);
	m_pIconPopup->AppendMenu(MF_STRING|MF_ENABLED, LAUNCH_NOTE, sNote);

	static CString sWilbur;
	sWilbur.LoadString(IDS_WILBUR);
	m_pIconPopup->AppendMenu(MF_STRING|MF_ENABLED, LAUNCH_WILBUR, sWilbur);

	static CString sWilbur2;
	sWilbur2.LoadString(IDS_WILBUR2);
	m_pIconPopup->AppendMenu(MF_STRING|MF_ENABLED, LAUNCH_WILBUR2, sWilbur2);

	static CString sSchedule;
	sSchedule.LoadString(IDS_SCHEDULE);
	m_pIconPopup->AppendMenu(MF_STRING|MF_ENABLED, SCHEDULE_DIALOG, sSchedule );

	static CString sExit;
	sExit.LoadString(IDS_EXIT);
	m_pIconPopup->AppendMenu(MF_STRING|MF_ENABLED, CLOSE_MESSAGE, sExit);

	static CString sCancel;
	sCancel.LoadString(IDS_CANCEL);
	m_pIconPopup->AppendMenu(MF_STRING|MF_ENABLED, IDCANCEL, sCancel);

	// Add the app to the system tray ...
	IconMessage(NIM_ADD);

	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

/////////////////////////////////////////////////////////////
//
// Process any operations to be performed on the system tray
//
void CIconFrame::IconMessage(DWORD dwMessage)
{
	NOTIFYICONDATA nid;

	lstrcpyn(nid.szTip, "Wilbur", sizeof(nid.szTip));

	switch (dwMessage)
	{
	case NIM_ADD:
		nid.uFlags = NIF_MESSAGE|NIF_ICON|NIF_TIP;
		break;
	case NIM_MODIFY:
		nid.uFlags = /*NIF_ICON|*/NIF_TIP;
		break;
	case NIM_DELETE:
		nid.uFlags = 0;
		break;
	}

	nid.uID = (UINT)IDR_MAINFRAME;
	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd = GetSafeHwnd();
	nid.uCallbackMessage = ICON_CALLBACK;
	nid.hIcon = m_okicon;

	// Do the business
	Shell_NotifyIcon(dwMessage,&nid);
}

/////////////////////////////////////////////////////////////
//
// Process any Callback operations i.e. mouse messages
//
void CIconFrame::IconCallback(WPARAM wParam, LPARAM lParam)
{
	UINT uID;
	UINT uMouseMsg;
	POINT pt;

	uID = (UINT)wParam;
	uMouseMsg = (UINT)lParam;

	switch (uMouseMsg)
	{
	case WM_LBUTTONUP:
	{
		launchWilbur();
		break;
	}

	case WM_RBUTTONUP:
		//Show Popup Menu*/
		GetCursorPos(&pt);

		m_pIconPopup->TrackPopupMenu(TPM_RIGHTALIGN|TPM_LEFTBUTTON,pt.x,pt.y,this);
		break;

	default:
		break;
	}

	return;
}


void CIconFrame::IconMenuClose()
{
	DestroyWindow();
}

void CIconFrame::IconClosePopup()
{
	// just dummy command to allow popup to close
}

void CIconFrame::launchWilbur()
{
	// launch Wilbur
	launch("wilbur.exe");
}

void CIconFrame::launch2ndWilbur()
{
	// launch Wilbur
	CString sPath = regGetString( HKEY_CURRENT_USER, "Settings", "Path", "c:\\Program Files\\Redtree\\Wilbur\\" );
	CString sWilbur = sPath + "\\wilbur.exe"; 
	unsigned long nShellReturn; 
	nShellReturn = (unsigned long) ShellExecute( NULL, NULL, sWilbur, "-2", NULL, SW_SHOW ); 

	if( nShellReturn <= 32 ) 
	{ 
		CString sMsg; 
		sMsg.Format("I could not run %s a second time\nfor some reason\nThe system reported error number %d\n", sWilbur, nShellReturn); 
		sMsg += "\nPlease report this error to us!\n\n"; 
		AfxMessageBox( sMsg ); 
	} 
}

void CIconFrame::launchNote()
{
	// launch WilburNote
	launch("WilburNote");
}

void CIconFrame::launch( CString sProg )
{
	// launch Wilbur
	CString sPath = regGetString( HKEY_CURRENT_USER, "Settings", "Path", "c:\\Program Files\\Redtree\\Wilbur\\" );
	CString sWilbur = sPath + "\\" + sProg; 
	unsigned long nShellReturn; 
	nShellReturn = (unsigned long) ShellExecute( NULL, NULL, sWilbur, NULL, NULL, SW_SHOW ); 

	if( nShellReturn <= 32 ) 
	{ 
		CString sMsg; 
		sMsg.Format("I could not run %s\nfor some reason\nThe system reported error number %d\n", sWilbur, nShellReturn); 
		sMsg += "\nPlease report this error to us!\n\n"; 
		AfxMessageBox( sMsg ); 
	} 
}

void CIconFrame::scheduleDialog()
{
	CScheduleListDialog dlg;

	dlg.m_pScheduleList = &m_scheduleList;

	if( dlg.DoModal() == IDOK )
	{
		CFile scheduleFile;
		if( scheduleFile.Open( "schedule.dat", CFile::modeCreate | CFile::modeWrite ) )
		{
			try
			{
				CArchive archive( &scheduleFile, CArchive::store );

				archive << 1;  // release number
				archive << m_scheduleList.GetCount();

				POSITION pos = m_scheduleList.GetHeadPosition();
				while( pos )
					m_scheduleList.GetNext( pos ).Serialize( archive );
			}
			catch( ... )
			{}
		}
		setUpTimer();
	}
}

void CIconFrame::setUpTimer()
{
	if( m_idTimer )
	{
		KillTimer( m_idTimer );
		m_idTimer = 0;
	}

	COleDateTime timeNow = COleDateTime::GetCurrentTime();

	int nItem = 0;
	int nID = 0;
	COleDateTimeSpan soonest = 45.0;

	POSITION pos = m_scheduleList.GetHeadPosition();
	while( pos )
	{
		nItem++;
		CSchedule schedule;
		schedule = m_scheduleList.GetNext( pos );

		COleDateTimeSpan span = schedule.getNextStart() - timeNow;
		if( (double) span > 0.0 )
		{
			if( span < soonest )
			{
				soonest = span;
				nID = nItem;
			}
		}
	}

	if( (double) soonest > 0.0 && (double) soonest < 45.0 )
	{
		UINT nElapsed = (UINT) soonest.GetTotalSeconds() * 1000;
		m_idTimer = SetTimer( nID, nElapsed, NULL );
	}
}

void CIconFrame::OnTimer(UINT nIDEvent) 
{
	KillTimer( m_idTimer );
	m_idTimer = 0;

	POSITION pos = m_scheduleList.GetHeadPosition();
	while( pos )
	{
		CSchedule schedule;
		schedule = m_scheduleList.GetNext( pos );
		if( nIDEvent-- == 1 )
		{
			CString sPath = regGetString( HKEY_CURRENT_USER, "Settings", "Path", "c:\\Program Files\\Redtree\\Wilbur\\" );
			CString sWilbur = sPath + "\\wilbur.exe";

			CString sParam = "-2 ";

			if( schedule.isUpdate() )
			{
				CFile testOpen;
				if( testOpen.Open( schedule.getIndexPath(), CFile::modeWrite ) )
				{
					// at least at this moment the file is available - go ahead
					sParam += "-u ";
					testOpen.Close();
				}
				else
					break; // file is unavailable - skip
			}
			else
				sParam += "-b ";

			sParam += schedule.getIndexPath();
			unsigned long nShellReturn; 
			nShellReturn = (unsigned long) ShellExecute( NULL, NULL, sWilbur, sParam, NULL, SW_SHOWMINNOACTIVE ); 

			if( nShellReturn <= 32 ) 
			{ 
				CString sMsg; 
				sMsg.Format("I could not run %s\nfor some reason\nThe system reported error number %d\n", sWilbur, nShellReturn); 
				sMsg += "\nPlease report this error to us!\n\n"; 
				AfxMessageBox( sMsg ); 
			}
			
			break;
		}
	}

	Sleep( 1000 );
	setUpTimer();

	CFrameWnd::OnTimer(nIDEvent);
}
