/*  Wilbur - file indexing and search utility - WilburUtil launcher and scheduler
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// schedule.cpp : schedule class
//
#include "stdafx.h"
#include <math.h>

#ifndef __SCHEDULE_H__
#include "schedule.h"
#endif

IMPLEMENT_SERIAL( CSchedule, CObject, 0 );

CSchedule::CSchedule() : m_intervalType( UNDEFINED )
{}

CSchedule::CSchedule( CString sIndexPath,
					  COleDateTime startTime,
					  int nInterval, 
					  IntervalType type,
					  bool bUpdate,
					  bool bSecondInstance )
	:
		m_sIndexPath( sIndexPath ), m_startTime( startTime ),
		m_nInterval( nInterval ), m_intervalType( type ),
		m_bUpdate( bUpdate ), m_bSecondInstance( bSecondInstance )
{}

void CSchedule::Serialize( CArchive& archive )
{
	if( archive.IsStoring() )
	{
		archive << 1;
		archive << m_sIndexPath << m_startTime << m_nInterval << m_intervalType;
		archive << m_bUpdate << m_bSecondInstance;
	}
	else
	{
		int nRelease;
		archive >> nRelease;
		if( nRelease < 1 || nRelease > 1 )
			AfxThrowArchiveException( CArchiveException::badSchema );

		int nType;
		archive >> m_sIndexPath >> m_startTime >> m_nInterval >> nType;
		m_intervalType = (IntervalType) nType;

		BOOL bFlag;
		archive >> bFlag;
		m_bUpdate = (bFlag == TRUE);
		archive >> bFlag;
		m_bSecondInstance = ( bFlag == TRUE );
	}
}

const CSchedule& CSchedule::operator=( const CSchedule& source )
{
	m_startTime = source.m_startTime;
	m_sIndexPath = source.m_sIndexPath;
	m_nInterval = source.m_nInterval;
	m_intervalType = source.m_intervalType;
	m_bUpdate = source.m_bUpdate;
	m_bSecondInstance = source.m_bSecondInstance;

	return *this;
}

COleDateTime CSchedule::getNextStart()
{
	double dInterval;

	switch( m_intervalType )
	{
	case MINUTE:
		dInterval = ((double) m_nInterval ) / 24.0 / 60.0;
		break;

	case HOUR:
		dInterval = ((double) m_nInterval ) / 24.0;
		break;

	case DAY:
		dInterval = (double) m_nInterval;
		break;

	case UNDEFINED:
	default:
		return COleDateTime( 4000.0 );
	}

	COleDateTime startTime = m_startTime;

	if( dInterval > 0.0 )
	{
		COleDateTime timeNow = COleDateTime::GetCurrentTime();
		COleDateTimeSpan delta = timeNow - startTime;
		double dDelta = delta;
		double dRepeats;
		modf( dDelta / dInterval, &dRepeats );
		startTime += (dRepeats + 1.0) * dInterval;
	}

	return startTime;
}

	