/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// dialogs.cpp : implementation file
//

#include "stdafx.h"

#include <direct.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __DIALOGS_H__
#include "dialogs.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSearchDialog dialog


CSearchDialog::CSearchDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CSearchDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSearchDialog)
	m_sSearch = _T("");
	m_bUseSoundex = FALSE;
	m_bReverse = FALSE;
	m_sStatus = _T("");
	m_sAttributes = _T("");
	m_sTemplate = _T("");
	m_sFolder = _T("");
	m_sBefore = _T("");
	m_sAfter = _T("");
	m_sMaxSize = _T("");
	m_sMinSize = _T("");
	m_sFileMask = _T("");
	m_sIndex = _T("");
	//}}AFX_DATA_INIT
}

BOOL CSearchDialog::OnInitDialog()
{
	ASSERT_VALID( m_pDocument );
	ASSERT_VALID( m_pCriteria );

	CString sTitle;
	sTitle.LoadString( IDS_SEARCHING_DASH );
	sTitle += m_pDocument->GetTitle();
	SetWindowText( sTitle );

	setFields();

	m_sTemplate = "";
	m_sIndex = "";

	CDialog::OnInitDialog();

	CString sPath = thisApp()->startDirectory() + "\\*.wtp";
	WIN32_FIND_DATA fileData;
	HANDLE hFind = FindFirstFile( sPath, &fileData );

	if( hFind != INVALID_HANDLE_VALUE )
	{
		m_cbTemplate.AddString("");
		do
		{
			CString sName = fileData.cFileName;
			sName = sName.Left( sName.GetLength() - 4 );
			m_cbTemplate.AddString( sName );
		}  while ( FindNextFile( hFind, &fileData ) );

		FindClose( hFind );
	}

	CList< CString, CString& > searchList;
	m_pDocument->readSearchHistory( searchList );

	for( POSITION pos = searchList.GetHeadPosition(); pos; )
	{
		CString sText = searchList.GetNext( pos );
		m_cbSearch.AddString( sText );
	}

	thisApp()->fillFileList( m_cbIndex );

	return TRUE;
}

void CSearchDialog::OnClear() 
{
	ASSERT_VALID( m_pCriteria );
	m_pCriteria->clear();
	setFields();
	UpdateData( FALSE );
}

void CSearchDialog::setFields()
{
	ASSERT_VALID( m_pCriteria );
	m_bUseSoundex	= m_pCriteria->getSoundex();
	m_sFileMask		= m_pCriteria->getFileName();
	m_bReverse		= m_pCriteria->getReverse();
	m_sFolder		= m_pCriteria->getFolderName();

	if( (DATE) m_pCriteria->getBeforeTime() != 0.0 )
		m_sBefore = m_pCriteria->getBeforeTime().Format();
	else
		m_sBefore.Empty();

	if( (DATE) m_pCriteria->getAfterTime() != 0.0 )
		m_sAfter = m_pCriteria->getAfterTime().Format();
	else
		m_sAfter.Empty();

	if( m_pCriteria->getMinSize() != 0 )
		m_sMinSize.Format("%d", m_pCriteria->getMinSize() );
	else
		m_sMinSize.Empty();

	if( m_pCriteria->getMaxSize() != 0 )
		m_sMaxSize.Format("%d", m_pCriteria->getMaxSize() );
	else
		m_sMaxSize.Empty();

	char attrib[10];
	char* pAttrib = attrib;
	DWORD dwAttrib = m_pCriteria->getAttributes();

	if( dwAttrib & 0x10000000 ) // NOT
		*pAttrib++ = '^';

	if( dwAttrib & FILE_ATTRIBUTE_COMPRESSED )
		*pAttrib++ = 'c';

	if( dwAttrib & FILE_ATTRIBUTE_ARCHIVE )
		*pAttrib++ = 'a';

	if( dwAttrib & FILE_ATTRIBUTE_DIRECTORY )
		*pAttrib++ = 'd';

	if( dwAttrib & FILE_ATTRIBUTE_SYSTEM )
		*pAttrib++ = 's';

	if( dwAttrib & FILE_ATTRIBUTE_HIDDEN )
		*pAttrib++ = 'h';

	if( dwAttrib & FILE_ATTRIBUTE_READONLY )
		*pAttrib++ = 'r';

	*pAttrib = '\0';

	m_sAttributes = attrib;
}

void CSearchDialog::OnSaveTemplate() 
{
	UpdateData();
	if( m_sTemplate.IsEmpty() )
	{
		simpleMessageBox( IDS_SEARCH_TEMPLATE_NAME );
		return;
	}

	CString sPath = thisApp()->startDirectory() + '\\' + m_sTemplate + ".wtp";

	CStdioFile templateFile;

	if( templateFile.Open( sPath, CFile::modeRead ) )
	{
		templateFile.Close();
		if( simpleMessageBox( IDS_REPLACE_TEMPLATE, MB_YESNO ) != IDYES )
			return;
	}

	if( !templateFile.Open( sPath, CFile::modeCreate | CFile::modeWrite ) )
	{
		CString sMsg;
		sMsg.LoadString( IDS_TEMPLATE_CREATE_ERR );
		sMsg += sPath;
		simpleMessageBox(sMsg);
		return;
	}

	try
	{
		templateFile.WriteString( "FileName=" + m_sFileMask + '\n' );
		templateFile.WriteString( "FolderName=" + m_sFolder + '\n' );
		templateFile.WriteString( "Before=" + m_sBefore + '\n' );
		templateFile.WriteString( "After=" + m_sAfter + '\n' );
		templateFile.WriteString( "Attributes=" + m_sAttributes + '\n' );
		templateFile.WriteString( "MinSize=" + m_sMinSize + '\n' );
		templateFile.WriteString( "MaxSize=" + m_sMaxSize + '\n' );
		CString sBool = m_bUseSoundex ? "T" : "F";
		templateFile.WriteString( "UseSoundex=" + sBool + '\n' );
		sBool = m_bReverse ? "T" : "F";
		templateFile.WriteString( "Reverse=" + sBool + '\n' );
	}
	catch( ... )
	{
		CString sMsg;
		sMsg.LoadString( IDS_TEMPLATE_SAVE_ERR );
		sMsg += sPath;
		simpleMessageBox(sMsg);
		return;
	}

	int nItem = m_cbTemplate.FindString( -1, m_sTemplate );
	if( nItem == CB_ERR )
		m_cbTemplate.AddString( m_sTemplate );

}

void CSearchDialog::OnDeleteTemplate() 
{
	UpdateData();
	if( m_sTemplate.IsEmpty() )
	{
		simpleMessageBox( IDS_TEMPLATE_DEL_NAME_ERR );
		return;
	}

	CString sPath = thisApp()->startDirectory() + '\\' + m_sTemplate + ".wtp";

	try
	{
		CFile::Remove( sPath );
	}
	catch( ... )
	{
		CString sMsg;
		sMsg.LoadString( IDS_TEMPLATE_DEL_ERR );
		sMsg += sPath;
		simpleMessageBox(sMsg);
	}

	int nItem = m_cbTemplate.FindString( -1, m_sTemplate );
	if( nItem != CB_ERR )
		m_cbTemplate.DeleteString( nItem );
}

void CSearchDialog::OnSelchangeTemplate() 
{
	UpdateData();

	int nSel = m_cbTemplate.GetCurSel();
	if( nSel == 0 || nSel == CB_ERR )
		return;

	CString sSel;
	m_cbTemplate.GetLBText( nSel, sSel );

	loadTemplate( sSel );
	m_sTemplate = sSel;
	UpdateData( FALSE );
}

void CSearchDialog::OnGetTemplate() 
{
	UpdateData();
	loadTemplate( m_sTemplate );	
	UpdateData( FALSE );
	SendMessage( DM_SETDEFID, IDOK );

	CWnd* pWnd = GetDlgItem( IDC_SEARCH_TEXT );
	ASSERT_VALID( pWnd );
	pWnd->SetFocus();
}

void CSearchDialog::OnEditchangeTemplate() 
{
	SendMessage( DM_SETDEFID, IDC_GET_TEMPLATE ); 
}

void CSearchDialog::OnKillfocusTemplate() 
{
	SendMessage( DM_SETDEFID, IDOK );
}

void CSearchDialog::loadTemplate( CString sTemplate )
{
	SendMessage( DM_SETDEFID, IDOK ); // make sure the OK is default

	CString sPath = thisApp()->startDirectory() + '\\' + sTemplate + ".wtp";
	CStdioFile templateFile;
	if( !templateFile.Open( sPath, CFile::modeRead | CFile::shareDenyWrite ) )
	{
		CString sMsg;
		sMsg.LoadString( IDS_TEMPLATE_LOAD_ERR );
		sMsg += sPath;
		simpleMessageBox(sMsg);
		return;
	}


	CString sText;
	try
	{
		templateFile.ReadString( sText);
		while( !sText.IsEmpty() )
		{
			int nEqual = sText.Find('=');

			if ( nEqual == -1 )
				break;

			CString sKey = sText.Left( nEqual );
			CString sValue = sText.Mid( nEqual + 1 );

			if( !sKey.Compare( "FileName" ) )
				m_sFileMask = sValue;
			
			else if( !sKey.Compare( "FolderName" ) )
				m_sFolder = sValue;

			else if( !sKey.Compare( "Before" ) )
				m_sBefore = sValue;
			else if( !sKey.Compare( "After" ) )
				m_sAfter = sValue;
			else if( !sKey.Compare( "Attributes" ) )
				m_sAttributes = sValue;
			else if( !sKey.Compare( "MinSize" ) )
				m_sMinSize = sValue;
			else if( !sKey.Compare( "MaxSize" ) )
				m_sMaxSize = sValue;
			else if( !sKey.Compare( "UseSoundex" ) )
				m_bUseSoundex = (sValue[ 0 ] == 'T');
			else if( !sKey.Compare( "Reverse" ) )
				m_bReverse = (sValue[ 0 ] == 'T');

			templateFile.ReadString( sText);
		}
	}
	catch( ... )
	{
		CString sMsg;
		sMsg.LoadString( IDS_TEMPLATE_READ_ERR );
		sMsg += sPath;
		simpleMessageBox(sMsg);
		return;
	}
	templateFile.Close();
}

void CSearchDialog::OnOK() 
{
	ASSERT_VALID( m_pCriteria );
	
	CDialog::OnOK();

	m_pCriteria->setFileName( m_sFileMask );
	m_pCriteria->setSoundex( m_bUseSoundex == TRUE );
	m_pCriteria->setReverse( m_bReverse == TRUE );
	m_pCriteria->setFolderName( m_sFolder );
	m_pCriteria->setBeforeTime( m_sBefore );
	m_pCriteria->setAfterTime( m_sAfter );

	m_pCriteria->setMinSize( atoi( m_sMinSize ) );
	m_pCriteria->setMaxSize( atoi( m_sMaxSize ) );

	m_pCriteria->setAttributes( m_sAttributes );
}

void CSearchDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSearchDialog)
	DDX_Control(pDX, IDC_INDEX, m_cbIndex);
	DDX_Control(pDX, IDC_TEMPLATE, m_cbTemplate);
	DDX_Control(pDX, IDC_SEARCH_TEXT, m_cbSearch);
	DDX_CBString(pDX, IDC_SEARCH_TEXT, m_sSearch);
	DDX_Check(pDX, IDC_SOUNDEX, m_bUseSoundex);
	DDX_Check(pDX, IDC_REVERSE, m_bReverse);
	DDX_Text(pDX, IDC_SEARCH_STATUS, m_sStatus);
	DDX_Text(pDX, IDC_ATTRIBUTES, m_sAttributes);
	DDX_CBString(pDX, IDC_TEMPLATE, m_sTemplate);
	DDX_Text(pDX, IDC_FOLDER_MASK, m_sFolder);
	DDX_Text(pDX, IDC_BEFORE, m_sBefore);
	DDX_Text(pDX, IDC_AFTER, m_sAfter);
	DDX_Text(pDX, IDC_MAX_SIZE, m_sMaxSize);
	DDX_Text(pDX, IDC_MIN_SIZE, m_sMinSize);
	DDX_Text(pDX, IDC_FILE_MASK, m_sFileMask);
	DDX_CBString(pDX, IDC_INDEX, m_sIndex);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSearchDialog, CDialog)
	//{{AFX_MSG_MAP(CSearchDialog)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_BN_CLICKED(IDC_SAVE_TEMPLATE, OnSaveTemplate)
	ON_CBN_SELCHANGE(IDC_TEMPLATE, OnSelchangeTemplate)
	ON_BN_CLICKED(IDC_GET_TEMPLATE, OnGetTemplate)
	ON_CBN_EDITCHANGE(IDC_TEMPLATE, OnEditchangeTemplate)
	ON_CBN_KILLFOCUS(IDC_TEMPLATE, OnKillfocusTemplate)
	ON_BN_CLICKED(IDC_DELETE_TEMPLATE, OnDeleteTemplate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CSearchDialog message handlers


/////////////////////////////////////////////////////////////////////////////
// CIncludeDialog dialog


CIncludeDialog::CIncludeDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CIncludeDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CIncludeDialog)
	m_sInclude = _T("");
	//}}AFX_DATA_INIT
}


void CIncludeDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIncludeDialog)
	DDX_Text(pDX, IDC_INCLUDEPATH, m_sInclude);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIncludeDialog, CDialog)
	//{{AFX_MSG_MAP(CIncludeDialog)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CIncludeDialog message handlers

void CIncludeDialog::OnBrowse() 
{
	CFileDialog dlg( TRUE, NULL, "*", OFN_NOVALIDATE );

	if( dlg.DoModal() == IDOK )
	{
		m_sInclude = dlg.GetPathName();

		UpdateData( FALSE );
	}
}


/////////////////////////////////////////////////////////////////////////////
// CExcludeDialog dialog


CExcludeDialog::CExcludeDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CExcludeDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExcludeDialog)
	m_sExclude = _T("");
	//}}AFX_DATA_INIT
}


void CExcludeDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExcludeDialog)
	DDX_Text(pDX, IDC_EXCLUDEPATH, m_sExclude);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CExcludeDialog, CDialog)
	//{{AFX_MSG_MAP(CExcludeDialog)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CExcludeDialog message handlers

void CExcludeDialog::OnBrowse() 
{
	CFileDialog dlg( TRUE, NULL, "*", OFN_NOVALIDATE );
	if( dlg.DoModal() == IDOK )
	{
		m_sExclude = dlg.GetPathName();
		UpdateData( FALSE );
	}
}

/////////////////////////////////////////////////////////////////////////////
// CPreferencesDialog dialog


CPreferencesDialog::CPreferencesDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CPreferencesDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPreferencesDialog)
	m_sEditor = _T("");
	m_sZipPath = _T("");
	m_nNear = 0;
	m_nTabSize = 0;
	m_nWordWrap = 0;
	m_sIndexPath = _T("");
	m_bOutline = FALSE;
	m_nFileHistory = 0;
	m_nMaxBuffer = 0;
	m_bCachePDF = FALSE;
	m_spdftotextArgs = _T("");
	m_nHorizontal = -1;
	m_nVertical = -1;
	//}}AFX_DATA_INIT
}


void CPreferencesDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPreferencesDialog)
	DDX_Control(pDX, IDC_HORIZONTAL, m_horizontal);
	DDX_Control(pDX, IDC_VERTICAL, m_vertical);
	DDX_Text(pDX, IDC_EDITOR, m_sEditor);
	DDX_Text(pDX, IDC_ZIP_PATH, m_sZipPath);
	DDX_Text(pDX, IDC_NEAR, m_nNear);
	DDV_MinMaxInt(pDX, m_nNear, 1, 999);
	DDX_Text(pDX, IDC_TABSIZE, m_nTabSize);
	DDV_MinMaxInt(pDX, m_nTabSize, 1, 80);
	DDX_Text(pDX, IDC_WORDWRAP, m_nWordWrap);
	DDV_MinMaxInt(pDX, m_nWordWrap, 10, 1000);
	DDX_Text(pDX, IDC_INDEXPATH, m_sIndexPath);
	DDX_Check(pDX, IDC_OUTLINE, m_bOutline);
	DDX_Text(pDX, IDC_FILE_HISTORY_SIZE, m_nFileHistory);
	DDV_MinMaxUInt(pDX, m_nFileHistory, 0, 1000);
	DDX_Text(pDX, IDC_MAX_CONTENT_BUFFER, m_nMaxBuffer);
	DDX_Check(pDX, IDC_CACHEPDF, m_bCachePDF);
	DDX_Text(pDX, IDC_PDFTOTEXTARGS, m_spdftotextArgs);
	DDX_Radio(pDX, IDC_HORIZONTAL, m_nHorizontal);
	DDX_Radio(pDX, IDC_VERTICAL, m_nVertical);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPreferencesDialog, CDialog)
	//{{AFX_MSG_MAP(CPreferencesDialog)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(IDC_VERTICAL, OnVertical)
	ON_BN_CLICKED(IDC_HORIZONTAL, OnHorizontal)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CPreferencesDialog message handlers

void CPreferencesDialog::OnBrowse() 
{
	CFileDialog dlg( TRUE );
	if( dlg.DoModal() == IDOK )
	{
		m_sEditor = dlg.GetPathName();
		UpdateData( FALSE );
	}
}

/////////////////////////////////////////////////////////////////////////////
// CIndexOptionsPage property page

IMPLEMENT_DYNCREATE(CIndexOptionsPage, CPropertyPage)

CIndexOptionsPage::CIndexOptionsPage() : CPropertyPage(CIndexOptionsPage::IDD)
{
	//{{AFX_DATA_INIT(CIndexOptionsPage)
	m_bUseZip = FALSE;
	m_sAddedChars = _T("");
	m_sAddedTrailing = _T("");
	m_nMaxWordLength = 0;
	m_nMinWordLength = 0;
	m_bUseRanking = FALSE;
	m_bTrackAll = FALSE;
	//}}AFX_DATA_INIT
}

CIndexOptionsPage::~CIndexOptionsPage()
{
}

void CIndexOptionsPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIndexOptionsPage)
	DDX_Control(pDX, IDC_USE_ZIP, m_ckbZip);
	DDX_Control(pDX, IDC_NUMBERS, m_NumberHandler);
	DDX_Check(pDX, IDC_USE_ZIP, m_bUseZip);
	DDX_Text(pDX, IDC_ADDEDCHARS, m_sAddedChars);
	DDX_Text(pDX, IDC_ADDEDTRAILING, m_sAddedTrailing);
	DDX_Text(pDX, IDC_MAXWORDSIZE, m_nMaxWordLength);
	DDV_MinMaxInt(pDX, m_nMaxWordLength, 0, 255);
	DDX_Text(pDX, IDC_MINWORDSIZE, m_nMinWordLength);
	DDV_MinMaxInt(pDX, m_nMinWordLength, 0, 255);
	DDX_Check(pDX, IDC_USE_RANKING, m_bUseRanking);
	DDX_Check(pDX, IDC_TRACKALL, m_bTrackAll);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIndexOptionsPage, CPropertyPage)
	//{{AFX_MSG_MAP(CIndexOptionsPage)
	ON_CBN_SELENDOK(IDC_NUMBERS, OnSelendokNumbers)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIndexOptionsPage message handlers


void CIndexOptionsPage::OnSelendokNumbers() 
{
	m_nNumber = m_NumberHandler.GetCurSel();
}

BOOL CIndexOptionsPage::OnInitDialog() 
{
	
	ASSERT_VALID( m_pIndex );
	CPropertyPage::OnInitDialog();

	m_ckbZip.EnableWindow( TRUE );
	CString sNumber;
	sNumber.LoadString(IDS_NONUMBERS);
	m_NumberHandler.AddString(sNumber);
	sNumber.LoadString(IDS_NOSTARTNUMBERS);
	m_NumberHandler.AddString(sNumber);
	sNumber.LoadString(IDS_NUMBERSANYWHERE);
	m_NumberHandler.AddString(sNumber);

	m_NumberHandler.SetCurSel( m_nNumber );
	UpdateData( FALSE );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


/////////////////////////////////////////////////////////////////////////////
// CIndexFilesPage property page

IMPLEMENT_DYNCREATE(CIndexFilesPage, CPropertyPage)

CIndexFilesPage::CIndexFilesPage()
      : CPropertyPage(CIndexFilesPage::IDD, IDS_INCLUDEFILES )
{
	//{{AFX_DATA_INIT(CIndexFilesPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


CIndexFilesPage::CIndexFilesPage( UINT nIDCaption )
      : CPropertyPage(CIndexFilesPage::IDD, nIDCaption ), m_nIDCaption( nIDCaption )
{
}

CIndexFilesPage::~CIndexFilesPage()
{
}


void CIndexFilesPage::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIndexFilesPage)
	DDX_Control(pDX, IDC_INCLUDELIST, m_lbInclude);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIndexFilesPage, CDialog)
	//{{AFX_MSG_MAP(CIndexFilesPage)
	ON_BN_CLICKED(IDC_ADD_INCLUDE, OnAddInclude)
	ON_BN_CLICKED(IDC_CLEAR_INCLUDES, OnClearIncludes)
	ON_BN_CLICKED(IDC_DELETE_INCLUDE, OnDeleteInclude)
	ON_BN_CLICKED(IDC_EDIT_INCLUDE, OnEditInclude)
	ON_BN_CLICKED(IDC_COPY_INCLUDES, OnCopyIncludes)
	ON_BN_CLICKED(IDC_PASTE_INCLUDES, OnPasteIncludes)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIndexFilesPage message handlers

void CIndexFilesPage::OnAddInclude() 
{
	if( m_nIDCaption == IDS_INCLUDEFILES )
	{
		CIncludeDialog dlg;
		if( dlg.DoModal() == IDOK )
		{
			m_pIndex->addInclude( dlg.m_sInclude );	

			fillLists();
			m_pDocument->SetModifiedFlag();
		}
	}
	else
	{
		CExcludeDialog dlg;
		if( dlg.DoModal() == IDOK )
		{
			m_pIndex->addExclude( dlg.m_sExclude );	

			fillLists();
			m_pDocument->SetModifiedFlag();
		}
	}
}

void CIndexFilesPage::OnClearIncludes() 
{
	if( m_nIDCaption == IDS_INCLUDEFILES )
		m_pIndex->clearIncludes();
	else
		m_pIndex->clearExcludes();

	m_pDocument->SetModifiedFlag();
	fillLists();
}

void CIndexFilesPage::OnDeleteInclude() 
{
	int nSelCount = m_lbInclude.GetSelCount();
	if( nSelCount == 0 )
		nothingSelected();
	else
	{
		int* pnIndexes = new int[ nSelCount ];
		VERIFY( nSelCount == m_lbInclude.GetSelItems( nSelCount, pnIndexes ) );

		for( int i = 0; i < nSelCount; i++ )
		{
			int nSel = pnIndexes[ i ];

			CString sItem;
			m_lbInclude.GetText( nSel, sItem );
			if( m_nIDCaption == IDS_INCLUDEFILES )
				m_pIndex->removeInclude( sItem );
			else
				m_pIndex->removeExclude( sItem );
		}

		delete[] pnIndexes;

		fillLists();
		m_pDocument->SetModifiedFlag();
	}
}

void CIndexFilesPage::OnEditInclude() 
{
	int nSel = m_lbInclude.GetCaretIndex();
	if( m_nIDCaption == IDS_INCLUDEFILES )
	{
		CString sItem;
		m_lbInclude.GetText( nSel, sItem );

		CIncludeDialog dlg;
		dlg.m_sInclude = sItem;
		if( dlg.DoModal() == IDOK )
		{
			m_pIndex->removeInclude( sItem );
			m_pIndex->addInclude( dlg.m_sInclude );	
			fillLists();
			m_pDocument->SetModifiedFlag();
		}
	}
	else
	{
		CString sItem;
		m_lbInclude.GetText( nSel, sItem );

		CExcludeDialog dlg;
		dlg.m_sExclude = sItem;
		if( dlg.DoModal() == IDOK )
		{
			m_pIndex->removeExclude( sItem );
			m_pIndex->addExclude( dlg.m_sExclude );	
			fillLists();
			m_pDocument->SetModifiedFlag();
		}
	}
}

void CIndexFilesPage::OnCopyIncludes() 
{
	int nSelCount = m_lbInclude.GetSelCount();
	if( nSelCount == 0 )
		nothingSelected();
	else
	{
		int* pnIndexes = new int[ nSelCount ];
		VERIFY( nSelCount == m_lbInclude.GetSelItems( nSelCount, pnIndexes ) );

		CString sSelection;
		for( int i = 0; i < nSelCount; i++ )
		{
			int nSel = pnIndexes[ i ];

			CString sItem;
			m_lbInclude.GetText( nSel, sItem );

			sSelection += sItem + '\n';
		}

		delete[] pnIndexes;

		OpenClipboard();
		EmptyClipboard();

		int nSize = sSelection.GetLength() + nSelCount + 1;
		HGLOBAL hText = GlobalAlloc( GMEM_MOVEABLE | GMEM_DDESHARE, nSize );
		char* pszClip = (char*) GlobalLock( hText );
		const char* pszText = sSelection;

		while( *pszText )
		{
			if( *pszText == '\n' )
				*pszClip++ = '\r';
			*pszClip++ = *pszText++;
		}
		*pszClip = '\0';
				
		SetClipboardData( CF_TEXT, hText );
		GlobalUnlock( hText );
		CloseClipboard();
	}
}

void CIndexFilesPage::OnPasteIncludes() 
{
	OpenClipboard();
	HGLOBAL hText = GetClipboardData( CF_TEXT );
	const char* pszClip = (const char*) GlobalLock( hText );

	if( m_nIDCaption == IDS_INCLUDEFILES )
		m_pIndex->addInclude( pszClip );
	else
		m_pIndex->addExclude( pszClip );

	GlobalUnlock( hText );
	CloseClipboard();

	fillLists();
	m_pDocument->SetModifiedFlag();
}

BOOL CIndexFilesPage::OnInitDialog() 
{
	ASSERT_VALID( m_pIndex );
	CPropertyPage::OnInitDialog();

	fillLists();

	UpdateData( FALSE );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CIndexFilesPage::fillLists()
{
	m_lbInclude.ResetContent();
	if( m_nIDCaption == IDS_INCLUDEFILES )
	{
		for( POSITION pos = m_pIndex->getFirstInclude(); pos; )
			m_lbInclude.AddString( m_pIndex->getNextInclude( pos ) );
	}
	else
	{
		for( POSITION pos = m_pIndex->getFirstExclude(); pos; )
			m_lbInclude.AddString( m_pIndex->getNextExclude( pos ) );
	}
}

void CIndexFilesPage::nothingSelected()
{
	AfxMessageBox( IDS_NOTHING_SELECTED );
}
/////////////////////////////////////////////////////////////////////////////
// CIndexLanguagePage dialog

IMPLEMENT_DYNCREATE(CIndexLanguagePage, CPropertyPage)

CIndexLanguagePage::CIndexLanguagePage(): CPropertyPage(CIndexLanguagePage::IDD)
{
	//{{AFX_DATA_INIT(CIndexLanguagePage)
	m_bDanish = FALSE;
	m_bFinnish = FALSE;
	m_bFrench = FALSE;
	m_bGerman = FALSE;
	m_bInternationalChar = FALSE;
	m_bSpanish = FALSE;
	m_bSwedish = FALSE;
	m_sSkipFile = _T("");
	m_bOEMChar = FALSE;
	//}}AFX_DATA_INIT
}


void CIndexLanguagePage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIndexLanguagePage)
	DDX_Check(pDX, IDC_DANISH, m_bDanish);
	DDX_Check(pDX, IDC_FINNISH, m_bFinnish);
	DDX_Check(pDX, IDC_FRENCH, m_bFrench);
	DDX_Check(pDX, IDC_GERMAN, m_bGerman);
	DDX_Check(pDX, IDC_INTERNATIONALCHAR, m_bInternationalChar);
	DDX_Check(pDX, IDC_SPANISH, m_bSpanish);
	DDX_Check(pDX, IDC_SWEDISH, m_bSwedish);
	DDX_Text(pDX, IDC_SKIP_FILE, m_sSkipFile);
	DDX_Check(pDX, IDC_OEMCHAR, m_bOEMChar);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIndexLanguagePage, CPropertyPage)
	//{{AFX_MSG_MAP(CIndexLanguagePage)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIndexLanguagePage message handlers
/////////////////////////////////////////////////////////////////////////////
// CIndexEncrypt property page

IMPLEMENT_DYNCREATE(CIndexEncrypt, CPropertyPage)

CIndexEncrypt::CIndexEncrypt() : CPropertyPage(CIndexEncrypt::IDD)
{
	//{{AFX_DATA_INIT(CIndexEncrypt)
	m_bEncryptNew = FALSE;
	m_sPassword = _T("");
	m_bRestrictCopies = FALSE;
	m_bStorePassword = FALSE;
	m_sPasswordCheck = _T("");
	//}}AFX_DATA_INIT
}

CIndexEncrypt::~CIndexEncrypt()
{
}

void CIndexEncrypt::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIndexEncrypt)
	DDX_Check(pDX, IDC_ENCRYPTNEW, m_bEncryptNew);
	DDX_Text(pDX, IDC_PASSWORD, m_sPassword);
	DDX_Check(pDX, IDC_RESTRICTCOPIES, m_bRestrictCopies);
	DDX_Check(pDX, IDC_STOREPW, m_bStorePassword);
	DDX_Text(pDX, IDC_PASSWORDCHECK, m_sPasswordCheck);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIndexEncrypt, CPropertyPage)
	//{{AFX_MSG_MAP(CIndexEncrypt)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIndexEncrypt message handlers
/////////////////////////////////////////////////////////////////////////////
// CConfirmPassword dialog


CConfirmPassword::CConfirmPassword(CWnd* pParent /*=NULL*/)
	: CDialog(CConfirmPassword::IDD, pParent)
{
	//{{AFX_DATA_INIT(CConfirmPassword)
	m_sPassword = _T("");
	m_sMessage = _T("");
	//}}AFX_DATA_INIT
}


void CConfirmPassword::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfirmPassword)
	DDX_Text(pDX, IDC_CHECKPASSWORD, m_sPassword);
	DDX_Text(pDX, IDC_CONFIRMMSG, m_sMessage);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfirmPassword, CDialog)
	//{{AFX_MSG_MAP(CConfirmPassword)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfirmPassword message handlers

void CPreferencesDialog::OnVertical() 
{
	m_horizontal.SetCheck(0);
}

void CPreferencesDialog::OnHorizontal() 
{
	m_vertical.SetCheck(0);
}
