/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// mainfrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include <afxpriv.h>

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __MAINFRM_H__
#include "mainfrm.h"
#endif

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_COMMAND(ID_CONTEXT_HELP, OnContextHelp)
	//}}AFX_MSG_MAP
	// Global help commands
	ON_COMMAND(ID_HELP_INDEX, CFrameWnd::OnHelpIndex)
	ON_COMMAND(ID_HELP_USING, CFrameWnd::OnHelpUsing)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpIndex)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// arrays of IDs used to initialize control bars
	
// toolbar buttons - IDs are command buttons
static UINT BASED_CODE buttons[] =
{
	// same order as in the bitmap 'toolbar.bmp'
	ID_FILE_NEW,
	ID_FILE_OPEN,
	ID_FILE_SAVE,
		ID_SEPARATOR,
	ID_EDIT_COPY,
		ID_SEPARATOR,
	ID_FILE_PRINT,
		ID_SEPARATOR,
	ID_EDIT_INDEX,
	ID_INDEX_BUILD,
	ID_INDEX_UPDATE,
		ID_SEPARATOR,
	ID_INDEX_SEARCH,
	ID_INDEX_FINDNEXT,
	ID_SEARCH_FINDPREVIOUS,
		ID_SEPARATOR,
	ID_FILE_EDIT,
	ID_FILE_COPY,
	ID_FILE_COMMAND,
		ID_SEPARATOR,
	ID_VIEW_FONT,
	ID_VIEW_OUTLINE,
		ID_SEPARATOR,
	ID_INDEX_STOP,
		ID_SEPARATOR,
	ID_APP_ABOUT,
	ID_CONTEXT_HELP
};

static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadBitmap(IDR_MAINFRAME) ||
		!m_wndToolBar.SetButtons(buttons,
		  sizeof(buttons)/sizeof(UINT)))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// TODO: Remove this if you don't want tool tips
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY);

	LoadBarState("WilburBarState");

	thisApp()->setMainFrame( this );

	WINDOWPLACEMENT placement;

	if( regGetBinary( HKEY_CURRENT_USER, "", "WindowPlacement", &placement, sizeof( placement ) ) )
	{
		placement.length = sizeof( placement );
		placement.flags = 0;
		
		placement.showCmd = SW_SHOWNORMAL;
		SetWindowPlacement( &placement );
	}

	return 0;
}

void CMainFrame::setStatus( const CString& sStatus )
{
	CWnd* pStatusWnd = CWnd::FromHandle( m_wndStatusBar.m_hWnd );

	if( sStatus.GetLength() > MaxStatusLength )
		strcpy( pszStatus, sStatus.Left( MaxStatusLength ) );
	else
		strcpy( pszStatus, sStatus );

	pStatusWnd->SetWindowText( pszStatus );
	pStatusWnd->SendMessage(WM_IDLEUPDATECMDUI);
	pStatusWnd->UpdateWindow();
}


void CMainFrame::updateInterface()
{
	m_wndToolBar.OnUpdateCmdUI( thisApp()->getMainFrame(), TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnDestroy() 
{
	if( !thisApp()->isBatch() )
	{
		SaveBarState("WilburBarState");
		WINDOWPLACEMENT placement;
		placement.length = sizeof( placement );
		if( GetWindowPlacement( &placement ) )
			regPutBinary( HKEY_CURRENT_USER, "", "WindowPlacement", &placement, sizeof( placement ) );
	}

	CFrameWnd::OnDestroy();
}

const TCHAR szWilburClass[] = _T("WilburClass");
const TCHAR szWilburClass2[] = _T("WilburClass2");

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs) 
{
	WNDCLASS wndcls;

	BOOL bRes = CFrameWnd::PreCreateWindow(cs);
	HINSTANCE hInst = AfxGetInstanceHandle();

	// see if the class already exists
	const TCHAR* pszClassName;
	if( thisApp()->canReplace() )
		pszClassName = szWilburClass;
	else
		pszClassName = szWilburClass2;

	if (!::GetClassInfo(hInst, pszClassName, &wndcls))
	{
		// get default stuff
		::GetClassInfo(hInst, cs.lpszClass, &wndcls);
		wndcls.style &= ~(CS_HREDRAW|CS_VREDRAW);
		// register a new class
		wndcls.lpszClassName = pszClassName;
		wndcls.hIcon = ::LoadIcon(hInst, MAKEINTRESOURCE(IDR_MAINFRAME));
		ASSERT(wndcls.hIcon != NULL);
		if (!AfxRegisterClass(&wndcls))
			AfxThrowResourceException();
	}
	cs.lpszClass = pszClassName;

	return bRes;
}

void CMainFrame::OnContextHelp() 
{
	CFrameWnd::OnContextHelp();
	
}

void CMainFrame::WinHelp(DWORD dwData, UINT nCmd) 
{
	CString sPath = regGetString( HKEY_CURRENT_USER, "Settings", "Path", "c:\\Program Files\\Redtree\\Wilbur\\" ); 
	sPath += "\\help";
	sPath = regGetString( HKEY_CURRENT_USER, "Settings", "HelpPath", sPath );

	int nTargetID = dwData & 0xffff;

	CString sIDs = sPath + "\\HelpIDs.txt";
	CStdioFile idFile;
	if( idFile.Open( sIDs, CFile::modeRead |CFile::shareDenyNone ) )
	{
		CString sLine;
		CString sHelp;
		if( nCmd == 4 )
			sHelp = sPath + "\\gettingstarted.htm";  // default if ID but command is using_help
		else
			sHelp = sPath + "\\index.htm";  // default if ID not found

		try
		{
			while( idFile.ReadString( sLine ) )
			{
				int nIndex = sLine.Find( ' ' );
				if( nIndex > 0 )
				{
					int nID = atoi( sLine.Left( nIndex ) );
					if( nID == nTargetID )
					{
						sHelp = sPath + '\\' + sLine.Mid(nIndex + 1 );
						break;
					}
				}
			}

			unsigned long nShellReturn = (unsigned long) ShellExecute( NULL, NULL, sHelp, NULL, NULL, SW_SHOW );
			
			if( nShellReturn <= 32 )  
			{  
				CString sMsg;  
				sMsg.Format(IDS_FIND_HELP_ERR,
							  sHelp, nShellReturn);  

				AfxMessageBox( sMsg );  
			}
		}
		catch( ... )
		{
			CString sMsg, sFormat;
			sFormat.LoadString( IDS_HELPIDS_READ_ERR );
			sMsg.Format( IDS_HELPIDS_READ_ERR, sIDs );
			simpleMessageBox( sMsg );
		}
	}
	else
	{
		CString sMsg;
		sMsg.Format( IDS_HELP_INDEX_OPEN_ERR, sIDs );
		simpleMessageBox(sMsg);
	}
}
