/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
//
//  mmfile.h
// 
//
//      Header file for CMemoryMappedFile. Class supports memory mapped files
//

class CMemoryMappedFile : public CMemFile
{
public:

	CMemoryMappedFile();
	virtual ~CMemoryMappedFile();

	virtual BOOL Open(LPCTSTR szName, 
		HANDLE hFile = INVALID_HANDLE_VALUE,
		DWORD flProtect = PAGE_READWRITE, 
		DWORD dwMaximumSizeHigh = 0, 
		DWORD dwMaximumSizeLow = 1024 * 4,
		DWORD dwNumberOfBytesToMap = 0,
		DWORD dwDesiredAccess = FILE_MAP_WRITE | FILE_MAP_READ,
		DWORD dwFileOffsetHigh = 0, 
		DWORD dwFileOffsetLow = 0);

	BOOL Attach(HANDLE *hFile, 
		DWORD dwNumberOfBytesToMap = 0,
		DWORD dwDesiredAccess = FILE_MAP_WRITE | FILE_MAP_READ, 
		DWORD dwFileOffsetHigh = 0, 
		DWORD dwFileOffsetLow = 0);

	BOOL MapViewOfFile(DWORD  dwDesiredAccess, 
		DWORD  dwFileOffsetHigh = 0, 
		DWORD  dwFileOffsetLow = 0,
		DWORD  dwNumberOfBytesToMap = 0);

	BOOL UnmapViewOfFile();

	HANDLE Detach();
	void Close(void);

	operator BYTE* () { return m_lpBuffer; } 

	virtual void Flush(DWORD startBytePosition = 0, DWORD  dwNumberOfBytesToFlush = 0);

	BOOL ReadFile(void* lpBuf, UINT nCount) { return CFile::Read(lpBuf, nCount); }
	void WriteFile(const void* lpBuf, UINT nCount) { CFile::Write(lpBuf, nCount); }

    // prevent direct buffering - undocumented function
    virtual UINT GetBufferPtr(UINT nCommand, UINT nCount,void** ppBufStart, void**ppBufMax)
        { return 0;}



protected:
    BOOL m_bAutoDelete;
   
#if _MFC_VER < 0x400
    BYTE* BaseDetach() 
    { 	
        BYTE* lpBuffer = m_lpBuffer;
	    m_lpBuffer = NULL;
	    m_nFileSize = 0;
	    m_nBufferSize = 0;
	    m_nPosition = 0;

    	return lpBuffer;
    }
#endif

    void BaseAttach( BYTE* lpBuffer, UINT nBufferSize);


protected:
	virtual BYTE* Alloc(DWORD nBytes) { return NULL; }
	virtual BYTE* Realloc(BYTE* lpMem, DWORD nBytes) { return NULL; }
	virtual void Free(BYTE* lpMem) { ASSERT(FALSE); }
    virtual void GrowFile(DWORD dwNewLen) { ASSERT(FALSE);}
	virtual void SetLength(DWORD dwNewLen) { ASSERT(FALSE);}


};


#if _MFC_VER < 0x400

#define BASEDETACH  CMemoryMappedFile::BaseDetach()

#define BASEATTACH(lpBuffer, nBUfferSize, nGrow)  	ASSERT(m_lpBuffer == NULL);\
    m_nGrowBytes = nGrowBytes;\
	m_nPosition = 0;\
	m_nBufferSize = nBufferSize;\
	m_nFileSize = nGrowBytes == 0 ? nBufferSize : 0;\
	m_lpBuffer = lpBuffer;

#else

#define BASEDETACH()  CMemFile::Detach()

#define BASEATTACH(lpBuffer, nBUfferSize, nGrow)   CMemFile::Attach(lpBuffer, nBUfferSize, 0)
	
#endif

