/*  Wilbur - file indexing and search utility
 *  Copyright (C) 1995-1999  Craig Morris and RedTree Development Inc.
 *        craig@redtree.com - <http://wilbur.redtree.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  or at http://www.fsf.org
*/
// wilbudoc.h : interface of the CWilburDoc class
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __WILBUDOC_H__
#define __WILBUDOC_H__

#ifndef __FILEINDX_H__
#include "fileindx.h"
#endif

class CFileListView;
class CContentsView;

class CWilburDoc : public CDocument
{
	protected: // create from serialization only
		CWilburDoc();

	public:
		// Generated OLE dispatch map functions
		//{{AFX_DISPATCH(CWilburDoc)
	afx_msg void oleQuit();
	afx_msg BOOL oleOpenIndex(LPCTSTR sIndexPath);
	afx_msg LPDISPATCH oleSearchFor(LPCTSTR sSearchString);
	afx_msg BOOL oleShowWindow();
	afx_msg BOOL oleSaveIndex(LPCTSTR sIndexPath);
	afx_msg BOOL oleBuild(long nWaitTime);
	afx_msg BOOL oleUpdate(long nWatTime);
	afx_msg void oleAddInclude(LPCTSTR sPath);
	afx_msg void oleAddExclude(LPCTSTR sPath);
	afx_msg void oleRemoveInclude(LPCTSTR sPath);
	afx_msg void oleRemoveExclude(LPCTSTR sPath);
	afx_msg void ClearIncludes();
	afx_msg void oleClearExcludes();
	afx_msg BSTR oleGetInclude(long nInclude);
	afx_msg BSTR oleGetExclude(long nExclude);
	afx_msg void oleSetNumberHandling(short nHandling);
	afx_msg short oleGetNumberHandling();
	afx_msg long oleGetInternational();
	afx_msg void oleSetInternational(long nInternational);
	afx_msg void oleSetDOSText(BOOL bDOSText);
	afx_msg BOOL oleGetDOSText();
	afx_msg void oleSetAddedCharacters(LPCTSTR sAdded, LPCTSTR sTrailing);
	afx_msg BSTR oleGetAddedCharacters();
	afx_msg BSTR oleGetAddedTrailing();
	afx_msg BOOL oleGetZip();
	afx_msg void oleSetZip(BOOL bZip);
	afx_msg long oleGetNumberOfFiles();
	afx_msg long oleGetNumberOfWords();
	afx_msg long oleGetNumberOfBytes();
	afx_msg DATE oleGetTimeCreated();
	afx_msg BSTR oleGetMessages();
	afx_msg void oleAddMessage(LPCTSTR pszMessage);
	afx_msg void oleClearMessages();
	afx_msg BOOL oleGetTrackAll();
	afx_msg void oleSetTrackAll(BOOL bNewValue);
	afx_msg LPDISPATCH oleSearchForEx(LPCTSTR sSearchString, LPDISPATCH criteria);
	afx_msg BOOL oleChangeDrive(short driveNumber);
	afx_msg BOOL oleSetZipEncrypt(LPCTSTR sOldPassword, LPCTSTR sNewPassword, BOOL bStorePassword, BOOL bEncryptNew, BOOL bRestrictCopying);
	afx_msg long oleGetZipEncryptFlags();
	afx_msg void oleHalt();
	afx_msg BOOL oleIsThreadRunning();
	//}}AFX_DISPATCH
		DECLARE_DISPATCH_MAP();
		DECLARE_INTERFACE_MAP();

		DECLARE_OLECREATE(CWilburDoc);
		DECLARE_DYNCREATE(CWilburDoc)

	// Attributes
	public:
		enum { NewFileSelection, FindNextOccurence } updateType;

		CWinThread* runBuild();
		CWinThread* runUpdate();

		void    setCurrentFile( const CString& sFile, BOOL bReverse );
		CString getCurrentFile();
		CString getContentsPath();  // may be a temp file if viewing zip
		void    markFileObsolete( FileID nFile );
		FileID  getFileIDFromPath( const CString& sPath );
		void    prevFile();
		CString getSearchText();
		BOOL    isSoundex() { return thisApp()->searchCriteria().getSoundex(); }

		void    setSearchText( CString& sSearchText );
		void    searchFor( const CString& sSearchText, CSearchCriteria& criteria );
		void    doSearch( BOOL bUseDialog = TRUE );

		void    indexingFinished();
		BOOL    getHaltStatus();
		int     getNear() { return m_nNear; }
		void    setNear( int nNear );
		void    setTabSize( int nSize );
		void    setWordWrap( int nChar );
		void    refreshContents();
		void	quickView();
		BOOL    isAutomaticQuickView() { return m_bQuickView; }

		void	minimizeWindow();
		void	setFocusOnList();
		void	setFocusOnContents();

		void    doSaveAfterIndex() { m_bDoSave = TRUE; }

		void addSearchText( CString& sText );
		void readSearchHistory( CList< CString, CString& > &searchHistory );
		void writeSearchHistory( CList< CString, CString& > &searchHistory );

		static BOOL fileNameMatch( const char* pszFilePath, const char* pszFileMask );
		static BOOL termMatch( const char* pszFilePath, const char* pszFileMask );

		CString getDocFileName() { return m_sDocFileName; }
		CString getIndexAttributes();
		void wordCount( const CString listPath ) { m_fileIndex.wordCount( listPath, 0.0, false ); }
		CFileInfo* getFileInfo( FileID nID ) { return m_fileIndex.getFileInfo( nID ); }
		CString getPassword( bool& bSave, bool& bEncryptNew, bool& bRestrictCopies );
		CFileIndex* getIndex() { return &m_fileIndex; }

	// Operations
	public:

	// Overrides
		// ClassWizard generated virtual function overrides
		//{{AFX_VIRTUAL(CWilburDoc)
	public:
		virtual BOOL OnNewDocument();
		virtual void DeleteContents();
	virtual BOOL CanCloseFrame(CFrameWnd* pFrame);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

	// Implementation
	public:
		virtual ~CWilburDoc();
		virtual void Serialize(CArchive& ar);   // overridden for document i/o
	#ifdef _DEBUG
		virtual void AssertValid() const;
		virtual void Dump(CDumpContext& dc) const;
	#endif

	protected:

	// Generated message map functions
	protected:
		//{{AFX_MSG(CWilburDoc)
		afx_msg void OnIndexBuild();
		afx_msg void OnIndexSearch();
		afx_msg void OnIndexFindnext();
		afx_msg void OnEditIndex();
		afx_msg void OnFileEdit();
		afx_msg void OnUpdateFileEdit(CCmdUI* pCmdUI);
		afx_msg void OnFileLaunch();
		afx_msg void OnUpdateFileLaunch(CCmdUI* pCmdUI);
		afx_msg void OnViewFont();
		afx_msg void OnFilePrint();
		afx_msg void OnUpdateFilePrint(CCmdUI* pCmdUI);
		afx_msg void OnUpdateEditIndex(CCmdUI* pCmdUI);
	afx_msg void OnIndexUpdate();
	afx_msg void OnUpdateIndexUpdate(CCmdUI* pCmdUI);
	afx_msg void OnUpdateIndexBuild(CCmdUI* pCmdUI);
	afx_msg void OnUpdateIndexSearch(CCmdUI* pCmdUI);
	afx_msg void OnUpdateIndexFindnext(CCmdUI* pCmdUI);
	afx_msg void OnSearchQuickfind();
	afx_msg void OnUpdateSearchQuickfind(CCmdUI* pCmdUI);
	afx_msg void OnViewInternational();
	afx_msg void OnUpdateViewInternational(CCmdUI* pCmdUI);
	afx_msg void OnViewNext();
	afx_msg void OnUpdateViewNext(CCmdUI* pCmdUI);
	afx_msg void OnViewPrev();
	afx_msg void OnUpdateViewPrev(CCmdUI* pCmdUI);
	afx_msg void OnSearchFindprevious();
	afx_msg void OnUpdateSearchFindprevious(CCmdUI* pCmdUI);
	afx_msg void OnViewQuick();
	afx_msg void OnUpdateViewQuick(CCmdUI* pCmdUI);
	afx_msg void OnViewAutoquick();
	afx_msg void OnUpdateViewAutoquick(CCmdUI* pCmdUI);
	afx_msg void OnViewOemtext();
	afx_msg void OnUpdateViewOemtext(CCmdUI* pCmdUI);
	afx_msg void OnViewOutline();
	afx_msg void OnUpdateViewOutline(CCmdUI* pCmdUI);
	afx_msg void OnFileSave();
	afx_msg void OnFileSaveAs();
	afx_msg void OnViewhidehtml();
	afx_msg void OnUpdateViewhidehtml(CCmdUI* pCmdUI);
	afx_msg void OnWordCount();
	afx_msg void OnImportSkip();
	afx_msg void OnEditSelectall();
	//}}AFX_MSG
		DECLARE_MESSAGE_MAP()

	private:
		CFileIndex m_fileIndex;
		CString    m_sCurrentFile;
		CString    m_sSearchText;

		BOOL       m_bQuickView;
		BOOL       m_bCanSetFile;
		BOOL       m_bDoSave;
		int        m_nNear;
		CString    m_sDocFileName;
		CString    m_sMessages;
		CString    m_sSavedTitle;
		CString    m_sIndexRegName;
		bool       m_bBeenNagged;

		CFile*		m_pCurrentDocFile;  // this is held open to prevent writes when mem map files are open

		CFileListView* m_pFileListView;
		CContentsView* m_pContentsView;

		CSearchResults m_results;

		void findViews();
		void findNext( BOOL bReverse );
		BOOL viewNext();
		BOOL viewPrevious();
		BOOL parenOK( const CString& sText );
		void removeExtras( CString sSearchText, CSearchResults& results );
};


/////////////////////////////////////////////////////////////////////////////
#endif
