// ziputils - routines that cannot be included in open source due to InnerMedia license
//

#include "stdafx.h"

#include <io.h>
#include <direct.h>

#ifndef __WILBUR_H__
#include "wilbur.h"
#endif

#ifndef __MAINFRM_H__
#include "mainfrm.h"
#endif

#ifndef __WILBUDOC_H__
#include "wilbudoc.h"
#endif

#ifndef __FILEINDX_H__
#include "fileindx.h"
#endif

#ifndef __FILELISTVIEW_H__
#include "FileListView.h"
#endif

// info-zip stuff

#ifdef __RSXNT__
#  include "unzip/win32/rsxntwin.h"
#endif

extern "C" {
#define WINDLL
#include "zip/api.h"
#include "unzip/windll/decs.h"
};

/* Defines */

typedef int (WINAPI * _DLL_UNZIP)(int, char **, int, char **,
                                  LPDCL, LPUSERFUNCTIONS);
typedef int (WINAPI * _USER_FUNCTIONS)(LPUSERFUNCTIONS);


static void initUnzipDCL(DCL& dcl)
{
	dcl.ExtractOnlyNewer = 0; /* Do not extract only newer */
	dcl.SpaceToUnderscore = 0; // retain spaces
	dcl.PromptToOverwrite = 0; /* "Overwrite all" selected -> no query mode */

	dcl.fQuiet = 0; /* We want all messages.
                      1 = fewer messages,
                      2 = no messages */
	dcl.ncflag = 0; /* Write to stdout if true */
	dcl.ntflag = 0; /* test zip file if true */
	dcl.nvflag = 0; /* give a verbose listing if true */
	dcl.nzflag = 0; /* display a zip file comment if true */
	dcl.ndflag = 1; /* Recreate directories if true */
	dcl.naflag = 0; /* Do not convert CR to CRLF */
	dcl.nfflag = 0; /* Do not freshen existing files only */
	dcl.noflag = 1; /* Over-write all files if true */
	dcl.naflag = 0; // end of line translation
	dcl.nZIflag = 0;  // get zip info flag
	dcl.C_flag = 0; // case insensitive if true
	dcl.fPrivilege = 2; //  => restore Acl's, 2 => Use privileges
	dcl.lpszZipFN = NULL; /* The archive name - to be filled in later */
	dcl.lpszExtractDir = NULL; /* The directory to extract to. This is set
                                 to NULL if you are extracting to the
                                 current directory.
                               */
}

static char szRootDir[8];

static void initZipOptions(ZPOPT& zOpt)
{
	zOpt.Date = NULL;            /* Not using, set to NULL pointer */
	strcat(szRootDir, "\\.");    /* need this because zip library writes to end of string ??? */
	zOpt.szRootDir = szRootDir;  /* Directory to use as base for zipping */
	zOpt.szTempDir = NULL;       /* Temporary directory used during zipping */
	zOpt.fTemp = FALSE;          /* Use temporary directory '-b' during zipping */
	zOpt.fSuffix = FALSE;        /* include suffixes (not yet implemented) */
	zOpt.fEncrypt = FALSE;       /* true if encryption wanted */
	zOpt.fSystem = FALSE;        /* true to include system/hidden files */
	zOpt.fVolume = FALSE;        /* true if storing volume label */
	zOpt.fExtra = FALSE;         /* true if including extra attributes */
	zOpt.fNoDirEntries = FALSE;  /* true if ignoring directory entries */
	zOpt.fVerbose = FALSE;       /* true if full messages wanted */
	zOpt.fQuiet = FALSE;         /* true if minimum messages wanted */
	zOpt.fCRLF_LF = FALSE;       /* true if translate CR/LF to LF */
	zOpt.fLF_CRLF = FALSE;       /* true if translate LF to CR/LF */
	zOpt.fJunkDir = FALSE;       /* true if junking directory names */
	zOpt.fGrow = FALSE;          /* true if allow appending to zip file */
	zOpt.fForce = FALSE;         /* true if making entries using DOS names */
	zOpt.fMove = FALSE;          /* true if deleting files added or updated */
	zOpt.fDeleteEntries = FALSE; /* true if deleting files from archive */
	zOpt.fUpdate = FALSE;        /* true if updating zip file--overwrite only
									  if newer */
	zOpt.fFreshen = FALSE;       /* true if freshening zip file--overwrite only */
	zOpt.fJunkSFX = FALSE;       /* true if junking sfx prefix*/
	zOpt.fLatestTime = FALSE;    /* true if setting zip file time to time of
									  latest file in archive */
	zOpt.fComment = FALSE;       /* true if putting comment in zip file */
	zOpt.fOffsets = FALSE;       /* true if updating archive offsets for sfx
									  files */
	zOpt.fRecurse = 0;           /* subdir recursing mode: 1 = "-r", 2 = "-R" */
	zOpt.fRepair = 0;            /* archive repair mode: 1 = "-F", 2 = "-FF" */
	zOpt.fLevel = '5';           /* Compression level (0 - 9) */
}


// need a place to stash the index and some parameters that they can be used by the unzip call back
static struct IndexParams {
	CFileIndex* pIndex;
	CString sZipPath;
	CString sIndexPath;
	CMaskList* pMaskList;
	FileID nVolume;
} indexParams;

/* Procedure Calls */
void WINAPI ReceiveDllMessage(unsigned long, unsigned long, unsigned,
   unsigned, unsigned, unsigned, unsigned, unsigned,
   char, LPSTR, LPSTR, unsigned long, char);

void WINAPI ReceiveDllMessage(unsigned long ucsize, unsigned long csiz,
    unsigned cfactor,
    unsigned mo, unsigned dy, unsigned yr, unsigned hh, unsigned mm,
    char c, LPSTR fileName, LPSTR methbuf, unsigned long crc, char fCrypt)
{
	unsigned int nSize;
	if( ucsize > MAXDWORD ) nSize = MAXDWORD;
	else nSize = (unsigned int) ucsize;

	// the zip libary switches day month and year so a printout in dmy form
	// honors the locale - need to undo this to get actual values
	TCHAR df[2];  /* LOCALE_IDATE has a maximum value of 2 */

	if (GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_IDATE, df, 2) != 0) {
		int dummy;
		switch (df[0])
		{
		case '1':
			// DMY
			dummy = mo;
			mo = dy;
			dy = dummy;
			break;
		case '2':
			// YMD
			dummy = dy;
			dy = yr;
			yr = mo;
			mo = dummy;
			break;
		}
	}
  
    // yr is two digit - grrr
	if(yr < 70) yr += 2000;
	else yr += 1900;

	if(yr > 2038) {
		yr =2038; mo = 1; dy = 1; // last valid Windows year
	}

	CTime timeLastWritten( yr, mo, dy, hh, mm, 0 );

	indexParams.pIndex->indexZipEntry(fileName, indexParams.sZipPath, indexParams.sIndexPath,
		*indexParams.pMaskList, indexParams.nVolume, timeLastWritten, nSize);
}

// used in place of ReceiveDllMessage to avoid recursion
void WINAPI IgnoreDllMessage(unsigned long ucsize, unsigned long csiz,
    unsigned cfactor,
    unsigned mo, unsigned dy, unsigned yr, unsigned hh, unsigned mm,
    char c, LPSTR filename, LPSTR methbuf, unsigned long crc, char fCrypt)
{
}

/* Password entry routine - see password.c in the wiz directory for how
   this is actually implemented in WiZ. If you have an encrypted file,
   this will probably give you great pain.
 */
int WINAPI password(char *p, int n, const char *m, const char *name)
{
	bool bSave, bEncryptNew, bRestrictCopies;
	strcpy(p, indexParams.pIndex->getPassword(bSave, bEncryptNew, bRestrictCopies).Left(n));
	return IZ_PW_ENTERED;
}

// password call that always fails
int WINAPI failPassword(char *p, int n, const char *m, const char *name)
{
	return IZ_PW_CANCEL;
}

/* Dummy "print" routine that simply outputs what is sent from the dll */
int WINAPI DisplayBuf(LPSTR buf, unsigned long size)
{
//printf("%s", (char *)buf);
return (unsigned int) size;
}

int WINAPI GetReplaceDlgRetVal(char *filename)
{
/* This is where one would decide if you want to replace, rename etc existing
   files.
 */
return 1;
}

int WINAPI ServiceCallBack(LPCSTR fileName, unsigned long size)
{
	return 0;
}

int WINAPI DummyComment(char far *szBuf)
{
szBuf[0] = '\0';
return TRUE;
}

int CFileIndex::getZipExtIndex( const CString& sPath )
{
	int nZip;
	(nZip = sPath.Find( ".zip\\" )) != -1 ||
	(nZip = sPath.Find( ".qwk\\" )) != -1 ||
	(nZip = sPath.Find( ".cab\\" )) != -1 ||
	(nZip = sPath.Find( ".odt\\" )) != -1 ||
	(nZip = sPath.Find( ".ods\\" )) != -1 ||
	(nZip = sPath.Find( ".odp\\" )) != -1 ||
	(nZip = sPath.Find( ".odb\\" )) != -1 ||
	(nZip = sPath.Find( ".sxw\\" )) != -1 ||
	(nZip = sPath.Find( ".sxc\\" )) != -1 ||
	(nZip = sPath.Find( ".sxi\\" )) != -1 ||
	(nZip = sPath.Find( ".mmap\\" )) != -1;
	return nZip;
}

void CFileIndex::indexZip( CString sZipPath, const CString& sIndexPath, CMaskList& maskList, FileID nVolume )
{
	// sZipPath is the path of the actual zip file - might be temporary for nested zips
	// sIndexPath is the path name to use in the index


	if( sZipPath[1] != ':' && sZipPath[1] != '\\' )
	{
		if( sZipPath[0] == '\\' )
		{
			// just need drive- prepend current drive
			char cDriveLetter = 'a' + _getdrive() - 1;
			sZipPath = ':' + sZipPath;
			sZipPath = cDriveLetter + sZipPath;
		}
		else
		{
			// prepend current drive and directory
			char pathBuf[ MAX_PATH];
			if( GetCurrentDirectory( MAX_PATH, pathBuf) )
			{
				if( !sZipPath.IsEmpty() && sZipPath[ 0 ] == '\\' )
					sZipPath = pathBuf + sZipPath.Mid(1);
				else
					sZipPath = pathBuf + sZipPath;
			}
		}
	}

	// stash stuff for call back
	indexParams.pIndex = this;
	indexParams.sZipPath = sZipPath;
	indexParams.sIndexPath = sIndexPath;
	indexParams.pMaskList = &maskList;
	indexParams.nVolume = nVolume;


	char currentDir[ MAX_PATH ];
	GetCurrentDirectory( MAX_PATH, currentDir );

	try
	{
		DCL dcl;
		initUnzipDCL(dcl);

		USERFUNCTIONS userFuncs;
		userFuncs.password = password;
		userFuncs.print = DisplayBuf;
		userFuncs.sound = NULL;
		userFuncs.replace = GetReplaceDlgRetVal;
		userFuncs.SendApplicationMessage = ReceiveDllMessage;
		userFuncs.ServCallBk = ServiceCallBack;

		// set dcl for listing
		dcl.nvflag = 1;
		dcl.lpszZipFN = sZipPath.GetBuffer(1);
		char extractPath[MAX_PATH];
		strcpy(extractPath, thisApp()->getZipPath());
		dcl.lpszExtractDir = extractPath;

		int retCode = Wiz_SingleEntryUnzip( 0, NULL, 0, NULL, &dcl, &userFuncs);

	}
	catch( ... )
	{
		SetCurrentDirectory( currentDir );
		thisApp()->removeZipDirectories();
		throw;
	}

	SetCurrentDirectory( currentDir );
	thisApp()->removeZipDirectories();
}

void CFileIndex::indexZipEntry(char* fileName,  CString sZipPath, CString sIndexPath,
		CMaskList& maskList, FileID nVolume, CTime& timeLastWritten, unsigned int nSize)
{

	CString sFilePath = fileName;
	sFilePath.Replace('/','\\');
	sFilePath = sIndexPath + '\\' + sFilePath;

	if( !isExcluded( sFilePath ) )
	{
		int nLastSlash = sFilePath.ReverseFind( '\\' );
		CString sDirPath = sFilePath.Left( nLastSlash + 1 );
		CString sFileName = sFilePath.Mid( nLastSlash + 1 );

		FileID nDir = m_pFiles->lookupFile( sDirPath, nVolume );
		if( !m_bIsUpdate || m_timeBase < timeLastWritten || !nDir || !m_pFiles->lookupFile( sFileName, nDir ) )
		{
			bool bMatchesMask = matchesMask( sFilePath, maskList );
			BOOL bIsZip = isZip(sFileName);

			if( !nDir && (bMatchesMask || bIsZip || m_bTrackAll) )
			{
				nDir = m_pFiles->addText( sDirPath, m_nFilesIndexed );

				CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nDir );
				pFileInfo->m_nParent = nVolume;
				pFileInfo->m_timeLastWritten = timeLastWritten;
				pFileInfo->m_dwAttributes = 0;
				pFileInfo->m_nSize = 0;
			}
			
			if( bMatchesMask || bIsZip )
			{
				DCL dcl;
				initUnzipDCL(dcl);
				dcl.lpszZipFN = sZipPath.GetBuffer(1);
				char extractPath[MAX_PATH];
				strcpy(extractPath, thisApp()->getZipPath());
				dcl.lpszExtractDir = extractPath;

				USERFUNCTIONS userFuncs;
				userFuncs.password = password;
				userFuncs.print = DisplayBuf;
				userFuncs.sound = NULL;
				userFuncs.replace = GetReplaceDlgRetVal;
				userFuncs.SendApplicationMessage = IgnoreDllMessage;
				userFuncs.ServCallBk = ServiceCallBack;

				int retCode = Wiz_SingleEntryUnzip( 1, &fileName, 0, NULL, &dcl, &userFuncs);

				if( !retCode && !sFileName.IsEmpty())
				{
					CString sTempFile = dcl.lpszExtractDir;
					sTempFile += fileName;

					if( bIsZip )
					{
						indexZip( sTempFile, sFilePath, maskList, nVolume );
						// need to reset the staticly saved parameters - ugly
						indexParams.pIndex = this;
						indexParams.sZipPath = sZipPath;
						indexParams.sIndexPath = sIndexPath;
						indexParams.pMaskList = &maskList;
						indexParams.nVolume = nVolume;
					}
					else if( bMatchesMask )
					{
						FileID nFile = m_pFiles->lookupFile( sFileName, nDir );

						if( !nFile )
							nFile = m_pFiles->addText( sFileName, m_nFilesIndexed );

						CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nFile );
						pFileInfo->m_timeLastWritten = timeLastWritten;
						pFileInfo->m_dwAttributes = 0;  // for now
						pFileInfo->m_nParent = nDir;

						pFileInfo->m_nSize = nSize;

						indexFile(sTempFile, nFile );
					}

					removeTemporaryFile( sTempFile, 7 );
				}
			}
			else if( m_bTrackAll )
			{
				FileID nFile = m_pFiles->addText( sFileName, m_nFilesIndexed );

				CFileInfo* pFileInfo = (CFileInfo*) m_pFiles->getData( nFile );
				pFileInfo->m_timeLastWritten = timeLastWritten;
				pFileInfo->m_dwAttributes = 0;  // for now
				pFileInfo->m_nParent = nDir;

				pFileInfo->m_nSize = nSize;
			}

		}
	}
}

CString CFileIndex::getTempZip( const CString& sFileName, int nZip, bool bCheckRestriction )
{
	CString sPath = sFileName.Mid( nZip + 5 );
	CString sZipName = sFileName.Left( nZip + 4 );

	int nNestedZip = CFileIndex::getZipExtIndex(makeLower(sPath));

	if( nNestedZip != -1 )
		 sPath = sPath.Left( nNestedZip + 4 );

	sPath.Replace('/','\\');
	
	if( sZipName[1] != ':' && sZipName[1] != '\\' )
	{
		if( sZipName[0] == '\\' )
		{
			// just need drive- prepend current drive
			char cDriveLetter = 'a' + _getdrive() - 1;
			sZipName = ':' + sZipName;
			sZipName = cDriveLetter + sZipName;
		}
		else
		{
			// get current directory
			char pathBuf[ MAX_PATH];
			if( GetCurrentDirectory( MAX_PATH, pathBuf) )
			{
				if( !sZipName.IsEmpty() && sZipName[ 0 ] == '\\' )
					sZipName = pathBuf + sZipName.Mid(1);
				else
					sZipName = pathBuf + sZipName;
			}
		}
	}

	// CString sDummy = '\"' + sPath + '\"';

	// stash stuff for call back - only pIndex will be used since just needed for passwords
	CWilburDoc* pDoc = thisApp()->getDocument();
	indexParams.pIndex = pDoc->getIndex();

	DCL dcl;
	initUnzipDCL(dcl);
	dcl.lpszZipFN = sZipName.GetBuffer(1);
	char extractPath[MAX_PATH];
	strcpy(extractPath, thisApp()->getZipPath());
	dcl.lpszExtractDir = extractPath;

	USERFUNCTIONS userFuncs;
	bool bSave, bEncryptNew, bRestrictCopy;
	CString sPassword = pDoc->getPassword(bSave, bEncryptNew, bRestrictCopy );
	if( !(bCheckRestriction && bRestrictCopy) && !sPassword.IsEmpty() )
		userFuncs.password = password;
	else
		userFuncs.password = failPassword;
	userFuncs.print = DisplayBuf;
	userFuncs.sound = NULL;
	userFuncs.replace = GetReplaceDlgRetVal;
	userFuncs.SendApplicationMessage = IgnoreDllMessage;
	userFuncs.ServCallBk = ServiceCallBack;

	CString sUnixPath = sPath;
	sUnixPath.Replace('\\','/');
	char* lpszPath = sUnixPath.GetBuffer(1);
	int retCode = Wiz_SingleEntryUnzip( 1, &lpszPath, 0, NULL, &dcl, &userFuncs);

	if( !retCode )
	{
		CString sTemp = dcl.lpszExtractDir;
		sPath = sTemp + sPath;

		if( nNestedZip != -1 )
		{
			CString sNestedPath = sFileName.Mid( nZip + 5 );
			sNestedPath = sNestedPath.Mid( nNestedZip + 4 );
			sNestedPath = getTempZip( sPath + sNestedPath, sPath.GetLength() - 4, bCheckRestriction );

			removeTemporaryFile( sPath, 0 );
			thisApp()->removeZipDirectories();
			sPath = sNestedPath;
		}
	}
	else
		sPath = "";

	return sPath;
}

bool CFileListView::addToZip( CString sFileList, CString sTargetZip )
{
	if( sFileList.IsEmpty() )
		return true;

	int nStart = sFileList.Find("\"");
	int nFiles = 0;
	CList<CString, CString&> lFileList;
	while( nStart != -1)
	{
		nStart++;
		int nEnd = sFileList.Find("\"", nStart);
		if( nEnd == -1 )
			break;  // really shouldn't happen as quotes should be paired
		CString fileName = sFileList.Mid(nStart, nEnd - nStart);
		lFileList.AddTail(fileName);
		nFiles++;
		nStart = sFileList.Find("\"", nEnd + 1);
	}

	if( !nFiles ) return false;

	char** ppFiles = NULL;	

	try
	{
		ZPOPT zOpt;

		initZipOptions( zOpt );

		ppFiles = new char*[nFiles];
		char** pIter = ppFiles;
		for(POSITION pos = lFileList.GetHeadPosition(); pos;)
			*(pIter++) = lFileList.GetNext(pos).GetBuffer(1);

		ZIPUSERFUNCTIONS zipFuncs;
		zipFuncs.comment = DummyComment;
		zipFuncs.password = password;
		zipFuncs.print = DisplayBuf;
		zipFuncs.ServiceApplication = ServiceCallBack;

		bool bSave, bNew, bRestrict;
		CString sPassword = GetDocument()->getPassword( bSave, bNew, bRestrict );
		if( !sPassword.IsEmpty() && bNew )
		{
			// stash stuff for call back
			indexParams.pIndex = thisApp()->getDocument()->getIndex();
			zOpt.fEncrypt = TRUE;
		}

		if( !ZpSetOptions(&zOpt) || !ZpInit(&zipFuncs) )
		{
			delete ppFiles;
			return false;
		}

		ZCL zcl;
		zcl.argc = lFileList.GetCount();
		zcl.FNV = ppFiles;
		zcl.lpszZipFN = sTargetZip.GetBuffer(1);

		if( ZpArchive(zcl) != 0 )
		{
			delete ppFiles;
			return false;
		}

		delete ppFiles;
		return true;
	}
	catch( ... )
	{
		delete ppFiles;
		return false;
	}
}


